/**
 * @addtogroup KAL
 * @{
 * 
 * @brief Defines the kernel adaptation layer (KAL), which provides compatible interfaces across different types of HarmonyOS devices.
 * 
 *
 * @since 1.0
 * @version 1.0
 */

/**
 * @file kal.h
 * 
 * @brief Defines the kernel adaptation layer (KAL), which provides compatible interfaces across different types of HarmonyOS devices.
 *
 * 
 * @since 1.0
 * @version 1.0
 */
 
#ifndef KAL_H_
#define KAL_H_

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Indicates the maximum length of a thread name.
 */
#define KAL_TASK_NAME_LEN 32

typedef void (*KalTimerProc)(void *arg);
typedef void *KalTimerId;

/**
 * @brief Enumerates timer types.
 */
typedef enum {
    /** One-shot timer */
    KAL_TIMER_ONCE = 0,
    /** Repetitive timer */
    KAL_TIMER_PERIODIC = 1
} KalTimerType;

/**
 * @brief Enumerates return values of the KAL function.
 */
typedef enum {
    /** A successful execution */
    KAL_OK = 0,
    /** Incorrect parameter */
    KAL_ERR_PARA = 1,
    /** Internal execution error */
    KAL_ERR_INNER = 2,
    /** Timer status error */
    KAL_ERR_TIMER_STATE = 0x100,
} KalErrCode;

/**
 * @brief Describes a thread.
 */
typedef struct {
    /** Thread name */
    char name[KAL_TASK_NAME_LEN];
    /** Thread ID */
    unsigned int id;
    /** Thread status */
    unsigned short status;
    /** Thread priority */
    unsigned short priority;
    /** Thread semaphore */
    void *taskSem;
    /** Thread mutex */
    void *taskMutex;
    /** Thread event */
    unsigned int eventStru[3];
    /** Thread event mask */
    unsigned int eventMask;
    /** Thread stack size */
    unsigned int stackSize;
    /** Thread stack top */
    unsigned int topOfStack;
    /** Thread stack bottom */
    unsigned int bottomOfStack;
    /** Current thread status */
    unsigned int mstatus;
    /** Current thread MEPC */
    unsigned int mepc;
    /** Current thread TP */
    unsigned int tp;
    /** Current thread RA */
    unsigned int ra;
    /** Thread stack pointer */
    unsigned int sp;
    /** Current stack space usage */
    unsigned int currUsed;
    /** Peak stack space usage */
    unsigned int peakUsed;
    /** Thread stack overflow flag */
    unsigned int overflowFlag;
} ThreadInfo;

/**
 * @brief Describes a memory pool.
 */
typedef struct {
    /** Total space of the memory pool, in bytes */
    unsigned int total;
    /** Used space of the memory pool, in bytes */
    unsigned int used;
    /** Available space of the memory pool, in bytes */
    unsigned int free;
    /** Number of available nodes in the memory pool */
    unsigned int freeNodeNum;
    /** Number of used nodes in the memory pool */
    unsigned int usedNodeNum;
    /** Size of the largest available node in the memory pool, in bytes */
    unsigned int maxFreeNodeSize;
    /** Number of memory allocation failures */
    unsigned int mallocFailCount;
    /** Peak memory usage of the memory pool */
    unsigned int peekSize;
    /** Total space of the small memory pool, in bytes */
    unsigned int totalLmp;
    /** Used space of the small memory pool, in bytes */
    unsigned int usedLmp;
    /** Available space of the small memory pool, in bytes */
    unsigned int freeLmp;
}MemInfo;

/**
 * @brief Obtains thread information.
 *
 * @param threadId Indicates the thread ID.
 * @param info Indicates the pointer to the buffer for storing the obtained thread information.
* @return Returns <b>0</b> if the thread information is obtained; returns a negative value otherwise.
 * @since 1.0
 * @version 1.0
 */
unsigned int KalThreadGetInfo(unsigned int threadId, ThreadInfo *info);

/**
 * @brief Delays a thread.
 *
 * The delay unit is microsecond. The actual delay precision can reach only the tick level.
 *
 * @param us Indicates the number of microseconds to delay. The actual delay precision is one tick.
 * @since 1.0
 * @version 1.0
 */
void KalDelayUs(unsigned int us);

/**
 *@brief Creates a timer.
 *
 * @param func Indicates the entry to timer callback. The callback is triggered when the timer expires.
 * @param type Indicates the timer type.
 * @param arg Indicates the pointer to the argument used in timer callback.
 * @param ticks Indicates the duration of the timer to create.
 * @return Returns the timer ID; returns <b>NULL</b> in the case of an error.
 * @since 1.0
 * @version 1.0
 */
KalTimerId KalTimerCreate(KalTimerProc func, KalTimerType type, void *arg, unsigned int ticks);

/**
 * @brief Starts a timer.
 *
 * @param timerId Indicates the ID of the timer to start, which is the value returned by {@link KalTimerCreate}.
 * @return Returns an enumerated value defined by {@link KalErrCode}, where {@link KAL_OK} indicates a success.
 * @since 1.0
 * @version 1.0
 */
KalErrCode KalTimerStart(KalTimerId timerId);

/**
 * @brief Changes the duration of a timer.
 *
 * The timer duration can be changed only before the timer is started.
 *
 * @param timerId Indicates the ID of the timer whose duration is to change, which is the value returned by {@link KalTimerCreate}.
 * @param ticks Indicates the new duration of the timer.
 * @return Returns an enumerated value defined by {@link KalErrCode}, where {@link KAL_OK} indicates a success.
 * @since 1.0
 * @version 1.0
 */
KalErrCode KalTimerChange(KalTimerId timerId, unsigned int ticks);

/**
 * @brief Stops a timer.
 *
 * @param timerId Indicates the ID of the timer to stop, which is the value returned by {@link KalTimerCreate}.
 * @return Returns an enumerated value defined by {@link KalErrCode}, where {@link KAL_OK} indicates a success.
 * @since 1.0
 * @version 1.0
 */
KalErrCode KalTimerStop(KalTimerId timerId);

/**
 * @brief Deletes a timer.
 *
 * @param timerId Indicates the ID of the timer to delete, which is the value returned by {@link KalTimerCreate}.
 * @return Returns an enumerated value defined by {@link KalErrCode}, where {@link KAL_OK} indicates a success.
 * @since 1.0
 * @version 1.0
 */
KalErrCode KalTimerDelete(KalTimerId timerId);

/**
 * @brief Checks whether a timer is running.
 *
 * @param timerId Indicates the ID of the timer to check, which is the value returned by {@link KalTimerCreate}.
 * @return Returns <b>1</b> if the timer is running; returns <b>0</b> if the timer is not running or the input parameter is incorrect.
 * @since 1.0
 * @version 1.0
 */
unsigned int KalTimerIsRunning(KalTimerId timerId);

/**
 * @brief Converts system ticks into milliseconds.
 *
 * @param ticks Indicates the number of ticks to convert.
 * @return Returns the milliseconds generated.
 * @since 1.0
 * @version 1.0
 */
unsigned int KalTickToMs(unsigned int ticks);

/**
 * @brief Converts milliseconds into system ticks.
 *
 * @param millisec Indicates the milliseconds to convert.
 * @return Returns the system ticks generated.
 * @since 1.0
 * @version 1.0
 */
unsigned int KalMsToTick(unsigned int millisec);

/**
 * @brief Obtains memory information.
 *
 * @param pmemInfo Indicates the pointer to the memory information.
 * @return Returns an enumerated value defined by {@link KalErrCode}, where {@link KAL_OK} indicates a success.
 * @since 1.0
 * @version 1.0
 */
KalErrCode KalGetMemInfo(MemInfo *pmemInfo);

#ifdef __cplusplus
}
#endif

#endif // KAL_H_

/* * @} */
