/**
* @addtogroup NET
* @{
*
* @brief Provides functions and data structures related to network operations.
*
* @since 1.0
* @version 1.0
*/

/**
* @file inet.h
*
* @brief Provides functions and data structures related to network operations.
*
* For example, you can use the functions to send and receive network data, manage network addresses, and convert bytes.
*
* @since 1.0
* @version 1.0
*/
#ifndef _ARPA_INET_H
#define	 _ARPA_INET_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Converts an integer from the host byte order to the network byte order.
*
* @param n Indicates the integer in host byte order.
* @return Returns the original data if the system is in big-endian mode; returns the converted data otherwise.
* @since 1.0
* @version 1.0
*/
uint32_t htonl(uint32_t n);

/**
* @brief Converts a 16-bit integer from the host byte order to the network byte order.
* @param n Indicates the integer in host byte order.
* @return Returns the original data if the system is in big-endian mode; returns the converted data otherwise.
* @since 1.0
* @version 1.0
*/
uint16_t htons(uint16_t n);

/**
* @brief Converts an integer from the network byte order to the host byte order.
* @param n Indicates the integer in network byte order.
* @return Returns the original data if the system is in big-endian mode; returns the converted data otherwise.
* @since 1.0
* @version 1.0
*/
uint32_t ntohl(uint32_t n);

/**
* @brief Converts a 16-bit integer from the network byte order to the host byte order.
* @param n Indicates the integer in network byte order.
* @return Returns the original data if the system is in big-endian mode; returns the converted data otherwise.
* @since 1.0
* @version 1.0
*/
uint16_t ntohs(uint16_t n);

/**
* @brief Converts a string from the IPv4 numbers-and-dots notation to the binary data in network byte order.
* @param p Indicates the pointer to the string to be converted.
* @return Returns the converted data if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
in_addr_t inet_addr(const char *p);

/**
* @brief Converts a string from the IPv4 numbers-and-dots notation to the binary data in host byte order.
* @param p Indicates the pointer to the string to be converted.
* @return Returns the converted data if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
in_addr_t inet_network(const char *p);

/**
* @brief Converts a network address to a string in dotted-decimal format.
* @param in Indicates the network address to be converted.
* @return Returns the string.
* @since 1.0
* @version 1.0
*/
char *inet_ntoa(struct in_addr in);

/**
* @brief Converts a string to a network address in the specified address family.
* @param af Indicates the address family. Currently, only <b>AF_INET</b> and <b>AF_INET6</b> are supported.
* @param s Indicates the pointer to the string to be converted.
* @param a0 Indicates the pointer to the converted data.
* @return Returns <b>1</b> if the operation is successful; returns <b>0</b> if the input address family is invalid; returns <b>-1</b> if the operation fails. The error codes are set as follows:
* errno          | Description
* -------------- | ---------------------------------
* EAFNOSUPPORT   | Unsupported address family
* @since 1.0
* @version 1.0
*/
int inet_pton(int af, const char *__restrict s, void *__restrict a0);

/**
* @brief Converts a network address in the specified address family to a string.
* @param af Indicates the address family. Currently, only <b>AF_INET</b> and <b>AF_INET6</b> are supported.
* @param a0 Indicates the pointer to the data to be converted.
* @param s Indicates the pointer to the string to be converted.
* @param l Indicates the size of the string to be converted.
* @return Returns the converted string if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* ENOSPC         | Insufficient space for conversion
* EAFNOSUPPORT   | Unsupported address family
* @since 1.0
* @version 1.0
*/
const char *inet_ntop(int af, const void *restrict a0, char *restrict s, socklen_t l);

/**
* @brief Converts an IP address from the string format to the 32-bit binary format in network byte order.
* @param s0 Indicates the pointer to the IP address to be converted.
* @param dest Indicates the pointer to the structure for conversion.
* @return Returns <b>1</b> if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int inet_aton(const char *s0, struct in_addr *dest);

/**
* @brief Converts the network number and host address to the network address.
* @param n Indicates the network number.
* @param h Indicates the host address.
* @return Returns the converted network address.
* @since 1.0
* @version 1.0
*/
struct in_addr inet_makeaddr(in_addr_t n, in_addr_t h);

/**
* @brief Converts an IP address into a host ID in host byte order without network bits.
* @param in Indicates the network address structure.
* @return Returns the converted host address.
* @since 1.0
* @version 1.0
*/
in_addr_t inet_lnaof(struct in_addr in);

/**
* @brief Extracts the network number from the <b>in_addr</b> structure and converts it to the host byte order.
*
* @param in Indicates the network address structure.
* @return Returns the converted network number.
* @since 1.0
* @version 1.0
*/
in_addr_t inet_netof(struct in_addr in);

#ifdef __cplusplus
}
#endif

#endif/** @} */