/**
* @addtogroup MATH
* @{
*
* @brief Provides math functions, such as trigonometric, exponential, logarithmic functions,and perform operations on complex numbers.
*
* @since 1.0
* @version 1.0
*/

/**
* @file complex.h
*
* @brief Declares functions to perform operations on complex numbers.
*
* You can use the functions provided in this file to perform operations on complex numbers during development. The functions include complex trigonometric, inverse trigonometric, hyperbolic, inverse hyperbolic, exponential, and logarithmic functions, and the functions to obtain the real and imaginary parts of a complex number.  \n
* @since 1.0
* @version 1.0
*/

#ifndef _COMPLEX_H
#define _COMPLEX_H

#ifdef __cplusplus
extern "C" {
#endif

/**@brief Indicates the type of a complex number. */
#define complex _Complex

/**
* @brief Calculates the arc cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose arc cosine is to be obtained.
* @return Returns the arc cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link M_PI_2} - 0i</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link M_PI_2} + {@link NAN}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is an infinity, this function returns <b>{@link M_PI_2} - {@link HUGE_VAL}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI} - {@link HUGE_VAL}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>3*{@link M_PI_4} - {@link HUGE_VAL}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_4} - {@link HUGE_VAL}i</b>. \n
* If <b>x</b> is a positive or negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link HUGE_VAL}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} - {@link HUGE_VAL}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex cacos(double complex z);

/**
* @brief Calculates the arc cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose arc cosine is to be obtained.
* @return Returns the arc cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link M_PI_2} - 0i</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link M_PI_2} + {@link NAN}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is an infinity, this function returns <b>{@link M_PI_2}-{@link HUGE_VALF}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI} - {@link HUGE_VALF}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>3*{@link M_PI_4} - {@link HUGE_VALF}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_4} - {@link HUGE_VALF}i</b>. \n
* If <b>x</b> is a positive or negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link HUGE_VALF}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} - {@link HUGE_VALF}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex cacosf(float complex z);

/**
* @brief Calculates the arc cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose arc cosine is to be obtained.
* @return Returns the arc cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link M_PI_2} - 0i</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link M_PI_2} + {@link NAN}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is an infinity, this function returns <b>{@link M_PI_2}-{@link HUGE_VALL}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI} - {@link HUGE_VALL}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>3*{@link M_PI_4} - {@link HUGE_VALL}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_4} - {@link HUGE_VALL}i</b>. \n
* If <b>x</b> is a positive or negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link HUGE_VALL}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} - {@link HUGE_VALL}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex cacosl(long double complex z);

/**
* @brief Calculates the arc sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1)
* @param z Indicates the complex number (a double value) whose arc sine is to be obtained.
* @return Returns the arc sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL}</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VAL} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex casin(double complex z);

/**
* @brief Calculates the arc sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1)
* @param z Indicates the complex number (a float value) whose arc sine is to be obtained.
* @return Returns the arc sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF}</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VALF} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex casinf(float complex z);

/**
* @brief Calculates the arc sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1)
* @param z Indicates the complex number (a long double value) whose arc sine is to be obtained.
* @return Returns the arc sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL}</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VALL} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex casinl(long double complex z);

/**
* @brief Calculates the arc tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose arc tangent is to be obtained.
* @return Returns the arc tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is <b>1</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link HUGE_VAL}</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex catan(double complex z);

/**
* @brief Calculates the arc tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose arc tangent is to be obtained.
* @return Returns the arc tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is <b>1</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link HUGE_VALF}</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex catanf(float complex z);

/**
* @brief Calculates the arc tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose arc tangent is to be obtained.
* @return Returns the arc tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is <b>1</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link HUGE_VALL}</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex catanl(long double complex z);

/**
* @brief Calculates the cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose cosine is to be obtained.
* @return Returns the cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns {@link HUGE_VAL}. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns {@link NAN}. \n
* If <b>x</b> is a negative infinity and <b>y</b> is <b>0</b>, this function returns {@link NAN} and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive or negative infinity, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex ccos(double complex z);

/**
* @brief Calculates the cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose cosine is to be obtained.
* @return Returns the cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns {@link HUGE_VALF}. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns {@link NAN}. \n
* If <b>x</b> is a negative infinity and <b>y</b> is <b>0</b>, this function returns {@link NAN} and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive or negative infinity, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex ccosf(float complex z);

/**
* @brief Calculates the cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose cosine is to be obtained.
* @return Returns the cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns {@link HUGE_VALL}. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns {@link NAN}. \n
* If <b>x</b> is a negative infinity and <b>y</b> is <b>0</b>, this function returns {@link NAN} and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive or negative infinity, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex ccosl(long double complex z);

/**
* @brief Calculates the sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose sine is to be obtained.
* @return Returns the sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VAL}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex csin(double complex z);

/**
* @brief Calculates the sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose sine is to be obtained.
* @return Returns the sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALF}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-) {@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex csinf(float complex z);

/**
* @brief Calculates the sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose sine is to be obtained.
* @return Returns the sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALL}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex csinl(long double complex z);

/**
* @brief Calculates the tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose tangent is to be obtained.
* @return Returns the tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>1</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>1</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex ctan(double complex z);

/**
* @brief Calculates the tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose tangent is to be obtained.
* @return Returns the tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>1</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>1</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex ctanf(float complex z);

/**
* @brief Calculates the tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose tangent is to be obtained.
* @return Returns the tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>1</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>1</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex ctanl(long double complex z);

/**
* @brief Calculates the inverse hyperbolic cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose inverse hyperbolic cosine is to be obtained.
* @return Returns the inverse hyperbolic cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link M_PI_2}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL} + {@link M_PI}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL}</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + 3*{@link M_PI_4}i</b>. \n
* If <b>x</b> is an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex cacosh(double complex z);

/**
* @brief Calculates the inverse hyperbolic cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose inverse hyperbolic cosine is to be obtained.
* @return Returns the inverse hyperbolic cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link M_PI_2}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF} + {@link M_PI}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF}</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + 3*{@link M_PI_4}i</b>. \n
* If <b>x</b> is an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex cacoshf(float complex z);

/**
* @brief Calculates the inverse hyperbolic cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose inverse hyperbolic cosine is to be obtained.
* @return Returns the inverse hyperbolic cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link M_PI_2}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL} + {@link M_PI}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL}</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + 3*{@link M_PI_4}i</b>. \n
* If <b>x</b> is an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex cacoshl(long double complex z);

/**
* @brief Calculates the inverse hyperbolic sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose inverse hyperbolic sine is to be obtained.
* @return Returns the inverse hyperbolic sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL}</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VAL} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex casinh(double complex z);

/**
* @brief Calculates the inverse hyperbolic sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose inverse hyperbolic sine is to be obtained.
* @return Returns the inverse hyperbolic sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF}</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VALF} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex casinhf(float complex z);

/**
* @brief Calculates the inverse hyperbolic sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose inverse hyperbolic sine is to be obtained.
* @return Returns the inverse hyperbolic sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL}</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VALL} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex casinhl(long double complex z);

/**
* @brief Calculates the inverse hyperbolic tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose inverse hyperbolic tangent is to be obtained.
* @return Returns the inverse hyperbolic tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is <b>1</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link HUGE_VAL}</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex catanh(double complex z);

/**
* @brief Calculates the inverse hyperbolic tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose inverse hyperbolic tangent is to be obtained.
* @return Returns the inverse hyperbolic tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is <b>1</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link HUGE_VALF}</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex catanhf(float complex z);

/**
* @brief Calculates the inverse hyperbolic tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose inverse hyperbolic tangent is to be obtained.
* @return Returns the inverse hyperbolic tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is <b>1</b> and <b>y</b> is <b>0</b>, this function returns <b>{@link HUGE_VALL}</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is {@link NAN}, this returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link M_PI_2}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex catanhl(long double complex z);

/**
* @brief Calculates the hyperbolic cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose hyperbolic cosine is to be obtained.
* @return Returns the hyperbolic cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns {@link NAN}. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VAL}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive or negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex ccosh(double complex z);

/**
* @brief Calculates the hyperbolic cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose hyperbolic cosine is to be obtained.
* @return Returns the hyperbolic cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns {@link NAN}. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALF}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive or negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex ccoshf(float complex z);

/**
* @brief Calculates the hyperbolic cosine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose hyperbolic cosine is to be obtained.
* @return Returns the hyperbolic cosine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns {@link NAN}. \n
* If <b>x</b> is neither an infinity nor <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALL}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive infinity, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive or negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex ccoshl(long double complex z);

/**
* @brief Calculates the hyperbolic sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose hyperbolic sine is to be obtained.
* @return Returns the hyperbolic sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VAL}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex csinh(double complex z);

/**
* @brief Calculates the hyperbolic sine of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose hyperbolic sine is to be obtained.
* @return Returns the hyperbolic sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALF}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-) {@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex csinhf(float complex z);

/**
* @brief Calculates the hyperbolic sine of the complex number <b>z</b>.
*
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose hyperbolic sine is to be obtained.
* @return Returns the hyperbolic sine of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is a positive infinity, this function returns <b>{@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN}i</b>. \n
* If <b>x</b> is a positive finite value and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive finite value and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALL}. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex csinhl(long double complex z);

/**
* @brief Calculates the hyperbolic tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose hyperbolic tangent is to be obtained.
* @return Returns the hyperbolic tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>1</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>1</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex ctanh(double complex z);

/**
* @brief Calculates the hyperbolic tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose hyperbolic tangent is to be obtained.
* @return Returns the hyperbolic tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>1</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>1</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex ctanhf(float complex z);

/**
* @brief Calculates the hyperbolic tangent of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose hyperbolic tangent is to be obtained.
* @return Returns the hyperbolic tangent of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>1</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>1</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex ctanhl(long double complex z);

/**
* @brief Calculates the base-e exponential of the complex number <b>z</b>.
*
* The mathematical formula is e^z. z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose base-e exponential is to be obtained.
* @return Returns the base-e exponential of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VAL}. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an infinity or {@link NAN}, this function returns <b>0</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link HUGE_VAL}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-){@link HUGE_VAL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex cexp(double complex z);

/**
* @brief Calculates the base-e exponential of the complex number <b>z</b>.
*
* The mathematical formula is e^z. z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose base-e exponential is to be obtained.
* @return Returns the base-e exponential of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALF}. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an infinity or {@link NAN}, this function returns <b>0</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link HUGE_VALF}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-) {@link HUGE_VALF} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex cexpf(float complex z);

/**
* @brief Calculates the base-e exponential of the complex number <b>z</b>.
*
* The mathematical formula is e^z. z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose base-e exponential is to be obtained.
* @return Returns the base-e exponential of <b>z</b> if the operation is successful. \n
* If <b>x</b> and <b>y</b> are both <b>0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is <b>0</b>, this function returns {@link HUGE_VALL}. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an infinity or {@link NAN}, this function returns <b>0</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is neither an infinity nor <b>0</b>, this function returns <b>{@link HUGE_VALL}*({@link cos}(y) + i{@link sin}(y))</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain) and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>+(-){@link HUGE_VALL} + {@link NAN}i</b> (the sign bit of the real part of the returned value is uncertain). \n
* If <b>x</b> is {@link NAN} and <b>y</b> is <b>0</b>, this function returns {@link NAN}. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not <b>0</b>, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b> and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex cexpl(long double complex z);

/**
* @brief Calculates the natural (base-e) logarithm of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose natural (base-e) logarithm is to be obtained.
* @return Returns the natural (base-e) logarithm of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>-0</b> and <b>y</b> is <b>+0</b>, this function returns <b>-{@link HUGE_VAL} + {@link M_PI}i</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> and <b>y</b> are both <b>+0</b>, this function returns <b>-{@link HUGE_VAL} + 0i</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL} + {@link M_PI}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + 3*{@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL} + 0i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VAL} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex clog(double complex z);

/**
* @brief Calculates the natural (base-e) logarithm of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose natural (base-e) logarithm is to be obtained.
* @return Returns the natural (base-e) logarithm of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>-0</b> and <b>y</b> is <b>+0</b>, this function returns <b>-{@link HUGE_VALF} + {@link M_PI}i</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> and <b>y</b> are both <b>+0</b>, this function returns <b>-{@link HUGE_VALF} + 0i</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF} + {@link M_PI}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + 3*{@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF} + 0i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VALF} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex clogf(float complex z);

/**
* @brief Calculates the natural (base-e) logarithm of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose natural (base-e) logarithm is to be obtained.
* @return Returns the natural (base-e) logarithm of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>-0</b> and <b>y</b> is <b>+0</b>, this function returns <b>-{@link HUGE_VALL} + {@link M_PI}i</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> and <b>y</b> are both <b>+0</b>, this function returns <b>-{@link HUGE_VALL} + 0i</b> and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is not an infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + {@link M_PI_2}i</b>. \n
* If <b>x</b> is not an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL} + {@link M_PI}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + 3*{@link M_PI_4}i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL} + 0i</b>. \n
* If <b>x</b> and <b>y</b> are both positive infinity, this function returns <b>{@link HUGE_VALL} + {@link M_PI_4}i</b>. \n
* If <b>x</b> is an infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is not an infinity, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + {@link NAN}i</b>. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex clogl(long double complex z);

/**
* @brief Calculates the absolute value of the complex number <b>z</b>.
*
* @attention If an error occurs or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link hypot}({@link cimag}(z) + {@link creal}(z)).
* @param z Indicates the complex number (a double value) whose absolute value is to be obtained.
* @return Returns the absolute value of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
double cabs(double complex z);

/**
* @brief Calculates the absolute value of the complex number <b>z</b>.
*
* @attention If an error occurs or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link hypot}({@link cimag}(z) + {@link creal}(z)).
* @param z Indicates the complex number (a float value) whose absolute value is to be obtained.
* @return Returns the absolute value of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
float cabsf(float complex z);

/**
* @brief Calculates the absolute value of the complex number <b>z</b>.
*
* @attention If an error occurs or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link hypot}({@link cimag}(z) + {@link creal}(z)).
* @param z Indicates the complex number (a long double value) whose absolute value is to be obtained.
* @return Returns the absolute value of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
long double cabsl(long double complex z);

/**
* @brief Calculates the value of <b>x</b> raised to the <b>z</b> power.
*
* The mathematical formula is x^z.  \n
* @attention If an error occurs or <b>x</b> or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link cexp}(z*{@link clog}(x)).
* @param x Indicates the base (a double complex number).
* @param z Indicates the exponent (a double complex number).
* @return Returns the value of <b>x</b> raised to the <b>z</b> power if the operation is successful.
* @since 1.0
* @version 1.0
*/
double complex cpow(double complex x, double complex z);

/**
* @brief Calculates the value of <b>x</b> raised to the <b>z</b> power.
*
* The mathematical formula is x^z.  \n
* @attention If an error occurs or <b>x</b> or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link cexp}(z*{@link clog}(x)).
* @param x Indicates the base (a float complex number).
* @param z Indicates the exponent (a float complex number).
* @return Returns the value of <b>x</b> raised to the <b>z</b> power if the operation is successful.
* @since 1.0
* @version 1.0
*/
float complex cpowf(float complex x, float complex z);

/**
* @brief Calculates the value of <b>x</b> raised to the <b>z</b> power.
*
* The mathematical formula is x^z.  \n
* @attention If an error occurs or <b>x</b> or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link cexp}(z*{@link clog}(x)).
* @param x Indicates the base (a long double complex number).
* @param z Indicates the exponent (a long double complex number).
* @return Returns the value of <b>x</b> raised to the <b>z</b> power if the operation is successful.
* @since 1.0
* @version 1.0
*/
long double complex cpowl(long double complex x, long double complex z);

/**
* @brief Calculates the square root of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose square root is to be obtained.
* @return Returns the square root of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is <b>+0</b>, this function returns <b>+0 + 0i</b>. \n
* If <b>x</b> is neither an infinity nor {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VAL} + {@link HUGE_VAL}i</b>. \n
* If <b>x</b> is a finite value or {@link NAN} and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link HUGE_VAL}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VAL} + 0i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VAL} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is any value, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
double complex csqrt(double complex z);

/**
* @brief Calculates the square root of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose square root is to be obtained.
* @return Returns the square root of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is <b>+0</b>, this function returns <b>+0 + 0i</b>. \n
* If <b>x</b> is neither an infinity nor {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALF} + {@link HUGE_VALF}i</b>. \n
* If <b>x</b> is a finite value or {@link NAN} and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link HUGE_VALF}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALF} + 0i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALF} + {@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is any value, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
float complex csqrtf(float complex z);

/**
* @brief Calculates the square root of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose square root is to be obtained.
* @return Returns the square root of <b>z</b> if the operation is successful. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is <b>+0</b>, this function returns <b>+0 + 0i</b>. \n
* If <b>x</b> is neither an infinity nor {@link NAN} and <b>y</b> is a positive infinity, this function returns <b>{@link HUGE_VALL} + {@link HUGE_VALL}i</b>. \n
* If <b>x</b> is a finite value or {@link NAN} and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL}i</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link NAN} +(-) {@link HUGE_VALL}i</b> (the sign bit of the imaginary part of the returned value is uncertain). \n
* If <b>x</b> is a positive infinity and <b>y</b> is a positive finite value, this function returns <b>{@link HUGE_VALL} + 0i</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is {@link NAN}, this function returns <b>{@link HUGE_VALL}+{@link NAN}i</b>. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is any value, this function returns <b>{@link NAN} + {@link NAN}i</b> and may throw the {@link FE_INVALID} exception. \n
* If <b>x</b> and <b>y</b> are both {@link NAN}, this function returns <b>{@link NAN} + {@link NAN}i</b>. \n
* @since 1.0
* @version 1.0
*/
long double complex csqrtl(long double complex z);

/**
* @brief Calculates the phase angle of the complex number <b>z</b>.
*
* @attention If an error occurs or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link atan2}({@link cimag}(z) + {@link creal}(z)).
* @param z Indicates the complex number (a double value) whose phase angle is to be obtained.
* @return Returns the phase angle of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
double carg(double complex z);

/**
* @brief Calculates the phase angle of the complex number <b>z</b>.
*
* @attention If an error occurs or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link atan2}({@link cimag}(z) + {@link creal}(z)).
* @param z Indicates the complex number (a float value) whose phase angle is to be obtained.
* @return Returns the phase angle of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
float cargf(float complex z);

/**
* @brief Calculates the phase angle of the complex number <b>z</b>.
*
* @attention If an error occurs or <b>z</b> is a positive or negative infinity or {@link NAN}, the returned value is calculated based on the formula {@link atan2}({@link cimag}(z) + {@link creal}(z)).
* @param z Indicates the complex number (a long double value) whose hyperbolic sine is to be obtained.
* @return Returns the phase angle of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
long double cargl(long double complex z);

/**
* @brief Calculates the imaginary part of the complex number <b>z</b>.
*
* \n
* @param z Indicates the complex number (a double value) whose imaginary part is to be obtained.
* @return Returns the imaginary part of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
double cimag(double complex z);

/**
* @brief Calculates the imaginary part of the complex number <b>z</b>.
*
* \n
* @param z Indicates the complex number (a float value) whose imaginary part is to be obtained.
* @return Returns the imaginary part of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
float cimagf(float complex z);

/**
* @brief Calculates the imaginary part of the complex number <b>z</b>.
*
* \n
* @param z Indicates the complex number (a long double value) whose imaginary part is to be obtained.
* @return Returns the imaginary part of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
long double cimagl(long double complex z);

/**
* @brief Calculates the conjugate of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose conjugate is to be obtained.
* @return Returns the conjugate of <b>z</b>, which is equal to x - yi.
* @since 1.0
* @version 1.0
*/
double complex conj(double complex z);

/**
* @brief Calculates the conjugate of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a float value) whose conjugate is to be obtained.
* @return Returns the conjugate of <b>z</b>, which is equal to x - yi.
* @since 1.0
* @version 1.0
*/
float complex conjf(float complex z);

/**
* @brief Calculates the conjugate of the complex number <b>z</b>.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose conjugate is to be obtained.
* @return Returns the conjugate of <b>z</b>, which is equal to x - yi.
* @since 1.0
* @version 1.0
*/
long double complex conjl(long double complex z);

/**
* @brief Calculates the projection of the complex number <b>z</b> onto the Riemann sphere.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a double value) whose projection onto the Riemann sphere is to be obtained.
* @return Returns the projection of <b>z</b> onto the Riemann sphere if the operation is successful. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is an infinity, this function returns {@link HUGE_VAL}. \n
* If <b>x</b> is an infinity, this function returns {@link HUGE_VAL} + i*{@link copysign}(0.0, {@link cimag}(z)). \n
* @since 1.0
* @version 1.0
*/
double complex cproj(double complex z);

/**
* @brief Calculates the projection of the complex number <b>z</b> onto the Riemann sphere.
*
* @param z Indicates the complex number (a float value) whose projection onto the Riemann sphere is to be obtained. \n
* @return Returns the projection of <b>z</b> onto the Riemann sphere if the operation is successful. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is an infinity, this function returns {@link HUGE_VALF}. \n
* If <b>x</b> is an infinity, this function returns {@link HUGE_VALF} + i*{@link copysign}(0.0, {@link cimag}(z)). \n
* @since 1.0
* @version 1.0
*/
float complex cprojf(float complex z);

/**
* @brief Calculates the projection of the complex number <b>z</b> onto the Riemann sphere.
*
* z = x + yi, where i = {@link sqrt}(-1) \n
* @param z Indicates the complex number (a long double value) whose projection onto the Riemann sphere is to be obtained. \n
* @return Returns the projection of <b>z</b> onto the Riemann sphere if the operation is successful. \n
* If <b>x</b> is {@link NAN} and <b>y</b> is an infinity, this function returns {@link HUGE_VALL}. \n
* If <b>x</b> is an infinity, this function returns {@link HUGE_VALL} + i*{@link copysign}(0.0, {@link cimag}(z)). \n
* @since 1.0
* @version 1.0
*/
long double complex cprojl(long double complex z);

/**
* @brief Calculates the real part of the complex number <b>z</b>.
*
* @param z Indicates the complex number (a double value) whose real part is to be obtained.
* @return Returns the real part of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
double creal(double complex z);

/**
* @brief Calculates the real part of the complex number <b>z</b>.
*
* @param z Indicates the complex number (a float value) whose real part is to be obtained.
* @return Returns the real part of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
float crealf(float complex z);

/**
* @brief Calculates the real part of the complex number <b>z</b>.
*
* @param z Indicates the complex number (a long double value) whose real part is to be obtained.
* @return Returns the real part of <b>z</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
long double creall(long double complex z);

#ifdef __cplusplus
}
#endif
#endif
/** @} */