/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/


/**
* @file ctype.h
*
* @brief Provides functions used for parameter identification.
*
* You can use the functions in this file to perform the mathematical operations required during development, for example, check whether the passed parameter is an alphabetic character, a blank character, not a number (NaN), a control character, or a decimal digit. \n
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _CTYPE_H
#define	 _CTYPE_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Checks whether a parameter is an alphabetic character or a decimal digit.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is an alphabetic character or a decimal digit; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isalnum(int c);

/**
* @brief Checks whether a parameter is an alphabetic character.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is an alphabetic character; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isalpha(int c);

/**
* @brief Checks whether a parameter is a blank character (space or tap).
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a blank character; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isblank(int c);

/**
* @brief Checks whether a parameter is a control character. A control character is invisible and does not occupy a printing position on a display.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a control character; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int iscntrl(int c);

/**
* @brief Checks whether a parameter is a decimal digit (0-9).
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a decimal digit; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isdigit(int c);

/**
* @brief Checks whether a parameter is any printable character except the space character.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a printable character except the space character; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isgraph(int c);

/**
* @brief Checks whether a parameter is a lowercase letter.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a lowercase letter; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   islower(int c);

/**
* @brief Checks whether a parameter is a printable character (including space).
*
* A printable character is a character that occupies a printing position on a display. \n
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a printable character; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isprint(int c);

/**
* @brief Checks whether a parameter is a punctuation or special character.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a punctuation or special character; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   ispunct(int c);

/**
* @brief Checks whether a parameter is a space character.
*
* The space characters are: space, horizontal tab (<b>\t</b>), vertical tab (<b>\v</b>), form feed (<b>\f</b>), carriage return (<b>\r</b>), and line feed (<b>\n</b>).
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a space character; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isspace(int c);

/**
* @brief Checks whether a parameter is an uppercase letter.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is an uppercase letter; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isupper(int c);

/**
* @brief Checks whether a parameter is a hexadecimal digit.
*
* @param c Indicates the parameter to check.
* @return Returns a non-zero value if <b>c</b> is a hexadecimal digit; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isxdigit(int c);

/**
* @brief Converts an uppercase letter specified by <b>c</b> to its lowercase equivalent.
*
* @param c Indicates the parameter to convert.
* @return Returns the converted letter if the conversion is successful; returns <b>c</b> otherwise.
* @since  1.0
* @version  1.0
*/
int   tolower(int c);

/**
* @brief Converts a lowercase letter specified by <b>c</b> to its uppercase equivalent.
*
* @param c Indicates the parameter to convert.
* @return Returns the converted letter if the conversion is successful; returns <b>c</b> otherwise.
* @since  1.0
* @version  1.0
*/
int   toupper(int c);

/**
* @brief Checks whether a parameter is an alphabetic character or digit for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is an alphabetic character or digit; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isalnum_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is an alphabetic character for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is an alphabetic character; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isalpha_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a blank character (including spaces and tabs) for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a blank character; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isblank_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a control character for the specified locale.
*
* A control character is invisible and does not occupy a printing position on a display. \n
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a control character; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int iscntrl_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a decimal digit for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a decimal digit; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isdigit_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is any printable character except the space character for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a printable character except the space character; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
int isgraph_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a character of lowercase letters for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a lowercase letter; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   islower_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a printable character (including space) for the specified locale. A printable character is visible and occupies a printing position on a display.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a printable character; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isprint_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a punctuation or special character for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a punctuation or special character; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   ispunct_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a blank character for the specified locale.
*
* Blank characters refer to space, horizontal tab (<b>\t</b>), vertical tab (<b>\v</b>), form feed (<b>\f</b>), carriage return (<b>\r</b>), and line feed (<b>\n</b>).
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is a blank character; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isspace_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a character of uppercase letters for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if <b>c</b> is in uppercase; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isupper_l(int c, locale_t locale);

/**
* @brief Checks whether a parameter is a hexadecimal digit for the specified locale.
*
* @param c Indicates the parameter to check.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the parameter is a hexadecimal digit; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int   isxdigit_l(int c, locale_t locale);

/**
* @brief Converts an upper letter specified by <b>c</b> to its lowercase equivalent for the specified locale.
*
* @param c Indicates the parameter to convert.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the converted value if the conversion is successful; returns <b>c</b> unchanged otherwise.
* @since  1.0
* @version  1.0
*/
int   tolower_l(int c, locale_t locale);

/**
* @brief Converts a lowercase letter specified by <b>c</b> to its uppercase equivalent for the specified locale.
*
* @param c Indicates the parameter to convert.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the converted value if the conversion is successful; returns <b>c</b> unchanged otherwise.
* @since  1.0
* @version  1.0
*/
int   toupper_l(int c, locale_t locale);


/**
* @brief Converts a parameter of the integer type to an ASCII code.
*
* @attention The function clears all but the lower seven bits of <b>c</b>.
* @param c Indicates the parameter to convert.
* @return Returns the ASCII code if the conversion is successful.
* @since  1.0
* @version  1.0
*/
int   toascii(int c);

/**
* @brief Converts an uppercase letter to its lowercase equivalent.
*
* @param a Indicates the parameter to convert.
* @return Returns the converted lowercase letter if the conversion is successful.
* @since  1.0
* @version  1.0
*/
#define _tolower(a) ((a)|0x20)

/**
* @brief Converts a lowercase letter to its uppercase equivalent.
*
* @param a Indicates the parameter to convert.
* @return Returns the converted uppercase letter if the conversion is successful.
* @since  1.0
* @version  1.0
*/
#define _toupper(a) ((a)&0x5f)

/**
* @brief Checks whether a parameter is an ASCII character.
*
* @param a Indicates the parameter to check.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
#define isascii(a) (0 ? isascii(a) : (unsigned)(a) < 128)

#endif

#ifdef __cplusplus
}
#endif

#endif
/** @} */