/**
* @addtogroup FS
* @{
*
* @brief Provides functions for file system, include file operations, directory operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file dirent.h
*
* @brief Provides functions and structures used for performing file system operations, including file read/write, directory traversal, and file system mounting.
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _DIRENT_H
#define	 _DIRENT_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Opens the handle of the folder directory.
*/
typedef struct __dirstream DIR;

/**
* @brief Defines the content of the directory.
*/
struct dirent {
/** Inode number */
ino_t d_ino;
/** Offset in the directory stream */
off_t d_off;
/** File name length */
unsigned short d_reclen;
/** File type */
unsigned char d_type;
/** File name */
char d_name[256];
};

/**
* @brief Closes a specified directory.
*
* @param dirp Indicates the pointer to the structure representing the directory to close.
* @return Returns <b>0</b> if the directory is successfully closed; returns <b>-1</b> and sets <b>errno</b> to the following value otherwise.
* errno | Description
* ----- | ---------------------------
* EBADF | <b>dirp</b> is not a valid opened directory stream.
* @since 1.0
* @version 1.0
*/
int            closedir(DIR *dirp);

/**
* @brief Opens a specified directory.
*
* @param dirname Indicates the pointer to the name of directory to open.
* @return Returns the pointer to the <b>DIR</b> structure representing the directory if it is opened successfully; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | -----------------------------
* EINVAL | The <b>dirname</b> is empty.
* ENAMETOOLONG | The length of the path name string is longer than the value of {@link NAME_MAX}.
* ENOENT | The path component does not exist.
* ENOMEM | The storage memory is insufficient.
* ENOTDIR | <b> dirname </b> is not a directory.
* @since 1.0
* @version 1.0
*/
DIR           *opendir(const char *dirname);

/**
* @brief Reads a specified directory.
*
* @param dirp Indicates the pointer to the directory structure.
* @return Returns the pointer to the <b>dirent</b> structure representing the directory if it is read successfully; returns <b>NULL</b> if reaching the end of the directory stream or the directory fails to be read. If the directory fails to be read, the corresponding error code is returned.
* errno | Description
* ------------ | --------------------------
* EACCES | This feature is not supported by the file system.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOENT | The current position in the directory stream is invalid.
* EBADF | <b>dirp</b> is not an opened directory stream.
* @since 1.0
* @version 1.0
*/
struct dirent *readdir(DIR *dirp);

/**
* @brief Reads a specified directory (thread-safe version).
*
* @param dirp Indicates the pointer to the directory structure.
* @param entry Indicates the entry read from the <b>dirp</b> stream.
* @param result Indicates the pointer to the returned buffer.
* @return Returns <b>0</b>. If the directory fails to be read, <b>errno</b> is sets to a value in the following table.
* errno | Description
* ------------ | --------------------------
* EACCES | This feature is not supported by the file system.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOENT | The current position in the directory stream is invalid.
* EBADF | <b>dirp</b> is not an opened directory stream.
* @since 1.0
* @version 1.0
*/
int            readdir_r(DIR *__restrict dirp, struct dirent *__restrict entry, struct dirent **__restrict result);

/**
* @brief Resets a directory stream read position.
*
* This function is used to change the current read position of the specified directory stream to the original read position (that is, the start position after the directory stream is opened by {@link opendir}). \n
* @param dirp Indicates the pointer to the directory stream whose read position needs to be reset.
* errno | Description
* ----- | ---------------------
* EBADF | <b>dirp</b> is not an opened directory stream.
* @since 1.0
* @version 1.0
*/
void           rewinddir(DIR *dirp);

/**
* @brief Sorts the directory entries in alphabetical order.
*
* This function is used to sort the two directory entries passed through the input parameters in alphabetical order. Generally, these parameters are passed in {@link scandir} as its last parameter. \n
* @param a Indicates the pointer to directory entry a.
* @param a Indicates the pointer to directory entry b.
* @return Returns an integer greater than, equal to, or less than <b>0</b>, respectively, based on whether the name of the directory entry to which <b>a</b> points is lexically greater than, equal to, or less than the directory.
* @since 1.0
* @version 1.0
*/
int alphasort(const struct dirent **a, const struct dirent **b);

/**
* @brief Scans directory to find the target entries.
*
* @param dir Indicates the pointer to the directory to scan.
* @param namelist Indicates the array storing the directory entries filtered by the function referenced by the <b>sel</b> parameter.
* @param sel Indicates the filtering condition. You can customize the function referenced by this parameter. Value <b>0</b> is returned if the filtering condition is not met; a non-zero value is returned otherwise.
* @param compar Indicates the pointer to the function for sorting the directory entries.
* @return Returns the number of entries copied to the <b>namelist</b> array if the scanning is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | --------------------------
* EINVAL | <b>dir</b> is empty.
* ENAMETOOLONG | The length of the path name string is greater than the value of {@link NAME_MAX}.
* ENOENT | The directory component does not exist.
* ENOMEM | The storage memory is insufficient.
* ENOTDIR | <b>dir</b> is not a directory.
* EACCES | This feature is not supported by the file system.
* @since 1.0
* @version 1.0
*/
int scandir(const char *dir, struct dirent *** namelist, int (*sel)(const struct dirent *), int (*compar)(const struct dirent **, const struct dirent **));

/**
* @brief Sets the position of the next {@link readdir} call in the directory stream.
*
* @param dirp Indicates the pointer to the directory stream in which the position of the next {@link readdir} call needs to be set.
* @param offset Indicates the offset of the position relative to the current position.
* @since 1.0
* @version 1.0
*/
void           seekdir(DIR *dirp, long offset);

/**
* @brief Obtains the current position in the specified directory stream.
*
* @param dirp Indicates the pointer to the directory stream.
* @return Returns the current position in the directory stream if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to the value in the following table otherwise.
* errno | Description
* ----- | ---------------------
* EBADF | <b>drip</b> is not an opened directory stream.
* @since 1.0
* @version 1.0
*/
long           telldir(DIR *dirp);


#ifdef __cplusplus
}
#endif

#endif/** @} */