/**
* @addtogroup LINKER
* @{
*
* @brief Provides functions and structures related to linking and loading.
*
* @since 1.0
* @version 1.0
*/

/**
* @file dlfcn.h
*
* @brief Provides functions and structures related to linking and loading.
*
* This file provides functions and structures for opening and closing a dynamic-link library (DLL) and searching for the specified symbols based on the DLL.
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef	 _DLFCN_H
#define	 _DLFCN_H

#ifdef __cplusplus
extern "C" {
#endif


/**@brief All undefined symbols in the shared object are not resolved before {@link dlopen()} returns. */
#define RTLD_LAZY   1
/**@brief All undefined symbols in the shared object are resolved before {@link dlopen()} returns. */
#define RTLD_NOW    2
/**@brief The shared object is not loaded. It can be used to test whether the object resides in the memory. */
#define RTLD_NOLOAD 4
/**@brief The shared object is not deleted during {@link dlclose()}. */
#define RTLD_NODELETE 4096
/**@brief The symbols defined by the loaded shared object can be resolved by other shared objects opened later. */
#define RTLD_GLOBAL 256
/**@brief The symbols defined by the loaded shared object cannot be resolved by other shared objects opened later. */
#define RTLD_LOCAL  0
/**@brief The next match of the required symbol is searched in the search order after the current object. */
#define RTLD_NEXT    ((void *)-1)
/**@brief The first match of the required symbol is searched in the default shared object search order. The search scope includes the global symbols of the program and its dependencies, and the global symbols loaded by the {@link dlopen()} function. */
#define RTLD_DEFAULT ((void *)0)
/**@brief The pointer to the {@link link_map} structure of the specified handle. */
#define RTLD_DI_LINKMAP 2

/**
* @brief Closes the DLL of a specified handle.
*
* @attention The symbols associated with the handle may not be removed from the caller's address space even if the <b>dlclose()</b> function returns.
* The symbols are removed only when the reference count of the DLL is 0. The reference count is incremented (by 1) each time the <b>dlopen()</b> function is explicitly called.
* In addition, the reference of DLL depended on by other DLLs or programs will be counted if it has been implicitly loaded. A shared object can be removed from the address space only after all references are released.
*
* @param handle Indicates the loaded DLL handle.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value if the operation fails.
* @since 1.0
* @version 1.0
*/
int    dlclose(void *handle);

/**
* @brief Obtains the latest error that occurred during a DLL function execution failure.
*
* @return Returns the error if occurred; returns <b>NULL</b> if the DLL function execution is successful.
* @since 1.0
* @version 1.0
*/
char  *dlerror(void);

/**
* @brief Opens a DLL and returns its handle.
*
* @param filename Indicates the pointer to the file name of the DLL.
* @param flags Indicates the flag of the DLL to be loaded.
* @return Returns the DLL handle if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
void  *dlopen(const char *filename, int flags);

/**
* @brief Searches for a specified symbol based on the DLL handle.
*
* You can also use this function to obtain the loading addresses of global variables. \n
* @param handle Indicates the loaded DLL handle.
* @param symbol Indicates the symbol name.
* @return Returns the symbol address if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
void  *dlsym(void *handle , const char *symbol);

#ifdef __cplusplus
}
#endif

#endif

/** @} */