/**
* @addtogroup IO
* @{
*
* @brief Provides data structures and functions for performing input/output (I/O) operations, such as printf and scanf.
*
* @since 1.0
* @version 1.0
*/

/**
* @file err.h
*
* @brief Provides common functions for error messages.
*
* You can use the functions provided in this file to implement error output operations required during development. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _ERR_H
#define _ERR_H

#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Displays a formatted error message on the standard error (stderr) output.
*
* If the <b>fmt</b> parameter is not <b>NULL</b>, stderr displays the error code description obtained from strerror(errno), followed by a space and a colon (:), and the error message to print. \n
* @param fmt Indicates the pointer to the string to format.
* @param ... Indicates the parameters corresponding to the format declaration.
* @since  1.0
* @version  1.0
*/
void warn(const char *fmt, ...);

/**
* @brief Displays a formatted error message on the stderr output.
*
* If the <b>fmt</b> parameter is not <b>NULL</b>, stderr displays the error code description obtained from strerror(errno), followed by a space and a colon (:), and the error message to print. \n
* @param fmt Indicates the pointer to the string to format.
* @param args Indicates the parameters initialized by using {@link va_start}.
* @since  1.0
* @version  1.0
*/
void vwarn(const char *fmt, va_list args);

/**
* @brief Displays a formatted error message on the stderr output.
*
* If the <b>fmt</b> parameter is not <b>NULL</b>, stderr displays the error code description obtained from strerror(errno), followed by a space and a colon (:), and the error message to print. \n
* @param fmt Indicates the pointer to the string to format.
* @param ... Indicates the parameters corresponding to the format declaration.
* @since  1.0
* @version  1.0
*/
void warnx(const char *fmt, ...);

/**
* @brief Displays a formatted error message on the stderr output.
*
* If the <b>fmt</b> parameter is not <b>NULL</b>, stderr displays the error code description obtained from strerror(errno), followed by a space and a colon (:), and the error message to print. \n
* @param fmt Indicates the pointer to the string to format.
* @param args Indicates the parameters initialized by using {@link va_start}.
* @since  1.0
* @version  1.0
*/
void vwarnx(const char *fmt, va_list args);

/**
* @brief Displays error information.
*
* If the <b>fmt</b> parameter is not <b>NULL</b>, stderr displays the error code description obtained from strerror(errno), followed by a space and a colon (:), and the error message to print. \n
* @param eval This function does not return, but exits with the value of <b>eval</b>.
* @param fmt Indicates the pointer to the string to format.
* @param args Indicates the parameters initialized by using {@link va_start}.
* @since  1.0
* @version  1.0
*/
void err(int eval, const char *fmt, va_list args);

/**
* @brief Displays error information.
*
* This function outputs the executable file name <b>argv[0]</b> and the accepted second formal parameter to stderr. \n
* @param eval This function does not return, but exits with the value of <b>eval</b>.
* @param fmt Indicates the pointer to the string to format.
* @since  1.0
* @version  1.0
*/
void errx(int eval, const char *fmt, ...);

/**
* @brief Displays error information.
*
* This function outputs the executable file name <b>argv[0]</b> and the accepted second formal parameter to stderr. \n
* @param eval This function does not return, but exits with the value of <b>eval</b>.
* @param fmt Indicates the pointer to the string to format.
* @param args Indicates the parameters initialized by using {@link va_start}.
* @since  1.0
* @version  1.0
*/
void verr(int eval, const char *fmt, va_list args);

/**
* @brief Displays a formatted error message.
*
* @param eval This function does not return, but exits with the value of <b>eval</b>.
* @param fmt Indicates the pointer to the string to format.
* @param args Indicates the parameters initialized by using {@link va_start}.
* @since  1.0
* @version  1.0
*/
void verrx(int eval, const char *fmt, va_list args);

#ifdef __cplusplus
}
#endif

#endif
/** @} */