/**
* @addtogroup FS
* @{
*
* @brief Provides functions for file system, include file operations, directory operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file fcntl.h
*
* @brief Declares structures and functions used for file and directory operations such as opening a file, creating a file, and allocating space.
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _FCNTL_H
#define	 _FCNTL_H

#ifdef __cplusplus
extern "C" {
#endif

#include <features.h>

/**
* @brief Creates a file or rewrites an existing file.
*
* @param path Indicates the pointer to the name of the file to create or rewrite.
* @param mode Indicates permissions on the file.
* @return Returns <b>0</b> if the file is created or rewritten successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | -------------------------------------------------
* EINVAL | <b>path</b> points to an invalid path.
* ENAMETOOLONG | The length of <b>path</b> is greater than the value of {@link NAME_MAX}.
* ENXIO | Invalid inode or the inode is not a mount point.
* EMFILE | All file descriptors that can be used by the process are opened.
* EPERM | The file structure corresponding to the file descriptor cannot be obtained.
* EACCES | Access denied by the permission bit of file mode
* ENOMEM | The memory is insufficient.
* ENODEV | The device does not exist.
* EEXIST | {@link O_CREAT} and {@link O_EXCL} are set.
* EIO | A hardware error occurs at the low-level disk I/O layer. As a result, the physical drive cannot work.
* ENFILE | The number of opened files in the system has reached the maximum.
* ENOTDIR | A component of the path prefix is the name of an existing file and is not a directory.
* EBUSY | The operation is rejected according to the file sharing policy.
* ENOTEMPTY | The directory is not empty.
* ELOOP | Too many symbolic links are encountered.
* EFAULT | The IP address is incorrect.
* @since 1.0
* @version 1.0
*/
int creat(const char *path, mode_t mode);

/**
* @brief Opens a file.
*
* @attention This function can NOT be used in the PROC file system to create a file.
* This function is used to open a file. If the file does not exist, this function creates a file and opens it. \n
* @param path Indicates the pointer to the name of the file to open.
* @param oflags Indicates the file access mode. Available values are as follows:
* oflags | Description
* ----------- |-------------------------------------------------------
* O_RDONLY | Read-only
* O_WRONLY | Write-only
* O_RDWR | Read and write
* O_APPEND | If set, the file offset will be set to the end of the file prior to each write.
* O_CREAT | If the file does not exist, it will be created.
* O_LARGEFILE | A file exceeding 2 GB can be created.
* O_TRUNC | If the file is opened successfully with {@link O_WRONLY} or {@link O_RDWR}, its length is truncated to 0.
* O_EXCL | Used together with {@link O_CREAT}. This function fails if the file exists.
* O_DIRECTORY | This flag must be added for opening a directory. Otherwise, the directory fails to be opened.
* @return Returns the file descriptor if the file is opened successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | -----------------------------------------------------------------------------------------------------
* EACCES | Search permission is denied on a component of the path prefix.
* EINVAL | The path name format is invalid.
* ENAMETOOLONG | The length of <b>path</b> is greater than the value of {@link NAME_MAX}.
* ENOENT | The {@link O_CREAT} flag is not set and the named file does not exist.
* ENXIO | Inode is invalid, not a normal character driver, or not a mount point.
* EMFILE | All available file descriptors of the process are opened.
* EPERM | The file structure corresponding to the file descriptor cannot be obtained.
* EEXIST | {@link O_CREAT} and {@link O_EXCL} are set.
* EIO | A hardware error occurs at the low-level disk I/O layer. As a result, the physical drive cannot work.
* EROFS | The named file resides on a read-only file system, and either {@link O_WRONLY}, {@link O_RDWR}, {@link O_CREAT} (if file does not exist), or {@link O_TRUNC} is set in the <b>oflags</b> parameter.
* ENOSPC | The directory or file system that would contain the new file cannot be expanded, the file does not exist, and {@link O_CREAT} is specified.
* ENFILE | The number of opened files in the system has reached the maximum.
* EISDIR | The named file is a directory, and <b>oflags</b> includes {@link O_WRONLY}, {@link O_RDWR}, or {@link O_CREAT} without {@link O_DIRECTORY}.
* ENOTDIR | A component of the path prefix names an existing file that is not a directory.
* EBUSY | The operation is rejected according to the file sharing policy.
* ENODEV | The device does not exist.
* ENOTEMPTY | The directory is not empty.
* ELOOP | Too many symbolic links are encountered.
* EFAULT | The IP address is incorrect.
* @since 1.0
* @version 1.0
*/
int open(const char *path, int oflags, ...);

/**@brief Access mode */
#define O_ACCMODE (03|O_SEARCH)
/**@brief Read-only */
#define O_RDONLY  00
/**@brief Write-only */
#define O_WRONLY  01
/**@brief Read and write */
#define O_RDWR    02


/**
* @brief Pre-allocates space to a file.
*
* @attention This function can be used only in the FAT file system.
* @param fd Indicates the file descriptor.
* @param mode Indicates the operation mode. Only {@link FALLOC_FL_KEEP_SIZE} is supported.
* @param offset Indicates the offset of the file to allocate.
* @param len Indicates the size of the disk space to be allocated to the file.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* --------- | ------------------------------------------------------------------------
* EINVAL | The value of <b>len</b> or <b>offset</b> is less than 0 or greater than that of {@link INT_MAX}, or the value of <b>mode</b> is not {@link FALLOC_FL_KEEP_SIZE}.
* EBADF | <b>fd</b> is not a valid file descriptor, or the basic file system does not support this operation.
* EAGAIN | The file list is null.
* EACCES | Access denied by the authority bit of file mode
* EIO | An I/O error occurs when data is read from or written to the file system.
* EROFS | The physical drive is write-protected.
* EPERM | The operation is not allowed or the access is rejected.
* EBUSY | The operation is rejected according to the file sharing policy.
* @since 1.0
* @version 1.0
*/
int fallocate(int fd, int mode, off_t offset, off_t len);

/**
* @brief Pre-allocates space to a file. See {@link fallocate}.
*/
#define fallocate64 fallocate

/**
* @brief Manipulates a specified file descriptor.
*
* @param fd Indicates the file descriptor.
* @param cmd Indicates the command used to manipulate the file descriptor.
* cmd | Description
* --------- | ------------------------------------------------------------------------
* F_DUPFD | Copy a new file descriptor (<b>fd</b> is a file).
* F_GETFL | Obtain the status of the file specified by <b>fd</b>. (<b>fd</b> is a file or sockets are available.)
* F_SETFL | Set the file descriptor status. (<b>fd</b> is a file or sockets are available.)
* @return Returns the value depending on <b>cmd</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* --------- | ------------------------------------------------------------------------
* EAGAIN | The operation is prohibited.
* EBADF | <b>fd</b> is an invalid file descriptor.
* EFAULT | Beyond the accessible address space.
* EINVAL | The command specified by <b>cmd</b> cannot be recognized.
* ENOLCK | Too many segment locks open, lock table is full, or a remote locking protocol failed.
* EPERM | Attempt to clear the {@link O_APPEND} flag on a file that has the append-only attribute set.
* @since 1.0
* @version 1.0
*/
int fcntl(int fd, int cmd, ...);

#ifdef __cplusplus
}
#endif

#endif
/** @} */