/**
* @addtogroup MATH
* @{
*
* @brief Provides math functions, such as trigonometric, exponential, logarithmic functions,and perform operations on complex numbers.
*
* @since 1.0
* @version 1.0
*/

/**
* @file fenv.h
*
* @brief Declares specific functions for performing operations for floating-point exceptions and rounding modes.
*
* You can use the functions provided in this file to set, clear, and store the current floating-point exception and to obtain and set the current rounding mode. \n
* @since 1.0
* @version 1.0
*/

#ifndef _FENV_H
#define _FENV_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Indicates an invalid floating-point exception. At least one parameter value is not defined in the function.
*/
#define FE_INVALID    1

/**
* @brief Indicates a pole error exception. The return value is asymptotically infinite when the divisor is zero or the input is asymptotic to zero.
*/
#define FE_DIVBYZERO  2

/**
* @brief Indicates an overflow floating-point exception. The result is too large to be represented in the normal return value.
*/
#define FE_OVERFLOW   4

/**
* @brief Indicates an underflow floating-point exception. The result is too small to be represented in the normal return value.
*/
#define FE_UNDERFLOW  8

/**
* @brief Indicates an inexact floating-point exception.
*/
#define FE_INEXACT    16

/**
* @brief Indicates bitwise OR of all the preceding supported floating-point exceptions.
*/
#define FE_ALL_EXCEPT 31

/**
* @brief Indicates rounding towards the nearest integer.
*/
#define FE_TONEAREST  0

/**
* @brief Indicates rounding towards negative infinity.
*/
#define FE_DOWNWARD   0x800000

/**
* @brief Indicates rounding towards positive infinity.
*/
#define FE_UPWARD     0x400000

/**
* @brief Indicates rounding towards zero.
*/
#define FE_TOWARDZERO 0xc00000

/**
* @brief Floating-point exception type.
*/
typedef unsigned long fexcept_t;

/**
* @brief Defines the floating-point environment.
*/
typedef struct {
/** Floating-point environment parameters */
unsigned long __cw;
} fenv_t;

/** @brief Defines the default floating-point environment. */
#define FE_DFL_ENV      ((const fenv_t *) -1)

/**
* @brief Clears floating-point exceptions specified by <b>excepts</b>.
*
* @param excepts Indicates the floating-point exceptions to clear.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int feclearexcept(int excepts);

/**
* @brief Stores the floating-point exceptions specified by <b>excepts</b> into the <b>flagp</b> variable address.
*
* @param flagp Indicates the pointer to the floating-point exceptions.
* @param excepts Indicates the current floating-point exceptions to store.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int fegetexceptflag(fexcept_t *flagp, int excepts);

/**
* @brief Obtains the floating-point exceptions specified by <b>excepts</b>.
*
* @param excepts Indicates the floating-point exceptions to obtain.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int feraiseexcept(int excepts);

/**
* @brief Sets the exception status flag by copying the floating-point exceptions stored in the <b>flagp</b> address to the <b>excepts</b> parameter.
*
* @attention Before using this function, ensure that you have successfully called {@link fegetexceptflag}.
* @param flagp Indicates the pointer to the floating-point exceptions.
* @param excepts Indicates the current floating-point exceptions to store.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int fesetexceptflag(const fexcept_t *flagp, int excepts);

/**
* @brief Checks whether the floating-point exceptions specified by <b>excepts</b> have been raised.
*
* For example, if the value of <b>excepts</b> is <b>{@link FE_OVERFLOW} | {@link FE_INVALID}</b>, this function checks whether the current floating-point operation has thrown an overflow exception or is invalid.
* @param excepts Indicates the floating-point exceptions.
* @return Returns the macros describing the specified exceptions if they have been raised; returns <b>0</b> if the specified exceptions do not exist.
* @since 1.0
* @version 1.0
*/
int fetestexcept(int excepts);

/**
* @brief Obtains the macro corresponding to the current rounding mode.
*
* The rounding mode can be {@link FE_TONEAREST}, {@link FE_UPWARD}, {@link FE_DOWNWARD}, or {@link FE_TOWARDZERO}. \n
* @return Returns the macro corresponding to the current rounding mode.
* @since 1.0
* @version 1.0
*/
int fegetround(void);

/**
* @brief Sets the rounding mode.
*
* The rounding mode can be {@link FE_TONEAREST}, {@link FE_UPWARD}, {@link FE_DOWNWARD}, or {@link FE_TOWARDZERO}. \n
* @param rounding_mode Indicates the rounding mode to set.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int fesetround(int rounding_mode);

/**
* @brief Stores the current floating-point environment into the object specified by <b>envp</b>.
*
* The floating-point environment contains the current control modes and status flags of floating-point operations. \n
* @param envp Indicates the pointer to the floating-point environment.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int fegetenv(fenv_t *envp);

/**
* @brief Stores the current floating-point environment into the object specified by <b>envp</b>, with all status flags cleared.
*
*
* @attention This function clears all exception status flags and sets the environment to the non-stop mode (to continue the execution of the exceptions), if any.
* @param envp Indicates the pointer to the floating-point environment.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int feholdexcept(fenv_t *envp);

/**
* @brief Sets the current floating-point environment.
*
* This function restores the floating-point environment from the object specified by <b>envp</b>. The floating-point environment contains the current control modes and status flags of floating-point operations. \n
* @attention The object specified by <b>envp</b> must be valid.
* @param envp Indicates the pointer to the floating-point environment.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int fesetenv(const fenv_t *envp);

/**
* @brief Updates the current floating-point environment from the object specified by <b>envp</b>.
*
*
* @attention The object specified by <b>envp</b> must be valid.
* @param envp Indicates the pointer to the floating-point environment.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int feupdateenv(const fenv_t *envp);

#ifdef __cplusplus
}
#endif
#endif
/** @} */