/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file fmtmsg.h
*
* @brief Provides a function for printing formatted messages.
*
* You can use the function in the file to write messages (especially error messages) into the output device specified by <b>classification</b>.
*
* @since 1.0
* @version 1.0
*/

#ifndef _FMTMSG_H
#define _FMTMSG_H

#ifdef __cplusplus
extern "C" {
#endif

/**@brief A hardware error occurred. */
#define MM_HARD		 1

/**@brief A software error occurred. */
#define MM_SOFT		 2

/**@brief A firmware error occurred. */
#define MM_FIRM		 4

/**@brief An error is detected by an application. */
#define MM_APPL		 8

/**@brief An error is detected by a utility. */
#define MM_UTIL		 16

/**@brief An error is detected by the operating system. */
#define MM_OPSYS	 32

/**@brief The error is recoverable. */
#define MM_RECOVER	 64

/**@brief The error is non-recoverable. */
#define MM_NRECOV	 128

/**@brief Prints error messages on standard error (stderr). */
#define MM_PRINT	 256

/**@brief Prints error messages on the system console. */
#define MM_CONSOLE	 512

/**@brief Ignores the <b>classification</b> parameter. */
#define MM_NULLMC	 0L


/**@brief Fatal fault */
#define MM_HALT		 1

/**@brief Error-level fault */
#define MM_ERROR	 2

/**@brief Warning condition */
#define MM_WARNING	 3

/**@brief Informative message */
#define MM_INFO		 4

/**@brief No severity level is set, equivalent to {@link MM_NULLSEV}. */
#define MM_NOSEV	 0


/**@brief The function succeeded. */
#define MM_OK		 0

/**@brief The function failed. */
#define MM_NOTOK	 (-1)

/**@brief Error writing to stderr. */
#define MM_NOMSG	 1

/**@brief Error writing to the console. */
#define MM_NOCON	 4



/**@brief Ignores the <b>label</b> parameter. */
#define MM_NULLLBL	 ((char*)0)

/**@brief Ignores the <b>text</b> parameter. */
#define MM_NULLTXT	 ((char*)0)

/**@brief Ignores the <b>action</b> parameter. */
#define MM_NULLACT	 ((char*)0)

/**@brief Ignores the <b>tag</b> parameter. */
#define MM_NULLTAG	 ((char*)0)

/**@brief Ignores the <b>severity</b> parameter. */
#define MM_NULLSEV	 0


/**
* @brief Prints formatted messages.
*
* This function is used to print formatted messages to the output device specified by <b>classification</b>. The messages include the message definition, error severity, possible recovery steps, and reference to online documents. \n
* @attention All parameters in this function can be ignored based on the following definitions: {@link MM_NULLLBL}, {@link MM_NULLSEV}, {@link MM_NULLTXT}, {@link MM_NULLACT}, and {@link MM_NULLTAG}.
* @param classification Indicates four types of information, including the output device, error source, problem detector, and problem severity. This parameter is of the long integer type.
* Message Type | Macro Definition Corresponding to the Message Description
* -----------| ---------------------
* Message output device | {@link MM_PRINT}, {@link MM_CONSOLE} or both
* Message source | {@link MM_HARD}, {@link MM_SOFT}, or {@link MM_FIRM}
* Message detector | {@link MM_APPL}, {@link MM_UTIL}, or {@link MM_OPSYS}
* Severity of the incident | {@link MM_RECOVER} or {@link MM_NRECOV}
* @param label Indicates the pointer to the message source. This parameter consists of two fields separated by a colon (:). The first field is up to 10 characters, the second is up to 14 characters.
* @param severity Indicates the severity level of an error message. For details, see {@link MM_HALT}, {@link MM_ERROR}, {@link MM_WARNING}, {@link MM_INFO}, and {@link MM_NOOSEV}.
* @param text Indicates the condition of the error.
* @param action Describes possible steps to recover from the error.
* @param tag Indicates the referenced online documents where more information can be found.
* @return The function can return 4 values: {@link MM_OK}, {@link MM_NOTOK}, {@link MM_NOMSG}, and {@link MM_NOCON}.
* @since 1.0
* @version 1.0
*/
int fmtmsg(long classification, const char *label, int severity, const char *text, const char *action, const char *tag);

#ifdef __cplusplus
}
#endif

#endif

/** @} */