/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file fnmatch.h
*
* @brief Provides a function for checking whether a specified string matches a string that contains wildcards.
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _FNMATCH_H
#define	 _FNMATCH_H

#ifdef __cplusplus
extern "C" {
#endif

/**@brief If this flag is set, a slash (/) in <b>string</b> can be matched by a slash (/) in <b>pattern</b>, but not by an asterisk (*), or a question mark (?), or a bracket expression ([]) containing a slash. */
#define	 FNM_PATHNAME 0x1

/**@brief If this flag is set, the backslash (\) is treated as an ordinary character, instead of an escape character. */
#define	 FNM_NOESCAPE 0x2

/**@brief If this flag is set, a leading period (.) in <b>string</b> can be exactly matched by the period (.) in <b>pattern</b>. A period is considered to be leading if it is the first character in <b>string</b>, or if both <b>FNM_PATHNAME</b> is set and the period immediately follows a slash.*/
#define	 FNM_PERIOD   0x4

/**@brief If this flag is set, a trailing sequence of characters starting with a slash (/) is ignored in <b>string</b>. For example, if this flag is set, either <b>foo*</b> or <b>foobar</b> as a pattern would match the string <b>foobar/frobozz</b>. */
#define	 FNM_LEADING_DIR	 0x8

/**@brief If this flag is set, the pattern is matched case-insensitively. */
#define	 FNM_CASEFOLD	 0x10

/**@brief The definition is similar to that of {@link FNM_PATHNAME}. */
#define	 FNM_FILE_NAME	 FNM_PATHNAME

/**@brief The <b>string</b> parameter does not match the <b>pattern</b> parameter. */
#define	 FNM_NOMATCH 1

/**@brief The function does not support the operation. */
#define FNM_NOSYS   (-1)

/**
* @brief Matches a file name or a path name.
*
* This function is used to check whether the value specified by <b>string</b> matches the value specified by <b>pattern</b>. Wildcards can be used in <b>pattern</b>, for example, <b>./foo*</b> and <b>foo*.boo*</b>. You can use <b>flags</b> to set your required matching rule. \n
* @param pattern Indicates the pointer to the pattern to match.
* @param string Indicates the pointer to the string to match against <b>pattern</b>.
* @param flags Indicates a combination of flag bits that specify the matching rule. For details, see {@link FNM_NOESCAPE}, {@link FNM_PATHNAME}, {@link FNM_PERIOD}, {@link FNM_LEADING_DIR}, {@link FNM_CASEFOLD}, and {@link FNM_FILE_NAME}.
* @return Returns <b>0</b> if the matching is successful; returns {@link FNM_NOMATCH} if the matching fails; returns a non-zero value if an error occurs.
* @since 1.0
* @version 1.0
*/
int fnmatch(const char *pattern, const char *string, int flags);

#ifdef __cplusplus
}
#endif

#endif
/** @} */