/**
* @addtogroup FS
* @{
*
* @brief Provides functions for file system, include file operations, directory operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file ftw.h
*
* @brief Provides functions and structures related to file tree traversal.
*
* You can use the functions and structures to traverse files in a specified directory to obtain or process related information.
*
* @since 1.0
* @version 1.0
*/

#ifndef _FTW_H
#define	 _FTW_H


#ifdef __cplusplus
extern "C" {
#endif

#include <sys/stat.h>

/**@brief Regular file */
#define FTW_F   1
/**@brief Directory */
#define FTW_D   2
/**@brief Unreadable directory. Its subdirectories will not be traversed. */
#define FTW_DNR 3
/**@brief The data of the <b>stat</b> structure cannot be obtained. The possible cause is that the permission is incorrect. */
#define FTW_NS  4
/**@brief Symbolic link */
#define FTW_SL  5
/**@briefDirectory. All subdirectories in this directory have been traversed. */
#define FTW_DP  6
/**@brief Symbolic link that names a non-existent file */
#define FTW_SLN 7

/**@brief Symbolic links that are not traversed. If not set, symbolic links are traversed by default. */
#define FTW_PHYS  1
/**@brief Stay within the same file system. */
#define FTW_MOUNT 2
/**@brief {@link chdir} is called to go to the directory before reading the directory. */
#define FTW_CHDIR 4
/**@brief All subdirectories will be traversed before the directory itself. */
#define FTW_DEPTH 8

/**
* @brief Defines the file tree walker (FTW).
*
* <b>ftw.h</b> describes the <b>FTW</b> structure. The file location can be determined based on the offset and the depth relative to the root node. \n
*/
struct FTW {
/** Offset of the file name in the path name */
int base;
/** Depth of the file name, relative to the root node of the tree */
int level;
};

/**
* @brief Traverses a file tree.
*
* This function is used to traverse a file tree for search. \n
* @param path Indicates the pointer to the file directory to traverse.
* @param fn Indicates the pointer to the function to be called for the found entry. For the <b>fn</b> function, <b>file</b> identifies the path of the file relative to <b>path</b>, and <b>sb</b> indicates the pointer to the {@link stat} structure. <b>flag</b> indicates the flag. The values of <b>flag</b> are as follows:
* value | Description
* ------- | --------------------------------------
* FTW_F | Regular file
* FTW_D | Directory
* FTW_DNR | Unreadable directory. Its subdirectories will not be traversed.
* FTW_SL | Symbolic link
* FTW_NS | The data of the <b>stat</b> structure cannot be obtained. The possible cause is that the permission is incorrect.
* @param depth Indicates the number of files that can be opened during traversal.
* @return Returns <b>0</b> if the tree is traversed; returns the value that is returned by the <b>fn()</b> function if the traversal is interrupted; returns <b>-1</b> if an error occurs.
* @since 1.0
* @version 1.0
*/
int ftw(const char *path, int (*fn)(const char *file, const struct stat *sb, int flag), int depth);

/**
* @brief Traverses a file tree.
*
* This function is used to traverse a file tree. It has a similar effect to {@link ftw} except that it takes an additional parameter <b>flags</b>. \n
* @param path Indicates the pointer to the file directory to traverse.
* @param fn Indicates the pointer to the function to be called for the found entry. For the <b>fn</b> function, <b>file</b> identifies the path of the file relative to <b>path</b>, and <b>sb</b> indicates the pointer to the {@link stat} structure. <b>flag</b> indicates the flag. The values of <b>flag</b> are as follows:
* value | Description
* ------- | ----------------------------------------
* FTW_F | Regular file
* FTW_D | Directory
* FTW_DNR | Unreadable directory. Its subdirectories will not be traversed.
* FTW_SL | Symbolic link
* FTW_NS | The data of the <b>stat</b> structure cannot be obtained. The possible cause is that the permission is incorrect.
* FTW_DP | Directory. All subdirectories have been traversed.
* FTW_SLN | Symbolic link that names a non-existent file
* The fourth argument of <b>fn()</b> is a pointer to a {@link FTW} structure.
* @param fd_limit Indicates the number of files that can be opened during traversal.
* @param flags Indicates the traversal action. It is a bitwise inclusive-OR of zero or more of the following flags:
* value     | Description
* --------- | ----------------------------------------
* FTW_CHDIR | Go to the directory through {@link chdir} before reading the directory.
* FTW_DEPTH | All subdirectories will be traversed before the directory itself.
* FTW_MOUNT | Stay within the same file system.
* FTW_PHYS | Symbolic links that are not traversed. If not set, symbolic links are traversed by default.
* @return Returns <b>0</b> if the tree is traversed; returns the value that is returned by the <b>fn()</b> function if the traversal is interrupted; returns <b>-1</b> if an error occurs.
* @since 1.0
* @version 1.0
*/
int nftw(const char *path, int (*fn)(const char *file, const struct stat *sb, int flag, struct FTW *s), int fd_limit, int flags);

#ifdef __cplusplus
}
#endif

#endif
/** @} */