/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file getopt.h
*
* @brief Provides functions and data structures, such as command-line argument parsing.
*
* You can use the functions provided in this file to perform operations related to wide character types during development, such as, command-line argument parsing. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _GETOPT_H
#define _GETOPT_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Defines the command parsing option.
*/
struct option {
/** Long argument name */
const char *name;
/** <b>has_arg</b> has three values. <b>0</b> indicates that the argument is not followed by an argument value, <b>1</b> indicates that the argument must be followed by an argument value,
and <b>2</b> indicates that the argument can be followed or not followed by an argument value. */
int has_arg;
/** Determines the returned value of the <b>getopt()</b> function.
If <b>flag</b> is <b>NULL</b>, the <b>getopt()</b> function returns the <b>val</b> value that matches the option.
If <b>flag</b> is not <b>NULL</b>, assign the <b>val</b> value to the memory to which <b>flag</b> points, and set the returned value to <b>0</b>. */
int *flag;
/** The returned value is determined by <b>flag</b>. */
int val;
};

/**
* @brief Parses the command-line arguments.
*
* @param argc Indicates the number of command-line arguments.
* @param argv Indicates the command-line argument content.
* @param optstring Indicates the string containing the optional arguments of the executable file.
* Character Option Expression | Description
*---------------------  | -------------------
* Only characters that indicate different options are available. | <b>abc</b> indicates that the file has three options <b>-a</b>, <b>-b</b>, and <b>-c</b>.
* The character is followed by a colon. | <b>b</b> indicates that <b>-b</b> must be followed by an argument in the executable file that contains the <b>getopt()</b> function.
* The character is followed by two colons. | <b>b</b> indicates that <b>-b</b> can be followed by or without the corresponding argument in the executable file that contains the <b>getopt()</b> function.
* @param longopts Indicates the long argument structure.
* @param longindex Indicates the index of the current long argument in <b>longopts</b>.
* @return Returns the parsed option character if the parsing is successful; returns <b>-1</b> if all options have been parsed; returns <b>?</b> if an option character that is not in <b>optstring</b> is encountered during parsing. When an option is encountered but there is no argument during parsing, this function returns <b>:</b> if the first character of <b>optstring</b> is a colon (:); returns <b>?</b> otherwise.
* @since  1.0
* @version  1.0
*/
int getopt_long(int argc, char *const *argv, const char *optstring, const struct option *longopts, int *longindex);

/**
* @brief Parses the command-line arguments.
*
* @attention Different from {@link getopt_long}, this function matches <b>--name</b> and <b>-name</b> as long arguments.
* @param argc Indicates the number of command-line arguments.
* @param argv Indicates the command-line argument content.
* @param optstring Indicates the string containing the optional arguments of the executable file.
* Character Option Expression | Description
*---------------------  | -------------------
* Only characters that indicate different options are available. | <b>abc</b> indicates that the file has three options <b>-a</b>, <b>-b</b>, and <b>-c</b>.
* The character is followed by a colon. | <b>b</b> indicates that <b>-b</b> must be followed by an argument in the executable file that contains the <b>getopt()</b> function.
* The character is followed by two colons. | <b>b</b> indicates that <b>-b</b> can be followed by or without the corresponding argument in the executable file that contains the <b>getopt()</b> function.
* @param longopts Indicates the long argument structure.
* @param longindex Indicates the index of the current long argument in <b>longopts</b>.
* @return Returns the parsed option character if the parsing is successful; returns <b>-1</b> if all options have been parsed; returns <b>?</b> if an option character that is not in <b>optstring</b> is encountered during parsing. When an option is encountered but there is no argument during parsing, this function returns <b>:</b> if the first character of <b>optstring</b> is a colon (:); returns <b>?</b> otherwise.
* @since  1.0
* @version  1.0
*/
int getopt_long_only(int argc, char *const *argv, const char *optstring, const struct option *longopts, int *longindex);

#ifdef __cplusplus
}
#endif

#endif

/** @} */