/**
* @addtogroup FS
* @{
*
* @brief Provides functions for file system, include file operations, directory operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file glob.h
*
* @brief Provides functions and structures used to find path names matching a pattern and release the memory that was applied for during the search.
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _GLOB_H
#define	 _GLOB_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Defines the search information.
*
* This structure is used to store information about matched files. \n
*/
typedef struct {
/** Number of matched paths so far */
size_t gl_pathc;
/** List of matched path names */
char **gl_pathv;
/** Slots to reserve in <b>gl_pathv</b> */
size_t gl_offs;
int __dummy1;
void *__dummy2[5];
} glob_t;

/**
* @brief Searches for path names that meet a specified pattern.
*
* @param pattern Indicates the pattern you want to match.
* @param flags Indicates the matching mode.
* flags | Description
* ---------------- | ----------------------------
* GLOB_ERR         | {@link GLOB_ERR}
* GLOB_MARK        | {@link GLOB_MARK}
* GLOB_NOSORT      | {@link GLOB_NOSORT}
* GLOB_DOOFFS      | {@link GLOB_DOOFFS}
* GLOB_NOCHECK     | {@link GLOB_NOCHECK}
* GLOB_APPEND      | {@link GLOB_APPEND}
* GLOB_NOESCAPE    | {@link GLOB_NOESCAPE}
* GLOB_PERIOD      | {@link GLOB_PERIOD}
* GLOB_TILDE       | {@link GLOB_TILDE}
* GLOB_TILDE_CHECK | {@link GLOB_TILDE_CHECK}
* @param errfunc Indicates the pointer to the error processing callback function. If <b>errfunc</b> is not a null pointer, <b>errfunc()</b> is called when an error occurs ({@link opendir()}, {@link readdir()}, or {@link stat()} fails) during the search. If the function returns a non-zero value or {@link GLOB_ERR} is set, <b>glob()</b> terminates the entire search process.
* @param pglob Indicates the pointer to the address storing the matched path names.
* @return Returns <b>0</b> on successful completion; returns {@link GLOB_NOSPACE} for running out of memory; returns {@link GLOB_ABORTED} for a read error; returns {@link GLOB_NOMATCH} for no match found.
* @since 1.0
* @version 1.0
*/
int  glob(const char *__restrict pattern, int flags, int (*errfunc)(const char *, int), glob_t *__restrict pglob);

/**
* @brief Frees allocated memory from a call to <b>glob()</b>.
*
* @param pglob Indicates the pointer to the address of memory dynamically allocated for a {@link glob} call.
* @since 1.0
* @version 1.0
*/
void globfree(glob_t *pglob);

/**@brief Return upon a read error. */
#define GLOB_ERR      0x01
/**@brief Append a slash (/) to each path, which corresponds to a directory. */
#define GLOB_MARK     0x02
/**@brief Do not sort the returned path names. */
#define GLOB_NOSORT   0x04
/**@brief In the <b>pglob</b> parameter of the {@link glob()} function, reserve a string that has a length of <b>pglob->gl_offs</b> and ends with <b>\0</b> at the beginning of the string list in <b>pglob->pathv</b>. That is, the newly matched string is appended to a position where the distance from the beginning of <b>pglob->pathv</b> is the length of <b>pglob->gl_offs</b>. */
#define GLOB_DOOFFS   0x08
/**@brief If no pattern matches, return the original pattern. */
#define GLOB_NOCHECK  0x10
/**@brief Append the results of this call to the results returned by a previous call to <b>glob()</b>. */
#define GLOB_APPEND   0x20
/**@brief Do not use a backslash (\) as an escape character. */
#define GLOB_NOESCAPE 0x40
/**@brief Allow a leading period to be matched by metacharacters. */
#define	 GLOB_PERIOD   0x80

/**@brief Carry out tilde expansion. If a tilde (~) is the only character in the pattern, or an initial tilde is followed immediately by a slash (/), then the home directory of the caller is substituted for the tilde. */
#define GLOB_TILDE       0x1000
/**@brief Similar to {@link GLOB_TILDE}. The difference is that if the username is invalid, or the home directory cannot be determined, {@link glob()} returns {@link GLOB_NOMATCH} to indicate an error. */
#define GLOB_TILDE_CHECK 0x4000

/**@brief Memory insufficient */
#define GLOB_NOSPACE 1
/**@brief Read error */
#define GLOB_ABORTED 2
/**@brief No matching item is found. */
#define GLOB_NOMATCH 3
/**@brief The function is not supported. */
#define GLOB_NOSYS   4

#ifdef __cplusplus
}
#endif

#endif
/** @} */