/**
* @addtogroup IO
* @{
*
* @brief Provides data structures and functions for performing input/output (I/O) operations, such as printf and scanf.
*
* @since 1.0
* @version 1.0
*/


/**
* @file iconv.h
*
* @brief Declares functions related to the I/O character set conversion.
*
* @since 1.0
* @version 1.0
*/

#ifndef _ICONV_H
#define _ICONV_H

#ifdef __cplusplus
extern "C" {
#endif


/**@brief Type of the character set conversion descriptor */
typedef void *iconv_t;

/**
* @brief Obtain a character set conversion descriptor.
* @param to Indicates the pointer to the source character set for conversion.
* @param from Indicates the pointer to the target character set for conversion.
* @return Returns the character set conversion descriptor if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
iconv_t iconv_open(const char *to, const char *from);

/**
* @brief Converts a character set.
* @param cd Indicates the conversion descriptor obtained via {@link iconv_open}.
* @param in Indicates the double pointer to the buffer for storing the input character.
* @param inb Indicates the pointer to the length of the character input buffer.
* @param out Indicates the double pointer to the buffer for storing the output character.
* @param outb Indicates the pointer to length of the character output buffer.
* @return Returns the number of irreversibly converted characters if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------
* EILSEQ | An invalid multi-byte sequence is encountered in the input.
* E2BIG | The output buffer space is insufficient.
* EINVAL | An incomplete multi-byte sequence is encountered in the input.
* @since 1.0
* @version 1.0
*/
size_t iconv(iconv_t cd, char **restrict in, size_t *restrict inb, char **restrict out, size_t *restrict outb);

/**
* @brief Closes a character set conversion descriptor.
* @param cd Indicates the conversion descriptor to close.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int iconv_close(iconv_t cd);

#ifdef __cplusplus
}
#endif

#endif
/** @} */