/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file inttypes.h
*
* @brief Provides functions and related data structures for conversion between character strings and plural.
*
* You can use the functions provided in this file to perform operations related to wide character types during development, for example, performing a division operation, converting a character string to an imax-type parameter, or converting a character string to an umax-type parameter. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _INTTYPES_H
#define _INTTYPES_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Stores the division result.
*/
typedef struct {
/** Quotient */
intmax_t quot;
/** Remainder */
intmax_t rem;
} imaxdiv_t;

/**
* @brief Calculates the absolute value of an input parameter of the integer type.
*
* This function is used to calculate the absolute value of the input parameter specified by <b>j</b> of the <b>intmax_t</b> type. \n
* @attention Pay attention to the parameter type.
* @param j Indicates the input parameter.
* @return Returns the absolute value of the input parameter if the operation is successful.
* @since 1.0
* @version 1.0
*/
intmax_t imaxabs(intmax_t j);

/**
* @brief Calculates the quotient and remainder after the division operation is performed on an integer.
*
* @param numerator Indicates the numerator.
* @param denominator Indicates the denominator.
* @return Returns a structure variable that has been defined in the function.
* @since 1.0
* @version 1.0
*/
imaxdiv_t imaxdiv(intmax_t numerator, intmax_t denominator);

/**
* @brief Parses a string to a value of the <b>intmax_t</b> type.
*
* This function is used to convert the string pointed to by <b>str</b> to a value of the <b>intmax_t</b> type. If <b>endptr</b> is not a null pointer, the function also sets the value of <b>endptr</b> to point to the first character following the integer. \n
* @attention This function discards as many spaces as possible until the first non-space character is found.
* @param str Indicates the pointer to the string to parse.
* @param endptr Indicates the double pointer to the first character following the integer obtained from <b>str</b>.
* @param base Indicates the conversion of number system. The common types are <b>2</b>, <b>8</b>, <b>10</b>, and <b>16</b>, which indicate binary, octal, decimal, and hexadecimal, respectively.
* @return Returns a floating-point number if the operation is successful; returns <b>0</b> if no conversion was performed; returns the positive or negative value of {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE} if the value falls out of the floating-point number range.
* @since  1.0
* @version  1.0
*/
intmax_t strtoimax(const char *str, char **endptr, int base);

/**
* @brief Parses a string to a value of the <b>uintmax_t</b> type.
*
* This function is used to convert the string pointed to by <b>str</b> to a value of the <b>uintmax_t</b> type. If <b>endptr</b> is not a null pointer, the function also sets the value of <b>endptr</b> to point to the first character following the integer. \n
* @attention This function discards as many spaces as possible until the first non-space character is found.
* @param str Indicates the pointer to the string to parse.
* @param endptr Indicates the double pointer to the first character following the integer obtained from <b>str</b>.
* @param base Indicates the conversion of number system.
* @return Returns a floating-point number if the operation is successful; returns <b>0</b> if no conversion was performed; returns the positive or negative value of {@link HUGE_VAL} and sets <b>errno</b> to <b>ERANGE</b> if the value falls out of the floating-point number range; returns a value that is not greater than the smallest positive normalized number if the correct value causes an overflow.
* @since  1.0
* @version  1.0
*/
uintmax_t strtoumax(const char *str, char **endptr, int base);

/**
* @brief Parses a string to a value of the <b>intmax_t</b> type.
*
* This function is used to convert the string pointed to by <b>str</b> to a value of the <b>intmax_t</b> type. If <b>endptr</b> is not a null pointer, the function also sets the value of <b>endptr</b> to point to the first character following the integer. \n
* @attention This function discards as many spaces as possible until the first non-space character is found.
* @param str Indicates the pointer to the string to parse.
* @param endptr Indicates the double pointer to the first character following the integer obtained from <b>str</b>.
* @param base Indicates the conversion of number system.
* @return Returns a floating-point number if the operation is successful; returns <b>0</b> if no conversion was performed; returns the positive or negative value of {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE} if the value falls out of the floating-point number range; returns a value that is not greater than the smallest positive normalized number if the correct value causes an overflow.
* @since  1.0
* @version  1.0
*/
intmax_t wcstoimax(const wchar_t *str, wchar_t **endptr, int base);

/**
* @brief Parses a string to a value of the <b>uintmax_t</b> type.
*
* This function is used to convert the string pointed to by <b>str</b> to a value of the <b>uintmax_t</b> type. If <b>endptr</b> is not a null pointer, the function also sets the value of <b>endptr</b> to point to the first character following the integer. \n
* @attention This function discards as many spaces as possible until the first non-space character is found.
* @param str Indicates the pointer to the string to parse.
* @param endptr Indicates the double pointer to the first character following the integer obtained from <b>str</b>.
* @param base Indicates the conversion of number system.
* @return Returns a floating-point number if the operation is successful; returns <b>0</b> if no conversion was performed; returns the positive or negative value of {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE} if the value falls out of the floating-point number range; returns a value that is not greater than the smallest positive normalized number if the correct value causes an overflow.
* @since  1.0
* @version  1.0
*/
uintmax_t wcstoumax(const wchar_t *str, wchar_t **endptr, int base);


#ifdef __cplusplus
}
#endif

#endif
/** @} */