/**
* @addtogroup IO
* @{
*
* @brief Provides data structures and functions for performing input/output (I/O) operations, such as printf and scanf.
*
* @since 1.0
* @version 1.0
*/

/**
* @file locale.h
*
* @brief Provides functions to set and obtain the locale language environment.
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _LOCALE_H
#define	 _LOCALE_H

#ifdef __cplusplus
extern "C" {
#endif

/**@brief Language symbol type, which affects character classification and case conversion */
#define LC_CTYPE    0

/**@brief Number type, which affects the number format */
#define LC_NUMERIC  1

/**@brief Time type, which affects date and time formats */
#define LC_TIME     2

/**@brief Character sorting type, which affects string comparison and sorting habits */
#define LC_COLLATE  3

/**@brief Monetary type, which affects monetary information */
#define LC_MONETARY 4

/**@brief Message type, which affects prompt information, error information, status information, titles, labels, buttons, and menus */
#define LC_MESSAGES 5

/**@brief All types, which affect all content of locale settings. If <b>LC_ALL</b> is set, the locale information of all types is set to be consistent with <b>LC_ALL</b>. */
#define LC_ALL      6


/**
* @brief Indicates format information about a number.
*/
struct lconv {
/** Separator between the integer part and decimal part of decimal values. The default separator is <b>.</b>. */
char *decimal_point;
/** The decimal values are separated by thousands, that is, three digits form a group. No separator is used in the system by default. */
char *thousands_sep;
/** Specifies that decimal values are separated by a group of digits. No separator is used in the system by default. */
char *grouping;
/** International currency symbol, for example, USD for dollar and CNY for Chinese Yuan. For details, see ISO-4217. */
char *int_curr_symbol;
/** Local currency symbol, for example, $ for dollar and ¥ for Chinese Yuan. */
char *currency_symbol;
/** Separator between the integer part and decimal part of currency values. The system does not support this separator. */
char *mon_decimal_point;
/** The currency values are separated by thousands, that is, three digits form a group. The system does not support this separator. */
char *mon_thousands_sep;
/** Specifies that currency values are separated by a group of digits. No separator is used in the system by default. */
char *mon_grouping;
/** Symbol of a non-negative currency value, which is not supported in the current system */
char *positive_sign;
/** Symbol of a negative currency value, which is not supported in the current system */
char *negative_sign;
/** Maximum number of decimal parts of a value, which is not supported in the current system */
char int_frac_digits;
/** Maximum number of decimal parts of a currency value, which is not supported in the current system */
char frac_digits;
/** Specifies whether to add a currency symbol before a non-negative currency value. This parameter is not supported in the current system. */
char p_cs_precedes;
/** Specifies whether to add a space before a non-negative currency value as a separator. This parameter is not supported in the current system. */
char p_sep_by_space;
/** Specifies whether to add a currency symbol before a negative currency value. This parameter is not supported in the current system. */
char n_cs_precedes;
/** Specifies whether to add a space before a negative currency value as a separator. This parameter is not supported in the current system. */
char n_sep_by_space;
/** Symbol position of a non-negative currency value, which is not supported in the current system */
char p_sign_posn;
/** Symbol position of a negative currency value, which is not supported in the current system */
char n_sign_posn;
/** Specifies whether to add an international currency symbol before a non-negative currency value. This parameter is not supported in the current system. */
char int_p_cs_precedes;
/** Specifies whether to add a space before a non-negative currency value as a separator. This parameter is not supported in the current system.  */
char int_p_sep_by_space;
/** Specifies whether to add an international currency symbol before a negative currency value. This parameter is not supported in the current system. */
char int_n_cs_precedes;
/** Specifies whether to add a space before a negative currency value as a separator. This parameter is not supported in the current system. */
char int_n_sep_by_space;
/** Symbol position of a non-negative currency value, which is not supported in the current system */
char int_p_sign_posn;
/** Symbol position of a negative currency value, which is not supported in the current system */
char int_n_sign_posn;
};

/**
* @brief Sets the current locale information based on input parameters.
*
* If <b>locale</b> is <b>NULL</b>, it can be used to obtain the current region settings. \n
* @param category Indicates the type of locale information used to set the impact scope of region settings. The value can be {@link LC_CTYPE}, {@link LC_NUMERIC}, {@link LC_TIME}, {@link LC_COLLATE}, {@link LC_MONETARY}, {@link LC_MESSAGES}, or {@link LC_ALL}. The region settings include the date format, numeric format, currency format, character processing, and character comparison.
* The current region settings can affect only one aspect of the content or all the content.
* @param locale Indicates the string containing region names. These names are system-specific, but at least the following three language environments must exist:
* Language Name | Description
* -------------|------------
* <b>C</b> | Indicates the C language execution environment.
* <b>""</b> | Uses the default region settings of the current system.
* <b>NULL</b> | No name needs to be specified.
* @return Returns a pointer to the string representation of the language environment set for the type if the operation is successful;
* returns a null pointer and the locale is not changed if the operation fails.
* @since 1.0
* @version 1.0
*/
char *setlocale(int category, const char *locale);

/**
* @brief Obtains the locale information about a region and stores them in {@link lconv}.
*
* @return Returns a pointer to <b>lconv</b> that stores region information.
* @since 1.0
* @version 1.0
*/
struct lconv *localeconv(void);

#ifdef __cplusplus
}
#endif

#endif
/** @} */
