/**
* @addtogroup MEM
* @{
*
* @brief Provides structures and functions for performing memory operations, including memory mapping, remapping, allocation, and release.
*
* @since 1.0
* @version 1.0
*/

/**
* @file malloc.h
*
* @brief Declares APIs for allocating and releasing memory.
*
*
*
* @since 1.0
* @version 1.0
*/
#ifndef _MALLOC_H
#define _MALLOC_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Dynamically allocates a block of uninitialized memory with the specified size.
*
* The allocated memory can be initialized by calling {@link memset()}. \n
* @param size Indicates the size of the memory block to allocate, in bytes.
* @return Returns the pointer to the allocated memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails or <b>size</b> is set to <b>0</b>.
* errno | Description
* -------|---------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *malloc(size_t size);

/**
* @brief Dynamically allocates multiple blocks of memory with the specified size.
*
* @param nmemb Indicates the number of memory blocks to allocate.
* @param size Indicates the size of the memory block to allocate, in bytes.
* @return Returns the pointer to the allocated memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails or <b>nmemb</b> or <b>size</b> is set to <b>0</b>.
* errno | Description
* -------|---------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *calloc(size_t nmemb, size_t size);

/**
* @brief Changes the size of a previously allocated memory block pointed to by <b>ptr</b> to the specified size.
*
* The contents of the memory block will be retained from the beginning of the memory pointed to by <b>ptr</b> to the lesser of the old and new sizes.
* If the new size is larger than the old size, the added memory portion will not be initialized. The behavior of this function varies depending on the input parameters:
* <ul>
*   <li>If <b>ptr</b> is not <b>NULL</b>, it should be a pointer returned by an earlier call to {@link malloc()}, {@link calloc()}, or {@link realloc}.
* If the area pointed to by <b>ptr</b> was moved, this function call is equivalent to {@link free()}. </li>
*   <li>If <b>ptr</b> is <b>NULL</b>, this call is equivalent to {@link malloc()}. </li>
*   <li>If <b>size</b> is <b>0</b> and <b>ptr</b> is not <b>NULL</b>, this call is equivalent to {@link free()}. </li>
* </ul>
* @param ptr Indicates the pointer to a memory block previously allocated with {@link malloc}, {@link calloc} or {@link realloc}.
* @param size Indicates the new size for the memory block, in bytes.
* @return Returns the pointer to the new memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *realloc(void *ptr, size_t size);

/**
* @brief Frees the memory space pointed to by <b>ptr</b>.
*
* @param ptr Indicates the pointer to a memory block previously allocated with {@link malloc}, {@link calloc} or {@link realloc}.
* @since 1.0
* @version 1.0
*/
void free(void *ptr);

/**
* @brief Allocates a block of memory with the specified size based on the given alignment mode.
*
* The value of <b>alignment</b> must be a power of two. \n
* @param alignment Indicates the alignment size of the allocated memory.
* @param size Indicates the size of the memory block to allocate, in bytes.
* @return Returns the pointer to the allocated memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------------------------
* EINVAL | Invalid alignment value (not a power of two).
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *memalign(size_t alignment, size_t size);

/**
* @brief Obtains the size of the memory block pointed to by <b>ptr</b>.
*
* @param ptr Indicates the pointer to a block of memory previously allocated by functions such as {@link malloc()}.
* @return Returns the number of usable bytes in the block of allocated memory pointed to by <b>ptr</b>; returns <b>0</b> if <b>ptr</b> is <b>NULL</b>.
* @see malloc() | calloc() | realloc()
* @since 1.0
* @version 1.0
*/
size_t malloc_usable_size(void *ptr);

#ifdef __cplusplus
}
#endif

#endif/** @} */