/**
* @addtogroup MATH
* @{
*
* @brief Provides math functions, such as trigonometric, exponential, logarithmic functions,and perform operations on complex numbers.
*
* @since 1.0
* @version 1.0
*/

/**
* @file math.h
*
* @brief Provides common math functions.
*
* You can use the functions provided in this file to perform mathematical operations during development. The functions include trigonometric,
* inverse trigonometric, hyperbolic, inverse hyperbolic, rounding, exponential, logarithmic, and Bessel functions. \n
* When using these functions, pay attention to the value range of their input parameters. \n
* @since 1.0
* @version 1.0
*/

#ifndef _MATH_H
#define _MATH_H

#ifdef __cplusplus
extern "C" {
#endif

#if 100*__GNUC__+__GNUC_MINOR__ >= 303
/**@brief Indicates an undefined or unrepresentable value. */
#define NAN       __builtin_nanf("")
/**@brief Indicates an infinity. */
#define INFINITY  __builtin_inff()
#else
/**@brief Indicates an undefined or unrepresentable value. */
#define NAN       (0.0f/0.0f)
/**@brief Indicates an infinity. */
#define INFINITY  1e5000f
#endif

/**@brief Indicates a float infinity. */
#define HUGE_VALF INFINITY
/**@brief Indicates a double infinity. */
#define HUGE_VAL  ((double)INFINITY)
/**@brief Indicates a long double infinity. */
#define HUGE_VALL ((long double)INFINITY)

/**@brief Indicates an error that occurs in floating-point operations. */
#define MATH_ERRNO  1
/**@brief Indicates a floating-point exception. */
#define MATH_ERREXCEPT 2
/**@brief Indicates the error handling mechanism employed by math functions. */
#define math_errhandling 2

/**@brief Indicates the calculation result of {@link ilogb}({@link NAN}). */
#define FP_ILOGBNAN (-1-0x7fffffff)
/**@brief Indicates the calculation result of {@link ilogb}(0). */
#define FP_ILOGB0 FP_ILOGBNAN

/**@brief Indicates a Not-a-Number (NaN) value. */
#define FP_NAN       0
/**@brief Indicates an infinity. */
#define FP_INFINITE  1
/**@brief Indicates value <b>0</b>. */
#define FP_ZERO      2
/**@brief Indicates a subnormal. */
#define FP_SUBNORMAL 3
/**@brief Indicates a normal value. */
#define FP_NORMAL    4

/**
* @brief Checks whether the floating-point value <b>x</b> is an infinity.
*
* @param x Indicates the floating-point value <b>x</b> to check.  \n
* @return Returns <b>1</b> if <b>x</b> is an infinity; returns <b>0</b> otherwise.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
#define isinf(x) ( \
sizeof(x) == sizeof(float) ? (__FLOAT_BITS(x) & 0x7fffffff) == 0x7f800000 : \
sizeof(x) == sizeof(double) ? (__DOUBLE_BITS(x) & -1ULL>>1) == 0x7ffULL<<52 : \
__fpclassifyl(x) == FP_INFINITE)

/**
* @brief Checks whether the floating-point value <b>x</b> is a NaN value.
*
* @param x Indicates the floating-point value <b>x</b> to check. \n
* @return Returns a non-zero value if <b>x</b> is a NaN value; returns <b>0</b> otherwise. \n
* @since 1.0  \n
* @version 1.0  \n
*/
#define isnan(x) ( \
sizeof(x) == sizeof(float) ? (__FLOAT_BITS(x) & 0x7fffffff) > 0x7f800000 : \
sizeof(x) == sizeof(double) ? (__DOUBLE_BITS(x) & -1ULL>>1) > 0x7ffULL<<52 : \
__fpclassifyl(x) == FP_NAN)

/**
* @brief Checks whether the floating-point value <b>x</b> is a normal value.
*
* A normal value refers to any value that is not an infinity, a NaN value, or <b>0</b>. \n
* @param x Indicates the floating-point value <b>x</b> to check.  \n
* @return Returns a non-zero value if <b>x</b> is a normal value; returns <b>0</b> otherwise.  \n
* @since 1.0
* @version 1.0
*/
#define isnormal(x) ( \
sizeof(x) == sizeof(float) ? ((__FLOAT_BITS(x)+0x00800000) & 0x7fffffff) >= 0x01000000 : \
sizeof(x) == sizeof(double) ? ((__DOUBLE_BITS(x)+(1ULL<<52)) & -1ULL>>1) >= 1ULL<<53 : \
__fpclassifyl(x) == FP_NORMAL)

/**
* @brief Checks whether the floating-point value <b>x</b> is finite.
*
* A finite value is any value that is neither an infinity nor a NaN value. \n
* @param x Indicates the floating-point value <b>x</b> to check. \n
* @return Returns <b>1</b> if <b>x</b> is finite; returns <b>0</b> otherwise. \n
* @since  1.0
* @version  1.0
*/
#define isfinite(x) ( \
sizeof(x) == sizeof(float) ? (__FLOAT_BITS(x) & 0x7fffffff) < 0x7f800000 : \
sizeof(x) == sizeof(double) ? (__DOUBLE_BITS(x) & -1ULL>>1) < 0x7ffULL<<52 : \
__fpclassifyl(x) > FP_INFINITE)

/**
* @brief Checks whether the sign of the floating-point value <b>x</b> is negative.
*
* @param x Indicates the floating-point value <b>x</b> to check. \n
* @return Returns a non-zero value if the sign of <b>x</b> is negative; returns <b>0</b> otherwise. \n
* @since 1.0 \n
* @version 1.0 \n
*/
#define signbit(x) ( \
sizeof(x) == sizeof(float) ? (int)(__FLOAT_BITS(x)>>31) : \
sizeof(x) == sizeof(double) ? (int)(__DOUBLE_BITS(x)>>63) : \
__signbitl(x) )

/**
* @brief Checks whether floating-point value <b>x</b> or <b>y</b> is unordered.
*
* An unordered value refers to a NaN value. \n
* @param x Indicates the floating-point value <b>x</b> to check. \n
* @param y Indicates the floating-point value <b>y</b> to check. \n
* @return Returns <b>1</b> if either <b>x</b> or <b>y</b> is unordered; returns <b>0</b> otherwise. \n
* @since 1.0 \n
* @version 1.0 \n
*/
#define isunordered(x,y) (isnan((x)) ? ((void)(y),1) : isnan((y)))

/**
* @brief Checks whether floating-point value <b>x</b> is less than <b>y</b>.
*
* @attention If either or both of <b>x</b> and <b>y</b> are a NaN value, an exception occurs. This function requires hardware support. \n
* @param x Indicates the floating-point value <b>x</b> to check.  \n
* @param y Indicates the floating-point value <b>y</b> to check.  \n
* @return Returns <b>1</b> if <b>x</b> is less than <b>y</b>; returns <b>0</b> otherwise.  \n
* @since 1.0
* @version 1.0
*/
#define isless(x, y)            __tg_pred_2(x, y, __isless)

/**
* @brief Checks whether floating-point value <b>x</b> is less than or equal to <b>y</b>.
*
* @attention If either or both of <b>x</b> and <b>y</b> are a NaN value, an exception occurs. This function requires hardware support.  \n
* @param x Indicates the floating-point value <b>x</b> to check.  \n
* @param y Indicates the floating-point value <b>y</b> to check.  \n
* @return Returns <b>1</b> if <b>x</b> is less than or equal to <b>y</b>; returns <b>0</b> otherwise.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
#define islessequal(x, y)       __tg_pred_2(x, y, __islessequal)

/**
* @brief Checks whether floating-point value <b>x</b> is either less than or greater than <b>y</b>.
*
* @attention If either or both of <b>x</b> and <b>y</b> are a NaN value, an exception occurs. This function requires hardware support. \n
* @param x Indicates the floating-point value <b>x</b> to check.  \n
* @param y Indicates the floating-point value <b>y</b> to check.  \n
* @return Returns <b>1</b> if <b>x</b> is either less than or greater than <b>y</b>; returns <b>0</b> otherwise.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
#define islessgreater(x, y)     __tg_pred_2(x, y, __islessgreater)

/**
* @brief Checks whether floating-point value <b>x</b> is greater than <b>y</b>.
*
* @attention If either or both of <b>x</b> and <b>y</b> are a NaN value, an exception occurs. This function requires hardware support.  \n
* @param x Indicates the floating-point value <b>x</b> to check.  \n
* @param y Indicates the floating-point value <b>y</b> to check.  \n
* @return Returns <b>1</b> if <b>x</b> is greater than <b>y</b>; returns <b>0</b> otherwise.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
#define isgreater(x, y)         __tg_pred_2(x, y, __isgreater)

/**
* @brief Checks whether floating-point value <b>x</b> is greater than or equal to <b>y</b>.
*
* @attention If either or both of <b>x</b> and <b>y</b> are a NaN value, an exception occurs. This function requires hardware support.  \n
* @param x Indicates the floating-point value <b>x</b> to check.  \n
* @param y Indicates the floating-point value <b>y</b> to check.  \n
* @return Returns <b>1</b> if <b>x</b> is greater than or equal to <b>y</b>; returns <b>0</b> otherwise.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
#define isgreaterequal(x, y)    __tg_pred_2(x, y, __isgreaterequal)

/**
* @brief Calculates the arc cosine of the double value <b>x</b>.
*
* @param x Indicates the double value whose arc cosine is to be calculated. \n
* @return Returns the arc cosine of <b>x</b> in radians, ranging from 0 to {@link M_PI}. \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+1</b>, this function returns <b>+0</b>.  \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value and reports a domain error.  \n
* If <b>x</b> is out of the range [-1, 1], this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
double acos(double x);

/**
* @brief Calculates the arc cosine of the float value <b>x</b>.
*
* @param x Indicates the float value whose arc cosine is to be calculated.  \n
* @return Returns the arc cosine of <b>x</b> in radians, ranging from 0 to {@link M_PI}.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+1</b>, this function returns <b>+0</b>.  \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value and reports a domain error.  \n
* If <b>x</b> is out of the range [-1, 1], this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
float acosf(float x);

/**
* @brief Calculates the arc cosine of the long double value <b>x</b>.
*
* @param x Indicates the long double value whose arc cosine is to be calculated.  \n
* @return Returns the arc cosine of <b>x</b> in radians, ranging from 0 to {@link M_PI}.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+1</b>, this function returns <b>+0</b>.  \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value and reports a domain error.  \n
* If <b>x</b> is out of the range [-1, 1], this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
long double acosl(long double x);

/**
* @brief Calculates the inverse hyperbolic cosine of the double value <b>x</b>.
*
* @param x Indicates the double value whose inverse hyperbolic cosine is to be calculated.  \n
* @return Returns the inverse hyperbolic cosine of <b>x</b>.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+1</b>, this function returns <b>+0</b>.  \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity.  \n
* If <b>x</b> is less than <b>1</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
double acosh(double x);

/**
* @brief Calculates the inverse hyperbolic cosine of float value <b>x</b>.
*
* @param x Indicates the float value whose inverse hyperbolic cosine is to be calculated.  \n
* @return Returns the inverse hyperbolic cosine of <b>x</b>.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+1</b>, this function returns <b>+0</b>.  \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity.  \n
* If <b>x</b> is less than <b>1</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
float acoshf(float x);

/**
* @brief Calculates the inverse hyperbolic cosine of the long double value <b>x</b>.
*
* @param x Indicates the long double value whose inverse hyperbolic cosine is to be calculated.  \n
* @return Returns the inverse hyperbolic cosine of <b>x</b>.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+1</b>, this function returns <b>+0</b>.  \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity.  \n
* If <b>x</b> is less than <b>1</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
long double acoshl(long double x);

/**
* @brief Calculates the arc sine of the double value <b>x</b>.
*
* @param x Indicates the double value whose arc sine is to be calculated.  \n
* @return Returns the arc sine of <b>x</b> in radians, ranging from -{@link M_PI_2} to {@link M_PI_2}.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>.  \n
* If <b>x</b> is out of the range [-1, 1], this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
double asin(double x);

/**
* @brief Calculates the arc sine of the float value <b>x</b>.
*
* @param x Indicates the float value whose arc sine is to be calculated.  \n
* @return Returns the arc sine of <b>x</b> in radians, ranging from -{@link M_PI_2} to {@link M_PI_2}.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>.  \n
* If <b>x</b> is out of the range [-1, 1], this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
float asinf(float x);

/**
* @brief Calculates the arc sine of the long double value <b>x</b>.
*
* @param x Indicates the long double value whose arc sine is to be calculated.  \n
* @return Returns the arc sine of <b>x</b> in radians, ranging from -{@link M_PI_2} to {@link M_PI_2}.  \n
* If <b>x</b> is a NaN value, this function returns a NaN value.  \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>.  \n
* If <b>x</b> is out of the range [-1, 1], this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception.  \n
* @since 1.0  \n
* @version 1.0  \n
*/
long double asinl(long double x);

/**
* @brief Calculates the inverse hyperbolic sine of the double value <b>x</b>.
*
* @param x Indicates the double value whose inverse hyperbolic sine is to be calculated.
* @return Returns the inverse hyperbolic sine of <b>x</b>. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* @since 1.0
* @version 1.0
*/
double asinh(double x);

/**
* @brief Calculates the inverse hyperbolic sine of the float value <b>x</b>.
*
* @param x Indicates the float value whose inverse hyperbolic sine is to be calculated.
* @return Returns the inverse hyperbolic sine of <b>x</b>. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* @since 1.0
* @version 1.0
*/
float asinhf(float x);

/**
* @brief Calculates the inverse hyperbolic sine of the long double value <b>x</b>.
*
* @param x Indicates the long double value whose inverse hyperbolic sine is to be calculated.
* @return Returns the inverse hyperbolic sine of <b>x</b>. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* @since 1.0
* @version 1.0
*/
long double asinhl(long double x);

/**
* @brief Calculates the arc tangent of the double value <b>x</b>.
*
* @param x Indicates the double value whose arc tangent is to be calculated.
* @return Returns the principal value of the arc tangent of <b>x</b> in radians, ranging from -{@link M_PI_2} to {@link M_PI_2}. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns {@link M_PI_2}. \n
* @since 1.0
* @version 1.0
*/
double atan(double x);

/**
* @brief Calculates the arc tangent of the float value <b>x</b>.
*
* @param x Indicates the float value whose arc tangent is to be calculated.
* @return Returns the principal value of the arc tangent of <b>x</b> in radians, ranging from -{@link M_PI_2} to {@link M_PI_2}. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns {@link M_PI_2} or -{@link M_PI_2}, respectively. \n
* @since 1.0
* @version 1.0
*/
float atanf(float x);

/**
* @brief Calculates the arc tangent of the long double value <b>x</b>.
*
* @param x Indicates the long double value whose arc tangent is to be calculated.
* @return Returns the principal value of the arc tangent of <b>x</b> in radians, ranging from -{@link M_PI_2} to {@link M_PI_2}. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns {@link M_PI_2} or -{@link M_PI_2}, respectively. \n
* @since 1.0
* @version 1.0
*/
long double atanl( long double x);

/**
* @brief Calculates the arc tangent of two double values <b>x</b> and <b>y</b>.
*
* @param y Indicates the double value <b>y</b> for calculating the arc tangent.
* @param x Indicates the double value <b>x</b> for calculating the arc tangent.
* @return Returns the principal value of the arc tangent of y/x in radians, ranging from <b>-{@link M_PI}</b> to <b>{@link M_PI}</b>. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is less than <b>0</b>, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is greater than <b>0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is less than <b>0</b> and <b>x</b> is either <b>+0</b> or <b>-0</b>, this function returns <b>-{@link M_PI_2}</b>. \n
* If <b>y</b> is greater than <b>0</b> and <b>x</b> is either <b>+0</b> or <b>-0</b>, this function returns <b>{@link M_PI_2}</b>. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is <b>-0</b>, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is <b>+0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is a finite value greater than or less than <b>0</b>, and <b>x</b> is a negative infinity, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is a finite value greater than or less than <b>0</b>, and <b>x</b> is a positive infinity, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a finite value, this function returns <b>{@link M_PI_2}</b> or <b>-{@link M_PI_2}</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a negative infinity, this function returns <b>+3*{@link M_PI_4}</b> or <b>-3*{@link M_PI_4}</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a positive infinity, this function returns <b>+{@link M_PI_4}</b> or <b>-{@link M_PI_4}</b>, respectively. \n
* @since 1.0
* @version 1.0
*/
double atan2(double y, double x);

/**
* @brief Calculates the arc tangent of two float values <b>x</b> and <b>y</b>.
*
* @param y Indicates the float value <b>y</b> for calculating the arc tangent.
* @param x Indicates the float value <b>x</b> for calculating the arc tangent.
* @return Returns the principal value of the arc tangent of y/x in radians, ranging from <b>-{@link M_PI}</b> to <b>{@link M_PI}</b>. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is less than <b>0</b>, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is greater than <b>0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is less than <b>0</b> and <b>x</b> is either <b>+0</b> or <b>-0</b>, this function returns <b>-{@link M_PI_2}</b>. \n
* If <b>y</b> is greater than <b>0</b> and <b>x</b> is either <b>+0</b> or <b>-0</b>, this function returns <b>{@link M_PI_2}</b>. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is <b>-0</b>, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is <b>+0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is a finite value greater than or less than <b>0</b>, and <b>x</b> is a negative infinity, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is a finite value greater than or less than <b>0</b>, and <b>x</b> is a positive infinity, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a finite value, this function returns <b>{@link M_PI_2}</b> or <b>-{@link M_PI_2}</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a negative infinity, this function returns <b>+3*{@link M_PI_4}</b> or <b>-3*{@link M_PI_4}</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a positive infinity, this function returns <b>+{@link M_PI_4}</b> or <b>-{@link M_PI_4}</b>, respectively. \n
* @since 1.0
* @version 1.0
*/
float atan2f(float y, float x);

/**
* @brief Calculates the arc tangent of two long double values <b>x</b> and <b>y</b>.
*
* @param y Indicates the long double value <b>y</b> for calculating the arc tangent.
* @param x Indicates the long double value <b>x</b> for calculating the arc tangent.
* @return Returns the principal value of the arc tangent of y/x in radians, ranging from <b>-{@link M_PI}</b> to <b>{@link M_PI}</b>. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is less than <b>0</b>, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is greater than <b>0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is less than <b>0</b> and <b>x</b> is either <b>+0</b> or <b>-0</b>, this function returns <b>-{@link M_PI_2}</b>. \n
* If <b>y</b> is greater than <b>0</b> and <b>x</b> is either <b>+0</b> or <b>-0</b>, this function returns <b>{@link M_PI_2}</b>. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is <b>-0</b>, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is <b>+0</b> or <b>-0</b>, and <b>x</b> is <b>+0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is a finite value greater than or less than <b>0</b>, and <b>x</b> is a negative infinity, this function returns <b>+{@link M_PI}</b> or <b>-{@link M_PI}</b>, respectively. \n
* If <b>y</b> is a finite value greater than or less than <b>0</b>, and <b>x</b> is a positive infinity, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a finite value, this function returns <b>{@link M_PI_2}</b> or <b>-{@link M_PI_2}</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a negative infinity, this function returns <b>+3*{@link M_PI_4}</b> or <b>-3*{@link M_PI_4}</b>, respectively. \n
* If <b>y</b> is a positive or negative infinity, and <b>x</b> is a positive infinity, this function returns <b>+{@link M_PI_4}</b> or <b>-{@link M_PI_4}</b>, respectively. \n
* @since 1.0
* @version 1.0
*/
long double atan2l(long double y, long double x);

/**
* @brief Calculates the inverse hyperbolic tangent of the double value <b>x</b>.
*
* @param x Indicates the double value whose inverse hyperbolic tangent is to be calculated.
* @return Returns the inverse hyperbolic tangent of <b>x</b>. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is <b>+1</b> or <b>-1</b>, this function returns {@link HUGE_VAL}, reports a pole error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* If the absolute value of <b>x</b> is greater than <b>1</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
double atanh(double x);

/**
* @brief Calculates the inverse hyperbolic tangent of the float value <b>x</b>.
*
* @param x Indicates the float value whose inverse hyperbolic tangent is to be calculated.
* @return Returns the inverse hyperbolic tangent of <b>x</b>. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is <b>+1</b> or <b>-1</b>, this function returns {@link HUGE_VAL}, reports a pole error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* If the absolute value of <b>x</b> is greater than <b>1</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
float atanhf(float x);

/**
* @brief Calculates the inverse hyperbolic tangent of the long double value <b>x</b>.
*
* @param x Indicates the long double value whose inverse hyperbolic tangent is to be calculated.
* @return Returns the inverse hyperbolic tangent of <b>x</b>. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is <b>+1</b> or <b>-1</b>, this function returns {@link HUGE_VAL}, reports a pole error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* If the absolute value of <b>x</b> is greater than <b>1</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
long double atanhl(long double x);

/**
* @brief Calculates the cube root of the double value <b>x</b>.
*
* @param x Indicates the double value whose cube root is to be calculated.
* @return Returns the cube root if the operation is successful; returns the passed <b>x</b> if <b>x</b> is <b>+0</b>, <b>-0</b>, a positive or negative infinity, or a NaN value.
* @since 1.0
* @version 1.0
*/
double      cbrt(double x);

/**
* @brief Calculates the cube root of the float value <b>x</b>.
*
* @param x Indicates the float value whose cube root is to be calculated.
* @return Returns the cube root if the operation is successful; returns the passed <b>x</b> if <b>x</b> is <b>+0</b>, <b>-0</b>, a positive or negative infinity, or a NaN value.
* @since 1.0
* @version 1.0
*/
float       cbrtf(float x);

/**
* @brief Calculates the cube root of the long double value <b>x</b>.
*
* @param x Indicates the long double value whose cube root is to be calculated.
* @return Returns the cube root if the operation is successful; returns the passed <b>x</b> if <b>x</b> is <b>+0</b>, <b>-0</b>, a positive or negative infinity, or a NaN value.
* @since 1.0
* @version 1.0
*/
long double cbrtl(long double x);

/**
* @brief Calculates the minimum integer greater than or equal to the double value <b>x</b>.
*
* @param x Indicates the double value <b>x</b> to compare with.
* @return Returns the minimum integer greater than or equal to <b>x</b> if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
double      ceil(double x);

/**
* @brief Calculates the minimum integer greater than or equal to the float value <b>x</b>.
*
* @param x Indicates the float value <b>x</b> to compare with.
* @return Returns the minimum integer greater than or equal to <b>x</b> if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
float       ceilf(float x);

/**
* @brief Calculates the minimum integer greater than or equal to the long double value <b>x</b>.
*
* @param x Indicates the long double value <b>x</b> to compare with.
* @return Returns the minimum integer greater than or equal to <b>x</b> if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
long double ceill(long double x);

/**
* @brief Generates a value by combining the magnitude of the double value <b>x</b> and the sign of the double value <b>y</b>.
*
* For example, if <b>x</b> is <b>42.0</b> and <b>y</b> is <b>-1.0</b>, value <b>-42.0</b> will be generated. \n
* @param x Indicates the double value <b>x</b> whose magnitude will be used.
* @param y Indicates the double value <b>y</b> whose sign will be used.
* @return Returns the generated value if the operation is successful; returns a NaN value with the sign of <b>y</b> if <b>x</b> is a NaN value.
* @since 1.0
* @version 1.0
*/
double      copysign(double x, double y);

/**
* @brief Generates a value by combining the magnitude of the float value <b>x</b> and the sign of the float value <b>y</b>.
*
* For example, if <b>x</b> is <b>42.0</b> and <b>y</b> is <b>-1.0</b>, value <b>-42.0</b> will be generated. \n
* @param x Indicates the float value <b>x</b> whose magnitude will be used.
* @param y Indicates the float value <b>y</b> whose sign will be used.
* @return Returns the generated value if the operation is successful; returns a NaN value with the sign of <b>y</b> if <b>x</b> is a NaN value.
* @since 1.0
* @version 1.0
*/
float       copysignf(float x, float y);

/**
* @brief Generates a value by combining the magnitude of the long double value <b>x</b> and the sign of the long double value <b>y</b>.
*
* For example, if <b>x</b> is <b>42.0</b> and <b>y</b> is <b>-1.0</b>, value <b>-42.0</b> will be generated. \n
* @param x Indicates the long double value <b>x</b> whose magnitude will be used.
* @param y Indicates the long double value <b>y</b> whose magnitude will be used.
* @return Returns the generated value if the operation is successful; returns a NaN value with the sign of <b>y</b> if <b>x</b> is a NaN value.
* @since 1.0
* @version 1.0
*/
long double copysignl(long double x, long double y);

/**
* @brief Calculates the cosine of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the cosine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
double      cos(double x);

/**
* @brief Calculates the cosine of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the cosine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
float       cosf(float x);

/**
* @brief Calculates the cosine of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the cosine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
long double cosl(long double x);

/**
* @brief Calculates the hyperbolic cosine of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the hyperbolic cosine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive infinity. \n
* If an overflow range error occurs, this function returns {@link HUGE_VAL}, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      cosh(double x);

/**
* @brief Calculates the hyperbolic cosine of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the hyperbolic cosine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive infinity. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF}, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       coshf(float x);

/**
* @brief Calculates the hyperbolic cosine of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the hyperbolic cosine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive infinity. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL}, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double coshl(long double x);

/**
* @brief Calculates the error function of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the error function if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns <b>+1</b> or <b>-1</b>, respectively. \n
* If an underflow range error occurs, this function returns <b>2*x/sqrt(pi)</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      erf(double x);

/**
* @brief Calculates the error function of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the error function if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns <b>+1</b> or <b>-1</b>, respectively. \n
* If an underflow range error occurs, this function returns <b>2*x/sqrt(pi)</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       erff(float x);

/**
* @brief Calculates the error function of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the error function if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns <b>+1</b> or <b>-1</b>, respectively. \n
* If an underflow range error occurs, this function returns <b>2*x/sqrt(pi)</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double erfl(long double x);

/**
* @brief Calculates the complementary error function of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the complementary error function if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity, this function returns <b>+0</b>. \n
* If <b>x</b> is a negative infinity, this function returns <b>+2</b>. \n
* If an underflow range error occurs and an unrepresentable value is generated, this function returns <b>0.0</b>. \n
* If an underflow range error occurs and a representable value is generated, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      erfc(double x);

/**
* @brief Calculates the complementary error function of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the complementary error function if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity, this function returns <b>+0</b>. \n
* If <b>x</b> is a negative infinity, this function returns <b>+2</b>. \n
* If an underflow range error occurs and an unrepresentable value is generated, this function returns <b>0.0</b>. \n
* If an underflow range error occurs and a representable value is generated, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       erfcf(float x);

/**
* @brief Calculates the complementary error function of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the complementary error function if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>1</b>. \n
* If <b>x</b> is a positive infinity, this function returns <b>+0</b>. \n
* If <b>x</b> is a negative infinity, this function returns <b>+2</b>. \n
* If an underflow range error occurs and an unrepresentable value is generated, this function returns <b>0.0</b>. \n
* If an underflow range error occurs and a representable value is generated, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double erfcl(long double x);

/**
* @brief Calculates the base-e exponential function of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the base-e exponential function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>+0</b>. \n
* If an underflow range error occurs, this function returns <b>0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VAL}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      exp(double x);

/**
* @brief Calculates the base-e exponential function of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the base-e exponential function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>+0</b>. \n
* If an underflow range error occurs, this function returns <b>0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VALF}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       expf(float x);

/**
* @brief Calculates the base-e exponential function of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the base-e exponential function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>+0</b>. \n
* If an underflow range error occurs, this function returns <b>0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VALL}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double expl(long double x);

/**
* @brief Calculates the base-2 exponential function of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the base-2 exponential function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>+0</b>. \n
* If an underflow range error occurs, this function returns <b>0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VALL}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      exp2(double x);

/**
* @brief Calculates the base-2 exponential function of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the base-2 exponential function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>+0</b>. \n
* If an underflow range error occurs, this function returns <b>0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VALL}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       exp2f(float x);

/**
* @brief Calculates the base-2 exponential function of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the base-2 exponential function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>+0</b>. \n
* If an underflow range error occurs, this function returns <b>0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VALL}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double exp2l(long double x);

/**
* @brief Calculates e raised to the power of the double value <b>x</b> minus one, that is, (e^x)-1.
*
* @param x Indicates the double value.
* @return Returns e raised to the power of <b>x</b> minus one if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>-1</b>. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VAL}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      expm1(double x);

/**
* @brief Calculates e raised to the power of the float value <b>x</b> minus one.
*
* @param x Indicates the float value.
* @return Returns e raised to the power of <b>x</b> minus one if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>-1</b>. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VALF}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       expm1f(float x);

/**
* @brief Calculates e raised to the power of the long double value <b>x</b> minus one.
*
* @param x Indicates the long double value.
* @return Returns e raised to the power of <b>x</b> minus one if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity, this function returns <b>-1</b>. \n
* If an overflow range error occurs, this function returns <b>+{@link HUGE_VALL}</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double expm1l(long double x);

/**
* @brief Calculates the absolute value of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the absolute value of <b>x</b> if the operation is successful; returns a NaN value if <b>x</b> is a NaN value; returns <b>+0</b> if <b>x</b> is <b>-0</b>; returns a positive infinity if <b>x</b> is a positive or negative infinity.
* @since 1.0
* @version 1.0
*/
double      fabs(double x);

/**
* @brief Calculates the absolute value of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the absolute value of <b>x</b> if the operation is successful; returns a NaN value if <b>x</b> is a NaN value; returns <b>+0</b> if <b>x</b> is <b>-0</b>; returns a positive infinity if <b>x</b> is a positive or negative infinity.
* @since 1.0
* @version 1.0
*/
float       fabsf(float x);

/**
* @brief Calculates the absolute value of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the absolute value of <b>x</b> if the operation is successful; returns a NaN value if <b>x</b> is a NaN value; returns <b>+0</b> if <b>x</b> is <b>-0</b>; returns a positive infinity if <b>x</b> is a positive or negative infinity.
* @since 1.0
* @version 1.0
*/
long double fabsl(long double x);

/**
* @brief Calculates the positive difference between the double value <b>x</b> and <b>y</b>.
*
* @param x Indicates the double value <b>x</b>.
* @param y Indicates the double value <b>y</b>.
* @return Returns the positive difference between <b>x</b> and <b>y</b>.
* @since 1.0
* @version 1.0
*/
double      fdim(double x, double y);

/**
* @brief Calculates the positive difference between the float value <b>x</b> and <b>y</b>.
*
* @param x Indicates the float value <b>x</b>.
* @param y Indicates the float value <b>y</b>.
* @return Returns the positive difference between <b>x</b> and <b>y</b>.
* @since 1.0
* @version 1.0
*/
float       fdimf(float x, float y);

/**
* @brief Calculates the positive difference between the long double value <b>x</b> and <b>y</b>.
*
* @param x Indicates the long double value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @return Returns the positive difference between <b>x</b> and <b>y</b>.
* @since 1.0
* @version 1.0
*/
long double fdiml(long double x, long double y);

/**
* @brief Obtains the largest integer less than or equal to the double value <b>x</b>.
*
* @param x Indicates the double value <b>x</b> to compare with.
* @return Returns the largest integer less than or equal to <b>x</b>.
* @since 1.0
* @version 1.0
*/
double      floor(double x);

/**
* @brief Obtains the largest integer less than or equal to the float value <b>x</b>.
*
* @param x Indicates the float value <b>x</b> to compare with.
* @return Returns the largest integer less than or equal to <b>x</b>.
* @since 1.0
* @version 1.0
*/
float       floorf(float x);

/**
* @brief Obtains the largest integer less than or equal to the long double value <b>x</b>.
*
* @param x Indicates the long double value <b>x</b> to compare with.
* @return Returns the largest integer less than or equal to <b>x</b>.
* @since 1.0
* @version 1.0
*/
long double floorl(long double x);

/**
* @brief Calculates the value of <b>x*y+z</b> and rounds the result up.
*
* @attention The rounding mode is the same as that set in the system.
* @param x Indicates the double value <b>x</b>.
* @param y Indicates the double value <b>y</b>.
* @param z Indicates the double value <b>z</b>.
* @return Returns the calculation result.
* @since 1.0
* @version 1.0
*/
double      fma(double x, double y, double z);

/**
* @brief Calculates the value of <b>x*y+z</b> and rounds the result up.
*
* @attention The rounding mode is the same as that set in the system.
* @param x Indicates the float value <b>x</b>.
* @param y Indicates the float value <b>y</b>.
* @param z Indicates the float value <b>z</b>.
* @return Returns the calculation result.
* @since 1.0
* @version 1.0
*/
float       fmaf(float x, float y, float z);

/**
* @brief Calculates the value of <b>x*y+z</b> and rounds the result up.
*
* @attention The rounding mode is the same as that set in the system.
* @param x Indicates the long double value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @param z Indicates the long double value <b>z</b>.
* @return Returns the calculation result.
* @since 1.0
* @version 1.0
*/
long double fmal(long double x, long double y, long double z);

/**
* @brief Obtains the larger value of two double values <b>x</b> and <b>y</b>.
*
* @param x Indicates the double value <b>x</b>.
* @param y Indicates the double value <b>y</b>.
* @return Returns the larger value.
* @since 1.0
* @version 1.0
*/
double      fmax(double x, double y);

/**
* @brief Obtains the larger value of two float values <b>x</b> and <b>y</b>.
*
* @param x Indicates the float value <b>x</b>.
* @param y Indicates the float value <b>y</b>.
* @return Returns the larger value.
* @since 1.0
* @version 1.0
*/
float       fmaxf(float x, float y);

/**
* @brief Obtains the larger value of two long double values <b>x</b> and <b>y</b>.
*
* @param x Indicates the long double value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @return Returns the larger value.
* @since 1.0
* @version 1.0
*/
long double fmaxl(long double x, long double y);

/**
* @brief Obtains the smaller value of two double values <b>x</b> and <b>y</b>.
*
* @param x Indicates the double value <b>x</b>.
* @param y Indicates the double value <b>y</b>.
* @return Returns the smaller value.
* @since 1.0
* @version 1.0
*/
double      fmin(double x, double y);

/**
* @brief Obtains the smaller value of two float values <b>x</b> and <b>y</b>.
*
* @param x Indicates the float value <b>x</b>.
* @param y Indicates the float value <b>y</b>.
* @return Returns the smaller value.
* @since 1.0
* @version 1.0
*/
float       fminf(float x, float y);

/**
* @brief Obtains the smaller value of two long double values <b>x</b> and <b>y</b>.
*
* @param x Indicates the long double value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @return Returns the smaller value.
* @since 1.0
* @version 1.0
*/
long double fminl(long double x, long double y);

/**
* @brief Calculates the remainder of the double value <b>x</b> divided by the double value <b>y</b>.
*
*  \n
* @param x Indicates the double value <b>x</b>, which is used as the numerator.
* @param y Indicates the double value <b>y</b>, which is used as the denominator.
* @return Returns the remainder.
* @since 1.0
* @version 1.0
*/
double      fmod(double x, double y);

/**
* @brief Calculates the remainder of the float value <b>x</b> divided by the float value <b>y</b>.
*
* @param x Indicates the float value <b>x</b>, which is used as the numerator.
* @param y Indicates the float value <b>y</b>, which is used as the denominator.
* @return Returns the remainder.
* @since 1.0
* @version 1.0
*/
float       fmodf(float x, float y);

/**
* @brief Calculates the remainder of the long double value <b>x</b> divided by the long double value <b>y</b>.
*
* @param x Indicates the long double value <b>x</b>, which is used as the numerator.
* @param y Indicates the long double value <b>y</b>, which is used as the denominator.
* @return Returns the remainder.
* @since 1.0
* @version 1.0
*/
long double fmodl(long double x, long double y);

/**
* @brief Decomposes the double value <b>x</b> into a significand and an integral exponent for 2.
*
* The absolute value of the significand ranges from 0.5 (included) to 1 (excluded). x = significand * 2^exponent \n
* @param x Indicates the double value <b>x</b> to decompose.
* @param exp Indicates the pointer to the exponent.
* @return Returns the binary significand of <b>x</b> if the operation is successful; returns <b>0</b> if <b>x</b> is <b>0</b> (<b>exp</b> is also <b>0</b>); returns a NaN value if <b>x</b> is a NaN value (<b>exp</b> is not specified);
* returns a positive or negative infinity if <b>x</b> is a positive or negative infinity, respectively (<b>exp</b> is not specified).
* @since 1.0
* @version 1.0
*/
double      frexp(double x, int *exp);

/**
* @brief Decomposes float value <b>x</b> into a binary significand and an integral exponent for 2.
*
* The absolute value of the significand ranges from 0.5 (included) to 1 (excluded). x = significand * 2^exponent \n
* @param x Indicates the float value to decompose.
* @param exp Indicates the pointer to an integer where the exponent is stored.
* @return Returns the binary significand of <b>x</b> if the operation is successful; returns <b>0</b> if <b>x</b> is <b>0</b> (<b>exp</b> is also <b>0</b>); returns a NaN value if <b>x</b> is a NaN value (<b>exp</b> is not specified);
* returns a positive or negative infinity if <b>x</b> is a positive or negative infinity (<b>exp</b> is not specified).
* @since 1.0
* @version 1.0
*/
float       frexpf(float x, int *exp);

/**
* @brief Decomposes long double value <b>x</b> into a binary significand and an integral exponent for 2.
*
* The absolute value of the significand ranges from 0.5 (included) to 1 (excluded). x = significand * 2^exponent \n
* @param x Indicates the long double value to decompose.
* @param exp Indicates the pointer to an integer where the exponent is stored.
* @return Returns the binary significand of <b>x</b> if the operation is successful; returns <b>0</b> if <b>x</b> is <b>0</b> (<b>exp</b> is also <b>0</b>); returns a NaN value if <b>x</b> is a NaN value (<b>exp</b> is not specified);
* returns a positive or negative infinity if <b>x</b> is a positive or negative infinity (<b>exp</b> is not specified).
* @since 1.0
* @version 1.0
*/
long double frexpl(long double x, int *exp);

/**
* @brief Calculates the hypotenuse of a right triangle whose legs are <b>x</b> and <b>y</b>.
*
* @param x Indicates the length of one side of the right triangle, represented using a double value.
* @param y Indicates the length of the other side of the right triangle, represented using a double value.
* @return Returns the hypotenuse of the right triangle.
* @since 1.0
* @version 1.0
*/
double      hypot(double x, double y);

/**
* @brief Calculates the hypotenuse of a right triangle whose legs are <b>x</b> and <b>y</b>.
*
* @param x Indicates the length of one side of the right triangle, represented using a float value.
* @param y Indicates the length of the other side of the right triangle, represented using a float value.
* @return Returns the hypotenuse of the right triangle.
* @since 1.0
* @version 1.0
*/
float       hypotf(float x, float y);

/**
* @brief Calculates the hypotenuse of a right triangle whose legs are <b>x</b> and <b>y</b>.
*
* @param x Indicates the length of one side of the right triangle, represented using a long double value.
* @param y Indicates the length of the other side of the right triangle, represented using a long double value.
* @return Returns the hypotenuse of the right triangle.
* @since 1.0
* @version 1.0
*/
long double hypotl(long double x, long double y);

/**
* @brief Obtains the integral part of the logarithm of double value <b>x</b>.
*
* {@link FLT_RADIX} is used as the base for the logarithm. \n
* @param x Indicates the double value.
* @return Returns the integral part of the logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is <b>0</b>, this function returns {@link FP_ILOGB0} and reports a domain error. \n
* If <b>x</b> is a NaN value, this function returns {@link FP_ILOGBNAN} and reports a domain error. \n
* If <b>x</b> is positive or negative infinity, this function returns {@link INT_MAX} and reports a domain error. \n
* @since 1.0
* @version 1.0
*/
int         ilogb(double x);

/**
* @brief Obtains the integral part of the logarithm of float value <b>x</b>.
*
* {@link FLT_RADIX} is used as the base for the logarithm. \n
* @param x Indicates the float value.
* @return Returns the integral part of the logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is <b>0</b>, this function returns {@link FP_ILOGB0} and reports a domain error. \n
* If <b>x</b> is a NaN value, this function returns {@link FP_ILOGBNAN} and reports a domain error. \n
* If <b>x</b> is positive or negative infinity, this function returns {@link INT_MAX} and reports a domain error. \n
* @since 1.0
* @version 1.0
*/
int         ilogbf(float x);

/**
* @brief Obtains the integral part of the logarithm of the long double value <b>x</b>.
*
* {@link FLT_RADIX} is used as the base for the logarithm. \n
* @param x Indicates the long double value.
* @return Returns the integral part of the logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is <b>0</b>, this function returns {@link FP_ILOGB0} and reports a domain error. \n
* If <b>x</b> is a NaN value, this function returns {@link FP_ILOGBNAN} and reports a domain error. \n
* If <b>x</b> is positive or negative infinity, this function returns {@link INT_MAX} and reports a domain error. \n
* @since 1.0
* @version 1.0
*/
int         ilogbl(long double x);

/**
* @brief Multiplies the double value <b>x</b> by 2 raised to the power of <b>exp</b>.
*
* The calculation formula is x*2^exp. \n
* @param x Indicates the double value.
* @param exp Indicates the exponent.
* @return Returns the calculation result.
* @since 1.0
* @version 1.0
*/
double      ldexp(double x, int exp);

/**
* @brief Multiplies the float value <b>x</b> by 2 raised to the power of <b>exp</b>.
*
* The calculation formula is x*2^exp. \n
* @param x Indicates the float value.
* @param exp Indicates the exponent.
* @return Returns the calculation result.
* @since 1.0
* @version 1.0
*/
float       ldexpf(float x, int exp);

/**
* @brief Multiplies the long double value <b>x</b> by 2 raised to the power of <b>exp</b>.
*
* The calculation formula is x*2^exp. \n
* @param x Indicates the long double value.
* @param exp Indicates the exponent.
* @return Returns the calculation result.
* @since 1.0
* @version 1.0
*/
long double ldexpl(long double x, int exp);

/**
* @brief Calculates the natural logarithm of the absolute value of the gamma function of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>1</b> or <b>2</b>, this function returns <b>+0</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b> or a negative value, this function returns one of +{@link HUGE_VAL}, reports a pole error, and sets <b>errno</b> to {@link ERANGE}). \n
* If an overflow range error occurs, this function returns one of {@link HUGE_VAL} * and sets <b>errno</b> to {@link ERANGE}. \n
* @since 1.0
* @version 1.0
*/
double      lgamma(double x);

/**
* @brief Calculates the natural logarithm of the absolute value of the gamma function of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>1</b> or <b>2</b>, this function returns <b>+0</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b> or a negative value, this function returns one of +{@link HUGE_VALF}, reports a pole error, and sets <b>errno</b> to {@link ERANGE}. \n
* If an overflow range error occurs, this function returns one of {@link HUGE_VALF} and sets <b>errno</b> to {@link ERANGE}. \n
* @since 1.0
* @version 1.0
*/
float       lgammaf(float x);

/**
* @brief Calculates the natural logarithm of the absolute value of the gamma function of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>1</b> or <b>2</b>, this function returns <b>+0</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b> or a negative value, this function returns one of +{@link HUGE_VALL}, reports a pole error, and sets <b>errno</b> to {@link ERANGE}. \n
* If an overflow range error occurs, this function returns one of {@link HUGE_VALL} and sets <b>errno</b> to {@link ERANGE}. \n
* @since 1.0
* @version 1.0
*/
long double lgammal(long double x);

/**
* @brief Rounds double value <b>x</b> to the nearest integer.
*
* @attention The type of <b>x</b>, rounding mode, and rounding direction are specified by {@link fegetround}.
* @param x Indicates the double value to round.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long long int   llrint(double x);

/**
* @brief Rounds float value <b>x</b> to the nearest integer.
*
* @attention The type of <b>x</b>, rounding mode, and rounding direction are specified by {@link fegetround}.
* @param x Indicates the float value.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long long int   llrintf(float x);

/**
* @brief Rounds long double value <b>x</b> to the nearest integer.
*
* @attention The type of <b>x</b>, rounding mode, and rounding direction are specified by {@link fegetround}.
* @param x Indicates the long double value.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long long int  llrintl(long double x);

/**
* @brief Rounds double value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>.
* @param x Indicates the double value.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long long int   llround(double x);

/**
* @brief Rounds float value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>.
* @param x Indicates the float value.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long long int   llroundf(float x);

/**
* @brief Rounds long double value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>.
* @param x Indicates the long double value.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long long int   llroundl(long double x);

/**
* @brief Calculates the natural logarithm of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the natural logarithm of <b>x</b> if the operation is successful. \n 
* If <b>x</b> is a NaN value, this function returns a NaN value. \n 
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VAL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
double      log(double x);

/**
* @brief Calculates the natural logarithm of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the natural logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALF}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
float       logf(float x);

/**
* @brief Calculates the natural logarithm of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the natural logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
long double logl(long double x);

/**
* @brief Calculates the common logarithm (logarithm with base 10) of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the natural logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VAL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
double      log10(double x);

/**
* @brief Calculates the common logarithm (logarithm with base 10) of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the natural logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALF}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
float       log10f(float x);

/**
* @brief Calculates the common logarithm (logarithm with base 10) of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the natural logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
long double log10l(long double x);

/**
* @brief Calculates the natural logarithm of one plus the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the natural logarithm of one plus <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>-1</b>, this function returns <b>-{@link HUGE_VAL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is less than <b>-1</b> (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
double      log1p(double x);

/**
* @brief Calculates the natural logarithm of one plus the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the natural logarithm of one plus <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>-1</b>, this function returns <b>-{@link HUGE_VALF}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is less than <b>-1</b> (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
float       log1pf(float x);

/**
* @brief Calculates the natural logarithm of one plus the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the natural logarithm of one plus <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>-1</b>, this function returns <b>-{@link HUGE_VALL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is less than <b>-1</b> (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
long double log1pl(long double x);

/**
* @brief Calculates the binary logarithm (logarithm with base 2) of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the binary logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VAL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
double      log2(double x);

/**
* @brief Calculates the binary logarithm (logarithm with base 2) of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the binary logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALF}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
float       log2f(float x);

/**
* @brief Calculates the binary logarithm (logarithm with base 2) of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the binary (base-2) logarithm of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is a negative value (including a negative infinity), this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
long double log2l(long double x);

/**
* @brief Calculates the logarithm of the absolute value of the double value <b>x</b>.
*
* {@link FLT_RADIX} is used as the base for the logarithm. \n
* @attention Set {@link FLT_RADIX} to <b>2</b> in the <b>float.h</b> file.
* @param x Indicates the double value.
* @return Returns the logarithm of the absolute value of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VAL}</b>, reports a pole error, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
double      logb(double x);

/**
* @brief Calculates the logarithm of the absolute value of the float value <b>x</b>.
*
* {@link FLT_RADIX} is used as the base for the logarithm. \n
* @attention Set {@link FLT_RADIX} to <b>2</b> in the <b>float.h</b> file.
* @param x Indicates the float value.
* @return Returns the logarithm of the absolute value of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALF}</b>, reports a pole error, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
float       logbf(float x);

/**
* @brief Calculates the logarithm of the absolute value of the long double value <b>x</b>.
*
* {@link FLT_RADIX} is used as the base for the logarithm. \n
* @attention Set {@link FLT_RADIX} to <b>2</b> in the <b>float.h</b> file.
* @param x Indicates the long double value.
* @return Returns the logarithm of the absolute value of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>0</b>, this function returns <b>-{@link HUGE_VALL}</b>, reports a pole error, and throws the {@link FE_DIVBYZERO} exception. \n
* @since 1.0
* @version 1.0
*/
long double logbl(long double x);

/**
* @brief Rounds the double value <b>x</b> to the nearest integer.
*
* @attention The type of <b>x</b>, rounding mode, and rounding direction are specified by {@link fegetround}.
* @param x Indicates the double value to round.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long int     lrint(double x);

/**
* @brief Rounds the float value <b>x</b> to the nearest integer.
*
* @attention The type of <b>x</b>, rounding mode, and rounding direction are specified by {@link fegetround}.
* @param x Indicates the float value to round.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long int      lrintf(float x);

/**
* @brief Rounds the long double value <b>x</b> to the nearest integer.
*
* @attention The type of <b>x</b>, rounding mode, and rounding direction are specified by {@link fegetround}.
* @param x Indicates the long double value.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long int     lrintl(long double x);

/**
* @brief Rounds double value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>.
* @param x Indicates the double value to round.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long int        lround(double x);

/**
* @brief Rounds the float value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>.
* @param x Indicates the float value to round.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long int        lroundf(float x);

/**
* @brief Rounds the long double value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>.
* @param x Indicates the long double value to round.
* @return Returns the nearest integer if the operation is successful; returns an unspecified value if <b>x</b> is a NaN value or an infinity, or the rounded value is outside the specified range, in which case a domain error or overflow range error occurs.
* @since 1.0
* @version 1.0
*/
long int        lroundl(long double x);

/**
* @brief Breaks the double value <b>x</b> into a signed integral and a fractional part.
*
* @param x Indicates the double value <b>x</b>.
* @param iptr Indicates the pointer to the object where the signed integral part is stored.
* @return Returns the fractional part of <b>x</b>.
* @since 1.0
* @version 1.0
*/
double modf(double x, double *iptr);

/**
* @brief Breaks the float value <b>x</b> into a signed integral and a fractional part.
*
* @param x Indicates the float value <b>x</b>.
* @param iptr Indicates the pointer to the object where the signed integral part is stored.
* @return Returns the fractional part of <b>x</b>.
* @since 1.0
* @version 1.0
*/
float modff(float x, float *iptr);

/**
* @brief Breaks the long double value <b>x</b> into a signed integral and a fractional part.
*
* @param x Indicates the long double value <b>x</b>.
* @param iptr Indicates the pointer to the object where the signed integral part is stored.
* @return Returns the fractional part of <b>x</b>.
* @since 1.0
* @version 1.0
*/
long double modfl(long double x, long double *iptr);

/**
* @brief Obtains a quiet NaN value of the double type.
*
* This function checks whether the input parameter is a NaN value and converts it to the double type. \n
* @param tagp Indicates the pointer to a character constant.
* @return Returns a quiet NaN value if the operation is successful; returns a NaN value otherwise.
* @since 1.0
* @version 1.0
*/
double nan(const char *tagp);

/**
* @brief Obtains a quiet NaN value of the float type.
*
* This function checks whether the input parameter is a NaN value and converts it to the float type. \n
* @param tagp Indicates the pointer to a character constant.
* @return Returns a quiet NaN value if the operation is successful; returns a NaN value otherwise.
* @since 1.0
* @version 1.0
*/
float nanf(const char *tagp);

/**
* @brief Obtains a quiet NaN value of the long double type.
*
* This function checks whether the input parameter is a NaN value and converts it to the long double type. \n
* @param tagp Indicates the pointer to a character constant.
* @return Returns a quiet NaN value if the operation is successful; returns a NaN value otherwise.
* @since 1.0
* @version 1.0
*/
long double nanl(const char *tagp);

/**
* @brief Rounds the double value <b>x</b> to an integer in floating-point format.
*
* @attention The rounding mode is specified by {@link fegetround}.
* @param x Indicates the double value <b>x</b>.
* @return Returns the rounded value.
* @since 1.0
* @version 1.0
*/
double nearbyint(double x);

/**
* @brief Rounds the float value <b>x</b> to an integer in floating-point format.
*
* @attention The rounding mode is specified by {@link fegetround}.
* @param x Indicates the float value <b>x</b>.
* @return Returns the rounded value.
* @since 1.0
* @version 1.0
*/
float nearbyintf(float x);

/**
* @brief Rounds the long double value <b>x</b> to an integer in floating-point format.
*
* @attention The rounding mode is specified by {@link fegetround}.
* @param x Indicates the long double value.
* @return Returns the rounded value.
* @since 1.0
* @version 1.0
*/
long double nearbyintl(long double x);

/**
* @brief Obtains the next representable value following the double value <b>x</b> in the direction of the double value <b>y</b>.
*
* @attention If <b>y</b> is less than <b>x</b>, this function returns the maximum representable value less than <b>x</b>.
* @param x Indicates the double value <b>x</b>.
* @param y Indicates the double value <b>y</b>.
* @return Returns the next representable value after <b>x</b> in the direction of <b>y</b> if the operation is successful; returns the passed <b>y</b> if <b>x</b> is equal to <b>y</b>; returns a NaN value if <b>x</b> is a NaN value;
* returns {@link HUGE_VAL} and reports an overflow range error if <b>x</b> is a finite value.
* @since 1.0
* @version 1.0
*/
double nextafter(double x, double y);

/**
* @brief Obtains the next representable value following the float value <b>x</b> in the direction of the float value <b>y</b>.
*
* @attention If <b>y</b> is less than <b>x</b>, this function returns the maximum representable value less than <b>x</b>.
* @param x Indicates the float value <b>x</b>.
* @param y Indicates the float value <b>y</b>.
* @return Returns the next representable value following <b>x</b> in the direction of <b>y</b> if the operation is successful; returns the passed <b>y</b> if <b>x</b> is equal to <b>y</b>; returns a NaN value if <b>x</b> is a NaN value;
* returns {@link HUGE_VALF} and reports an overflow range error if <b>x</b> is a finite value.
* @since 1.0
* @version 1.0
*/
float nextafterf(float x, float y);

/**
* @brief Obtains the next representable value following the long double value <b>x</b> in the direction of the long double value <b>y</b>.
*
* @attention If <b>y</b> is less than <b>x</b>, this function returns the maximum representable value less than <b>x</b>.
* @param x Indicates the long double value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @return Returns the next representable value after <b>x</b> in the direction of <b>y</b> if the operation is successful; returns the passed <b>y</b> if <b>x</b> is equal to <b>y</b>; returns a NaN value if <b>x</b> is a NaN value;
* returns {@link HUGE_VALL} and reports an overflow range error if <b>x</b> is a finite value.
* @since 1.0
* @version 1.0
*/
long double nextafterl(long double x, long double y);

/**
* @brief Obtains the next representable value following the double value <b>x</b> in the direction of the long double value <b>y</b>.
*
* @attention If <b>y</b> is less than <b>x</b>, this function returns the maximum representable value less than <b>x</b>.
* @param x Indicates the double value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @return Returns the next representable value following <b>x</b> in the direction of <b>y</b> if the operation is successful; returns the passed <b>y</b> if <b>x</b> is equal to <b>y</b>; returns a NaN value if <b>x</b> is a NaN value;
* returns {@link HUGE_VAL} and reports an overflow range error if <b>x</b> is a finite value.
* @since 1.0
* @version 1.0
*/
double nexttoward(double x, long double y);

/**
* @brief Obtains the next representable value following the float value <b>x</b> in the direction of the long double value <b>y</b>.
*
* @attention If <b>y</b> is less than <b>x</b>, this function returns the maximum representable value less than <b>x</b>.
* @param x Indicates the float value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @return Returns the next representable value after <b>x</b> in the direction of <b>y</b> if the operation is successful; returns the passed <b>y</b> if <b>x</b> is equal to <b>y</b>; returns a NaN value if <b>x</b> is a NaN value;
* returns {@link HUGE_VALF} and reports an overflow range error if <b>x</b> is a finite value.
* @version 1.0
*/
float nexttowardf(float x, long double y);

/**
* @brief Obtains the next representable value following the long double value <b>x</b> in the direction of the long double value <b>y</b>.
*
* @attention If <b>y</b> is less than <b>x</b>, this function returns the maximum representable value less than <b>x</b>.
* @param x Indicates the long double value <b>x</b>.
* @param y Indicates the long double value <b>y</b>.
* @return Returns the next representable value following <b>x</b> in the direction of <b>y</b> if the operation is successful; returns the passed <b>y</b> if <b>x</b> is equal to <b>y</b>; returns a NaN value if <b>x</b> is a NaN value;
* returns {@link HUGE_VALL} and reports an overflow range error if <b>x</b> is a finite value.
* @since 1.0
* @version 1.0
*/
long double nexttowardl(long double x, long double y);

/**
* @brief Obtains the double value <b>x</b> raised to the power of the double value <b>y</b>.
*
* @param x Indicates the double value <b>x</b>, which is used as the base.
* @param y Indicates the double value <b>y</b>, which is used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this returns a NaN value, except as specified below. \n
* If <b>x</b> is <b>+1</b> (even if <b>y</b> is a NaN value), this function returns <b>1.0</b>. \n
* If <b>y</b> is <b>0</b> (even if <b>x</b> is a NaN value), this function returns <b>1.0</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>y</b> is an odd integer greater than <b>0</b>, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is greater than <b>0</b> and not an odd integer, this function returns <b>+0</b>. \n
* If <b>x</b> is <b>-1</b> and <b>y</b> is a positive or negative infinity, this function returns <b>1.0</b>. \n
* If the absolute value of <b>x</b> is less than <b>1</b> and <b>y</b> is a negative infinity, this function returns a positive infinity. \n
* If the absolute value of <b>x</b> is greater than <b>1</b> and <b>y</b> is a negative infinity, this function returns <b>+0</b>. \n
* If the absolute value of <b>x</b> is less than <b>1</b> and <b>y</b> is a positive infinity, this function returns <b>+0</b>. \n
* If the absolute value of <b>x</b> is greater than <b>1</b> and <b>y</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an odd integer less than <b>0</b>, this function returns <b>-0</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is less than <b>0</b> and not an odd integer, this function returns <b>+0</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an odd integer greater than <b>0</b>, this function returns a negative infinity. \n
* If <b>x</b> is a negative infinity and <b>y</b> is greater than <b>0</b> and not an odd integer, this function returns a positive infinity. \n
* If <b>x</b> is a positive infinity and <b>y</b> is less than <b>0</b>, this function returns <b>+0</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is greater than <b>0</b>, this function returns a positive infinity. \n
* If <b>x</b> is a negative value and <b>y</b> is a finite non-integer, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is a negative odd integer, this function returns {@link HUGE_VAL}, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is a negative non-odd integer, this function returns <b>+{@link HUGE_VAL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VAL}, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0.0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      pow(double x, double y);

/**
* @brief Obtains the float value <b>x</b> raised to the power of the float value <b>y</b>.
*
* @param x Indicates the float value <b>x</b>, which is used as the base.
* @param y Indicates the float value <b>y</b>, which is used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this returns a NaN value, except as specified below. \n
* If <b>x</b> is <b>+1</b> (even if <b>y</b> is a NaN value), this function returns <b>1.0</b>. \n
* If <b>y</b> is <b>0</b> (even if <b>x</b> is a NaN value), this function returns <b>1.0</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>y</b> is an odd integer greater than <b>0</b>, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is greater than <b>0</b> and not an odd integer, this function returns <b>+0</b>. \n
* If <b>x</b> is <b>-1</b> and <b>y</b> is a positive or negative infinity, this function returns <b>1.0</b>. \n
* If the absolute value of <b>x</b> is less than <b>1</b> and <b>y</b> is a negative infinity, this function returns a positive infinity. \n
* If the absolute value of <b>x</b> is greater than <b>1</b> and <b>y</b> is a negative infinity, this function returns <b>+0</b>. \n
* If the absolute value of <b>x</b> is less than <b>1</b> and <b>y</b> is a positive infinity, this function returns <b>+0</b>. \n
* If the absolute value of <b>x</b> is greater than <b>1</b> and <b>y</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an odd integer less than <b>0</b>, this function returns <b>-0</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is less than <b>0</b> and not an odd integer, this function returns <b>+0</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an odd integer greater than <b>0</b>, this function returns a negative infinity. \n
* If <b>x</b> is a negative infinity and <b>y</b> is greater than <b>0</b> and not an odd integer, this function returns a positive infinity. \n
* If <b>x</b> is a positive infinity and <b>y</b> is less than <b>0</b>, this function returns <b>+0</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is greater than <b>0</b>, this function returns a positive infinity. \n
* If <b>x</b> is a negative value and <b>y</b> is a finite non-integer, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is a negative odd integer, this function returns {@link HUGE_VALF}, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is a negative non-odd integer, this function returns <b>+{@link HUGE_VALF}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF}, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0.0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       powf(float x, float y);

/**
* @brief Calculates the xth power of 10.
*
* @param x Indicates the float value x, which is used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If y is 0 (even if x is a NaN value), this function returns 1.0. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF}, sets errno to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns 0.0, sets errno to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float powf10(float x);

/**
* @brief Obtains the long double value <b>x</b> raised to the power of the long double value <b>y</b>.
*
*  \n
* @param x Indicates the long double value <b>x</b>, which is used as the base.
* @param y Indicates the long double value <b>y</b>, which is used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this returns a NaN value, except as specified below. \n
* If <b>x</b> is <b>+1</b> (even if <b>y</b> is a NaN value), this function returns <b>1.0</b>. \n
* If <b>y</b> is <b>0</b> (even if <b>x</b> is a NaN value), this function returns <b>1.0</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>y</b> is an odd integer greater than <b>0</b>, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>y</b> is greater than <b>0</b> and not an odd integer, this function returns <b>+0</b>. \n
* If <b>x</b> is <b>-1</b> and <b>y</b> is a positive or negative infinity, this function returns <b>1.0</b>. \n
* If the absolute value of <b>x</b> is less than <b>1</b> and <b>y</b> is a negative infinity, this function returns a positive infinity. \n
* If the absolute value of <b>x</b> is greater than <b>1</b> and <b>y</b> is a negative infinity, this function returns <b>+0</b>. \n
* If the absolute value of <b>x</b> is less than <b>1</b> and <b>y</b> is a positive infinity, this function returns <b>+0</b>. \n
* If the absolute value of <b>x</b> is greater than <b>1</b> and <b>y</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an odd integer less than <b>0</b>, this function returns <b>-0</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is less than <b>0</b> and not an odd integer, this function returns <b>+0</b>. \n
* If <b>x</b> is a negative infinity and <b>y</b> is an odd integer greater than <b>0</b>, this function returns a negative infinity. \n
* If <b>x</b> is a negative infinity and <b>y</b> is greater than <b>0</b> and not an odd integer, this function returns a positive infinity. \n
* If <b>x</b> is a positive infinity and <b>y</b> is less than <b>0</b>, this function returns <b>+0</b>. \n
* If <b>x</b> is a positive infinity and <b>y</b> is greater than <b>0</b>, this function returns a positive infinity. \n
* If <b>x</b> is a negative value and <b>y</b> is a finite non-integer, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is a negative odd integer, this function returns {@link HUGE_VALL}, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b> and <b>y</b> is a negative non-odd integer, this function returns <b>+{@link HUGE_VALL}</b>, reports a pole error, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_DIVBYZERO} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL}, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0.0</b>, sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double powl(long double x, long double y);

/**
* @brief Calculates the xth power of 10.
*
* @param x Indicates the long double value x, which is used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If x is 0, this function returns 1.0. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL}, sets errno to {@link ERANGE}, and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns 0.0, sets errno to {@link ERANGE}, and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double powl10(long double x);

/**
* @brief Calculates the remainder of the double value <b>x</b> divided by the double value <b>y</b>.
*
* @attention The return value is <b>x-n*y</b>, where <b>n</b> is the value of <b>x</b>/<b>y</b>, rounded toward the nearest integer (with halfway values rounded toward the even number). For example, if the decimal part of <b>x</b>/<b>y</b> is <b>0.5</b>, the value is rounded to an even number and <b>0</b> is returned.
* @param x Indicates the double value <b>x</b>, which is used as the numerator.
* @param y Indicates the double value <b>y</b>, which is used as the denominator.
* @return Returns the remainder if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is an infinity and <b>y</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* If <b>y</b> is <b>0</b> and <b>x</b> is not a NaN value, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVAID} exception. \n
* @since 1.0
* @version 1.0
*/
double      remainder(double x, double y);

/**
* @brief Calculates the remainder of the float value <b>x</b> divided by the float value <b>y</b>.
*
* @attention The return value is <b>x-n*y</b>, where <b>n</b> is the value of <b>x</b>/<b>y</b>, rounded toward the nearest integer (with halfway values rounded toward the even number). For example, if the decimal part of <b>x</b>/<b>y</b> is <b>0.5</b>, the value is rounded to an even number and <b>0</b> is returned.
* @param x Indicates the float value <b>x</b>, which is used as the numerator.
* @param y Indicates the float value <b>y</b>, which is used as the denominator.
* @return Returns the remainder if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is an infinity and <b>y</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* If <b>y</b> is <b>0</b> and <b>x</b> is not a NaN value, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVAID} exception. \n
* @since 1.0
* @version 1.0
*/
float       remainderf(float x, float y);

/**
* @brief Calculates the remainder of the long double value <b>x</b> divided by the long double value <b>y</b>.
*
* @attention The return value is <b>x-n*y</b>, where <b>n</b> is the value of <b>x</b>/<b>y</b>, rounded toward the nearest integer (with halfway values rounded toward the even number). For example, if the decimal part of <b>x</b>/<b>y</b> is <b>0.5</b>, the value is rounded to an even number and <b>0</b> is returned.
* @param x Indicates the long double value <b>x</b>, which is used as the numerator.
* @param y Indicates the long double value <b>y</b>, which is used as the denominator.
* @return Returns the remainder if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is an infinity and <b>y</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* If <b>y</b> is <b>0</b> and <b>x</b> is not a NaN value, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVAID} exception. \n
* @since 1.0
* @version 1.0
*/
long double remainderl(long double x, long double y);

/**
* @brief Calculates the quotient and remainder of the double value <b>x</b> divided by the double value <b>y</b>.
*
* The return value is <b>x-n*y</b>, where <b>n</b> is the value of <b>x</b>/<b>y</b>, rounded toward the nearest integer (with halfway values rounded toward the even number). For example, if the decimal part of <b>x</b>/<b>y</b> is <b>0.5</b>, the value is rounded to an even number and <b>0</b> is returned. <b>n</b> is converted into a signed binary number, and the sign and the least significant three bits of the binary number are combined and stored to the pointer specified by <b>quo</b>. For example, if <b>x</b> is <b>29.0</b> and <b>y</b> is <b>3.0</b>, the return value is <b>-1.0</b> and <b>n</b> is <b>10</b>. <b>n</b> is converted into a signed binary number <b>00001010</b>. Therefore, <b>2</b> is stored to the pointer specified by <b>quo</b>.  \n
* @param x Indicates the double value <b>x</b>, which is used as the numerator.
* @param y Indicates the double value <b>y</b>, which is used as the denominator.
* @param quo Indicates the pointer that stores the quotient.
* @return Returns the remainder if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is an infinity and <b>y</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* If <b>y</b> is <b>0</b> and <b>x</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* @since 1.0
* @version 1.0
*/
double      remquo(double x, double y, int *quo);

/**
* @brief Calculates the quotient and remainder of the float value <b>x</b> divided by the float value <b>y</b>.
*
* The return value is <b>x-n*y</b>, where <b>n</b> is the value of <b>x</b>/<b>y</b>, rounded toward the nearest integer (with halfway values rounded toward the even number). For example, if the decimal part of <b>x</b>/<b>y</b> is <b>0.5</b>, the value is rounded to an even number and <b>0</b> is returned. <b>n</b> is converted into a signed binary number, and the sign and the least significant three bits of the binary number are combined and stored to the pointer specified by <b>quo</b>. For example, if <b>x</b> is <b>29.0</b> and <b>y</b> is <b>3.0</b>, the return value is <b>-1.0</b> and <b>n</b> is <b>10</b>. <b>n</b> is converted into a signed binary number <b>00001010</b>. Therefore, <b>2</b> is stored to the pointer specified by <b>quo</b>.  \n
* @param x Indicates the float value <b>x</b>, which is used as the numerator.
* @param y Indicates the float value <b>y</b>, which is used as the denominator.
* @param quo Indicates the pointer that stores the quotient.
* @return Returns the remainder if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is an infinity and <b>y</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* If <b>y</b> is <b>0</b> and <b>x</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* @since 1.0
* @version 1.0
*/
float       remquof(float x, float y, int *quo);

/**
* @brief Calculates the quotient and remainder of the long double value <b>x</b> divided by the long double value <b>y</b>.
*
* The return value is <b>x-n*y</b>, where <b>n</b> is the value of <b>x</b>/<b>y</b>, rounded toward the nearest integer (with halfway values rounded toward the even number). For example, if the decimal part of <b>x</b>/<b>y</b> is <b>0.5</b>, the value is rounded to an even number and <b>0</b> is returned. <b>n</b> is converted into a signed binary number, and the sign and the least significant three bits of the binary number are combined and stored to the pointer specified by <b>quo</b>. For example, if <b>x</b> is <b>29.0</b> and <b>y</b> is <b>3.0</b>, the return value is <b>-1.0</b> and <b>n</b> is <b>10</b>. <b>n</b> is converted into a signed binary number <b>00001010</b>. Therefore, <b>2</b> is stored to the pointer specified by <b>quo</b>.  \n
* @param x Indicates the long double value <b>x</b>, which is used as the numerator.
* @param y Indicates the long double value <b>y</b>, which is used as the denominator.
* @param quo Indicates the pointer that stores the quotient.
* @return Returns the remainder if the operation is successful. \n
* If <b>x</b> or <b>y</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is an infinity and <b>y</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* If <b>y</b> is <b>0</b> and <b>x</b> is not a NaN value, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVAID} exception. \n
* @since 1.0
* @version 1.0
*/
long double remquol(long double x, long double y, int *quo);

/**
* @brief Rounds the double value <b>x</b> to the nearest integer.
*
* @attention POSIX.1-2001 records overflow range errors. If there are two nearest integers, the even integer is returned. For example, if <b>x</b> is <b>0.5</b>, <b>0</b> is returned.
* @param x Indicates the double value.
* @return Returns the nearest integer if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
double      rint(double x);

/**
* @brief Rounds the float value <b>x</b> to the nearest integer.
*
* @attention POSIX.1-2001 records overflow range errors. If there are two nearest integers, the even integer is returned. For example, if <b>x</b> is <b>0.5</b>, <b>0</b> is returned.
* @param x Indicates the float value.
* @return Returns the nearest integer if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
float       rintf(float x);

/**
* @brief Rounds the long double value <b>x</b> to the nearest integer.
*
* @attention POSIX.1-2001 records overflow range errors. If there are two nearest integers, the even integer is returned. For example, if <b>x</b> is <b>0.5</b>, <b>0</b> is returned.
* @param x Indicates the long double value.
* @return Returns the nearest integer if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
long double rintl(long double x);

/**
* @brief Rounds the double value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>. For example, if <b>x</b> is <b>0.5</b>, the rounded value is <b>1</b>; if <b>x</b> is <b>-0.5</b>, the rounded value is <b>-1</b>.
* @param x Indicates the double value <b>x</b>.
* @return Returns the nearest integer if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
double      round(double x);

/**
* @brief Rounds the float value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>. For example, if <b>x</b> is <b>0.5</b>, the rounded value is <b>1</b>; if <b>x</b> is <b>-0.5</b>, the rounded value is <b>-1</b>.
* @param x Indicates the float value <b>x</b>.
* @return Returns the nearest integer if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
float       roundf(float x);

/**
* @brief Rounds the long double value <b>x</b> to the nearest integer, rounding away from <b>0</b>.
*
* @attention Halfway values are rounded away from <b>0</b>. For example, if <b>x</b> is <b>0.5</b>, the rounded value is <b>1</b>; if <b>x</b> is <b>-0.5</b>, the rounded value is <b>-1</b>.
* @param x Indicates the long double value <b>x</b>.
* @return Returns the nearest integer if the operation is successful; returns the passed <b>x</b> if <b>x</b> is an integer, <b>+0</b>, <b>-0</b>, a NaN value, or an infinity.
* @since 1.0
* @version 1.0
*/
long double roundl(long double x);

/**
* @brief Multiplies the double value <b>x</b> by {@link FLT_RADIX} raised to the power of the long int value <b>exp</b>, that is, <b>x*({@link FLT_RADIX})^exp</b>.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the double value <b>x</b>.
* @param exp Indicates the double value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VAL} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      scalbln(double x, long int exp);

/**
* @brief Multiplies the float value <b>x</b> by {@link FLT_RADIX} raised to the power of the long int value <b>exp</b>, that is, <b>x*({@link FLT_RADIX})^exp</b>.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the float value <b>x</b>.
* @param exp Indicates the float value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       scalblnf(float x, long int exp);

/**
* @brief Multiplies the long double value <b>x</b> by {@link FLT_RADIX} raised to the power of the long int value <b>exp</b>, that is, <b>x*({@link FLT_RADIX})^exp</b>.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the long double value <b>x</b>.
* @param exp Indicates the long double value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double scalblnl(long double x, long int exp);


/**
* @brief Multiplies the double value <b>x</b> by {@link FLT_RADIX} raised to the power of the int value <b>exp</b>, that is, <b>x*({@link FLT_RADIX})^exp</b>.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the double value <b>x</b>.
* @param exp Indicates the double value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VAL} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      scalbn(double x, int exp);

/**
* @brief Multiplies the float value <b>x</b> by {@link FLT_RADIX} raised to the power of the int value <b>exp</b>, that is, <b>x*({@link FLT_RADIX})^exp</b>.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the float value <b>x</b>.
* @param exp Indicates the float value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       scalbnf(float x, int exp);


/**
* @brief Multiplies the long double value <b>x</b> by {@link FLT_RADIX} raised to the power of the int value <b>exp</b>, that is, <b>x*({@link FLT_RADIX})^exp</b>.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the long double value <b>x</b>.
* @param exp Indicates the long double value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
long double scalbnl(long double x, int exp);

/**
* @brief Calculates the sine of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the sine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value, reports a domain error,
* sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
double      sin(double x);

/**
* @brief Calculates the sine of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the sine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value, reports a domain error,
* sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
float       sinf(float x);

/**
* @brief Calculates the sine of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the sine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, this function returns a NaN value, reports a domain error,
* sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
long double sinl(long double x);

/**
* @brief Calculates the hyperbolic sine of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the hyperbolic sine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF} with the sign of <b>x</b>,
* sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
double      sinh(double x);

/**
* @brief Calculates the hyperbolic sine of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the hyperbolic sine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF} with the sign of <b>x</b>,
* sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
float       sinhf(float x);

/**
* @brief Calculates the hyperbolic sine of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the hyperbolic sine of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns <b>+0</b> or <b>-0</b>, respectively. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive or negative infinity. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL} with the sign of <b>x</b>,
* sets <b>errno</b> to {@link ERANGE}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
long double sinhl(long double x);

/**
* @brief Calculates the square root of double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the square root of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
double      sqrt(double x);

/**
* @brief Calculates the square root of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the square root of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
float       sqrtf(float x);

/**
* @brief Calculates the square root of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the square root of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
long double sqrtl(long double x);

/**
* @brief Calculates the tangent of the double value <b>x</b>.
*
* @param x Indicates the double value, which represents an angle in radians.
* @return Returns the tangent of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
double      tan(double x);

/**
* @brief Calculates the tangent of the float value <b>x</b>.
*
* @param x Indicates the float value, which represents an angle in radians.
* @return Returns the tangent of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
float       tanf(float x);

/**
* @brief Calculates the tangent of the long double value <b>x</b>.
*
* @param x Indicates the long double value, which represents an angle in radians.
* @return Returns the tangent of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
long double tanl(long double x);

/**
* @brief Calculates the hyperbolic tangent of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the hyperbolic tangent of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
double      tanh(double x);

/**
* @brief Calculates the hyperbolic tangent of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the hyperbolic tangent of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
float       tanhf(float x);

/**
* @brief Calculates the hyperbolic tangent of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the hyperbolic tangent of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0</b>, this function returns <b>0</b>. \n
* If <b>x</b> is less than <b>-0</b>, this function returns a NaN value, reports a domain error, sets <b>errno</b> to {@link EDOM}, and throws the {@link FE_INVALID} exception. \n
* @since 1.0
* @version 1.0
*/
long double tanhl(long double x);

/**
* @brief Calculates the gamma function of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the gamma function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative integer or negative infinity, this function returns a NaN value, reports a domain error, and sets <b>errno</b> to {@link EDOM}. \n
* If an underflow range error occurs, this function returns <b>+0</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns {@link HUGE_VAL} with the sign of <b>x</b>, reports a pole error, and sets <b>errno</b> to {@link ERANGE}. \n
* If an overflow range error occurs, this function returns {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}. \n
* @since 1.0
* @version 1.0
*/
double      tgamma(double x);

/**
* @brief Calculates the gamma function of the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the gamma function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative integer or negative infinity, this function returns a NaN value, reports a domain error, and sets <b>errno</b> to {@link EDOM}. \n
* If an underflow range error occurs, this function returns <b>+0</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns {@link HUGE_VALF} with the sign of <b>x</b>, reports a pole error, and sets <b>errno</b> to {@link ERANGE}. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALF} and sets <b>errno</b> to {@link ERANGE}. \n
* @since 1.0
* @version 1.0
*/
float       tgammaf(float x);

/**
* @brief Calculates the gamma function of the long double value <b>x</b>.
*
* @param x Indicates the long double value.
* @return Returns the gamma function of <b>x</b> if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive infinity, this function returns a positive infinity. \n
* If <b>x</b> is a negative integer or negative infinity, this function returns a NaN value, reports a domain error, and sets <b>errno</b> to {@link EDOM}. \n
* If an underflow range error occurs, this function returns <b>+0</b>. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, this function returns {@link HUGE_VALL} with the sign of <b>x</b>, reports a pole error, and sets <b>errno</b> to {@link ERANGE}. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL} and sets <b>errno</b> to {@link ERANGE}. \n
* @since 1.0
* @version 1.0
*/
long double tgammal(long double x);

/**
* @brief Obtains the nearest integer whose absolute value is less than or equal to the absolute value of the double value <b>x</b>.
*
* @param x Indicates the double value <b>x</b>.
* @return Returns the nearest integer whose absolute value is less than or equal to the absolute value of <b>x</b>.
* @since 1.0
* @version 1.0
*/
double      trunc(double x);

/**
* @brief Obtains the nearest integer whose absolute value is less than or equal to the absolute value of the float value <b>x</b>.
*
* @param x Indicates the float value <b>x</b>.
* @return Returns the nearest integer whose absolute value is less than or equal to the absolute value of <b>x</b>.
* @since 1.0
* @version 1.0
*/
float       truncf(float x);

/**
* @brief Obtains the nearest integer whose absolute value is less than or equal to the absolute value of the long double value <b>x</b>.
*
* @param x Indicates the long double value <b>x</b>.
* @return Returns the nearest integer whose absolute value is less than or equal to the absolute value of <b>x</b>.
* @since 1.0
* @version 1.0
*/
long double truncl(long double x);

/**
* @brief Maximum value of type float
*/
#define MAXFLOAT        3.40282346638528859812e+38F

/**
* @brief Base e of natural logarithms
*/
#define M_E             2.7182818284590452354   /* e */

/**
* @brief Logarithm to base 2 of <b>M_E</b>
*/
#define M_LOG2E         1.4426950408889634074   /* log_2 e */

/**
* @brief Logarithm to base 10 of <b>M_E</b>
*/
#define M_LOG10E        0.43429448190325182765  /* log_10 e */

/**
* @brief Natural logarithm of <b>2</b>
*/
#define M_LN2           0.69314718055994530942  /* log_e 2 */

/**
* @brief Natural logarithm of <b>10</b>
*/
#define M_LN10          2.30258509299404568402  /* log_e 10 */

/**
* @brief Ratio of a circle's circumference to its diameter
*/
#define M_PI            3.14159265358979323846  /* pi */

/**
* @brief Pi divided by <b>2</b>
*/
#define M_PI_2          1.57079632679489661923  /* pi/2 */

/**
* @brief Pi divided by <b>4</b>
*/
#define M_PI_4          0.78539816339744830962  /* pi/4 */

/**
* @brief Reciprocal of pi
*/
#define M_1_PI          0.31830988618379067154  /* 1/pi */

/**
* @brief Two times the reciprocal of pi
*/
#define M_2_PI          0.63661977236758134308  /* 2/pi */

/**
* @brief Two times the reciprocal of the square root of pi
*/
#define M_2_SQRTPI      1.12837916709551257390  /* 2/sqrt(pi) */

/**
* @brief Square root of <b>2</b>
*/
#define M_SQRT2         1.41421356237309504880  /* sqrt(2) */

/**
* @brief Reciprocal of the square root of <b>2</b>
*/
#define M_SQRT1_2       0.70710678118654752440  /* 1/sqrt(2) */

extern int signgam;

/**
* @brief Calculates the Bessel function of the first kind of order <b>0</b> for the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the Bessel function of the first kind of order <b>0</b> for <b>x</b> if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>0</b> and sets <b>errno</b> to {@link ERANGE} if <b>x</b> is too large or an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
double      j0(double x);

/**
* @brief Calculates the Bessel function of the first kind of order <b>1</b> for the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the Bessel function of the first kind of order <b>1</b> for <b>x</b> if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>0</b> and sets <b>errno</b> to {@link ERANGE} if <b>x</b> is too large or an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
double      j1(double x);

/**
* @brief Calculates the Bessel function of the first kind of order <b>n</b> for the double value <b>x</b>.
*
* @param n Indicates the order of the Bessel function of the first kind.
* @param x Indicates the double value.
* @return Returns the Bessel function of the first kind of order <b>n</b> for <b>x</b> if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>0</b> and sets <b>errno</b> to {@link ERANGE} if <b>x</b> is too large or an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
double      jn(int n, double x);

/**
* @brief Calculates the Bessel function of the second kind of order <b>0</b> for the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the Bessel function of the second kind of order <b>0</b> for <b>x</b> if the operation is successful .\n If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0.0</b>, this function returns <b>-{@link HUGE_VAL}</b> and reports a pole error. \n
* If <b>x</b> is a negative value, this function returns <b>-{@link HUGE_VAL}</b> and reports a range error. \n
* If an overflow range error occurs, this function returns <b>0.0</b>. \n
* @since 1.0
* @version 1.0
*/
double      y0(double x);

/**
* @brief Calculates the Bessel function of the second kind of order <b>1</b> for the double value <b>x</b>.
*
* @param x Indicates the double value.
* @return Returns the Bessel function of the second kind of order <b>1</b> for <b>x</b> if the operation is successful. \n If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0.0</b>, this function returns <b>-{@link HUGE_VAL}</b> and reports a pole error. \n
* If <b>x</b> is a negative value, this function returns <b>-{@link HUGE_VAL}</b> and reports a range error. \n
* If an overflow range error occurs, this function returns <b>0.0</b>. \n
* @since 1.0
* @version 1.0
*/
double      y1(double x);

/**
* @brief Calculates the Bessel function of the second kind of order <b>n</b> for the double value <b>x</b>.
*
* @param n Indicates the order of the Bessel function of the first kind.
* @param x Indicates the double value.
* @return Returns the Bessel function of the second kind of order <b>n</b> for <b>x</b> if the operation is successful. \n If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>0.0</b>, this function returns <b>-{@link HUGE_VAL}</b> and reports a pole error. \n
* If <b>x</b> is a negative value, this function returns <b>-{@link HUGE_VAL}</b> and reports a range error. \n
* If an overflow range error occurs, this function returns <b>0.0</b>. \n
* @since 1.0
* @version 1.0
*/
double      yn(int n, double x);

/**
* @brief Checks whether the double value <b>x</b> is an infinity or a NaN value.
*
* @param x Indicates the double value to check.
* @return Returns <b>0</b> if <b>x</b> is an infinity or a NaN value; returns <b>1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int         finite(double x);

/**
* @brief Checks whether the float value <b>x</b> is an infinity or a NaN value.
*
* @param x Indicates the float value to check.
* @return Returns <b>0</b> if <b>x</b> is an infinity or a NaN value; returns <b>1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int         finitef(float x);

/**
* @brief Multiplies the double value <b>x</b> by {@link FLT_RADIX} raised to the power of the double value <b>exp</b>, that is, x*{@link FLT_RADIX}^exp.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the double value <b>x</b>.
* @param exp Indicates the double value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n 
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VAL} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
double      scalb(double x, double exp);

/**
* @brief Multiplies the float value <b>x</b> by {@link FLT_RADIX} raised to the power of the float value <b>exp</b>, that is, x*{@link FLT_RADIX}^exp.
*
* @attention {@link FLT_RADIX} is defined in <b>float.h</b> and is generally <b>2</b>.
* @param x Indicates the float value <b>x</b>.
* @param exp Indicates the float value used as the exponent.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> or <b>exp</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is a positive or negative infinity, and <b>exp</b> is not a negative infinity, this function returns a positive or negative infinity. \n
* If <b>x</b> is <b>+0</b> or <b>-0</b>, and <b>exp</b> is not a positive infinity, this function returns <b>+0</b> or <b>-0</b>. \n
* If <b>x</b> is <b>0</b> and <b>exp</b> is a positive infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If <b>x</b> is an infinity and <b>exp</b> is a negative infinity, this function returns a NaN value, reports a domain error, and throws the {@link FE_INVALID} exception. \n
* If an overflow range error occurs, this function returns {@link HUGE_VALL} with the sign of <b>x</b> and throws the {@link FE_OVERFLOW} exception. \n
* If an underflow range error occurs, this function returns <b>0</b> and throws the {@link FE_UNDERFLOW} exception. \n
* @since 1.0
* @version 1.0
*/
float       scalbf(float x, float exp);

/**
* @brief Obtains the significand of the double value <b>x</b> scaled to the range [1,2).
*
* The calculation formula is x*2^-{@link ilogb}(x). \n
* This function is mainly used in certain standardized tests for IEEE 754 conformance. \n
* @param x Indicates the double value <b>x</b>.
* @return Returns the significand of <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
double      significand(double x);


/**
* @brief Obtains the significand of the float value <b>x</b> scaled to the range [1,2).
*
* The calculation formula is x*2^-{@link ilogb}(x). \n
* This function is mainly used in certain standardized tests for IEEE 754 conformance. \n
* @param x Indicates the float value <b>x</b>.
* @return Returns the significand of <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
float       significandf(float x);


/**
* @brief Calculates the Bessel function of the first kind of order <b>0</b> for the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the calculation result if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>0</b> and sets <b>errno</b> to {@link ERANGE} if <b>x</b> is too large or an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
float       j0f(float x);

/**
* @brief Calculates the Bessel function of the first kind of order <b>1</b> for the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the calculation result if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>0</b> and sets <b>errno</b> to {@link ERANGE} if <b>x</b> is too large or an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
float       j1f(float x);

/**
* @brief Calculates the Bessel function of the first kind of order <b>n</b> for the float value <b>x</b>.
*
* @param n Indicates the order of the Bessel function of the first kind.
* @param x Indicates the float value.
* @return Returns the calculation result if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>0</b> and sets <b>errno</b> to {@link ERANGE} if <b>x</b> is too large or an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
float       jnf(int n, float x);

/**
* @brief Calculates the Bessel function of the second kind of order <b>0</b> for the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the calculation result if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>-{@link HUGE_VALF}</b> and reports a pole error if <b>x</b> is <b>0.0</b>;
* returns <b>-{@link HUGE_VALF}</b> and reports a domain error if <b>x</b> is a negative value;
* returns <b>0.0</b> if an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
float       y0f(float x);

/**
* @brief Calculates the Bessel function of the second kind of order <b>1</b> for the float value <b>x</b>.
*
* @param x Indicates the float value.
* @return Returns the calculation result if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>-{@link HUGE_VALF}</b> and reports a pole error if <b>x</b> is <b>0.0</b>;
* returns <b>-{@link HUGE_VALF}</b> and reports a domain error if <b>x</b> is a negative value;
* returns <b>0.0</b> if an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
float       y1f(float x);

/**
* @brief Calculates the Bessel function of the second kind of order <b>n</b> for the float value <b>x</b>.
*
* @param n Indicates the order of the Bessel function of the second kind.
* @param x Indicates the float value.
* @return Returns the calculation result if the operation is successful; returns a NaN value if <b>x</b> is a NaN value;
* returns <b>-{@link HUGE_VALF}</b> and reports a pole error if <b>x</b> is <b>0.0</b>;
* returns <b>-{@link HUGE_VALF}</b> and reports a domain error if <b>x</b> is a negative value;
* returns <b>0.0</b> if an overflow range error occurs.
* @since 1.0
* @version 1.0
*/
float       ynf(int n, float x);


/**
* @brief Calculates the natural logarithm of the absolute value of the gamma function of the double value <b>x</b>.
*
* @param x Indicates the double value.
* @param signp return value of sign.
* @return Returns the calculation result if the operation is successful. \n
* If <b>x</b> is a NaN value, this function returns a NaN value. \n
* If <b>x</b> is <b>1</b> or <b>2</b>, this function returns <b>+0</b>. \n
* If <b>x</b> is a positive or negative infinity, this function returns a positive infinity. \n
* If <b>x</b> is <b>0</b> or a negative value, this function returns one of +{@link HUGE_VAL}, reports a pole error, and sets <b>errno</b> to {@link ERANGE}). \n
* If an overflow range error occurs, this function returns one of {@link HUGE_VAL} * and sets <b>errno</b> to {@link ERANGE}. \n
* @since 1.0
* @version 1.0
*/
long double lgammal_r(long double x, int *signp);

/**
* @brief Calculates the sine and cosine of the double value <b>x</b>.
*
* @attention A domain error occurs if <b>x</b> is an infinity. In this case, the {@link FE_INVALID} exception is thrown, and <b>sin</b> and <b>cos</b> are both NaN values.
* If <b>x</b> is a NaN value, <b>sin</b> and <b>cos</b> are both NaN values.
* @param x Indicates the double value whose sine and cosine are to be calculated.
* @param sin Indicates the pointer to the obtained sine.
* @param cos Indicates the pointer to the obtained cosine.
* @since 1.0
* @version 1.0
*/
void sincos(double x, double *sin, double *cos);

/**
* @brief Calculates the sine and cosine of the float value <b>x</b>.
*
* @attention A domain error occurs if <b>x</b> is an infinity. In this case, the {@link FE_INVALID} exception is thrown, and <b>sin</b> and <b>cos</b> are both NaN values.
* If <b>x</b> is a NaN value, <b>sin</b> and <b>cos</b> are both NaN values.
* @param x Indicates the double value whose sine and cosine are to be calculated.
* @param sin Indicates the pointer to the obtained sine.
* @param cos Indicates the pointer to the obtained cosine.
* @since 1.0
* @version 1.0
*/
void        sincosf(float x, float *sin, float *cos);

/**
* @brief Calculates the sine and cosine of the long double value <b>x</b>.
*
* @attention A domain error occurs if <b>x</b> is an infinity. In this case, the {@link FE_INVALID} exception is thrown, and <b>sin</b> and <b>cos</b> are both NaN values.
* If <b>x</b> is a NaN value, <b>sin</b> and <b>cos</b> are both NaN values.
* @param x Indicates the double value whose sine and cosine are to be calculated.
* @param sin Indicates the pointer to the obtained sine.
* @param cos Indicates the pointer to the obtained cosine.
* @since 1.0
* @version 1.0
*/
void        sincosl(long double x, long double *sin, long double *cos);

/**
* @brief Calculates the base-10 exponential function of the double value <b>x</b>.
*
* @param x Indicates the double value <b>x</b> used as the exponent.
* @return Returns the base-10 exponential function of <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
double      exp10(double x);

/**
* @brief Calculates the base-10 exponential function of the float value <b>x</b>.
*
* @param x Indicates the float value <b>x</b> used as the exponent.
* @return Returns the base-10 exponential function of <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
float       exp10f(float x);

/**
* @brief Calculates the base-10 exponential function of the long double value <b>x</b>.
*
* @param x Indicates the long double value <b>x</b> used as the exponent.
* @return Returns the base-10 exponential function of <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
long double exp10l(long double x);

/**
* @brief Calculates the value of 10 raised to the power <b>x</b>, which is a double value.
*
* @param x Indicates the double value <b>x</b>.
* @return Returns the value of 10 raised to the power <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
double      pow10(double x);

/**
* @brief Calculates the value of 10 raised to the power <b>x</b>, which is a float value.
*
* @param x Indicates the float value <b>x</b>.
* @return Returns the value of 10 raised to the power <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
float       pow10f(float x);

/**
* @brief Calculates the value of 10 raised to the power <b>x</b>, which is a long double value.
*
* @param x Indicates the long double value <b>x</b>.
* @return Returns the value of 10 raised to the power <b>x</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
long double pow10l(long double x);

#ifdef __cplusplus
}
#endif

#endif
/** @} */
