/**
* @addtogroup IPC
* @{
*
* @brief Provides functions and structures related to inter-process communication (IPC), including signals, semaphores, and message queues.
*
* @since 1.0
* @version 1.0
*/

/**
* @file mqueue.h
*
* @brief Provides message queue operation functions and related structures.
*
* For example, you can use the functions to create, open, close, delete, read, and write a message queue, and to obtain and set its attributes.
*
* @since 1.0
* @version 1.0
*/
#ifndef _MQUEUE_H
#define _MQUEUE_H
#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Describes message queue attributes.
*/
struct mq_attr {
/** Message queue flags: 0 or {@link O_NONBLOCK} */
long mq_flags;
/** Maximum number of messages on queue */
long mq_maxmsg;
/** Maximum length of a message */
long mq_msgsize;
/** Number of messages currently queued */
long mq_curmsgs;
/** A reserved field */
long __unused[4];
};

/**
* @brief Closes a message queue that is no longer used.
*
* @param mqdes Indicates the descriptor of the message queue to be closed.
* @return Returns <b>0</b> if the message queue is closed; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the message queue fails to be closed.
* errno | Description
* -------|------------------------------------------
* EBADF | <b>mqdes</b> is invalid.
* EAGAIN | Failed to delete the message queue in the function.
* EFAULT | Failed to release the space allocated with the message queue.
* EINVAL | <b>mqdes</b> is invalid when the message queue is deleted in the function.
* @since 1.0
* @version 1.0
*/
int mq_close(mqd_t mqdes);

/**
* @brief Obtains the attributes of the message queue specified by the descriptor. The values of <b>mq_maxmsg</b>, <b>mq_msgsize</b>, and <b>mq_curmsgs</b> are fixed.
*
* @param mqdes Indicates the message queue descriptor.
* @param attr Indicates the pointer to the obtained message queue attributes.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|------------------------------------------
* EBADF | <b>mqdes</b> is invalid.
* EINVAL | <b>attr</b> is <b>NULL</b>.
* @since 1.0
* @version 1.0
*/
int mq_getattr(mqd_t mqdes, struct mq_attr *attr);

/**
* @brief Creates a message queue or opens an existing message queue.
*
* @attention The message consists of 64 bytes by default, and the maximum length of the message is 64 KB. A maximum of 1024 message queues can be created. The mq_curmsgs field in {@link mq_attr} is not supported.
* @param name Indicates the name of the message queue to be created or opened. The maximum value is 259 bytes.
* @param oflag Indicates the open flag, including {@link O_RDONLY}, {@link O_WRONLY}, {@link O_RDWR}, {@link O_CREAT}, {@link O_EXCL}, and {@link O_NONBLOCK}.
* @param mode Used for permission control (not supported currently). Its type is <b>mode_t</b>. This is parameter is required only during the creation of a message queue.
* @param attr Indicates the pointer to the queue attributes to be set. Its type is <b>struct mq_attr</b>. This parameter is optional and is used only during the creation of a message queue.
* @return Returns a message queue descriptor if the creation or opening is successful; returns <b>(mqd_t)-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------|-----------------------------------------------------------------------
* EEXIST | Both {@link O_CREAT} and {@link O_EXCL} are specified in <b>oflag</b>, and a queue with this name already exists.
* EINVAL | {@link O_CREAT} is specified in <b>oflag</b>, and <b>attr</b> is invalid.<br>To be specific, the attribute value of the <b>mq_maxmsg</b> field is less than 0 or greater than {@link USHRT_MAX}, or the attribute value of <b>mq_msgsize</b> is less than 0 or greater than {@link USHRT_MAX}-4.<br><b>name</b> is <b>NULL</b>, or the length of the string specified by <b>name</b> is <b>0</b>.
* ENAMETOOLONG | <b>name</b> is too long.
* ENOENT | {@link O_CREAT} is not specified in <b>oflag</b>, and no queue with this name exists.
* ENOSPC | Insufficient memory.
* ENFILE | The number of opened message queues exceeds the maximum.
* @since 1.0
* @version 1.0
*/
mqd_t mq_open(const char *name, int oflag, ...);

/**
* @brief Receives a message from a specified message queue.
*
* The message queue must have been open by {@link mq_open()}. \n
* @param mqdes Indicates the message queue descriptor.
* @param buffer Indicates the pointer to the buffer for receiving the message.
* @param size Indicates the size of the buffer for receiving the message.
* @param prioptr Indicates the priority of the received message if the value is not <b>NULL</b>. This parameter is not used because priority-based message processing is not supported.
* @return Returns the number of bytes in the received message if the message is received; returns <b>(mqd_t)-1</b> and sets <b>errno</b> to a value in the following table if the message fails to be received.
* errno | Description
* ---------|--------------------------------------------------
* EINTR | Message receiving is interrupted.
* EAGAIN | The message queue is empty, and {@link O_NONBLOCK} is set for the message queue associated with <b>mqdes</b>.
* EBADF | <b>mqdes</b> is invalid or you do not have the read permission.
* EMSGSIZE | The size of the buffer for receiving the message is less than the message size attribute of the message queue.
* EINVAL | <b>mqdes</b> is <b>NULL</b> or the value of <b>size</b> is <b>0</b>.
* @since 1.0
* @version 1.0
*/
ssize_t mq_receive(mqd_t mqdes, char *buffer, size_t size, unsigned *prioptr);

/**
* @brief Sends a message to a specified message queue.
*
* The message queue must have been opened using {@link mq_open()}. \n
* @param mqdes Indicates the message queue descriptor.
* @param buffer Indicates the pointer to the buffer for receiving the message.
* @param size Indicates the size of the buffer for receiving the message.
* @param prio Indicates the priority of the message to be sent. This parameter must be <b>0</b> because priority-based message processing is not supported.
* @return Returns <b>0</b> if the message is sent; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the message fails to be sent.
* errno | Description
* ---------|--------------------------------------------------
* EINTR | Message sending is interrupted.
* EAGAIN | The message queue is full, and {@link O_NONBLOCK} is set for the message queue associated with <b>mqdes</b>.
* EBADF | <b>mqdes</b> is invalid or you do not have the write permission.
* EINVAL | <b>mqdes</b> is <b>NULL</b> or the value of <b>size</b> is <b>0</b>.
* EMSGSIZE | The size of the message to be sent is greater than the message size attribute of the message queue.
* EOPNOTSUPP | The operation is not supported.
* ENOSPC | Insufficient space.
* @since 1.0
* @version 1.0
*/
int mq_send(mqd_t mqdes, const char *buffer, size_t size, unsigned prio);

/**
* @brief Sets the attributes of the message queue specified by the descriptor. The <b>mq_maxmsg</b>, <b>mq_msgsize</b>, and <b>mq_curmsgs</b> attributes cannot be modified. <b>mq_flags</b> supports the <b>O_NONBLOCK</b> attribute only.
*
* The message queue must have been opened using {@link mq_open()}. \n
* @param mqdes Indicates the message queue descriptor.
* @param newattr Indicates the pointer to the message queue attribute to be set.
* @param oldattr Indicates the pointer to the original attribute.
* @return Returns <b>0</b> if the setting is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the setting fails.
* errno | Description
* -------|---------------------------------------------------------------
* EBADF | <b>mqdes</b> is invalid.
* EINVAL | <b>newattr</b> is <b>NULL</b>.
* @since 1.0
* @version 1.0
*/
int mq_setattr(mqd_t mqdes, const struct mq_attr *__restrict newattr, struct mq_attr *__restrict oldattr);

/**
* @brief Receives a message from the message queue, with a timeout period specified.
*
* The message queue must have been open by {@link mq_open()}. \n
* @param mqdes Indicates the message queue descriptor.
* @param buffer Indicates the pointer to the buffer for receiving the message.
* @param size Indicates the size of the buffer for receiving the message.
* @param prioptr Indicates the priority of the received message if the value is not <b>NULL</b>. This parameter is not used because priority-based message processing is not supported.
* @param timeout Indicates the timeout duration for returning a request.
* @return Returns the number of bytes in the message if the message is received; returns <b>(mqd_t)-1</b> and sets <b>errno</b> to a value in the following table if the message fails to be received.
* errno | Description
* ----------|--------------------------------------------------------------
* EINTR | Message receiving is interrupted.
* EAGAIN | The message queue is empty, and {@link O_NONBLOCK} is set for the message queue associated with <b>mqdes</b>.
* EBADF | <b>mqdes</b> is invalid or you do not have the read permission.
* EINVAL | <b>mqdes</b> is <b>NULL</b> or the value of <b>size</b> is <b>0</b>.<br><b>timeout</b> is invalid. Specifically, the value of <b>tv_sec</b> is less than 0, and the value of <b>tv_nsec</b> is less than 0 or greater than the ninth power of 10.
* EMSGSIZE | The size of the buffer for receiving the message is less than the message size attribute of the message queue.
* ETIMEDOUT | No message is received from the queue before the specified timeout period expires.
* @since 1.0
* @version 1.0
*/
ssize_t mq_timedreceive(mqd_t mqdes, char *__restrict buffer, size_t size, unsigned *__restrict prioptr, const struct timespec *__restrict timeout);

/**
* @brief Sends a message to a specified message queue, with a timeout period specified.
*
* The message queue must have been open by {@link mq_open()}. \n
* @param mqdes Indicates the message queue descriptor.
* @param buffer Indicates the pointer to the buffer for sending the message.
* @param size Indicates the size of the message to be sent.
* @param prio Indicates the priority of the message to be sent. This parameter must be <b>0</b> because priority-based message processing is not supported.
* @param timeout Indicates the timeout duration for returning a request.
* @return Returns <b>0</b> if the message is sent; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the message fails to be sent.
* errno | Description
* ----------|------------------------------------------------------------
* EINTR | Message sending is interrupted.
* EAGAIN | The message queue is full, and {@link O_NONBLOCK} is set for the message queue associated with <b>mqdes</b>.
* EBADF | <b>mqdes</b> is invalid or you do not have the write permission.
* EINVAL | <b>mqdes</b> is <b>NULL</b> or the value of <b>size</b> is <b>0</b>.<br><b>timeout</b> is invalid. Specifically, the value of <b>tv_sec</b> is less than 0, and the value of <b>tv_nsec</b> is less than 0 or greater than the ninth power of 10.
* EMSGSIZE | The size of the message to be sent is greater than the message size attribute of the message queue.
* EOPNOTSUPP | The operation is not supported.
* ETIMEDOUT | No message is sent to the queue before the specified timeout period expires.
* @since 1.0
* @version 1.0
*/
int mq_timedsend(mqd_t mqdes, const char *buffer, size_t size, unsigned prio, const struct timespec *timeout);

/**
* @brief Decreases the reference count of the message queue by 1, or deletes the message queue if the reference count is <b>0</b>.
*
* @param name Indicates the message queue name.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------|------------------------------------------
* ENAMETOOLONG | <b>name</b> is too long.
* ENOENT | There is no message queue with the given name.
* EAGAIN | Failed to delete the message queue.
* EBUSY | The message queue to be deleted is being used.
* EINVAL | <b>name</b> is <b>NULL</b>, or the length of the string specified by <b>name</b> is <b>0</b>.
* @since 1.0
* @version 1.0
*/
int mq_unlink(const char *name);

#ifdef __cplusplus
}
#endif
#endif
/** @} */