/**
* @addtogroup NET
* @{
*
* @brief Provides functions and data structures related to network operations.
*
* @since 1.0
* @version 1.0
*/

/**
* @file netdb.h
*
* @brief Provides functions and data structures related to network operations. For example, you can use the functions to send and receive network data, manage network addresses, and convert bytes.
*
* @since 1.0
* @version 1.0
*/
#ifndef	 _NETDB_H
#define	 _NETDB_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Describes the host name and address information.
*/
struct hostent {
/** A pointer to the host name */
char *h_name;
/** A pointer to an array of pointers to alternative host names */
char **h_aliases;
/** IP address type*/
int h_addrtype;
/** Address length */
int h_length;
/** A pointer to an array of pointers to host addresses */
char **h_addr_list;
};

/**
* @brief Defines the first address in <b>h_addr_list</b> for backward compatibility.
*/
#define h_addr h_addr_list[0]

/**
* @brief Opens a connection to the database and sets the next entry to the first entry.
* @attention The <b>stayopen</b> parameter does not take effect. It is equivalent to nonzero for {@link getprotoent} and zero for {@link getprotobyname} and <b>getprotobynumbmer</b>.
* @param stayopen Reserved for interface compatibility (not used currently).
* @since 1.0
* @version 1.0
*/
void setprotoent(int stayopen);

/**
* @brief Retrieves the current protocol information.
* @attention The <b>/etc/protocols</b> configuration file is not supported. Only the following built-in protocols are supported:
* ip/icmp/igmp/ggp/ipencap/st/tcp/egp/pup/udp/hmp/xns-idp/rdp/iso-tp4/xtp/
* ddp/idpr-cmtp/ipv6/ipv6-route/ipv6-frag/idrp/rsvp/gre/esp/ah/skip/ipv6-icmp/
* ipv6-nonxt/ipv6-opts/rspf/vmtp/ospf/ipip/encap/pim/raw (This protocol does not have the <b>aliases</b> attribute, that is, <b>aliases</b> is empty.)
* @return Returns the protocol information if the operation is successful; returns <b>NULL</b> otherwise.
* @since 1.0
* @version 1.0
*/
struct protoent *getprotoent(void);

/**
* @brief Retrieves the information about a specified protocol.
*
* @attention The <b>/etc/protocols</b> configuration file is not supported. Only the following built-in protocols are supported:
* ip/icmp/igmp/ggp/ipencap/st/tcp/egp/pup/udp/hmp/xns-idp/rdp/iso-tp4/xtp/
* ddp/idpr-cmtp/ipv6/ipv6-route/ipv6-frag/idrp/rsvp/gre/esp/ah/skip/ipv6-icmp/
* ipv6-nonxt/ipv6-opts/rspf/vmtp/ospf/ipip/encap/pim/raw (This protocol does not have the <b>aliases</b> attribute, that is, <b>aliases</b> is empty.)
* @param addr Indicates the pointer to the protocol name.
* @return Returns the protocol information if the operation is successful; returns <b>NULL</b> otherwise.
* @since 1.0
* @version 1.0
*/
struct protoent *getprotobyname(const char *name);

/**
* @brief Retrieves the information about a protocol with the specified number.
*
* @attention The <b>/etc/protocols</b> configuration file is not supported. Only the following built-in protocols are supported:
* ip/icmp/igmp/ggp/ipencap/st/tcp/egp/pup/udp/hmp/xns-idp/rdp/iso-tp4/xtp/
* ddp/idpr-cmtp/ipv6/ipv6-route/ipv6-frag/idrp/rsvp/gre/esp/ah/skip/ipv6-icmp/
* ipv6-nonxt/ipv6-opts/rspf/vmtp/ospf/ipip/encap/pim/raw (This protocol does not have the <b>aliases</b> attribute, that is, <b>aliases</b> is empty.)
* @param num Indicates the protocol number.
* @return Returns the protocol information if the operation is successful; returns <b>NULL</b> otherwise.
* @since 1.0
* @version 1.0
*/
struct protoent *getprotobynumber(int num);

/**
* @brief Prints error information.
*
* @param msg Indicates the pointer to the string to be printed together with the error information. This parameter can be null.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Incorrect symbols or parameters
* EOVERFLOW      | Long item for printing
* @since 1.0
* @version 1.0
*/
void herror(const char *msg);

/**
* @brief Retrieves error information associated with the specified error number.
*
* @param ecode Indicates the error number.
* @return Returns the string representing the error information.
* @since 1.0
* @version 1.0
*/
const char *hstrerror(int ecode);

#ifdef __cplusplus
}
#endif

#endif
/** @} */