﻿/**
* @addtogroup NET
* @{
*
* @brief Provides functions and data structures related to network operations.
*
* @since 1.0
* @version 1.0
*/

/**
* @file ether.h
*
* @brief Provides functions and data structures related to network operations. For example, you can use the functions to send and receive network data, manage network addresses, and convert bytes.
*
* @since 1.0
* @version 1.0
*/
#ifndef _NETINET_ETHER_H
#define _NETINET_ETHER_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Converts binary data in network byte order into a standard 48-bit Ethernet host address in the colon hexadecimal notation.
* @attention Different from glibc (for example, 1:2:3:4:5:f), 0 is prefixed and letters are capitalized (for example, 01:02:03:04:05:0F).
* @param p_a Indicates the pointer to the binary data to be converted.
* @return Returns the host address if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
char *ether_ntoa(const struct ether_addr *p_a);

/**
* @brief Converts a standard 48-bit Ethernet host address in the colon hexadecimal notation into binary data in network byte order.
*
* @param x Indicates the pointer to the host address to be converted.
* @return Returns the binary data if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
struct ether_addr *ether_aton(const char *x);

/**
* @brief Converts binary data in network byte order into a standard 48-bit Ethernet host address in the colon hexadecimal notation. This function is reentrant.
*
* This function is used in multitasking scenarios. \n
* @attention Different from glibc (for example, 1:2:3:4:5:f), 0 is prefixed and letters are capitalized (for example, 01:02:03:04:05:0F).
* @param p_a Indicates the pointer to the binary data to be converted.
* @param x Indicates the pointer to the temporary buffer during conversion.
* @return Returns the host address if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
char *ether_ntoa_r(const struct ether_addr *p_a, char *x);

/**
* @brief Converts a standard 48-bit Ethernet host address in the colon hexadecimal notation into binary data in network byte order. This function is reentrant.
*
* This function is used in multitasking scenarios. \n
* @param x Indicates the pointer to the host address to be converted.
* @param p_a Indicates the pointer to the temporary buffer during conversion.
* @return Returns the binary data represented by <b>ether_addr</b> if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
struct ether_addr *ether_aton_r(const char *x, struct ether_addr *p_a);

#ifdef __cplusplus
}
#endif

#endif
/** @} */