/**
* @addtogroup IO
* @{
*
* @brief Provides data structures and functions for performing input/output (I/O) operations, such as printf and scanf.
*
* @since 1.0
* @version 1.0
*/

/**
* @file nl_types.h
*
* @brief Defines I/O multiplexing functions related to message catalog operations.
*
* @since 1.0
* @version 1.0
*/

#ifndef _NL_TYPES_H
#define _NL_TYPES_H

#ifdef __cplusplus
extern "C" {
#endif

/**@brief Type of message catalog descriptor */
typedef void *nl_catd;

/**
* @brief Opens a message catalog and returns its descriptor.
* @param name Indicates the pointer to the path name of the message catalog to open.
* @param oflag Indicates the source of the language to use.
* @return Returns the catalog descriptor if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------
* EIO | A hardware error occurs at the low-level disk I/O layer.
* EINVAL | The path name format is invalid.
* ENOTDIR| A part of the path prefix is the name of an existing file and is not a catalog.
* EBUSY | The operation is rejected as it does not conform to the file sharing policy.
* ENODEV | The corresponding device does not exist.
* ENFILE | The number of opened files in the system has reached the maximum.
* ELOOP | Too many symbolic links are encountered.
* EFAULT | The memory address is invalid.
* ENOENT | Any input parameter is incorrect.
* @since 1.0
* @version 1.0
*/
nl_catd catopen(const char *name, int oflag);

/**
* @brief Obtains information from a message catalog.
* @param catd Indicates the descriptor of the message catalog.
* @param set_id Indicates the setting ID of the message catalog.
* @param msg_id Indicates the ID of the message catalog.
* @param s Indicates the pointer to a string to be returned if the operation fails.
* @return Returns the information obtained from the message catalog if the operation is successful; returns <b>s</b> otherwise.
* @since 1.0
* @version 1.0
*/
char *catgets(nl_catd catd, int set_id, int msg_id, const char *s);

/**
* @brief Closes a catalog descriptor.
* @param catd Indicates the catalog descriptor to close.
* @return 0
* @since 1.0
* @version 1.0
*/
int catclose(nl_catd catd);

#ifdef __cplusplus
}
#endif

#endif

/** @} */