/**
* @addtogroup PROCESS
* @{
*
* @brief Provides process- and thread-related structures and functions.
*
* @since 1.0
* @version 1.0
*/

/**
* @file pthread.h
*
* @brief Provides process- and thread-related structures (providing fields such as thread attributes) and functions (including the functions for creating and destroying threads, and setting the thread detach state and blocking conditions).
*
*
*
* @since 1.0
* @version 1.0
*/

#ifndef _PTHREAD_H
#define _PTHREAD_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sched.h>
#include <time.h>


/**
* @brief Creates a thread.
*
* This function creates a thread in the calling process. The new thread starts execution from the entry point <b>star_routine</b>. <b>arg</b> is passed as the unique argument of the entry point. \n
* @param thread Indicates the pointer to the buffer for storing the thread ID.
* @param attr Indicates the pointer to the thread attribute object. If this parameter is set to <b>NULL</b>, the default thread attributes are used.
* @param start_routine Indicates the pointer to the entry point of the thread.
* @param arg Functions as the unique argument of <b>start_routine</b>.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid parameter.
* EAGAIN | Insufficient resource, or the maximum number of threads allowed by the system reached.
* ENOSYS | Unsupported system call.
* @since 1.0
* @version 1.0
*/
int pthread_create(pthread_t *thread, const pthread_attr_t *attr, void *(*start_routine)(void *), void *arg);

/**
* @brief Detaches a thread.
*
* \n
* @param thread Indicates the ID of the user-level thread to detach.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | The thread is not joinable.
* ESRCH | Invalid thread ID.
* @since 1.0
* @version 1.0
*/
int pthread_detach(pthread_t thread);

/**
* @brief Terminates the calling thread.
*
*
* @param retval Indicates the pointer to the return value after the thread is terminated.
* @since 1.0
* @version 1.0
*/
_Noreturn void pthread_exit(void *retval);

/**
* @brief Waits for a thread to terminate.
*
* This function returns a value immediately if the thread has already terminated. \n
* @param thread Indicates the target thread.
* @param retval Indicates the double pointer to the exit or cancellation status of the target thread. This parameter can be <b>NULL</b>.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | The thread is not joinable, or the target thread is the calling thread.
* @since 1.0
* @version 1.0
*/
int pthread_join(pthread_t thread, void **retval);

#ifdef __GNUC__
__attribute__((const))
#endif

/**
* @brief Obtains the ID of the calling thread.
*
* This function always succeeds.
* @return Returns the thread ID.
* @since 1.0
* @version 1.0
*/
pthread_t pthread_self(void);

/**
* @brief Compares whether two thread IDs are equal.
*
*
* @param t1 Indicates the first thread.
* @param t2 Indicates the second thread.
* @return Returns <b>0</b> if the two are not equal; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int pthread_equal(pthread_t t1, pthread_t t2);

/**
* @brief Sets the cancelability state for the calling thread.
*
* This function sets the cancelability state of the calling thread to the value specified by <b>state</b>. The previous cancelability state is stored in the buffer pointed to by </b>oldstate</b>. \n
* @param state Indicates the cancelability state to set. Available values are as follows:
* state | Description
* -----------------------|---------------------------------------
* PTHREAD_CANCEL_ENABLE | The thread is cancelable.
* PTHREAD_CANCEL_DISABLE | The thread is not cancelable.
* @param oldstate Indicates the pointer to the previous cancelability state before the setting.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid parameter.
* @since 1.0
* @version 1.0
*/
int pthread_setcancelstate(int state, int *oldstate);

/**
* @brief Sets the cancelability type for the calling thread.
*
* This function sets the cancelability type of the calling thread to the value specified by <b>type</b>. The previous cancelability type is stored in the buffer pointed to by </b>oldtype</b>. \n
* @param type Indicates the cancelability type to set. Available values are as follows:
* type | Description
* -----------------------|---------------------------------------
* PTHREAD_CANCEL_DEFERRED | The thread is canceled until the next cancellation point.
* PTHREAD_CANCEL_ASYNCHRONOUS | The thread is canceled immediately upon receiving a cancellation request.
* @param oldtype Indicates the pointer to the previous cancelability type before the setting.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid parameter.
* @since 1.0
* @version 1.0
*/
int pthread_setcanceltype(int type, int *oldtype);

/**
* @brief Requests delivery of any pending cancellation request.
*
* This function creates a cancellation point in the calling thread. In this way, the thread executing code that contains no cancellation point responds to the cancellation request. This function always succeeds. \n
* @return Returns the cancellation point.
* @since 1.0
* @version 1.0
*/
void pthread_testcancel(void);

/**
* @brief Sends a cancellation request to a thread.
*
*
* @param thread Indicates the thread to receive the cancellation request.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid parameter.
* @since 1.0
* @version 1.0
*/
int pthread_cancel(pthread_t thread);

/**
* @brief Sends a signal to a thread.
*
* If <b>sig</b> is <b>0</b>, no signal is sent, but error checking is still performed. Therefore, you can call this function with <b>sig</b> set to <b>0</b> to check whether a thread exists. \n
* @param thread Indicates the thread to receive the signal.
* @param sig Indicates the signal to send.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid signal.
* ESRCH | Invalid thread ID.
* @since 1.0
* @version 1.0
*/
int pthread_kill(pthread_t thread, int sig);

/**
* @brief Obtains the scheduling policy and parameters of a thread.
*
* @param thread Indicates the target thread.
* @param policy Indicates the pointer to the scheduling policy. The value can only be {@link SCHED_FIFO} or {@link SCHED_RR}.
* @param param Indicates the pointer to the scheduling parameters. Only the thread priority is supported. The priority ranges from 0 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* ESRCH | Invalid thread ID.
* EINVAL | Invalid parameter.
* EPERM | No permission to obtain the specified scheduling policy and parameters.
* @since 1.0
* @version 1.0
*/
int pthread_getschedparam(pthread_t thread, int *policy, struct sched_param *param);

/**
* @brief Sets a scheduling policy and parameters for a thread.
*
* @param thread Indicates the target thread.
* @param policy Indicates the scheduling policy to set. The value can only be {@link SCHED_FIFO} or {@link SCHED_RR}.
* @param param Indicates the pointer to the scheduling parameters to set. Only the thread priority is supported. The priority ranges from 0 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise. If the operation fails, the scheduling policy and parameters of the target thread remain unchanged.
* errno | Description
* -------|---------------------------------------
* ESRCH | Invalid thread ID.
* EINVAL | Invalid parameter.
* EPERM | No permission to set the specified scheduling policy and parameters.
* @since 1.0
* @version 1.0
*/
int pthread_setschedparam(pthread_t thread, int policy, const struct sched_param *param);

/**
* @brief Sets a static scheduling priority for a thread.
*
* @param thread Indicates the target thread.
* @param priority Indicates the static scheduling priority to set. The value ranges from 0 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise. If the operation fails, the scheduling policy of the target thread remains unchanged.
* errno | Description
* -------|---------------------------------------
* ESRCH | Invalid thread ID.
* EINVAL | Invalid parameter.
* EPERM | No permission to set the specified scheduling policy and parameters.
* @since 1.0
* @version 1.0
*/
int pthread_setschedprio(pthread_t thread, int prio);

/**
* @brief Enables the initialization function to be called only once.
*
* This function dynamically initializes the function specified by <b>init_routine</b> and ensures that it will be called only once.\n
* @param once_control Indicates the pointer to a variable specifying the execution status. The value <b>0</b> means <b>NEVER</b>, <b>1</b> means <b>IN PROGRESS</b>, and <b>2</b> means <b>DONE</b>.
* @param init_routine Indicates the pointer to the function that you want to call for any required initialization.
* @return Returns <b>0</b> if <b>once_control</b> is set to <b>0</b> or <b>2</b>. If <b>once_control</b> is <b>1</b>, the calling thread waits until the other thread completes initialization.
* @since 1.0
* @version 1.0
*/
int pthread_once(pthread_once_t *once_control, void (*init_routine)(void));

/**
* @brief Initializes a mutex.
*
* This function dynamically creates a mutex. The parameter <b>a</b> specifies the attributes of the mutex. This function always succeeds.\n
* @param m Indicates the pointer to the mutex to initialize.
* @param a Indicates the pointer to the mutex attribute object. If this parameter is set to <b>NULL</b>, the default mutex attributes are used. The default attributes indicate a fast mutex.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_mutex_init(pthread_mutex_t *__restrict m, const pthread_mutexattr_t *__restrict a);

/**
* @brief Locks a mutex.
*
* If the mutex is already locked by a thread, the call is blocked until the holding thread unlocks the mutex by calling {@link pthread_mutex_unlock()}. \n
* @param m Indicates the pointer to the mutex to lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The mutex has been damaged during the waiting.
* EDEADLK | The thread attempts to relock the mutex that it has already locked, and the mutex is of the error check mutex type.
* EAGAIN | The maximum number of recursive locks for the mutex has been exceeded.
* @since 1.0
* @version 1.0
*/
int pthread_mutex_lock(pthread_mutex_t *m);

/**
* @brief Unlocks a mutex.
*
* If the calling thread attempts to unlock a mutex that it has not locked (by calling {@link pthread_mutex_lock()}, {@link pthread_mutex_trylock()}, or {@link pthread_mutex_timedlock_np()}, the unlock request fails and the error code <b>EPERM</b> is returned. \n
* @param m Indicates the pointer to the mutex to unlock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EPERM | The mutex has not been locked by the calling thread.
* @since 1.0
* @version 1.0
*/
int pthread_mutex_unlock(pthread_mutex_t *m);

/**
* @brief Attempts to lock a mutex.
*
* This function attempts to acquire a mutex, without blocking the calling thread. If the mutex is already locked, the error code <b>EBUSY</b> is returned immediately. \n
* @param m Indicates the pointer to the mutex to lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The mutex has been damaged during the waiting.
* EBUSY | The mutex is already locked.
* EAGAIN | The maximum number of recursive locks for the mutex has been exceeded.
* @since 1.0
* @version 1.0
*/
int pthread_mutex_trylock(pthread_mutex_t *m);

/**
* @brief Blocks the calling thread to lock a mutex.
*
* If the mutex is already locked, the call is blocked until the specified timeout duration expires or the holding thread unlocks the mutex by calling {@link pthread_mutex_unlock()}. \n
* @param m Indicates the pointer to the mutex to lock.
* @param at Indicates the pointer to the maximum duration that the calling thread waits for the mutex.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The mutex has been damaged during the waiting.
* EBUSY | The mutex is already locked.
* EAGAIN | The maximum number of recursive locks for the mutex has been exceeded.
* ETIMEDOUT | The mutex cannot be acquired within the specified period of time.
* @since 1.0
* @version 1.0
*/
int pthread_mutex_timedlock(pthread_mutex_t *__restrict m, const struct timespec *__restrict at);

/**
* @brief Destroys a mutex.
*
* This function always succeeds.\n
* @param m Indicates the pointer to the mutex to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_mutex_destroy(pthread_mutex_t *m);

int pthread_mutex_consistent(pthread_mutex_t *);
int pthread_mutex_getprioceiling(const pthread_mutex_t *__restrict, int *__restrict);
int pthread_mutex_setprioceiling(pthread_mutex_t *__restrict, int, int *__restrict);

/**
* @brief Initializes a condition variable.
*
* This function always succeeds.\n
* @param c Indicates the pointer to the condition variable to initialize.
* @param a Indicates the pointer to the condition variable attribute object. If this parameter is set to <b>NULL</b>, the default condition variable attributes are used.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_cond_init(pthread_cond_t *__restrict c, const pthread_condattr_t *__restrict a);

/**
* @brief Destroys a condition variable.
*
* This function always succeeds.\n
* @param c Indicates the pointer to the condition variable to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_cond_destroy(pthread_cond_t *c);

/**
* @brief Blocks the calling thread to wait for the condition set by {@link pthread_con_signal()}.
*
* @param c Indicates the pointer to the condition variable to wait for.
* @param m Indicates the pointer to the mutex associated with the condition variable.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EPERM | The associated mutex is invalid.
* @since 1.0
* @version 1.0
*/
int pthread_cond_wait(pthread_cond_t *__restrict c, pthread_mutex_t *__restrict m);

/**
* @brief Blocks the calling thread to wait for the condition set by {@link pthread_con_signal()} for a period of time specified by <b>ts</b>.
*
*
* @param c Indicates the pointer to the condition variable to wait for.
* @param m Indicates the pointer to the mutex associated with the condition variable.
* @param ts Indicates the pointer to the absolute system time when the calling thread stops blocking.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid <b>ts</b> value.
* EPERM | The associated mutex is invalid.
* @since 1.0
* @version 1.0
*/
int pthread_cond_timedwait(pthread_cond_t *__restrict c, pthread_mutex_t *__restrict m, const struct timespec *__restrict ts);

/**
* @brief Unblocks all threads that are currently blocked on the condition variable <b>cond</b>.
*
* This function always succeeds. \n
* @param c Indicates the pointer to the condition variable to broadcast.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_cond_broadcast(pthread_cond_t *c);

/**
* @brief Unblocks a thread.
*
* If multiple threads are blocked on the condition variable <b>cond</b>, this function unblocks at least one thread. This function always succeeds. \n
* @param c Indicates the pointer to the condition variable to signal.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_cond_signal(pthread_cond_t *c);

/**
* @brief Initializes a read-write lock.
*
* This function always succeeds.\n
* @param rw Indicates the pointer to the read-write lock to initialize.
* @param a Indicates the pointer to the read-write lock attribute object. If this parameter is set to <b>NULL</b>, the default read-write lock attributes are used.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_init(pthread_rwlock_t *__restrict rw, const pthread_rwlockattr_t *__restrict a);

/**
* @brief Destroys a read-write lock.
*
* This function always succeeds.\n
* @param rw Indicates the pointer to the read-write lock to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_destroy(pthread_rwlock_t *rw);

/**
* @brief Applies a read lock to a read-write lock.
*
* @param rw Indicates the pointer to the target read-write lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The read-write lock has been damaged during the waiting.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_rdlock(pthread_rwlock_t *rw);

/**
* @brief Attempts to apply a read lock to a read-write lock.
*
* This function attempts to lock a read-write lock for reading, without blocking the calling thread. If the read-write lock is already locked, the error code <b>EBUSY</b> is returned immediately. \n
* @param rw Indicates the pointer to the target read-write lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The read-write lock has been damaged during the waiting.
* EBUSY | The read-write lock is already locked.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_tryrdlock(pthread_rwlock_t *rw);

/**
* @brief Blocks the calling thread to lock a read-write lock for reading.
*
* If the read-write lock is already locked, the calling thread is blocked until the specified timeout duration expires or the holding thread unlocks the read-write lock by calling {@link pthread_rwlock_unlock()}. \n
* @param rw Indicates the pointer to the read-write lock to lock.
* @param at Indicates the maximum duration that the calling thread waits for the read-write lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The read-write lock has been damaged during the waiting.
* EBUSY | The read-write lock is already locked.
* ETIMEDOUT | The read-write lock cannot be acquired within the specified period of time.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_timedrdlock(pthread_rwlock_t *__restrict rw, const struct timespec *__restrict at);

/**
* @brief Applies a write lock to a read-write lock.
*
*
* @param rw Indicates the pointer to the read-write lock to lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The read-write lock has been damaged during the waiting.
* EBUSY | The read-write lock is already locked.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_wrlock(pthread_rwlock_t *rw);

/**
* @brief Attempts to apply a write lock to a read-write lock.
*
* This function attempts to lock a read-write lock for writing, without blocking the calling thread. If the read-write lock is already locked, the error code <b>EBUSY</b> is returned immediately. \n
* @param rw Indicates the pointer to the read-write lock to lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The read-write lock has been damaged during the waiting.
* EBUSY | The read-write lock is already locked.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_trywrlock(pthread_rwlock_t *rw);

/**
* @brief Blocks the calling thread to lock a read-write lock for writing.
*
* If the read-write lock is already locked, the calling thread is blocked until the specified timeout duration expires or the holding thread unlocks the read-write lock by calling {@link pthread_rwlock_unlock()}. \n
* @param rw Indicates the pointer to the read-write lock to lock.
* @param at Indicates the pointer to the maximum duration that the calling thread waits for the read-write lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EBADF | The read-write lock has been damaged during the waiting.
* EBUSY | The read-write lock is already locked.
* ETIMEDOUT | The read-write lock cannot be acquired within the specified period of time.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_timedwrlock(pthread_rwlock_t *__restrict rw, const struct timespec *__restrict at);

/**
* @brief Unlocks a read-write lock.
*
* If the calling thread attempts to unlock a read-write lock that it has not locked, the unlock request fails and the error code <b>EPERM</b> is returned. \n
* @param rw Indicates the pointer to the read-write lock to unlock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Incorrect parameter value.
* EPERM | The read-write lock is not held by the calling thread.
* @since 1.0
* @version 1.0
*/
int pthread_rwlock_unlock(pthread_rwlock_t *rw);

/**
* @brief Initializes a spin lock.
*
* This function always succeeds.\n
* @param s Indicates the pointer to the spin lock to initialize.
* @param shared Indicates thread process-shared synchronization, which is not supported.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_spin_init(pthread_spinlock_t *s, int shared);

/**
* @brief Destroys a spin lock.
*
* This function always succeeds.\n
* @param s Indicates the pointer to the spin lock to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_spin_destroy(pthread_spinlock_t *s);

/**
* @brief Locks a spin lock.
*
* This function always succeeds.\n
* @param s Indicates the pointer to the spin lock to lock.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_spin_lock(pthread_spinlock_t *s);

/**
* @brief Attempts to lock a spin lock.
*
* This function attempts to lock the spin lock, without blocking the calling thread. If the spin lock is already locked, the error code <b>EBUSY</b> is returned immediately. \n
\n
* @param s Indicates the pointer to the spin lock to lock.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EBUSY | The spin lock has been held by another thread.
* @since 1.0
* @version 1.0
*/
int pthread_spin_trylock(pthread_spinlock_t *s);

/**
* @brief Unlocks a spin lock.
*
* This function always succeeds.\n
* @param s Indicates the pointer to the spin lock to unlock.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_spin_unlock(pthread_spinlock_t *s);

/**
* @brief Initializes a barrier.
*
*
* @param b Indicates the pointer to the barrier to initialize.
* @param a Indicates the pointer to the barrier attribute object. If this parameter is set to <b>NULL</b>, the default barrier attributes are used.
* @param count Indicates the number of threads that must call {@link pthread_barrier_wait()} before any of them successfully returns from the call.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | The value of <b>count </b> is greater than the maximum number of threads to block.
* @since 1.0
* @version 1.0
*/
int pthread_barrier_init(pthread_barrier_t *__restrict b, const pthread_barrierattr_t *__restrict a, unsigned count);

/**
* @brief Destroys a barrier.
*
* This function always succeeds.\n
* @param b Indicates the pointer to the barrier to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_barrier_destroy(pthread_barrier_t *b);

/**
* @brief Synchronizes participating threads at a barrier.
*
* The call is blocked until the required number of threads have called this function with the specified barrier. \n
* @param b Indicates the pointer to the barrier to be used for synchronization.
* @return Returns {@link PTHREAD_BARRIER_SERIAL_THREAD} for the first restored thread and <b>0</b> for other threads.
* @since 1.0
* @version 1.0
*/
int pthread_barrier_wait(pthread_barrier_t *b);

/**
* @brief Creates a key for thread data.
*
*
* @param key Indicates the pointer to the key to set for the thread data.
* @param destructor Indicates the pointer to the function to be bound to the key.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EAGAIN | The system lacks the necessary resources to create another thread-specific key, or the number of keys exceeds the limit specified by {@link PTHREAD_KEYS_MAX} for each process.
* @since 1.0
* @version 1.0
*/
int pthread_key_create(pthread_key_t *key, void (*destructor)(void *));

/**
* @brief Deletes a key for thread data.
*
*
* @param key Indicates the pointer to the key to delete for the thread data.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid key.
* @since 1.0
* @version 1.0
*/
int pthread_key_delete(pthread_key_t key);

/**
* @brief Obtains specific thread data.
*
*
* @param key Indicates the key bound to the thread data.
* @since 1.0
* @version 1.0
*/
void *pthread_getspecific(pthread_key_t key);

/**
* @brief Sets specific thread data.
*
* This function always succeeds.\n
* @param key Indicates the key bound to the thread data.
* @param value Indicates the pointer to the thread data to be bound to the key.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_setspecific(pthread_key_t key, const void *value);

/**
* @brief Initializes a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the thread attribute object that is successfully initialized.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_init(pthread_attr_t *attr);

/**
* @brief Destroys a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the thread attribute object to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_destroy(pthread_attr_t *attr);

/**
* @brief Obtains the guard size of a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param guardsize Indicates the pointer to the obtained guard size.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_getguardsize(const pthread_attr_t *attr, size_t *guardsize);

/**
* @brief Sets the guard size for a thread attribute object.
*
*
* @param attr Indicates the pointer to the target thread attribute object.
* @param guardsize Indicates the guard size to set.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid guard size.
*/
int pthread_attr_setguardsize(pthread_attr_t *attr, size_t guardsize);

/**
* @brief Obtains the stack size of a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param stacksize Indicates the pointer to the obtained stack size.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_getstacksize(const pthread_attr_t *attr, size_t *stacksize);

/**
* @brief Sets the stack size for a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param stacksize Indicates the stack size.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid stack size.
*/
int pthread_attr_setstacksize(pthread_attr_t *attr, size_t stacksize);

/**
* @brief Obtains the detach state of a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param detachstate Indicates the pointer to the obtained detach state.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_getdetachstate(const pthread_attr_t *attr, int *detachstate);

/**
* @brief Sets the detach state for a thread attribute object.
*
*
* @param attr Indicates the pointer to the target thread attribute object.
* @param detachstate Indicates the detach state to set. Available values are as follows:
* detachstate | Description
* --------------|------------------
* PTHREAD_CREATE_DETACHED | Threads using <b>attr</b> are created in the detached state.
* PTHREAD_CREATE_JOINABLE | Threads using <b>attr</b> are created in the joinable state.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid detach state.
*/
int pthread_attr_setdetachstate(pthread_attr_t *attr, int detachstate);

/**
* @brief Obtains stack attributes of a thread attribute object.
*
*
* @param attr Indicates the pointer to the target thread attribute object.
* @param stackaddr Indicates the double pointer to the start address of the buffer that stores the obtained stack attributes.
* @param stacksize Indicates the pointer to the size of the buffer that stores the obtained stack attributes.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid start address.
*/
int pthread_attr_getstack(const pthread_attr_t *attr, void **stackaddr, size_t *stacksize);

/**
* @brief Sets stack attributes for a thread attribute object.
*
*
* @param attr Indicates the pointer to the thread attribute object.
* @param stackaddr Indicates the pointer to the start address of the buffer that stores the stack attributes to set.
* @param stacksize Indicates the size of the buffer that stores the stack attributes to set.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid start address.
*/
int pthread_attr_setstack(pthread_attr_t *attr, void *stackaddr, size_t stacksize);

/**
* @brief Obtains contention scope attributes of a thread attribute object.
*
*
* @param attr Indicates the pointer to the target thread attribute object.
* @param scope Indicates the pointer to the start address of the buffer that stores the target thread attribute object.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid <b>scope</b> value.
*/
int pthread_attr_getscope(const pthread_attr_t *arrt, int *scope);

/**
* @brief Sets contention scope attributes for a thread attribute object.
*
* The contention scope attribute defines a set of threads against which a thread competes for resources such as the CPU.
POSIX.1-2001 specifies two values for <b>scope</b>: <ul><li>{@link PTHREAD_SCOPE_SYSTEM}: The thread competes for resources
with other threads in all processes that are in the same scheduling allocation domain (a group of one or more processors).</li>
<li>{@link PTHREAD_SCOPE_PROCESS}: These threads are scheduled relative to other threads in the process according to their scheduling policy and priority.</li></ul>
POSIX.1-2001 does not specify how these threads contend with other threads in other process on the system or with other threads in the same process that were created with the {@link PTHREAD_SCOPE_SYSTEM} contention scope.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param scope Indicates the start address of the buffer that stores the target thread attribute object. By default, only {@link PTHREAD_SCOPE_PROCESS} is supported.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid <b>scope</b> value.
*/
int pthread_attr_setscope(pthread_attr_t *arrt, int scope);

/**
* @brief Obtains scheduling policy attributes of a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param schedpolicy Indicates the pointer to the obtained scheduling policy attributes. Only {@link SCHED_FIFO} and {@link SCHED_RR} are supported.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_getschedpolicy(const pthread_attr_t *attr, int *schedpolicy);

/**
* @brief Sets scheduling policy attributes for a thread attribute object.
*
*
* @param attr Indicates the pointer to the target thread attribute object.
* @param schedpolicy Indicates the scheduling policy attributes to set. Only {@link SCHED_FIFO} and {@link SCHED_RR} are supported.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid scheduling policy attribute.
*/
int pthread_attr_setschedpolicy(pthread_attr_t *attr, int schedpolicy);

/**
* @brief Obtains scheduling parameter attributes of a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param param Indicates the pointer to the obtained scheduling parameter attributes. Only the thread priority is supported. The priority ranges from 0 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_getschedparam(const pthread_attr_t *attr, struct sched_param *param);

/**
* @brief Sets scheduling parameter attributes for a thread attribute object.
*
* @param attr Indicates the pointer to the target thread attribute object.
* @param param Indicates the pointer to the scheduling parameter attributes to set. Only the thread priority is supported. The priority ranges from 0 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid scheduling parameter attributes.
*/
int pthread_attr_setschedparam(pthread_attr_t *attr, const struct sched_param *param);

/**
* @brief Obtains inherit scheduler attributes of a thread attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target thread attribute object.
* @param inheritsched Indicates the pointer to the obtained inherit scheduler attributes.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_attr_getinheritsched(const pthread_attr_t *attr, int *inheritsched);

/**
* @brief Sets inherit scheduler attributes for a thread attribute object.
*
* @param attr Indicates the pointer to the target thread attribute object.
* @param inheritsched Indicates the inherit scheduler attributes to set.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid inherit scheduler attribute.
*/
int pthread_attr_setinheritsched(pthread_attr_t *attr, int inheritsched);

/**
* @brief Destroys a mutex attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target mutex attribute object.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_mutexattr_destroy(pthread_mutexattr_t *attr);
int pthread_mutexattr_getprioceiling(const pthread_mutexattr_t *__restrict, int *__restrict);
int pthread_mutexattr_getprotocol(const pthread_mutexattr_t *__restrict, int *__restrict);
int pthread_mutexattr_getpshared(const pthread_mutexattr_t *__restrict, int *__restrict);
int pthread_mutexattr_getrobust(const pthread_mutexattr_t *__restrict, int *__restrict);

/**
* @brief Obtains the mutex type attribute.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the mutex attribute object.
* @param type Indicates the pointer to the obtained mutex type attribute.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_mutexattr_gettype(const pthread_mutexattr_t *__restrict attr, int *__restrict type);

/**
* @brief Initializes a mutex attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the target mutex attribute object.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_mutexattr_init(pthread_mutexattr_t *attr);

/**
* @brief Sets the mutex type attribute.
*
*
* @param attr Indicates the pointer to the mutex attribute object.
* @param type Indicates the type of the mutex.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid mutex type attribute.
*/
int pthread_mutexattr_settype(pthread_mutexattr_t *attr, int type);

/**
* @brief Initializes a condition variable attribute object.
*
* This function always succeeds.\n
* @param a Indicates the pointer to the variable that contains the condition variable attributes.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_condattr_init(pthread_condattr_t *a);

/**
* @brief Destroys a condition variable attribute object.
*
* This function always succeeds.\n
* @param a Indicates the pointer to the variable that contains the condition variable attributes.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_condattr_destroy(pthread_condattr_t *a);

/**
* @brief Sets a clock for a condition variable attribute object.
*
* @param a Indicates the pointer to the variable that contains the condition variable attributes.
* @param clk Indicates the ID of the clock to set.
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid <b>clk> value.
* @since 1.0
* @version 1.0
*/
int pthread_condattr_setclock(pthread_condattr_t *a, clockid_t clk);

/**
* @brief Obtains the clock of a condition variable attribute object.
*
* This function always succeeds.\n
* @param a Indicates the pointer to the variable that contains the condition variable attributes.
* @param clk Indicates the pointer to the obtained clock ID.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_condattr_getclock(const pthread_condattr_t *__restrict a, clockid_t *__restrict clk);

/**
* @brief Initializes a read-write lock attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the read-write lock attribute object to initialize.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_rwlockattr_init(pthread_rwlockattr_t *attr);

/**
* @brief Destroys a read-write lock attribute object.
*
* This function always succeeds.\n
* @param attr Indicates the pointer to the read-write lock attribute object to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_rwlockattr_destroy(pthread_rwlockattr_t *attr);

/**
* @brief Destroys a barrier attribute object.
*
* This function always succeeds.\n
* @param a Indicates the pointer to the barrier attribute object to destroy.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_barrierattr_destroy(pthread_barrierattr_t *a);

/**
* @brief Initializes a barrier attribute object.
*
* This function always succeeds.\n
* @param a Indicates the pointer to the barrier attribute object to initialize.
* @return Returns <b>0</b>.
* @since 1.0
* @version 1.0
*/
int pthread_barrierattr_init(pthread_barrierattr_t *a);

/**
* @brief Registers a fork handler to be called before and after {@link fork()}.
*
*
* @param prepare Indicates the pointer to the fork handler to be called before {@link fork()}.
* @param parent Indicates the pointer to the fork handler to be called after {@link fork()} in the parent process.
* @param child Indicates the pointer to the fork handler to be called after {@link fork()} in the child process.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int pthread_atfork(void (*prepare)(void), void (*parent)(void), void (*child)(void));

/**
* @brief Pushes the routine to the top of the clean-up handler stack.
*
* @param routine Indicates the pointer to the routine used to complete the clean-up.
* @param arg Indicates the parameter to be passed to the routine.
* @since 1.0
* @version 1.0
*/
void pthread_cleanup_push(void (*routine)(void *), void *arg);

/**
* @brief Removes the routine at the top of the clean-up handler stack.
*
*
* @param execute Specifies whether the routine at the top of the clean-up handler stack should be executed. If this parameter is set to a non-zero value, the routine at the top of the clean-up handler stack must be popped and executed.
* @since 1.0
* @version 1.0
*/
void pthread_cleanup_pop(int execute);

/**
* @brief Obtains the attributes of a created thread.
*
* This function always succeeds.\n
* @param thread Indicates the thread that has been created.
* @param attr Indicates the pointer to the attribute values that describe the running thread.
* @return Returns <b>0</b>.
*/
int pthread_getattr_np(pthread_t thread, pthread_attr_t *attr);

/**
* @brief Sets the thread name.
*
* @attention Currently, a thread can change its own thread name only.\n
* @param thread Indicates the thread whose name is to be changed.
* @param name Indicates the pointer to the thread name to set. The value contains a maximum of 16 characters, including the terminating null byte ('\0').
* @return Returns <b>0</b> if the operation is successful; returns a value listed in <b>errno</b> otherwise.
* errno | Description
* -------|---------------------------------------
* ERANGE | The name is too long.
* EPERM | Failed to copy data from the user-level thread.
*/
int pthread_setname_np(pthread_t pthread, const char * name);

#ifdef __cplusplus
}
#endif

#endif
/** @} */