/**
* @addtogroup SYS
* @{
*
* @brief Provides functions for system operations.
*
* @since 1.0
* @version 1.0
*/

/**
* @file regex.h
*
* @brief Provides common functions for operating regular expressions.
*
* You can use functions provided in this file to perform operations on regular expressions during development. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _REGEX_H
#define _REGEX_H


#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Compiles a specified regular expression into a string of a specific format.
*
* @param preg Indicates the pointer to the compiled regular expression.
* @param regex Indicates the pointer to the regular expression to compile.
* @param cflags Indicates the regular expression matching flag. Available values are as follows:
* cflags       | description
* ------------ | ---------------------------------------------
* REG_EXTENDED | Support extended regular expressions.
* REG_ICASE | Ignore case during matching.
* REG_NOSUB | The matching result does not need to be stored.
* REG_NEWLINE | Identify the newline character.
* @return Returns <b>0</b> if the operation is successful; returns a value in the following table otherwise.
* value        | description
* ------------ | -------------------------------------------------
* REG_BADBR | Use of back reference operator is invalid.
* REG_BADPAT | Use of pattern operators such as group or list is invalid.
* REG_BADRPT | Use of repetition operators is invalid. For example, <b>*</b> is used as the first character.
* REG_EBRACE | Brace interval operators are unmatched.
* REG_EBRACK | Bracket list operators are unmatched.
* REG_ECOLLATE | The collation element is invalid.
* REG_ECTYPE | The character class name is invalid.
* REG_EESCAPE | The last character is a backslash (\).
* REG_EPAREN | Parenthesis group operators are unmatched.
* @since  1.0
* @version  1.0
*/
int regcomp(regex_t *preg, const char *regex, int cflags);

/**
* @brief Matches a regular expression.
*
* @param preg Indicates the pointer to the regular expression compiled using {@link regcomp}.
* @param string Indicates the pointer to the target string.
* @param nmatch Indicates the array length of the {@link regmatch_t} structure.
* @param pmatch Indicates the array of the {@link regmatch_t} structure. It stores the position information of the matched string.
* @param eflags Indicates the matching flag. Available values are as follows:
* value      | description
* ---------- | --------------------
* REG_NOTBOL | The circumflex character (^) will not match the beginning of a string.
* REG_NOTEOL | The dollar sign ($) will not match the end of a string.
* @return Returns <b>0</b> if the operation is successful; returns {@link REG_NOMATCH} otherwise.
* @since  1.0
* @version  1.0
*/
int regexec(const regex_t *preg, const char *string, size_t nmatch, regmatch_t pmatch[], int eflags);

/**
* @brief Releases a regular expression.
*
* @param preg Indicates the pointer to the regular expression compiled using {@link regcomp}.
* @since  1.0
* @version  1.0
*/
void regfree(regex_t *preg);

/**
* @brief Returns a string containing error information.
*
* If an error occurs during the execution of {@link regcomp} or {@link regexec}, this function can be called to return a string containing the error information. \n
* @param errcode Indicates the error code returned by {@link regcomp} and {@link regexec}.
* @param preg Indicates the pointer to the regular expression compiled using {@link regcomp}. This parameter value can be <b>NULL</b>.
* @param errbuf Indicates the pointer to the memory for storing the string of the error information.
* @param errbuf_size Indicates the length of the <b>errbuf</b> parameter. If the length of the string of the error information is greater than the value of this parameter, this function automatically truncates the extra string, but still returns the complete string length.
* @return Returns the length of the error information string.
* @since  1.0
* @version  1.0
*/
size_t regerror(int errcode, const regex_t *preg, char *errbuf, size_t errbuf_size);

#ifdef __cplusplus
}
#endif

#endif/** @} */