/**
* @addtogroup NET
* @{
*
* @brief Provides functions and data structures related to network operations.
*
* @since 1.0
* @version 1.0
*/

/**
* @file resolv.h
*
* @brief Provides functions and data structures related to network operations. For example, you can use the functions to send and receive network data, manage network addresses, and convert bytes.
*
* @since 1.0
* @version 1.0
*/
#ifndef _RESOLV_H
#define _RESOLV_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Compresses a network domain name.
*
* To reduce the size of DNS messages, domain names in the messages are compressed. For details about the compression scheme, see RFC 1035.
*
* @param src Indicates the pointer to the domain name to be compressed.
* @param dst Indicates the pointer to the compressed domain name.
* @param space Indicates the size of the compressed domain name.
* @param dnptrs Indicates the pointer to an array of pointers to previously compressed domain names. If the value is <b>NULL</b>, the current domain name will not be compressed.
* @param lastdnptr Indicates the pointer to the last element of the array. If the value is <b>NULL</b>, the array is not updated.
* @return Returns the length of the compressed domain name if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int dn_comp(const char *src, unsigned char *dst, int space, unsigned char **dnptrs, unsigned char **lastdnptr);

/**
* @brief Expands a compressed domain name to a full domain name.
*
* The compressed domain name is contained in a query or reply message.
*
* @param base Indicates the pointer to the start of the DNS message body.
* @param end Indicates the pointer to the end of the DNS message body.
* @param src Indicates the pointer to the compressed domain name.
* @param dest Indicates the pointer to the expanded domain name.
* @param space Indicates the memory size of the <b>dest</b> address.
* @return Returns the length of the compressed domain name if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int dn_expand(const unsigned char *base, const unsigned char *end, const unsigned char *src, char *dest, int space);

#ifdef __cplusplus
}
#endif

#endif
/** @} */