/**
* @addtogroup PROCESS
* @{
*
* @brief Provides process- and thread-related structures and functions.
*
* @since 1.0
* @version 1.0
*/

/**
* @file sched.h
*
* @brief Provides process- and thread-related structures and functions (for example, obtaining scheduling policies and parameters).
*
*
*
* @since 1.0
* @version 1.0
*/
#ifndef _SCHED_H
#define _SCHED_H
#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Defines process scheduling parameters.
*/
struct sched_param {
/** Process priority */
int sched_priority;
/** Reserved */
int __reserved1;
struct {
time_t __reserved1;
long __reserved2;
} __reserved2[2];
int __reserved3;
};

/**
* @brief Obtains the maximum static priority that can be used for a process.
*
* This function returns the lowest priority of process scheduling in a scheduling policy specified by <b>policy</b>. The value of <b>policy</b> must be a value defined in {@link sched.h}. \n
* @param policy Indicates the scheduling policy. The value can be {@link SCHED_FIFO} or {@link SCHED_RR}, but not {@link SCHED_OTHER}, {@link SCHED_BATCH}, {@link SCHED_IDLE}, or {@link SCHED_DEADLINE}.
* @return Returns the lowest priority of the scheduling policy if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------------------------------
* EINVAL | Invalid parameter. The value of <b>policy</b> is not a defined scheduling policy.
* ENOSYS | Unsupported scheduling policy.
* @since 1.0
* @version 1.0
*/
int sched_get_priority_max(int policy);

/**
* @brief Obtains the minimum static priority that can be used for a process.
*
* This function returns the highest priority of process scheduling in a scheduling policy specified by <b>policy</b>. The value of <b>policy</b> must be a value defined in {@link sched.h}. \n
* @param policy Indicates the scheduling policy. The value can be {@link SCHED_FIFO} or {@link SCHED_RR}, but not {@link SCHED_OTHER}, {@link SCHED_BATCH}, {@link SCHED_IDLE}, or {@link SCHED_DEADLINE}.
* @return Returns the highest priority of the scheduling policy if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid parameter. The value of <b>policy</b> is not a defined scheduling policy.
* ENOSYS | Unsupported scheduling policy.
* @since 1.0
* @version 1.0
*/
int    sched_get_priority_min(int policy);

/**
* @brief Obtains scheduling parameters of a process.
*
* @param pid Indicates the ID of the process for which the scheduling parameters are to be obtained. If this parameter is set to <b>0</b>, the scheduling parameters of the calling process are to be obtained.
* @param param Indicates the pointer to the scheduling parameters. Only the static priority is supported. The priority ranges from 10 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid parameter.
* ESRCH | The process specified by <b>pid</b> cannot be found.
* @since 1.0
* @version 1.0
*/
int    sched_getparam(pid_t pid, struct sched_param *param);

/**
* @brief Obtains the scheduling policy of a process.
*
* @param pid Indicates the ID of the process for which the scheduling policy is to be obtained. If this parameter is set to <b>0</b>, the scheduling policy of the calling process is to be obtained.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid parameter.
* EPERM | No permission.
* ESRCH | The process specified by <b>pid</b> cannot be found.
* @since 1.0
* @version 1.0
*/
int    sched_getscheduler(pid_t pid);

/**
* @brief Obtains the execution time limit of a process.
*
* This function updates the <b>timespec</b> structure referenced by the parameter <b>interval</b> to record the execution time limit of a process. \n
* @param pid Indicates the ID of the process for which the execution time limit is to be obtained. If this parameter is <0>0</b>, the time quantum of the calling process is obtained and written into the parameter <b>interval</b>.
* @param interval Indicates the pointer to the time structure that records the time limit. The time structure supports only seconds and nanoseconds.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-------------------------------
* EFAULT | An error occurred when copying information to the user space.
* EINVAL | Invalid parameter.
* ESRCH | The process specified by <b>pid</b> cannot be found.
* @since 1.0
* @version 1.0
*/
int    sched_rr_get_interval(pid_t pid, struct timespec *interval);

/**
* @brief Sets scheduling parameters related to a scheduling policy for a process.
*
* \n
* @param pid Indicates the ID of the process for which the scheduling parameters are to be set. If this parameter is set to <b>0</b>, the scheduling parameters of the calling process are to be set.
* @param param Indicates the pointer to the scheduling parameters to set. Only the static priority can be set. The priority ranges from 10 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|------------------------------------
* EINVAL | Invalid parameter.
* EPERM | No permission.
* ESRCH | The process specified by <b>pid</b> cannot be found.
* @since 1.0
* @version 1.0
*/
int    sched_setparam(pid_t pid, const struct sched_param * param);

/**
* @brief Sets a scheduling policy for a process.
*
* @param pid Indicates the ID of the process for which the scheduling policy and parameters are to be set. If this parameter is set to <b>0</b>, the scheduling policy and parameters of the calling process are to be set.
* @param policy Indicates the scheduling policy to set. The value can be {@link SCHED_FIFO} or {@link SCHED_RR}, but not {@link SCHED_OTHER}, {@link SCHED_BATCH}, {@link SCHED_IDLE}, or {@link SCHED_DEADLINE}.
* @param param Indicates the pointer to the scheduling parameters to set. Only the static priority can be set. The priority ranges from 10 (highest priority) to 31 (lowest priority).
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|----------------------------------------------------
* EINVAL | Invalid parameter.
* EPERM | No permission.
* ESRCH | The process specified by <b>pid</b> cannot be found.
* @since 1.0
* @version 1.0
*/
int    sched_setscheduler(pid_t pid, int policy, const struct sched_param *param);

/**
* @brief Yields the running process.
*
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int     sched_yield(void);

#endif

#ifdef __cplusplus
}
#endif
#endif
/** @} */