/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file search.h
*
* @brief Provides functions and related data structures, such as creating, destroying, or searching for a hash table.
*
* You can use the functions provided in this file to perform operations related to wide character types during development, such as, creating, destroying, or searching for a hash table, and creating or searching for a binary tree. \n
* @since 1.0
* @version 1.0
*/

#ifndef _SEARCH_H
#define _SEARCH_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Defines a hash table entry.
*/
typedef struct entry {
/** Key of the entry */
char *key;
/** Value of the entry */
void *data;
} ENTRY;

/**
* @brief Creates a queue from the doubly linked list for {@link insque} and {@link remque}.
*/
struct qelem {
/** Pointer to the previous {@link struct qelem} type */
struct qelem *q_forw;
/** Pointer to the next {@link struct qelem} type */
struct qelem *q_back;
/** Data saved in the queue */
char q_data[1];
};

/**
* @brief Creates a hash table based on the number of entries.
*
* \n
* @param nel Indicates the maximum number of entries allowed in the hash table.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> and sets <b>errno</b> to a value if the operation fails. \n
* @since  1.0
* @version  1.0
*/
int hcreate(size_t nel);

/**
* @brief Destroys a hash table.
*
* This function releases the memory for creating a hash table. \n
* @since  1.0
* @version  1.0
*/
void hdestroy(void);

/**
* @brief Adds or searches for a hash entry.
*
* This function searches for the entry in the hash table based on the specified action. If the action is <b>FIND</b>, only the search action is performed. If the action is <b>ENTER</b>, an entry not found is added to the hash table. \n
* @param item Indicates the hash table entry to be searched.
* @param action Indicates the function action. <b>ENTER</b> indicates that an entry is added, and <b>FIND</b> indicates that an entry is searched. For details, see {@link ACTION}.
* @return Returns the pointer to the hash table if the operation is successful.
* @since  1.0
* @version  1.0
*/
ENTRY *hsearch(ENTRY item, ACTION action);

/**
* @brief Defines a hash table.
*/
struct hsearch_data {
/** Pointer to the hash table search entry */
struct __tab *__tab;
/** Unused */
unsigned int __unused1;
/** Unused */
unsigned int __unused2;
};


/**
* @brief Creates a hash table based on the number of entries and its description.
*
* @attention The hash table must be initialized before use.
* @param nel Indicates the maximum number of entries allowed in the hash table.
* @param htab Indicates the hash table.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> and sets <b>errno</b> to a value if the operation fails.
* @since  1.0
* @version  1.0
*/
int hcreate_r(size_t nel, struct hsearch_data *htab);

/**
* @brief Destroys a hash table.
*
* This function releases the memory occupied by the specified hash table. \n
* @param htab Indicates the hash table.
* @since  1.0
* @version  1.0
*/
void hdestroy_r(struct hsearch_data *htab);

/**
* @brief Searches for a hash table.
*
* This function searches for the entry in the hash table based on the specified action. If the action is <b>FIND</b>, only the search action is performed. If the action is <b>ENTER</b>, an entry not found is added to the hash table. \n
* @param item Indicates the hash table entry to be searched.
* @param action Indicates the function action. <b>ENTER</b> indicates that an entry is added, and <b>FIND</b> indicates that an entry is searched. {@link ACTION}
* @param htab Indicates the hash table.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
int hsearch_r(ENTRY item, ACTION action, ENTRY **retval, struct hsearch_data *htab);

/**
* @brief Adds an entry to a queue.
*
* @param element Indicates the <b>element</b> to be added to the queue.
* @param pred After the position of <b>prev</b> to add the <b>element</b>.
* @since  1.0
* @version  1.0
*/
void insque(void *element, void *pred);

/**
* @brief Removes an entry from a queue.
*
* @param element Indicates the <b>element</b> to be removed from the queue.
* @since  1.0
* @version  1.0
*/
void remque(void *elem);

/**
* @brief Performs a linear search for a key in the array and adds a key to the end of the array if the key is not found.
*
* @param key Indicates the pointer to the key to be searched.
* @param base Indicates the pointer to the array buffer.
* @param nelp Indicates the pointer to the linear search length.
* @param width Indicates the pointer to the array buffer width.
* @param compar Indicates the pointer to the function to match.
* @return Returns the obtained key if the operation is successful.
* @since  1.0
* @version  1.0
*/
void *lsearch(const void *key, const void *base, size_t *nelp, size_t width, int (*compar)(const void *, const void *));

/**
* @brief Performs a linear search for a key in the array.
*
* @param key Indicates the pointer to the key to be searched.
* @param base Indicates the pointer to the array buffer.
* @param nelp Indicates the pointer to the linear search length.
* @param width Indicates the pointer to the array buffer width.
* @param compar Indicates the pointer to the function to match.
* @return Returns the obtained key if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
void *lfind(const void *key, const void *base, size_t *nelp, size_t width, int (*compar)(const void *, const void *));

/**
* @brief Deletes a key from the binary tree.
*
* @param key Indicates the pointer to the key to be deleted.
* @param rootp Indicates the pointer to the root node of the tree. If the tree is empty, the value of this parameter should be <b>NULL</b>.
* @param compar Indicates the judgment function for searching for a key in the tree.
* @return Returns the parent pointer to the deleted key if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
void *tdelete(const void *key, void **rootp, int (*compar)(const void *, const void *));

/**
* @brief Searches for a key in the binary tree.
*
* @param key Indicates the pointer to the key to be searched.
* @param rootp Indicates the pointer to the root node of the tree. If the tree is empty, the value of this parameter should be <b>NULL</b>.
* @param compar Indicates the judgment function for searching for a key in the tree.
* @return Returns the parent pointer to the deleted key if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
void *tfind(const void *key, void *const *rootp,int (*compar)(const void *, const void *));

/**
* @brief Searches for a key in the binary tree and adds a key to the tree if the key is not found.
*
* @param key Indicates the pointer to the key to be searched.
* @param rootp Indicates the pointer to the root node of the tree. If the tree is empty, the value of this parameter should be <b>NULL</b>.
* @param compar Indicates the judgment function for searching for a key in the tree.
* @since  1.0
* @version  1.0
*/
void *tsearch(const void *key, void *const *rootp,int (*compar)(const void *, const void *));

/**
* @brief Traverses a binary tree from left to right.
*
* <b>root </b> points to the start node for the traversal. If the node is not the root, only part of the tree will be visited. \n
* @param root Indicates the pointer to the start node to be searched.
* @param action Indicates the running function for each node access.
* @param nodep Indicates the pointer to the node to be accessed.
* @param which Indicates the value sequence, which is <b>preorder</b>, <b>inorder</b>, <b>postorder</b>, and <b>leaf</b>. For details, see {@link VISIT}.
* @param depth Indicates the depth of the node.
* @since  1.0
* @version  1.0
*/
void twalk(const void *root, void (*action)(const void *nodep, VISIT which, int depth));

/**
* @brief Releases all nodes in the binary tree.
*
* @param root Indicates the pointer to the root node of the binary tree to be released.
* @param free_node Indicates the callback function each time when a node is released.
* @since  1.0
* @version  1.0
*/
void tdestroy(void *root, void (*free_node)(void *nodep));


#ifdef __cplusplus
}
#endif

#endif

/** @} */