/**
* @addtogroup IPC
* @{
*
* @brief Provides functions and structures related to inter-process communication (IPC), including signals, semaphores, and message queues.
*
* @since 1.0
* @version 1.0
*/

/**
* @file semaphore.h
*
* @brief Provides functions and structures related to semaphore operations.
*
* For example, you can use the functions to create, open, close, and delete semaphores, and perform P and V operations on semaphores.
*
* @since 1.0
* @version 1.0
*/
#ifndef _SEMAPHORE_H
#define _SEMAPHORE_H
#ifdef __cplusplus
extern "C" {
#endif

#include <fcntl.h>

/**@brief Defines the semaphore failure flag. */
#define SEM_FAILED ((sem_t *)0)

/**
* @brief Defines semaphores.
*
* This structure is used to store semaphores. \n
*
*/
typedef struct {
/** Number of semaphores */
volatile int __val[4*sizeof(long)/sizeof(int)];
} sem_t;

/**
* @brief Closes a specified semaphore.
*
* @param sem Indicates the pointer to the semaphore to close.
* @return Returns <b>0</b> if the semaphore is closed.
* errno | Description
* @since 1.0
* @version 1.0
*/
int    sem_close(sem_t *sem);

/**
* @brief Destroys a specified anonymous semaphore that is no longer used.
*
* @param sem Indicates the pointer to the semaphore to destroy.
* @return Returns <b>0</b> if the destruction is successful.
* @since 1.0
* @version 1.0
*/
int    sem_destroy(sem_t *sem);

/**
* @brief Obtains the count value of a specified semaphore.
*
* @param sem Indicates the pointer to the semaphore.
* @param sval Indicates the pointer to the obtained count value.
* @return Returns <b>0</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
int    sem_getvalue(sem_t *__restrict sem, int *__restrict sval);

/**
* @brief Creates and initializes an anonymous semaphore.
*
* @param sem Indicates the pointer to the semaphore to be created or initialized.
* @param pshared Specifies whether the semaphore can be shared between the threads of a process. <b>0</b> indicates that the semaphore is shared between the threads of a process. A non-zero value indicates that the semaphore can be shared between processes.
* @param value Indicates the initial count value of the new semaphore.
* @return Returns <b>0</b> if the initialization is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the initialization fails.
* errno | Description
* -------|------------------------------------------
* EINVAL | The input parameters are invalid and the initial count value exceeds the limit.
* @since 1.0
* @version 1.0
*/
int    sem_init(sem_t *sem, int pshared, unsigned int value);

/**
* @brief Increments the semaphore count by 1.
*
* Generally, this function is used to increment the count of available shared resources when the process is to leave its critical section.
* This function is used together with {@link sem_wait()} to protect shared resources. \n
* @param sem Indicates the pointer to the semaphore to be operated.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|------------------------------------------
* EOVERFLOW | The number of semaphores exceeds the maximum number.
* @since 1.0
* @version 1.0
*/
int    sem_post(sem_t *sem);

/**
* @brief Obtains the semaphore, with a timeout period specified.
*
* If the semaphore count is greater than <b>0</b>, the shared resources can be obtained and the semaphore count decreases by 1.
* After the function returns <b>0</b>, the code execution enters the resource critical area.
* Otherwise, the shared resource is unavailable, the function keeps waiting, and code execution cannot enter the resource critical area. If no semaphore is obtained within the specified timeout period, <b>-1</b> is returned.
* This function is used together with {@link sem_post()} to protect shared resources. \n
* @param sem Indicates the pointer to the semaphore to be operated.
* @param timeout Indicates the timeout period to be set.
* @return Returns <b>0</b> if the semaphore is obtained; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the semaphore fails to be obtained.
* errno | Description
* ----------|------------------------------------------
* EINVAL | <b>sem</b> or <b>timeout</b> is invalid.
* EINTR | The waiting for the semaphore is interrupted.
* EAGAIN | The semaphore is set to non-blocking mode, and the semaphore count is <b>0</b>.
* ETIMEDOUT | Waiting for the semaphore timed out.
* @since 1.0
* @version 1.0
*/
int    sem_timedwait(sem_t *__restrict sem, const struct timespec *__restrict timeout);

/**
* @brief Attempts to obtain the semaphore.
*
* If the semaphore count is greater than <b>0</b>, the shared resources can be obtained and the semaphore count decreases by 1.
* After the function returns <b>0</b>, the code execution enters the resource critical area.
* Otherwise, the shared resource is unavailable, and the function returns <b>-1</b>. This function is used together with {@link sem_post()} to protect shared resources. \n
* @param sem Indicates the pointer to the semaphore to be operated.
* @return Returns <b>0</b> if the semaphore is obtained; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the semaphore fails to be obtained.
* errno | Description
* -------|------------------------------------------
* EINVAL | <b>sem</b> is invalid.
* EAGAIN | The semaphore is set to non-blocking mode, and the semaphore count is <b>0</b>.
* @since 1.0
* @version 1.0
*/
int    sem_trywait(sem_t *sem);

/**
* @brief Deletes a specified semaphore.
*
* This function is used to subtract 1 from the open count of the semaphore. If the open count is <b>0</b>, this function deletes the semaphore. \n
* @param name Indicates the name of the semaphore to be deleted.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------|------------------------------------------
* ENAMETOOLONG | <b>name</b> is too long.
* ENOENT | There is no semaphore with the given name.
* @since 1.0
* @version 1.0
*/
int    sem_unlink(const char *name);

/**
* @brief Obtains the semaphore.
*
* If the semaphore count is greater than <b>0</b>, the shared resources can be obtained and the semaphore count decreases by 1.
* After the function returns <b>0</b>, the code execution enters the resource critical area.
* Otherwise, the shared resource is unavailable, the function keeps waiting, and code execution cannot enter the resource critical area.
* This function is used together with {@link sem_post()} to protect shared resources. \n
* @param sem Indicates the pointer to the semaphore to be obtained.
* @return Returns <b>0</b> if the semaphore is obtained; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the semaphore fails to be obtained.
* errno | Description
* -------|------------------------------------------
* EINVAL | <b>sem</b> is invalid.
* EINTR | The waiting for the semaphore is interrupted.
* EAGAIN | The semaphore is set to non-blocking mode, and the semaphore count is <b>0</b>.
* @since 1.0
* @version 1.0
*/
int    sem_wait(sem_t *sem);

#ifdef __cplusplus
}
#endif
#endif
/** @} */