/**
* @addtogroup IPC
* @{
*
* @brief Provides functions and structures related to inter-process communication (IPC), including signals, semaphores, and message queues.
*
* @since 1.0
* @version 1.0
*/

/**
* @file signal.h
*
* @brief Provides signal operation functions and structures, including changing the signal processing mode and the process signal mask set.
*
* @since 1.0
* @version 1.0
*/

#ifndef _SIGNAL_H
#define _SIGNAL_H
#ifdef __cplusplus
extern"C" {
#endif

/**
* @brief Describes a signal stack.
*/
struct sigaltstack {
/** Base address of the signal stack */
void *ss_sp;
/** Flag */
int ss_flags;
/** Number of bytes in the signal stack */
size_t ss_size;
};

/**
* @brief Describes the user execution context.
*/
typedef struct __ucontext {
/** Flag */
unsigned long uc_flags;
/** Context linked-list */
struct __ucontext *uc_link;
/** User stack */
stack_t uc_stack;
/** Related signal context */
mcontext_t uc_mcontext;
/** User signal mask */
sigset_t uc_sigmask;
/** User register space */
unsigned long long uc_regspace[64];
} ucontext_t;

/**@brief Stops a child process without sending the {@link SIGCHLD} signal. */
#define SA_NOCLDSTOP  1
/**@brief Sets the {@link SIGCHLD} flag to prevent zombie processes. */
#define SA_NOCLDWAIT  2
/**@brief Transfers the {@link siginfo_t} structure with the signal. */
#define SA_SIGINFO    4
/**@brief Uses the signal stack specified by {@link sigaltstack} for signal processing. */
#define SA_ONSTACK    0x08000000
/**@brief Obtains the restart signal. */
#define SA_RESTART    0x10000000
/**@brief The signals being processed are not masked during signal processing. */
#define SA_NODEFER    0x40000000
/**@brief The default processing mode is restored after the signal processing is complete. */
#define SA_RESETHAND  0x80000000
#define SA_RESTORER   0x04000000

/**@brief Defines the minimum size of the replaceable signal stack. */
#define MINSIGSTKSZ	 2048
/**@brief Defines the size of a signal stack. */
#define SIGSTKSZ	 8192
/**@brief Number of supported signals */
#define _NSIG		 65

/**@brief Number of bits occupied by a word. */
#define _NSIG_BPW	 __BITS_PER_LONG

/**@brief Number of words required by all signals*/
#define _NSIG_WORDS	 (_NSIG / _NSIG_BPW)


/**@brief Hangs up a process. */
#define SIGHUP		 1
/**@brief Interrupts a process from keyboard. */
#define SIGINT		 2
/**@brief Exits a process from keyboard. A core file is generated. */
#define SIGQUIT		 3
/**@brief Illegal instruction */
#define SIGILL		 4
/**@brief Breakpoint, used for debugging */
#define SIGTRAP		 5
/**@brief Abnormal termination */
#define SIGABRT		 6
/**@brief Equivalent to {@link SIGABRT} */
#define SIGIOT		 6
/**@brief Invalid address access */
#define SIGBUS		 7
/**@brief Floating-point exceptions */
#define SIGFPE		 8
/**@brief Forcibly stops a process. */
#define SIGKILL		 9
/**@brief User-defined signal 1 */
#define SIGUSR1		 10
/**@brief Illegal memory access */
#define SIGSEGV		 11
/**@brief User-defined signal 2 */
#define SIGUSR2		 12
/**@brief The pipe is broken. */
#define SIGPIPE		 13
/**@brief Timer signal */
#define SIGALRM		 14
/**@brief Termination signal */
#define SIGTERM		 15
/**@brief Coprocessor stack error */
#define SIGSTKFLT	 16
/**@brief Child process terminated or stopped */
#define SIGCHLD		 17
/**@brief Resumes a suspended process. */
#define SIGCONT		 18
/**@brief Suspends a process. */
#define SIGSTOP		 19
/**@brief Stops a process by entering the command on the terminal. */
#define SIGTSTP		 20
/**@brief Input required by the background process. */
#define SIGTTIN		 21
/**@brief Output required by the background process */
#define SIGTTOU		 22
/**@brief Urgent socket condition */
#define SIGURG		 23
/**@brief CPU time limit exceeded */
#define SIGXCPU		 24
/**@brief File size limit exceeded */
#define SIGXFSZ		 25
/**@brief Virtual timer, used to calculate the CPU occupation time of a process */
#define SIGVTALRM	 26
/**@brief Calculates the CPU time occupied by a process and the system calling time. */
#define SIGPROF		 27
/**@brief Window size change */
#define SIGWINCH	 28
/**@brief Input/Output */
#define SIGIO		 29
/**@brief Equivalent to {@link SIGIO} */
#define SIGPOLL		 SIGIO
/*
#define SIGLOST		 29
*/
/**@brief Power failure */
#define SIGPWR		 30
/**@brief Non-existent system call */
#define SIGSYS		 31
/**@brief Equivalent to {@link SIGSYS} */
#define	 SIGUNUSED	 31

/* These should not be considered constants from userland.  */
/**@brief Start of a reliable signal */
#define SIGRTMIN	 32
/**@brief End of a reliable signal */
#define SIGRTMAX	 _NSIG

/**@brief {@link SIGCHLD} is not generated when the child process is stopped. */
#define SA_NOCLDSTOP	 0x00000001
/**@brief Sets the {@link SIGCHLD} flag to prevent zombie processes. */
#define SA_NOCLDWAIT	 0x00000002
/**@brief Transfers the {@link siginfo_t} with the signal. */
#define SA_SIGINFO	 0x00000004
/**@brief Transfers signals in a 32-bit mode even if the task runs in a 26-bit mode */
#define SA_THIRTYTWO	 0x02000000
/**@brief Not intended for application use. This flag is used by C libraries to indicate that the <b>sa_restorer</b> field contains the address of a <b>trampoline</b>. */
#define SA_RESTORER	 0x04000000
/**@brief The signal stack specified by {@link sigaltstack} is used for signal processing. */
#define SA_ONSTACK	 0x08000000
/**@brief Obtains the restart signal. */
#define SA_RESTART	 0x10000000
/**@brief Signals being processed are not masked during signal processing. */
#define SA_NODEFER	 0x40000000
/**@brief The default processing mode is restored after the signal processing is complete. */
#define SA_RESETHAND	 0x80000000
/**@brief Signals being processed are not masked during signal processing. */
#define SA_NOMASK	 SA_NODEFER
/**@brief The default processing mode is restored after the signal processing is complete. */
#define SA_ONESHOT	 SA_RESETHAND

/**@brief Minimum size of the replaceable signal stack */
#define MINSIGSTKSZ	 2048
/**@brief Size of a signal stack */
#define SIGSTKSZ	 8192

/**@brief Defines the function behaviors such as {@link sigprocmask()} and {@link pthread_sigmask()}, which are used to add signals to the mask set. */
#define SIG_BLOCK     0
/**@brief Defines the function behaviors such as {@link sigprocmask()} and {@link pthread_sigmask()}, which are used to remove signals from the mask set. */
#define SIG_UNBLOCK   1
/**@brief Defines the function behaviors such as {@link sigprocmask()} and {@link pthread_sigmask()}, which are used to set the mask set. */
#define SIG_SETMASK   2

/**
* @brief Defines a signal set.
*/
struct __sigset_t {
/** Saves signals in the signal set. */
unsigned long __bits[128/sizeof(long)];
} sigset_t;

/**
* @brief Defines the signal processing actions and related attributes. You can use the structures to change the processing actions or attributes.
*/
struct sigaction {
/** Defines the callbacks for signal processing. You can use either of them based on the specific scenario. */
union {
/** Callback for the processing action */
__sighandler_t _sa_handler;
/** Callback with signal information for processing actions */
void (*_sa_sigaction)(int, struct siginfo *, void *);
} _u;
/** Signal to be masked */
sigset_t sa_mask;
/** Signal flag. Only <b>SA_SIGINFO</b> is supported. */
unsigned long sa_flags;
void (*sa_restorer)(void);
};

/** Indicates the <b>_u._sa_handler</b> field in {@link sigaction}. */
#define sa_handler	 _u._sa_handler
/** Indicates the <b>__sa_handler.sa_sigaction</b> field in {@link sigaction}. */
#define sa_sigaction	 _u._sa_sigaction

/**
* @brief Describes a signal stack.
*/
typedef struct sigaltstack {
/** Base address of the signal stack */
void *ss_sp;
/** Flag */
int ss_flags;
/** Number of bytes in the signal stack */
size_t ss_size;
} stack_t;


/**@brief Describes stack information for signal processing. */
typedef struct sigaltstack stack_t;


/**@brief Defines the function behaviors such as {@link sigset()} and adds a specified signal to the process signal mask without changing the current processing mode of the signal. */
#define SIG_HOLD ((void (*)(int)) 2)

/**
* @brief Describes signal values in different cases.
*
*/
union sigval {
/** Integer signal value */
int sival_int;
/** Pointer signal value */
void *sival_ptr;
};
/**
* @brief Describes signal information.
*
*/
typedef struct {
#ifdef __SI_SWAP_ERRNO_CODE
/** Signal number */
int si_signo;
/** Cause of signal generation, which is not supported currently */
int si_code;
/** Error code, which is not supported currently */
int si_errno;
#else
/** Signal number */
int si_signo;
/** Error code, which is not supported currently */
int si_errno;
/** Cause of signal generation, which is not supported currently */
int si_code;
#endif
union {
/** Alignment fields */
char__pad[128 - 2*sizeof(int) - sizeof(long)];
struct {
union {
/** This structure is not supported. */
struct {
pid_t si_pid;
uid_t si_uid;
} __piduid;
/** This structure is not supported. */
struct {
int si_timerid;
int si_overrun;
} __timer;
} __first;
union {
/** <b>sival_int</b> in the union is supported. */
union sigval si_value;
/** This structure is not supported. */
struct {
int si_status;
clock_t si_utime, si_stime;
} __sigchld;
} __second;
} __si_common;
} __si_fields;
} __si_fields;
} siginfo_t;

/**
* @brief Describes the signal processing actions and related attributes.
*
*/
struct sigaction {
union {
/** Signal callback function type and callback for the processing action */
void (*sa_handler)(int);
/** Signal callback function type, which is not supported */
void (*sa_sigaction)(int, siginfo_t*, void*);
} __sa_handler;
/** Signal to be masked */
sigset_t sa_mask;
/** Signal flag, which is not supported */
int sa_flags;
void (*sa_restorer)(void);
};
/**@brief Defines the simplified writing used to indicate the <b>__sa_handler.sa_handler</b> field in {@link sigaction}. */
#define sa_handler   __sa_handler.sa_handler
/**@brief Defines the simplified writing used to indicate the <b>__sa_handler.sa_sigaction</b> field in {@link sigaction}. */
#define sa_sigaction __sa_handler.sa_sigaction

/**
* @brief Describes asynchronous notifications.
*
*/
struct sigevent {
/** Data passed with notification */
union sigval sigev_value;
/** Notification signal */
int sigev_signo;
/**Notification method, which can be set to {@link SIGEV_SIGNAL}, {@link SIGEV_NONE}, or {@link SIGEV_THREAD} */
int sigev_notify;
/** Function used for thread notification */
void (*sigev_notify_function)(unionsigval);
/** Attributes for notification thread */
pthread_attr_t *sigev_notify_attributes;
/** A reserved field */
char__pad[56-3*sizeof(long)];
};
/**@brief Sets the notification method in {@link sigevent}: signal notification. */
#define SIGEV_SIGNAL 0
/**@brief Sets the notification method in {@link sigevent}: no notification. */
#define SIGEV_NONE 1
/**@brief Sets the notification method in {@link sigevent}: thread notification. */
#define SIGEV_THREAD 2
int__libc_current_sigrtmin(void);
int__libc_current_sigrtmax(void);
/**@brief Defines the lowest available signal number. */
#define SIGRTMIN  (__libc_current_sigrtmin())
/**@brief Defines the highest available signal number. */
#define SIGRTMAX  (__libc_current_sigrtmax())

/**
* @brief Sends a signal to a specified process.
*
* @attention The default signal behavior does not support <b>STOP</b> and <b>CONTINUE</b>, and <b>COREDUMP</b> is not involved.
* {@link SIGSTOP}, {@link SIGKILL}, and {@link SIGCONT} cannot be masked.
* If a process is killed but its parent process is not recycled, a zombie process will be generated.
* After an asynchronous signal is sent to a process, the signal callback is executed only after the process is scheduled. The kernel cannot forcibly kill the process by using the signal.
* After the process is killed, {@link SIGCHLD} is sent to its parent process. The sending action cannot be canceled. A sleeping process cannot be woken up by a signal. For example, a process invokes the <b>sleep</b> function to enter the sleeping state.
* Currently, suspended and zombie processes cannot be forcibly killed.
* @param pid Indicates the target process ID. Parameters <b>0</b> and <b>-1</b> are not supported.
* @param sig Indicates the signal to be sent.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------------------------
* EINVAL | <b>sig</b> is invalid.
* EPERM | The current process does not have permission to send the signal to the target process.
* @since 1.0
* @version 1.0
*/
int kill(pid_t pid,  int sig);

/**
* @brief Clears all signals in a specified signal set.
*
* @param set Indicates the pointer to the signal set to be cleared.
* @return Returns <b>0</b> if the operation is successful. If <b>set</b> is invalid, the process is abnormal.
* @since 1.0
* @version 1.0
*/
int sigemptyset(sigset_t *set);

/**
* @brief Adds all signals to a specified signal set.
*
* @param set Indicates the pointer to the signal set to hold signals.
* @return Returns <b>0</b> if the operation is successful. If <b>set</b> is invalid, the process is abnormal.
* @since 1.0
* @version 1.0
*/
int sigfillset(sigset_t *set);

/**
* @brief Adds a signal to a specified signal set.
*
* @param set Indicates the pointer to the signal set to hold signals.
* @param signum Indicates the signal to be added.
* @return Returns <b>0</b> if the operation is successful. If <b>set</b> or <b>signum</b> is invalid, the process is abnormal.
* @since 1.0
* @version 1.0
*/
int sigaddset(sigset_t *set, int signum);

/**
* @brief Delete a signal from a specified signal set.
*
* @param set Indicates the pointer to the signal set to hold signals.
* @param signum Indicates the signal to be deleted.
* @return Returns <b>0</b> if the operation is successful. If <b>set</b> or <b>signum</b> is invalid, the process is abnormal.
* @since 1.0
* @version 1.0
*/
int sigdelset(sigset_t *set, int signum);

/**
* @brief Checks whether a signal is in the signal set.
*
* @param set Indicates the pointer to the signal set to be checked.
* @param signum Indicates the signal to be checked.
* @return Returns <b>0</b> if the signal is not in the set; returns <b>1</b> if the signal is in the set. If <b>set</b> or <b>signum</b> is invalid, the process is abnormal.
* @since 1.0
* @version 1.0
*/
int sigismember(const sigset_t *set, int signum);

/**
* @brief Changes the signal mask of the calling thread.
*
* @param how Indicates the function behavior.
* flags | Description
* ------------|--------------------------------------
* SIG_BLOCK | Adds the signals in <b>set</b> to the mask set of the thread.
* SIG_UNBLOCK | Removes the signals in <b>set</b> from the mask set of the thread.
* SIG_SETMASK | Sets <b>set</b> of blocked signals as the mask set of the thread.
* @param set Indicates the pointer to the signal set to be changed.
* @param oldset Indicates the pointer to the original mask set of the calling thread.
* @return Returns <b>0</b> if the change is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the change fails.
* errno | Description
* -------|------------------------------------------
* EINVAL | <b>how</b> is invalid.
* EFAULT | The <b>set</b> or <b>oldset</b> points outside the process's allocated address space.
* @since 1.0
* @version 1.0
*/
int sigprocmask(int how, const sigset_t *__restrict set, sigset_t *__restrict oldset);

/**
* @brief Sets <b>mask</b> as the signal mask set of the current process and suspends the process until a signal processing action is triggered.
*
* @param mask Indicates the pointer to the signal set to be set.
* @return Returns <b>-1</b> and sets <b>errno</b> to a value in the following table.
* errno | Description
* -------|------------------------------------
* EFAULT | <b>mask</b> points to an invalid space.
* EINTR | The suspension wait is interrupted by the signal.
* EINVAL | <b>mask</b> is <b>NULL</b>.
* @since 1.0
* @version 1.0
*/
int sigsuspend(const sigset_t *mask);

/**
* @brief Checks or changes the processing action associated with a specified signal. Do not use the process creation function or thread creation function in the registered callback processing function. Otherwise, the process is abnormal. For example, do not use the <b>fork</b> or <b>pthread_create</b> function.
*
* @attention The <b>sa_flag</b> field in <b>act</b> supports the {@link SA_SIGINFO} flag only.
* @param signum Indicates the signal to be operated.
* @param act Indicates the new signal processing action.
* @param oldact Indicates the original signal processing action if the value is not <b>NULL</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------
* EINVAL | <b>signum</b> is invalid or cannot be captured.
* ^ | <b>act</b> is <b>NULL</b>.
* ENOMEM | Insufficient memory.
*
* @since 1.0
* @version 1.0
*/
int sigaction(int signum, const struct sigaction *__restrict act, struct sigaction *__restrict oldact);

/**
* @brief Obtains the signal suspended by the current thread.
*
* @param set Indicates the pointer to the signal set used to store the signal suspended by the current thread.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|--------------------------------
* EFAULT | <b>set</b> points to an invalid space.
* EINVAL | <b>set</b> is <b>NULL</b>.
* EINTR | The suspension wait is interrupted by the signal.
* @since 1.0
* @version 1.0
*/
int sigpending(sigset_t *set);

/**
* @brief Suspends the calling thread and waits for the signal.
*
* This function is used to suspend execution of the calling thread until one of the signals specified in the signal set becomes pending. \n
* @attention The signal to be waited must be in the mask set of the current thread. Once the signal is received, it is deleted from the pending list.
* @param set Indicates the pointer to the signal set to be waited.
* @param sig Used for saving the waited signal.
* @return Returns <b>0</b> if the signal is received; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the signal fails to be received.
* errno | Description
* -------|------------------------------
* EINTR | The suspension wait is interrupted by the signal.
* @since 1.0
* @version 1.0
*/
int sigwait(const sigset_t *__restrict set, int *__restrict sig);

/**
* @brief Suspends the calling thread and waits for the signal, with a timeout period specified.
*
* This function is used to suspend execution of the calling thread until one of the signals specified in the signal set becomes pending. This function does not specify the timeout behavior. \n
* @attention The signal to be waited must be in the mask set of the current thread.
* Once the signal is received, it is deleted from the pending list. The waiting for the signal cannot be interrupted by other signals, that is, <b>EINTR</b> is not supported.
* @param set Indicates the pointer to the signal set to be waited.
* @param info Used to save the information about the waited signal if the value is not <b>NULL</b>.
* @return Returns the value of the waited signal if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|--------------------------------
* @since 1.0
* @version 1.0
*/
int sigwaitinfo(const sigset_t *__restrict set, siginfo_t *__restrict info);

/**
* @brief Suspends the calling thread and waits for the signal, with a timeout period specified.
*
* This function is used to suspend execution of the calling thread until one of the signals specified in the signal set becomes pending. This function specifies the timeout behavior. \n
* @attention The signal to be waited must be in the mask set of the current thread.
* Once the signal is received, it is deleted from the pending list. The waiting for the signal cannot be interrupted by other signals, that is, <b>EINTR</b> is not supported.
* @param set Indicates the pointer to the signal set to be waited.
* @param info Used to save the information about the waited signal if the value is not <b>NULL</b>.
* @param timeout Indicates the timeout duration.
* @return Returns the value of the waited signal if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|--------------------------------
* EAGAIN | No signal specified by <b>set</b> is generated within the specified timeout period.
* EINVAL | <b>timeout</b> is invalid.
* @since 1.0
* @version 1.0
*/
int sigtimedwait(const sigset_t *__restrict set, siginfo_t *__restrict info, const struct timespec *__restrict timeout);

/**
* @brief Checks or changes the signal mask of the calling thread.
*
* @param how Indicates the function behavior.
* flags | Description
* ------------|--------------------------------------
* SIG_BLOCK | Adds the signals in <b>set</b> to the mask set of the thread.
* SIG_UNBLOCK | Removes the signals in <b>set</b> from the mask set of the thread.
* SIG_SETMASK | Sets <b>set</b> of blocked signals as the mask set of the thread.
* @param set Indicates the signal set to be changed.
* @param oldset Used for returning the original mask set of the calling thread.
* @return Returns <b>0</b> if the change is successful; returns <b>errno</b> as described below if the change fails.
* errno | Description
* -------|------------------------------------------
* EINVAL | <b>how</b> is invalid.
* EFAULT | The <b>set</b> or <b>oldset</b> points outside the process's allocated address space.
* @since 1.0
* @version 1.0
*/
int pthread_sigmask(int how, const sigset_t *__restrict set, sigset_t *__restrict oldset);

/**
* @brief Prints signal information.
*
* This function is used to print a message on the standard error output. The message consists of the character string <b>s</b>, a colon, a space, a string describing the signal information <b>signum</b>, and a newline character.
* If the string <b>s</b> is <b>NULL</b> or empty, the colon and space are omitted. If <b>signum</b> is invalid, the message displayed will indicate an unknown signal. \n
* @param signum Indicates the signal to be operated.
* @param s Indicates the customized string <b>s</b> to be printed.
* @since 1.0
* @version 1.0
*/
void psignal(int signum, const char *s);

/**
* @brief Deletes the signal specified by <b>signum</b> from the signal mask of the calling process and suspends the process until the signal is received.
*
* @attention After the signal is received, the original signal mask of the process is restored.
* @param signum Indicates the signal to be operated.
* @return Returns <b>-1</b> and sets <b>errno</b> to a value in the following table.
* errno | Description
* -------|-----------------
* EINTR | The suspension wait is interrupted by the signal.
* @since 1.0
* @version 1.0
*/
int sigpause(int signum);

/**
* @brief Adds the signal specified by <b>signum</b> to the signal mask of the calling process.
*
* @param signum Indicates the signal to be added.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
int sighold(int signum);

/**
* @brief Deletes the signal specified by <b>signum</b> from the signal mask of the calling process.
*
* @param signum Indicates the signal to be deleted.
* @return Returns <b>0</b> if the deletion is successful; returns <b>-1</b> if the deletion fails.
* @since 1.0
* @version 1.0
*/
int sigrelse(int signum);

/**
* @brief Checks whether the signal set is empty.
*
* @param set Indicates the pointer to the signal set to be checked.
* @return Returns <b>0</b> if the set is not empty; returns <b>1</b> if the set is empty; returns <b>-1</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
int sigisemptyset(const sigset_t *set);

/**
* @brief Adds the union set of <b>left</b> and <b>right</b> to <b>dest</b>.
*
* @param dest Indicates the pointer to the target signal set.
* @param left Indicates the <b>left</b> set to be added.
* @param right Indicates the <b>right</b> set to be added.
* @return Returns <b>0</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
int sigorset(sigset_t *dest, const sigset_t *left, const sigset_t *right);

/**
* @brief Adds the intersection of <b>left</b> and <b>right</b> to <b>dest</b>.
*
* @param dest Indicates the pointer to the target signal set.
* @param left Indicates the <b>left</b> set to be added.
* @param right Indicates the <b>right</b> set to be added.
* @return Returns <b>0</b> if the operation is successful.
* @since 1.0
* @version 1.0
*/
int sigandset(sigset_t *dest, const sigset_t *left, const sigset_t *right);

/**@brief Function pointer of signal handler */
typedef void (*sighandler_t)(int);

/**@brief Defines the value returned when a function such as {@link signal()} fails. */
#define SIG_ERR  ((__sighandler_t)-1)
/**@brief Defines the default signal processing mode. */
#define SIG_DFL	 ((__sighandler_t)0)
/**@brief Defines the signal processing mode in which the signal is ignored. */
#define SIG_IGN	 ((__sighandler_t)1)

/**
* @brief Changes the processing action of a specified signal. Do not use the process creation function or thread creation function in the registered callback processing function. Otherwise, the process is abnormal. For example, do not use the <b>fork</b> or <b>pthread_create</b> function.
*
* @param signum Indicates the signal to be operated.
* @param handler Indicates the pointer to the specified processing action ({@link SIG_IGN} or {@link SIG_DFL}, or to a custom processing action.
* @return Returns the pointer to the original processing action if the operation is successful; returns {@link SIG_ERR} and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|------------------
* EINVAL | <b>signum</b> is invalid.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
sighandler_t signal(int signum, sighandler_t handler);


/**
* @brief Sends a signal to the calling process.
*
* @param signum Indicates the signal to be sent.
* @return Returns 0 if the operation is successful; returns nonzero if the operation fails.
* @since 1.0
* @version 1.0
*/
int raise(int signum);

#ifdef __cplusplus
}
#endif

#endif/** @} */