/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file stdarg.h
*
* @brief Provides functions and data structures, such as applying for and ending a variable-length argument list and obtaining an argument type.
*
*
*
* @since 1.0
* @version 1.0
*/

#ifndef _STDARG_H
#define _STDARG_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Defines the start position of the variable-length argument list.
*
* This function initializes <b>v</b> for subsequent use by {@link va_arg} and {@link va_end}, and must be called first. \n
* @attention This function must be used together with {@link va_end}.
* @param v Indicates the argument to be initialized.
* @param l Indicates the name of the last argument before the variable argument list, that is, the last argument of which the calling function knows the type.
* @since  1.0
* @version  1.0
*/
#define va_start(v,l)   __builtin_va_start(v,l)

/**
* @brief Ends a variable-length argument list.
*
* This function releases a calling function. \n
* @attention After <b>va_end</b> is called, <b>v</b> is a random value.
* @param v Indicates the <b>va_list v</b> initialized by <b>va_start()</b>.
* @since  1.0
* @version  1.0
*/
#define va_end(v)       __builtin_va_end(v)

/**
* @brief Obtains the next argument in the variable-length argument list.
*
* @attention <b>va_start</b> must be called first.
* @param v Indicates the <b>va_list v</b> initialized by <b>va_start()</b>. Each call to <b>va_arg()</b> modifies <b>v</b> so that the next call returns the next argument.
* @param l Indicates the name of the last argument before the variable argument list, that is, the last argument of which the calling function knows the type.
* @since  1.0
* @version  1.0
*/
#define va_arg(v,l)     __builtin_va_arg(v,l)

/**
* @brief Copies the previously initialized variable argument list <b>s</b> to <b>d</b>.
*
* @param d Indicates the destination variable.
* @param s Indicates the source variable.
* @since  1.0
* @version  1.0
*/
#define va_copy(d,s)    __builtin_va_copy(d,s)

#ifdef __cplusplus
}
#endif

#endif
/** @} */