/**
* @addtogroup IO
* @{
*
* @brief Provides data structures and functions for performing input/output (I/O) operations, such as printf and scanf.
*
* @since 1.0
* @version 1.0
*/

/**
* @file stdio.h
*
* @brief Declares common functions used for performing input and output (I/O) operations.
*
* You can use the functions provided in this file to perform the I/O operations required during development. Distinguished by I/O targets, there are standard I/O, file stream I/O, and string I/O. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _STDIO_H
#define _STDIO_H

#ifdef __cplusplus
extern "C" {
#endif

/**@brief Struct of file stream */
typedef struct _IO_FILE FILE;

/**
* @brief Opens the file stored in the specified path based on the given mode.
*
* @attention This function can NOT be used in the PROC file system to create a file.
* @param path Indicates the pointer to the path of the file to open.
* @param mode Indicates the pointer to the file access mode. The following table describes the available values.
* mode | Description
* ----- | ------------------------------------------------------------------------------------------
* "r" | Open a file in read-only mode. The file must exist.
* "w" | Open a file in write-only mode. The file is created if it does not exist. If the file already exists, its existing contents will be cleared.
* "a" | Open a write-only file in append mode. The file is created if it does not exist. If the file already exists, new contents will be written at the end of the file.
* "r+" | Open a file in read and write mode. The file must exist.
* "w+" | Open a file for both reading and writing. The file is created if it does not exist. If the file already exists, its contents are cleared.
* "a+" | Open a file in append mode for both reading and writing. The file is created if it does not exist. If the file already exists, new contents will be written at the end of the file.
* "rb" | Open a binary file in read-only mode.
* "wb" | Open or create a binary file in write-only mode.
* "ab" | Open a binary file in append mode and write data at the end of the file.
* "rb+" | Open a binary file in read and write mode. The file must exist.
* "wb+" | Open or create a binary file in read and write mode.
* "ab+" | Open a binary file in append mode for wring data at the end of the file. The file is also readable.
* "rt" | Open a text file in read-only mode.
* "wt" | Open or create a text file in write-only mode.
* "at" | Open a text file in append mode and write data at the end of the file.
* "rt+" | Open a text file in read and write mode.
* "wt+" | Open or create a text file in read and write mode.
* "at+" | Open a text file in append mode for writing data at the end of the file. The file is also readable.
* @return Returns the pointer to the file if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ |------------------------
* EINVAL | Invalid mode.
* EMFILE | The number of opened files exceeds the maximum number.
* @since 1.0
* @version 1.0
*/
FILE *fopen(const char *__restrict path, const char *__restrict mode);

/**
* @brief Closes a file based on the specified file descriptor.
*
* @param fd Indicates the pointer to the file descriptor of the file to close.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -------------
* EINVAL | <b>fd</b> is <b>NULL</b>.
* @since 1.0
* @version 1.0
*/
int fclose(FILE *fd);

/**
* @brief Deletes a specified file or directory from the file system.
*
* @attention This function can NOT be used in the PROC file system.
* @param pathname Indicates the pointer to the file or directory to delete.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------------ | -------------------------------------------------------------
* EINVAL | <b>pathname</b> is a null pointer or an empty string.
* ENAMETOOLONG | The length of <b>pathname</b> is greater than {@link NAME_MAX}.
* ENOMEM | Insufficient memory.
* ENOENT | The file or directory pointed to by <b>pathname</b> does not exist.
* ENOSYS | The file system does not support this operation.
* EPERM | The path is a mount point or is neither a directory nor a file.
* ENOTEMPTY | When <b>pathname</b> points to a directory, the directory is not empty.
* ENXIO | No such device or address.
* EACCES | The search permission on the path prefix component is denied, or the write permission of the directory in the path is denied.
* @since 1.0
* @version 1.0
*/
int remove(const char *pathname);

/**
* @brief Renames a specified file.
*
* @attention This function can NOT be used in the PROC file system.
* @param oldpath Indicates the pointer to the old path name of the file to rename.
* @param newpath Indicates the pointer to the new path name of the file.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -----------------
* EINVAL | Invalid input parameter.
* ENOENT | The file or directory does not exist.
* EXDEV | <b>oldpath</b> and <b>newpath</b> are not on the same file system.
* ENOSYS | The function is unavailable.
* EBUSY | The device or resource is in use by the system or another process.
* EEXIST | The file path pointed to by <b>newpath</b> already exists.
* ENXIO | The device or address does not exist.
* @since 1.0
* @version 1.0
*/
int rename(const char *oldpath, const char *newpath);

/**
* @brief Tests the end-of-file indicator for a specified stream.
*
* @param fp Indicates the pointer to the file object that identifies a stream to test.
* @return Returns a non-zero value if the end-of-file indicator is set; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int feof(FILE *fp);

/**
* @brief Tests for the error identifier of a specified file stream.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns a non-zero value to indicate an error for the stream if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int ferror(FILE *stream);

/**
* @brief Flushes the output buffer of a specified stream.
*
* @param fp Indicates the pointer to the file object that identifies a stream to flush.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------
* EBADF | <b>fp</b> points to an invalid stream.
* @since 1.0
* @version 1.0
*/
int fflush(FILE *fp);

/**
* @brief Resets the error indicators.
*
* This function is used to set both the file error indicator and end-of-file indicator to <b>0</b>. \n
* @param fp Indicates the pointer to the file object that identifies a stream to set.
* @since 1.0
* @version 1.0
*/
void clearerr(FILE *fp);

/**
* @brief Changes the position of a specified stream to a new position.
*
* @param fp Indicates the pointer to the file object that identifies a stream for which the position is to set.
* @param offset Indicates the offset that determines the position.
* @param whence Indicates the start position of the offset. The following table describes the available values.
* value    | Description
* -------- | ---------
* SEEK_SET | Beginning of the file
* SEEK_CUR | Current position
* SEEK_END | End of the file
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | --------------------------------------------
* EBADF | <b>fp</b> points to an invalid stream.
* EINVAL | The value of <b>whence</b> is not <b>SEEK_SET</b>, <b>SEEK_END</b>, or <b>SEEK_CUR</b>.
* ESPIPE | Invalid seek.
* @since 1.0
* @version 1.0
*/
int fseek(FILE *fp, long offset, int whence);

/**
* @brief Obtains the current value of the file position indicator for a specified file stream.
*
* @param fp Indicates the pointer to the file object that identifies a stream for which the position is to obtain.
* @return Returns the current offset if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----- | --------------
* EBADF | <b>fp</b> points to an invalid stream.
* @since 1.0
* @version 1.0
*/
long ftell(FILE *fp);

/**
* @brief Sets the position indicator of a specified stream to the beginning of the file.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @since 1.0
* @version 1.0
*/
void rewind(FILE *stream);

/**
* @brief Obtains the current file position of a specified stream and writes it to the <b>pos</b> parameter.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param pos Indicates the pointer to the <b>fpos_t</b> object.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int fgetpos(FILE *stream, fpos_t *pos);

/**
* @brief Sets the file position indicator for a specified file stream.
*
* @param fp Indicates the pointer to the file object that identifies a stream for which the position is to set.
* @param pos Indicates the pointer to the position to set.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------
* EBADF | <b>fp</b> points to an invalid stream.
* ESPIPE | Invalid seek.
* @since 1.0
* @version 1.0
*/
int fsetpos(FILE *fp, const fpos_t *pos);

/**
* @brief Reads data from a specified stream.
*
* @param ptr Indicates the pointer to the array for storing the read data.
* @param size Indicates the size of each element to read, in bytes.
* @param nmemb Indicates the number of elements to read.
* @param stream Indicates the pointer to the file object that identifies a stream to read.
* @return Returns <b>0</b> if <b>size</b> or <b>nmemb</b> is <b>0</b>; returns the number of elements successfully read otherwise.
* This function sets <b>errno</b> to a value in the following table if a reading error occurs or the end-of-file is reached (<b>fread</b> does not distinguish the two error reporting situations, and you can call <b>feof</b> and <b>ferror</b> to determine the specific situation).
* errno | Description
* ----- | -------------------
* EBADF | <b>stream</b> points to an invalid stream.
* EEOF | The end-of-file is reached.
* @since 1.0
* @version 1.0
*/
size_t fread(void *__restrict ptr, size_t size, size_t nmemb, FILE *__restrict stream);

/**
* @brief Writes data to a specified stream.
*
* @param ptr Indicates the pointer to the data to write.
* @param size Indicates the size of each element to write, in bytes.
* @param nmemb Indicates the number of elements to write.
* @param stream Indicates the pointer to the file object that identifies a stream to which data is to write.
* @return Returns <b>0</b> if <b>size</b> or <b>nmemb</b> is <b>0</b>; returns the number of elements successfully written otherwise. If this number is not <b>nmemb</b>, this function sets <b>errno</b> to a value in the following table if the number is not <b>nmemb</b>.
* errno | Description
* ----- | -------------------
* EBADF | <b>stream</b> points to an invalid stream.
* @since 1.0
* @version 1.0
*/
size_t fwrite(const void *__restrict ptr, size_t size, size_t nmemb, FILE *__restrict stream);

/**
* @brief Reads a character from a specified file stream.
*
* @attention The difference between this function and <b>fgetc</b> is that this function can be implemented as a macro.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the character read as an int value if the operation is successful; returns <b>EOF</b> if the end-of-file is reached or a reading error occurs.
* @since 1.0
* @version 1.0
*/
int getc(FILE *stream);

/**
* @brief Obtains the next character from the standard input (stdin).
*
* This function waits for user input and does not return until any input is available. \n
* @return Returns the character read as an int value if the operation is successful; returns <b>EOF</b> if the end-of-file is reached or a reading error occurs.
* @since  1.0
* @version  1.0
*/
int getchar(void);

/**
* @brief Pushes a character back into a specified file stream.
*
* This function is usually used together with <b>getc</b>. \n
* @attention The pushed characters are returned in reverse order if multiple characters are pushed back. Only one pushback is guaranteed.
* @param c Indicates the int promotion of the character to be pushed back.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the character specified in <b>c</b> if the operation is successful; returns <b>EOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
int ungetc(int c, FILE *stream);

/**
* @brief Writes a character into the position pointed to by a specified stream.
*
* @param c Indicates the int promotion of the character to write.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the ASCII code of the character written if the operation is successful; returns <b>EOF</b> otherwise.
* @since  1.0
* @version  1.0
*/
int fputc(int c, FILE *stream);

/**
* @brief Writes a character to a specified file stream.
*
*
* @attention
* @param c Indicates the int promotion of the character to write.
* @param stream Indicates the pointer to the file stream.
* @return Returns the ASCII code of the character written if the operation is successful; returns <b>EOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
int putc(int c, FILE *stream);

/**
* @brief Writes a character into the standard output (stdout).
*
* @param c Indicates the int promotion of the character to write.
* @return Returns the ASCII code of the character written if the operation is successful; returns <b>EOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
int putchar(int c);

/**
* @brief Reads a line of characters from a specified file stream and stores it into the string pointed to by <b>s</b>.
*
* @param s Indicates the pointer to the character array for storing the obtained string.
* @param size Indicates the maximum number of characters to read (including the terminating null character). The value of this parameter is usually the length of the array pointed to by <b>s</b>.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the pointer to the read string if the operation is successful; returns a null pointer if the end-of-file is reached, no characters have been read, or if an error occurs.
* @since  1.0
* @version  1.0
*/
char *fgets(char *s, int size, FILE *stream);

/**
* @brief Reads a line of characters from the stdin and stores it in the buffer.
*
* @attention The <b>gets()</b> function cannot predict the size of the line to be read, which may cause buffer overflow. You are recommended not to use this function. POSIX.1-2008 and ISO C11 have marked this function as obsolete.
* @param s Indicates the pointer to the memory buffer for storing the obtained string.
* @return Returns the value of <b>s</b> if the operation is successful; returns <b>NULL</b> otherwise.
* @since 1.0
* @version 1.0
*/
char *gets(char *s);

/**
* @brief Writes a character string (excluding the terminating null character) to a specified file stream.
*
* @param s Indicates the pointer to the character string to write.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns a non-negative value if the operation is successful; returns <b>EOF</b> otherwise.
* @since  1.0
* @version  1.0
*/
int fputs(const char *s, FILE *stream);

/**
* @brief Writes a character string and appends a new-line character to the stdout.
*
* @param s Indicates the pointer to the start address of the output buffer.
* @return Returns a non-negative value if the operation is successful; returns <b>EOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
int puts(const char *s);

/**
* @brief Formats arguments based on <b>format</b> and prints formatted data to the stdout.
*
* @attention <b>format</b> must correspond to the arguments specified in <b>...</b>. Otherwise, an error is returned.
* @param format Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the total number of characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise.
* @since 1.0
* @version 1.0
*/
int printf(const char *format, ...);

/**
* @brief Formats arguments based on <b>format</b> and prints formatted data to a specified file stream.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param format Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the total number of characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise.
* @since  1.0
* @version 1.0
*/
int fprintf(FILE *stream, const char *format, ...);

/**
* @brief Prints formatted data from a variable argument list to a specified string.
*
* @param str Indicates the pointer to the string to which the data is to be written.
* @param format Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the total number of characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise.
* @since 1.0
* @version 1.0
*/
int sprintf(char *str, const char *format, ...);

/**
* @brief Prints formatted data of a maximum of <b>size</b> bytes from a variable argument list to a specified string.
*
* @attention This function can write a maximum of <i>size-1</i> valid characters to the destination string.
* @param str Indicates the pointer to the string to which the data is to be written.
* @param size Indicates the maximum length of the destination string.
* @param format Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the number of formatted characters written (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise. If the return value is a non-negative value smaller than the value of <b>size</b>, the string has been completely written.
*
* @since 1.0
* @version 1.0
*/
int snprintf(char *str, size_t size, const char *format, ...);

/**
* @brief Prints formatted data from a variable argument list to the stdout.
*
* @attention This function is generally used together with <b>va_start</b> and <b>va_end</b>.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the total number of characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise.
* @since 1.0
* @version 1.0
*/
int vprintf(const char *format, va_list ap);

/**
* @brief Prints formatted data from a variable argument list to a specified file stream.
*
* @attention This function is generally used together with <b>va_start</b> and <b>va_end</b>.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the total number of characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise.
* @since 1.0
* @version 1.0
*/
int vfprintf(FILE *stream, const char *format, va_list ap);

/**
* @brief Prints formatted data from a variable argument list to a specified string.
*
* @param str Indicates the pointer to the destination character string.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the total number of characters written (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise.
* @since 1.0
* @version 1.0
*/
int vsprintf(char *str, const char *format, va_list ap);

/**
* @brief Prints formatted data of a maximum of <b>size</b> bytes from a variable argument list to a specified string.
*
* @attention This function can write a maximum of <i>size-1</i> characters to the destination string.
* @param str Indicates the pointer to the destination character string.
* @param size Indicates the maximum length of the destination string.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of formatted characters written (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise. If the return value is a non-negative value smaller than the value of <b>size</b>, the string has been completely written.
*
* @since 1.0
* @version 1.0
*/
int vsnprintf(char *str, size_t size, const char *format, va_list ap);

/**
* @brief Reads formatted data from the stdin and stores the data to the additional arguments based on <b>format</b>.
*
* @attention The type of each argument must match a format specifier in <b>format</b>.
* @param format Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the number of variables that are successfully read.
* If the matching fails quickly, the return value may be less than the number of members in the argument list or even zero.
* If no complete conversion format is available or the input format fails to be matched, {@link EOF} is returned. The return value is also {@link EOF} if a reading error occurs. \n
* If the number of arguments exceeds the number of the format specifiers, the excess arguments are ignored.
* @since 1.0
* @version 1.0
*/
int scanf(const char *format, ...);

/**
* @brief Reads formatted data from a specified file stream and stores it based on <b>format</b> into the locations pointed to by the variable arguments.
*
* The operation ends when a space or line feed character is encountered. \n
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param format Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the number of variables that are successfully read. If the matching fails quickly, the return value may be less than the number of members in the argument list or even zero.
* If no complete conversion format is available or the input format fails to be matched, <b>EOF</b> is returned. The return value is also <b>EOF</b> if a reading error occurs.
* @since  1.0
* @version  1.0
*/
int fscanf(FILE *stream, const char *format, ...);

/**
* @brief Reads data from a character string pointed to by <b>str</b> and stores it based on <b>format</b> into the locations pointed to by the variable arguments.
*
* @attention An asterisk (*) right after the percent symbol (such as <b>%*d</b> and <b>%*s</b>) in the format denotes that the value for the format will be read but will not be stored into a variable.
* @param str Indicates the pointer to the character string to read.
* @param format Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int sscanf(const char *str, const char *format, ...);

/**
* @brief Reads data from the stdin and stores it based on <b>format</b> into the locations pointed to by the elements in the variable argument list identified by <b>ap</b>.
*
* @attention This function is generally used together with <b>va_start</b> and <b>va_end</b>.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int vscanf(const char *format, va_list ap);

/**
* @brief Reads data from a specified file stream and stores it based on <b>format</b> into the locations pointed to by the elements in the variable argument list identified by <b>ap</b>.
*
* @attention This function is generally used together with <b>va_start</b> and <b>va_end</b>.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int vfscanf(FILE *stream, const char *format, va_list ap);

/**
* @brief Reads data from a specified string and stores it based on <b>format</b> into the locations pointed to by the elements in the variable argument list identified by <b>ap</b>.
*
* @attention This function is generally used together with <b>va_start</b> and <b>va_end</b>.
* @param str Indicates the pointer to the character string to read.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int vsscanf(const char *str, const char *format, va_list ap);

/**
* @brief Prints the last error occurred during a call to a system or library function to the standard error output stream (stderr).
*
* @attention To make the output message more useful, <b>msg</b> should convey obvious information describing what happened.
* When a system call fails, it usually returns <b>-1</b> and sets <b>errno</b> to a value that describes what went wrong. The error codes can be found in <b>errno.h</b>.
* @param msg Indicates the pointer to the prompt message printed to stderr before the error message itself.
* @since 1.0
* @version 1.0
*/
void perror(const char *msg);

/**
* @brief Sets the mode and size of the buffer for a specified stream.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param buf Indicates the pointer to the start address of the buffer.
* @param mode Indicates the buffer mode to set. There are three modes: <b>_IONBF</b> (unbuffered), <b> _IOLBF</b> (line buffered), and <b>_IOFBF</b> (fully buffered).
* @param size Indicates the buffer block size when the buffer mode is <b>_IOFBF</b>.
* @return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
* @since 1.0
* @version 1.0
*/
int setvbuf(FILE *stream, char *buf, int mode, size_t size);

/**
* @brief Sets a buffer to be used by a specified stream.
*
* @attention When the input parameter <b>buf</b> is not null, the buffer mode is {@link _IOFBF}. The default buffer size is determined by the value of <b>BUFSIZ</b> defined in this file. When <b>buf</b> is null, the buffer mode is {@link _IONBF} (unbuffered).
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param buf Indicates the pointer to the start address of the buffer.
* @since 1.0
* @version 1.0
*/
void setbuf(FILE *stream, char *buf);

/**
* @brief Opens a memory stream that allows the access mode specified by <b>mode</b>.
*
* This function is used for applications that do not support memory operations but support file operations. After the function call, you can directly operate the returned file stream, which is equivalent to operating the memory stream. \n
* @param buf Indicates the pointer to the buffer.
* @param size Indicates the size of the buffer.
* @param mode Indicates the pointer to the file operation mode.
* @return Returns the opened memory stream as a file stream.
* @since  1.0
* @version  1.0
*/
FILE *fmemopen(void *buf, size_t size, const char *mode);

/**
* @brief Opens a stream for writing to a specified buffer.
*
* The buffer is dynamically allocated and grown. After the stream is closed, call {@link free()}
* to release the buffer. When {@link fclose()} or {@link fflush()} is called to close or flush the stream, the pointers to the allocated buffer address
* and buffer size are updated to <b>ptr</b> and <b>size</b>, respectively. \n
* @attentions This function is defined in POSIX.1-2008. It is not defined in POSIX.1-2001 and is not widely used in other systems.
* @param ptr Indicates the pointer to the buffer address.
* @param size Indicates the pointer to the buffer size.
* @return Returns the pointer to the stream if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to indicate an error otherwise.
* @since 1.0
* @version 1.0
*/
FILE *open_memstream(char **ptr, size_t *size);

/**
* @brief Closes a specified pipe.
*
*
* @param f Indicates the pointer to the file object that identifies a pipe to close.
* @return Returns the thread termination status code if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ----- | ---------------
* ECHILD | The child process does not exist, or the specified process group does not exist.
* ESRCH | The child process ends abnormally.
* @since 1.0
* @version 1.0
*/
int pclose(FILE *f);

/**
* @brief Obtains the file descriptor of a specified file stream.
*
* @param fp Indicates the pointer to the file object that identifies a stream for which the file descriptor is obtained.
* @return Returns the file descriptor if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ----- | ---------------
* EBADF | <b>fp</b> points to an invalid stream.
* @since 1.0
* @version 1.0
*/
int fileno(FILE *fp);

/**
* @brief Sets the position of the next input or output operation on a specified file stream.
*
* @param fp Indicates the pointer to the file object that identifies a stream for which the position is to set.
* @param offset Indicates the offset that determines the next operation.
* @param whence Indicates the start position of the offset. The following table describes the available values.
* value    | Description
* -------- | ---------
* SEEK_SET | Beginning of the file
* SEEK_CUR | Current position
* SEEK_END | End of the file
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -------------------------------------------
* EBADF | <b>fp</b> points to an invalid stream.
* EINVAL | The value of <b>whence</b> is not <b>SEEK_SET</b>, <b>SEEK_END</b>, or <b>SEEK_CUR</b>.
* ESPIPE | Invalid seek.
* @since 1.0
* @version 1.0
*/
int fseeko(FILE *fp, off_t offset, int whence);

/**
* @brief Obtains the current value of the file position indicator for a specified file stream.
*
* @param fp Indicates the pointer to the file object that identifies a stream for which the position is to obtain.
* @return Returns the current offset if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----- | ---------------
* EBADF | <b>fp</b> points to an invalid stream.
* @since 1.0
* @version 1.0
*/
off_t ftello(FILE *fp);

/**
* @brief Prints formatted data to a specified file.
*
* @param fd Indicates the file descriptor that identifies a file.
* @param format Indicates the pointer to the format string.
* @return Returns the total number of written characters if the operation is successful; returns a negative value otherwise.
* @since  1.0
* @version  1.0
*/
int dprintf(int fd, const char *format, ...);

/**
* @brief Prints formatted data from a variable argument list to a specified file.
*
* @param fd Indicates the file descriptor that identifies a file.
* @param format Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the total number of written characters if the operation is successful; returns a negative value otherwise.
* @since  1.0
* @version  1.0
*/
int vdprintf(int fd, const char *format, va_list ap);

/**
* @brief Locks a specified file.
*
* This function facilitates I/O operations in multiple threads. If the file has been locked by another thread, this function waits until the file object is no longer locked by any thread and then locks it.  \n
* @param filehandle Indicates the pointer to the file object.
* @return Returns <b>0</b> if the locking is successful; returns a non-zero value otherwise.
* @since 1.0
* @version  1.0
*/
int ftrylockfile(FILE *filehandle);

/**
* @brief Unlocks a specified file.
*
* This function is used to reduce the lock count.
* @param filehandle Indicates the pointer to the file whose lock count is to be reduced.
* @since 1.0
* @version 1.0
*/
void funlockfile(FILE *filehandle);

/**
* @brief Obtains a character from a specified file stream without locking the file stream or checking whether it is locked.
*
* @attention Use this function with caution in multi-thread scenarios.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the obtained character.
* @since  1.0
* @version  1.0
*/
int getc_unlocked(FILE *stream);

/**
* @brief Obtains the next character from stdin.
*
* This function waits for user input and does not return until any input is available. \n
* @attention Use this function with caution in multi-thread scenarios.
* @return Returns the character forcibly converted to an int value if the operation is successful; returns <b>EOF</b> if the end-of-file is reached or a reading error occurs.
* @since  1.0
* @version  1.0
*/
int getchar_unlocked(void);

/**
* @brief Writes a character to a specified file stream with the thread remains unlocked.
*
* @attention This function cannot be used in a multi-thread process because it does not use locking during the call.
* @param c Indicates the int promotion of the character to write.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the ASCII code of the character written if the operation is successful; returns <b>EOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
int putc_unlocked(int c, FILE *stream);

/**
* @brief Writes a character to the stdout with the thread remains unlocked.
*
* @attention This function cannot be used in a multi-thread process because it does not use locking during the call.
* @param c Indicates the int promotion of the character to write.
* @return Returns the ASCII code of the character written if the operation is successful; returns <b>EOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
int putchar_unlocked(int c);

/**
* @brief Reads an entire line from a specified file stream.
*
* @attention If the system dynamically allocates memory, the memory needs to be released after being used.
* @param lineptr Indicates the double pointer to the buffer for storing the characters in the line.
* @param n Indicates the pointer to the size of the buffer. If the system needs to dynamically allocate memory, set this parameter to <b>0</b>.
* @param delim Indicates the line delimiter.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the number of bytes read if the operation is successful; returns <b>-1</b> otherwise.
* @since  1.0
* @version  1.0
*/
ssize_t getdelim(char **lineptr, size_t *n, int delim, FILE *stream);

/**
* @brief Reads an entire line from a specified file stream.
*
* @attention If the system dynamically allocates memory, the memory needs to be released after being used.
* @param lineptr Indicates the double pointer to the buffer for storing the characters in the line.
* @param n Indicates the pointer to the size of the buffer. If the system needs to dynamically allocate memory, set this parameter to <b>0</b>.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the number of bytes read if the operation is successful; returns <b>-1</b> otherwise.
* @since  1.0
* @version  1.0
*/
ssize_t getline(char **lineptr, size_t *n, FILE *stream);

/**
* @brief Creates a temporary file with a unique file name.
*
* @attention Do not use this function. Use <b>mkstemp()</b> or <b>tmpfile()</b> instead.
* @param dir Indicates the pointer to directory for creating the temporary file.
* @param prefix Indicates the pointer to the prefix of the file name.
* @return Returns the pointer to the unique name of the created temporary file if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to indicate an error otherwise.
* @see mkstemp() |  tmpfile()
* @since 1.0
* @version 1.0
*/
char *tempnam(const char * dir, const char * prefix);

/**
* @brief Sets the buffer for a specified file stream.
*
*
* @param f Indicates the pointer to the file object that identifies a stream to set.
* @param buf Indicates the pointer to the buffer to set.
* @param size Indicates the buffer size.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> otherwise.
* @since  1.0
* @version 1.0
*/
void setbuffer(FILE *f, char *buf, size_t size);

/**
* @brief Sets the linear buffer for a specified file stream.
*
*
* @param f Indicates the pointer to the file object that identifies a stream to set.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> otherwise.
* @since  1.0
* @version 1.0
*/
void setlinebuf(FILE *f);

/**
* @brief Reads a character from the file pointed to by <b>stream</b>.
*
* This function reads the next character from the current position of the file stream and returns the character, then advances the file position indicator. You can read each character of the file by continuously calling this function. \n
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the character read as an int value if the operation is successful; returns <b>EOF</b> if the end-of-file is reached or a reading error occurs.
* @since  1.0
* @version  1.0
*/
int fgetc(FILE *stream);

/**
* @brief Reads a character from the file pointed to by <b>stream</b> with the thread remains unlocked.
*
* This function reads the next character from the current position of the file stream and returns the character, then advances the file position indicator. You can read each character of the file by continuously calling this function. \n
* @attention This function cannot be used in a multi-thread process because it does not use locking during the call. (It does not set locks itself, and does not test for the presence of locks set by others.) An error occurs upon a multi-thread operation.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the character read as an int value if the operation is successful; returns <b>EOF</b> if the end-of-file is reached or a reading error occurs.
* @since  1.0
* @version  1.0
*/
int fgetc_unlocked(FILE *stream);

/**
* @brief Writes a character to the file stream pointed to by <b>stream</b> at the current position.
*
* @attention This function cannot be used in a multi-thread process because it does not use locking during the call. (It does not set locks itself, and does not test for the presence of locks set by others.) An error occurs upon a multi-thread operation.
* @param c Indicates the int promotion of the character to write.
* @param stream Indicates the pointer to the file stream.
* @return Returns the ASCII code of the character written if the operation is successful; returns <b>EOF</b> otherwise.
* @since  1.0
* @version  1.0
*/
int fputc_unlocked(int c, FILE *stream);

/**
* @brief Tests for the error identifier of a specified file stream.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns a non-zero value to indicate an error for the stream if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int ferror_unlocked(FILE *stream);

/**
* @brief Reads a word (an int value) from a specified file stream.
*
* @attention This function is provided for compatibility with System V Release 4 (SVR4). You are advised to use {@link fread} instead.
* The value returned upon an error is also a valid data value.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the integer read if the operation is successful; returns <b>EOP</b> otherwise.
* @since 1.0
* @version 1.0
*/
int getw(FILE *stream);

/**
* @brief Writes a word (an int value) to a specified file stream.
*
* @attention This function is provided for compatibility with SVR4. You are advised to use {@link fwrite} instead.
* @param w Indicates the word to write.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns <b>0</b> if the operation is successful; returns <b>EOP</b> otherwise.
* @since 1.0
* @version 1.0
*/
int putw(int w, FILE *stream);

/**
* @brief Reads a line of data from the current position of a specified file stream.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param len Indicates the pointer to the length of the string that is read.
* @return Returns the string read.
* @since 1.0
* @version  1.0
*/
char *fgetln(FILE *stream, size_t *len);

/**
* @brief Prints formatted data to a specified character string.
*
* If the length of the string is uncertain, this function applies for sufficient memory space based on the length of the formatted character string. \n
* @attention If <b>strp</b> is no longer used, the applied space must be released.
* @param strp Indicates the double pointer to the start address of the target string.
* @param fmt Indicates the pointer to the format string.
* @param ... Indicates the list of arguments corresponding to the format specifiers.
* @return Returns the number of output characters if the operation is successful; returns <b>-1</b> otherwise.
* @since  1.0
* @version  1.0
*/
int asprintf(char **strp, const char *fmt, ...);

/**
* @brief Prints formatted data from a variable argument list to a specified character string.
*
* If the length of the string is uncertain, this function applies for sufficient memory space based on the length of the formatted character string. \n
* @attention If <b>strp</b> is no longer used, the applied space must be released.
* @param strp Indicates the double pointer to the start address of the target string.
* @param fmt Indicates the pointer to the format string.
* @param ap Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of output characters if the operation is successful; returns <b>-1</b> otherwise.
* @since  1.0
* @version  1.0
*/
int vasprintf(char **strp, const char *fmt, va_list ap);


#ifdef __cplusplus
}
#endif

#endif
/** @} */