/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file stdlib.h
*
* @brief Declares common functions used for performing I/O operations.
*
* You can use the functions provided in this file to complete string conversion, random number generation, array sorting, and tree-related operations during development. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _STDLIB_H
#define _STDLIB_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Converts an input string to an integer.
*
* The conversion stops when a non-numeric character is encountered. \n
* @param nptr Indicates the string to be converted.
* @return Returns the converted value if the operation is successful; returns <b>0</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
int atoi(const char *nptr);

/**
* @brief Converts an input string to a long integer.
*
* @param nptr Indicates the string to be converted.
* @return Returns the converted value if the operation is successful; returns <b>0</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
long atol(const char *nptr);

/**
* @brief Converts a string to an 8-byte long integer.
*
* The conversion stops when a non-numeric character is encountered. \n
* @param nptr Indicates the string to be converted.
* @return Returns the converted value if the operation is successful; returns <b>0</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
long long atoll(const char *nptr);

/**
* @brief Converts an input string to a double-precision floating-point number.
*
* The conversion stops when a non-numeric character is encountered. \n
* @param nptr Indicates the string to be converted.
* @return Returns the converted value if the operation is successful; returns <b>0</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
double atof(const char *nptr);

/**
* @brief Converts an input string to a floating-point number.
*
* This function skips the space characters, starts the conversion at a digit or a positive or negative sign, and ends at a non-digit or the terminating null byte <b>'\0'</b>. If the value of <b>endptr</b> is not <b>NULL</b>, the pointer to the character in <b>nptr</b> where the conversion is terminated due to an invalid condition is stored in <b>endptr</b>. \n
* @param nptr Indicates the input string.
* @param endptr Indicates the double pointer to the first unrecognized character.
* @return Returns the converted value if the operation is successful.
* @since 1.0
* @version 1.0
*/
float strtof(const char *nptr, char **endptr);

/**
* @brief Converts a string to <b>double</b>.
*
* This function skips the space characters, starts the conversion at a digit or a positive or negative sign, and ends at a non-digit or the terminating null byte <b>'\0'</b>. If the value of <b>endptr</b> is not <b>NULL</b>, the pointer to the character in <b>nptr</b> where the conversion is terminated due to an invalid condition is stored in <b>endptr</b>. \n
* @param nptr Indicates the input string.
* @param endptr Indicates the double pointer to the first unrecognized character.
* @return Returns the converted value.
* @since 1.0
* @version 1.0
*/
double strtod(const char *nptr, char **endptr);

/**
* @brief Converts a string to <b>long double</b>.
*
* This function skips the space characters, starts the conversion at a digit or a positive or negative sign, and ends at a non-digit or the terminating null byte <b>'\0'</b>. If the value of <b>endptr</b> is not <b>NULL</b>, the pointer to the character in <b>nptr</b> where the conversion is terminated due to an invalid condition is stored in <b>endptr</b>. \n
* @param nptr Indicates the input string.
* @param endptr Indicates the double pointer to the first unrecognized character.
* @return Returns the converted value.
* @since 1.0
* @version 1.0
*/
long double strtold(const char *nptr, char **endptr);

/**
* @brief Converts a string to a long integer according to the given <b>base</b>.
*
* @attention The value of <b>base</b> ranges from <b>2</b> to <b>36</b>. If the value of <b>base</b> is <b>0</b>, the decimal format is used for conversion.
* Otherwise, the hexadecimal format is used, or the octal format is used if the next character is <b>0</b> instead of <b>0x</b>.
* @param nptr Indicates the input string.
* @param endptr Indicates the first character that is unrecognized.
* @param base Indicates the number system.
* @return Returns the converted value.
* @since 1.0
* @version 1.0
*/
long strtol(const char *nptr, char **endptr, int base);

/**
* @brief Converts a string to an unsigned long integer according to the given <b>base</b>.
*
* @attention The value of <b>base</b> ranges from <b>2</b> to <b>36</b>. If the value of <b>base</b> is <b>0</b>, the decimal format is used for conversion.
* Otherwise, the hexadecimal format is used, or the octal format is used if the next character is <b>0</b> instead of <b>0x</b>.
* @param nptr Indicates the input string.
* @param endptr Indicates the first character that is unrecognized.
* @param base Indicates the number system.
* @return Returns the converted value.
* @since 1.0
* @version 1.0
*/
unsigned long strtoul(const char *nptr, char **endptr, int base);

/**
* @brief Converts a string to a long long integer according to the given <b>base</b>.
*
* @attention The value of <b>base</b> ranges from <b>2</b> to <b>36</b>. If the value of <b>base</b> is <b>0</b>, the decimal format is used for conversion.
* Otherwise, the hexadecimal format is used, or the octal format is used if the next character is <b>0</b> instead of <b>0x</b>.
* @param nptr Indicates the input string.
* @param endptr Indicates the first character that is unrecognized.
* @param base Indicates the number system.
* @return Returns the converted value.
* @since 1.0
* @version 1.0
*/
long long strtoll(const char *nptr, char **endptr, int base);

/**
* @brief Converts a string to an unsigned long long integer according to the given <b>base</b>.
*
* @attention The value of <b>base</b> ranges from <b>2</b> to <b>36</b>. If the value of <b>base</b> is <b>0</b>, the decimal format is used for conversion.
* Otherwise, the hexadecimal format is used, or the octal format is used if the next character is <b>0</b> instead of <b>0x</b>.
* @param nptr Indicates the input string.
* @param endptr Indicates the first character that is unrecognized.
* @param base Indicates the number system.
* @return Returns the converted value.
* @since 1.0
* @version 1.0
*/
unsigned long long strtoull(const char *nptr, char **endptr, int base);

/**
* @brief Generates a pseudo-random number.
*
* The random number ranges from <b>0</b> to {@link RAND_MAX}. \n
* @attention {@link RAND_MAX} is defined in this header file. This function is implemented using the linear congruence method. If the random number seed does not change, the generated random number complies with the normal distribution.
* The generated random number is not a real random number but a pseudo-random number. Therefore, you need to use the <b>srand()</b> function to initialize the random number seed to the unique values.
* @return Returns a non-negative integer that is evenly distributed in the interval [0, {@link RAND_MAX}] if the operation is successful.
* @since 1.0
* @version 1.0
*/
int rand(void);

/**
* @brief Initializes a random number generator.
*
* @attention If <b>seed</b> is <b>0</b>, this function is equivalent to <b>srand(1)</b>. Before the <b>rand()</b> function is executed, this function is used to initialize different algorithm seeds to generate true random numbers.
* @param seed Indicates the seed for generating a pseudo-random number.
* @since 1.0
* @version 1.0
*/
void srand(unsigned int seed);

/**
* @brief Dynamically allocates a memory block of <b>size</b>.
*
* The allocated memory is not initialized. \n
* @param size Indicates the size of the memory block to be allocated.
* @return Returns the pointer to the allocated memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *malloc(size_t size);

/**
* @brief Dynamically allocates <b>nmemb</b> memory blocks of <b>size</b>.
*
* @param nmemb Indicates the number of memory blocks to be allocated.
* @param size Indicates the size of the memory block to be allocated.
* @return Returns the pointer to the allocated memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *calloc(size_t nmemb, size_t size);

/**
* @brief Changes the size of the memory block pointed to by <b>ptr</b> to <b>size</b> bytes.
*
* The contents will remain unchanged in the range from the start of <b>ptr</b> to the minimum of the old and new sizes.
* If the new size is larger than the old size, the added memory will not be initialized.
* If <b>ptr</b> is <b>NULL</b>, this function call is equivalent to <b>malloc(size)</b>; if <b>size</b> is equal to <b>0</b> and <b>ptr</b> is not <b>NULL</b>, the function call is equivalent to <b>free(ptr)</b>.
* If <b>ptr</b> is not <b>NULL</b>, it must be the returned value of {@link malloc()}, {@link calloc()}, or {@link realloc}. If the memory block pointed to by <b>ptr</b> has been moved, <b>free(ptr)</b> is executed. \n
* @param ptr Indicates the pointer to the memory block to be adjusted.
* @param size Indicates the new size of the memory block.
* @return Returns the pointer to the new memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *realloc(void *ptr, size_t size);

/**
* @brief Releases the memory space pointed to by <b>ptr</b>.
*
* The memory space was allocated by using {@link calloc()}, {@link malloc()}, or {@link realloc()}. \n
* @param ptr Indicates the pointer to the memory space to be released.
* @since 1.0
* @version 1.0
*/
void free(void *ptr);

/**
*@brief Terminates an abnormal process and sends the <b>SIGABRT</b> signal to the caller.
*
*@since 1.0
*@version 1.0
*/
_Noreturn void abort(void);

/**
*@brief Registers a termination function.
*
* The registered function is called when the current process is terminated normally. The sequence of the termination function to be called is opposite to the registration sequence. \n
*@param func Indicates the pointer to the termination function to be called.
*@return Returns <b>0</b> if the operation is successful; returns a non-zero value otherwise.
*@since 1.0
*@version 1.0
*/
int atexit(void (*func)(void));

/**
*@brief Terminates the calling process, clears the used memory space and various data structures in the kernel, and sends the process end status to the parent process. All functions registered with atexit and on_exit are called in the reverse order.
*
*@param status Indicates the status sent to the parent process when the calling process ends.
*@since 1.0
*@version 1.0
*/
void exit(int status);

/**
* @brief Obtains the value of an environment variable.
*
* @param name Indicates the pointer to the name of the environment variable whose value needs to be obtained.
* @return Returns the pointer to the environment variable value if the operation is successful; returns <b>NULL</b> if there is no match.
* @since  1.0
* @version  1.0
*/
char *getenv(const char *name);

/**
*@brief Terminates the calling process, clears the used memory space and various data structures in the kernel, and sends the process end status to the parent process.
*
*@param status Indicates the status sent to the parent process when the calling process ends.
*@since 1.0
*@version 1.0
*/
void _Exit(int status);

/**
*@brief Aborts the program if assertion is false.
*
* If the <b>NDEBUG</b> macro is not defined and the value of <b>expression</b> is false, an error message is displayed and the process is terminated. In other cases, this macro is expanded but no operation is performed. \n
*@param expression Indicates the condition variable. If the value of <b>expression</b> is false, an error message is displayed and the process is terminated.
*@since 1.0
*@version 1.0
*/
void assert(scalar expression);

/**
* @brief Obtains the value of an environment variable.
*
* The feature of this function is the same as that of {@link getenv()} during non-secure execution. \n
* @param name Indicates the pointer to the name of the environment variable whose value needs to be obtained.
* @return Returns the pointer to the environment variable value if the operation is successful; returns <b>NULL</b> if there is no match.
* @since  1.0
* @version  1.0
*/
char *secure_getenv(const char *name);

/**
* @brief Searches for <b>key</b> using the binary search algorithm from the array element <b>base[0]</b> to <b>base[num-1]</b>.
*
* @attention Data must have been sorted, following the same rule as the comparison subfunction pointed to by <b>compar</b>.
* @param key Indicates the pointer to the keyword to be searched.
* @param base Indicates the array to be searched.
* @param nel Indicates the number of elements in the array to be searched.
* @param width Indicates the length of each element, in bytes.
* @param compare Indicates the pointer to the comparison subfunction used to define comparison rules.
* @return Returns the matching item if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
void *bsearch(const void *key, const void *base, size_t nel, size_t width, int (*compar)(const void *, const void *));

/**
* @brief Sorts array elements <b>base[0]</b> to <b>base[num-1]</b> based on the comparison rules of <b>compar</b>.
*
* @param base Indicates the array to be sorted.
* @param nel Indicates the number of elements in the array to be searched.
* @param width Indicates the length of each element, in bytes.
* @param compare Indicates the pointer to the comparison subfunction used to define comparison rules.
* @since  1.0
* @version  1.0
*/
void qsort(const void *base, size_t nel, size_t width, int (*compar)(const void *, const void *));

/**
* @brief Obtains the absolute value of an integer value.
*
* @param i Indicates the integer value.
* @return Returns the absolute value.
* @since 1.0
* @version 1.0
*/
int abs(int i);

/**
* @brief Calculates the absolute value of a long integer.
*
* @param i Indicates the long integer to be converted.
* @return Returns the absolute value.
* @since 1.0
* @version 1.0
*/
long labs(long i);

/**
* @brief Calculates the absolute value of a long long integer.
*
* @param i Indicates the long long integer to be converted.
* @return Returns the absolute value.
* @since 1.0
* @version 1.0
*/
long long llabs(long long i);

/**
* @brief Defines the structures of the division operation result.
*/
typedef struct {
/** Quotient */
int quot;
/** Remainder */
int rem;
} div_t;

/**
* @brief Defines the structures of the division operation result.
*/
typedef struct {
/** Quotient */
long quot;
/** Remainder */
long rem;
} ldiv_t;

/**
* @brief Defines the structures of the division operation result.
*/
typedef struct {
/** Quotient */
long long quot;
/** Remainder */
long long rem;
}  lldiv_t;

/**
* @brief Calculates the quotient and remainder of an integer after division.
*
* @param numerator Indicates the numerator.
* @param denominator Indicates the denominator.
* @return Returns a structure variable that has been defined in the function.
* @since 1.0
* @version 1.0
*/
div_t div(int numerator, int denominator);

/**
* @brief Calculates the quotient and remainder of a long integer after division.
*
* @param numerator Indicates the numerator.
* @param denominator Indicates the denominator.
* @return Returns a structure variable that has been defined in the function.
* @since 1.0
* @version 1.0
*/
ldiv_t ldiv(long numerator , long denominator);

/**
* @brief Calculates the quotient and remainder of a long long integer after division.
*
* @param numerator Indicates the numerator.
* @param denominator Indicates the denominator.
* @return Returns a structure variable that has been defined in the function.
* @since 1.0
* @version 1.0
*/
lldiv_t lldiv(long long numerator, long long denominator);

/**
* @brief Obtains the number of bytes in the next multi-byte string.
*
* If <b>s</b> is not <b>NULL</b>, this function inspects at most <b>n</b> bytes of the multi-byte string starting at <b>s</b>.
* @param s Indicates the pointer to a string constant.
* @param n Indicates the number of bytes to be checked. The value cannot be greater than <b>MB_CUR_MAX</b>.
* @return Returns the number of bytes of the string constant if the operation is successful; returns <b>0</b> if the string constant content is a wide character that is empty;
* returns <b>-1</b> if the string constant content is invalid or cannot be parsed.
* @since 1.0
* @version 1.0
*/
int mblen(const char *s, size_t n);

/**
* @brief Converts a string constant to a wide character.
*
* If <b>s</b> is not <b>NULL</b>, this function inspects at most <b>n</b> bytes of the multi-byte string starting at <b>s</b> and converts it to a wide character and stores it.
* @attention <b>s</b> and <b>pwc</b> cannot be <b>NULL</b>. If <b>s</b> is <b>NULL</b>, <b>pwc</b> and <b>n</b> are ignored.
* @param pwc Indicates the pointer to the string for storing the wide character.
* @param s Indicates the pointer to a string constant.
* @param n Indicates the number of bytes to be checked. The value cannot be greater than {@link MB_CUR_MAX}.
* @return Returns the number of bytes converted from <b>s</b> if the operation is successful; returns <b>0</b> if <b>s</b> points to an empty byte;
* returns <b>-1</b> if the string constant content is invalid or cannot be parsed.
* @since 1.0
* @version 1.0
*/
int mbtowc(wchar_t *pwc, const char *s, size_t n);

/**
* @brief Converts a wide character to its multi-byte sequence and stores it in a character array.
*
* @param s Indicates the pointer to an array that can store multi-byte characters.
* @param wc Indicates the wide character to be converted.
* @return Returns one of the following values:
* Input | Return Value
* -----------|  ------------
* <b>s</b> is not <b>NULL</b>. | The number of bytes written to the byte array.
* <b>wc</b> cannot be represented as a multi-byte sequence. | <b>-1</b>.
* <b>s</b> is <b>NULL</b>. | A non-zero value if the code has shift state or <b>0</b> if the code is stateless.
* @since  1.0
* @version  1.0
*/
int wctomb(char *s, wchar_t wc);

/**
* @brief Converts a multi-byte string to a wide-character string.
*
* @param dest Indicates the pointer to an array for the <b>wchar_t</b> element. The array length can store a wide-character string whose length is specified by <b>n</b>.
* @param src Indicates the pointer to the multi-byte string to convert.
* @param n Indicates the maximum number of wide characters to convert.
* @return Returns the number of wide characters, excluding the terminating null wide character if the operation is successful; returns <b>-1</b> if an invalid multibyte character is encountered.
* @since  1.0
* @version  1.0
*/
size_t mbstowcs(wchar_t *dest, const char *src, size_t n);

/**
* @brief Converts a wide-character string to a multi-byte string.
*
* @param dest Indicates the pointer to the destination string, which must have sufficient space, as specified by <b>n</b>.
* @param src Indicates the pointer to the wide-character string to be converted.
* @param n Indicates the maximum number of bytes to be written to the destination string.
* @return Returns the number of converted bytes, excluding the terminating null byte if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to <b>EILSEQ</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
size_t wcstombs(char *dest, const wchar_t *src, size_t n);

/**
* @brief Allocates memory with the specified size based on the given <b>alignment</b>.
*
* The value of <b>alignment</b> must be a power of 2 and a multiple of <b>sizeof(void *)</b>.
* The allocated memory address is stored in <b>*memptr</b>.
* If <b>size</b> is <b>0</b>, <b>*memptr</b> is either <b>NULL</b> or a unique pointer value that can later be successfully passed to {@link free()}. \n
* @param memptr Indicates the double pointer to the allocated memory block.
* @param alignment Indicates the alignment size of the allocated memory block.
* @param size Indicates the size of the memory block to be allocated.
* @return Returns <b>0</b> if the operation is successful; returns an error code if the operation fails.
* errno | Description
* -------|------------------------------------------------------
* EINVAL | The value of <b>alignment</b> is not a power of 2, or is not a multiple of <b>sizeof(void *)</b>.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int posix_memalign(void **memptr, size_t alignment, size_t size);


/**
* @brief Add or change an environment variable.
*
* This function adds the varible <b>name</b> to environment with string <b>value</b> if <b>name</b> is not exist.
* @param name Indicates the name of environment variable.
* @param value Indicates the value of environment variable.
* @param overwrite If the varible <b>name</b> does exit and the value <b>overwrite</b> is nonzero, then value of <b>name</b> will be change to new <b>value</b>; \n
* if <b>overwrite</b> is zero, then the value of <b>name</b> remains unchanged.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails
* errno | Description
* -------|------------------------------------------------------
* EINVAL | The name is <b>NULL</b>, or its length is zero, or contains an "=" character.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int setenv(const char *name, const char *value, int overwrite);

/**
* @brief Deletes an environment variable.
*
* @param name Indicates the name of environment variable to be deleted.
* if <b>overwrite</b> is zero, then the value of <b>name</b> remains unchanged.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails
* 
* errno | Description
* -------|------------------------------------------------------
* EINVAL | The name is <b>NULL</b>, or its length is zero, or contains an "=" character.
* @since 1.0
* @version 1.0
*/
int unsetenv(const char *name);

/**
* @brief Creates and opens a unique temporary file.
*
* @param template Indicates the name of the file to be created. The last six characters must be <b>XXXXXX</b>.
* @return Returns the descriptor of the opened file if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ------------------------
* EINVAL | The last six characters are not <b>XXXXXX</b>.
* EEXIST | Failed to create a unique temporary file name.
* Other | For details, see {@link open()}.
* @see open
* @since 1.0
* @version 1.0
*/
int mkstemp(char *template);

/**
* @brief Creates and opens a unique temporary file.
*
* Different from {@link mkstemp}, this function has the <b>flags</b> parameter. \n
* @param template Indicates the name of the file to be created. The last six characters must be <b>XXXXXX</b>.
* @param flags Indicates the flag for creating a file. For details, see {@link open}.
* @return Returns the descriptor of the opened file if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -------------------------
* EINVAL | The last six characters are not <b>XXXXXX</b>.
* EEXIST | Failed to create a unique temporary file name.
* Other | For details, see {@link open()}.
* @see open
* @since 1.0
* @version 1.0
*/
int mkostemp(char *template, int flags);

/**
* @brief Creates a unique temporary directory based on <b>template</b>.
*
* @param template Indicates the name of the directory to be created. The last six characters must be <b>XXXXXX</b>.
* @return Returns the pointer to the modified <b>template</b> string if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -------------------------
* EINVAL | The last six characters are not <b>XXXXXX</b>.
* Other | For details, see {@link mkdir}.
* @see mkdir
* @since 1.0
* @version 1.0
*/
char *mkdtemp(char *template);

/**
* @brief Processes the parameters of an option in the command line.
*
* Generally, this function is used together with the {@link getopt()} to process the parameters of an option.
* You are advised to use this function if the parameters of an option are complex. \n
* @param optionp Indicates the command word option.
* @param tokens Indicates the pointer to the queried token.
* @param valuep Indicates the suboption of the command word value.
* @return Returns the index of the matched suboption element in <b>tokens</b> if the operation is successful; returns <b>-1</b> if the operation fails.
* @link getopt
* @since  1.0
* @version  1.0
*/
int getsubopt(char **optionp, char *const *tokens, char **valuep);

/**
* @brief Generates a pseudo-random number.
*
* The random number ranges from <b>0</b> to {@link RAND_MAX}. \n
* @attention This function is implemented using the linear congruence method. If the random number seed does not change, the generated random number complies with the normal distribution.
* The generated random number is not a real random number but a pseudo-random number. Therefore, you need to use {@link srand} to initialize the random number seed to the unique values.
* @param seedp Indicates the random number seed.
* @return Returns a non-negative integer that is evenly distributed in the interval [0, {@link RAND_MAX}] if the operation is successful.
* @since 1.0
* @version 1.0
*/
int rand_r(unsigned *seedp);

/**
* @brief Obtains a normalized absolute path.
*
* @param path Indicates the name of the absolute path to be obtained.
* @param resolved Indicates the pointer to the obtained absolute path.
* @return Returns the pointer to <b>resolved_path</b> if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------------ | ----------------------------------
* EACCES | Read or search permission is denied on a component of the path prefix.
* EINVAL | <b>path</b> is <b>NULL</b>.
* EIO | An I/O error occurred while reading from the file system.
* ELOOP | Too many symbolic links are encountered when the path name is parsed.
* ENAMETOOLONG | The path name is too long.
* ENOMEM | Insufficient memory.
* ENOENT | The file does not exist.
* ENOTDIR | The component of the path prefix is not a directory.
* @since 1.0
* @version 1.0
*/
char *realpath(const char *__restrict path, char *__restrict resolved);

/**
* @brief Generates a pseudo-random number.
*
* @attention Generally, {@link srandom()} is called before this function to generate a random number seed. If {@link srandom()} is not called or <b>srandom(0)</b> is used to generate a random number seed, the random number seed generated by <b>srandom(1)</b> is used.
* @since 1.0
* @version 1.0
*/
long int random(void);

/**
* @brief Initializes a random number generator.
*
* @param seed Indicates the seed for generating a pseudo-random number.
* @since 1.0
* @version 1.0
*/
void srandom(unsigned int seed);

/**
* @brief Initializes a random number generator.
*
* This function initializes the state array for subsequent random use. \n
* @param seed Indicates the seed for initialization, which specifies the start point of the random number sequence and provides the information for restarting at the same point.
* @param Indicates the state array used to generate random numbers.
* @param n Indicates the length of the state array.
* @return Returns the pointer to the state array if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since  1.0
* @version  1.0
*/
char *initstate(unsigned int seed, char *state, size_t n);

/**
* @brief Sets the current state list for subsequent random use.
*
* @param state Indicates the pointer to the state list.
* @return Returns the pointer to the current state list if the operation is successful; returns <b>NULL</b> if the operation fails. <b>errno</b> is set to <b>EINVAL</b> if <b>state</b> is empty or the size of the state list is less than 8 bytes.
* @since 1.0
* @version 1.0
*/
char *setstate(char *state);

/**
* @brief Configures an environment variable.
*
* @param s Indicates the content of the environment variable to configure.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int putenv(char *s);

/**
* @brief Unlocks the secondary pseudo terminal corresponding to a primary pseudo terminal.
*
* @param fd Indicates the primary terminal descriptor.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------------------------------
* EBADF | Invalid descriptor
* EINVAL | <b>fd</b> is invalid.
* @since 1.0
* @version 1.0
*/
int unlockpt(int fd);

/**
* @brief Obtains the name of a pseudo terminal.
*
* @param fd Indicates the terminal descriptor.
* @return Returns the pointer to the name if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------------------------------
* ERANGE | <b>fd</b> is too long.
* @since 1.0
* @version 1.0
*/
char *ptsname(int fd);

/**
* @brief Converts a long integer to a 64-bit ASCII string.
*
* @param value Indicates the long integer to be converted.
* @return Returns the pointer to the static buffer if the operation is successful. The content in the buffer is overwritten each time the function is called. For multithreaded programs, this function returns a pointer to thread-specific data.
* @since 1.0
* @version 1.0
*/
char *l64a(long value);

/**
* @brief Converts between a 32-bit long integer and a little-endian 64-bit ASCII string.
*
* @attention If the length of <b>str64</b> is greater than <b>6</b>, only the first six bytes are used.
* @param str64 Indicates the pointer to the character constant.
* @return Returns the converted value.
* @since  1.0
* @version  1.0
*/
long a64l(const char *str64);

/**
* @brief Obtains a random number.
*
* This function returns a double-precision random number evenly distributed between (0.0, 1.0). \n
* @attention Before calling this function, you need to call one of the initialization functions <b>srand48()</b>, <b>seed48()</b>, and <b>lcong48()</b>.
* @return Returns the obtained double-precision random number.
* @since  1.0
* @version  1.0
*/
double drand48(void);

/**
* @brief Obtains a random number.
*
* This function returns nonnegative double-precision floating-point values evenly distributed between [0.0, 1.0).  \n
* @param xsubi Indicates the array of the initialized value of <b>Xi</b>.
* @since  1.0
* @version  1.0
*/
double erand48(unsigned short xsubi[3]);

/**
* @brief Generates pseudo-random numbers evenly distributed between [0, 2^31).
*
* @attention This function works by generating a 48-bit integer sequence, and <b>X(i+1)</b> must be generated by using the value of <b>Xi</b>. The correlation is obtained by using a linear congruential formula, Xn+1 = (aXn + c) mod m, where n = 0, m = 2^48, a = 0x5DEECE66D, and c = 0xB.
* Before this function is called, one of the functions <b>srand48()</b>, <b>seed48()</b>, and <b>lcong48()</b> must be used to initialize <b>Xi</b>.
* @return Returns nonnegative long integers evenly distributed between [0, 2^31) if the operation is successful.
* @since 1.0
* @version 1.0
*/
long int lrand48(void);

/**
* @brief Generates pseudo-random numbers evenly distributed between [0, 2^31).
*
* @attention This function works by generating a 48-bit integer sequence, and <b>X(i+1)</b> must be generated by using the value of <b>Xi</b>. The correlation is obtained by using a linear congruential formula, Xn+1 = (aXn + c) mod m, where n = 0, m = 2^48, a = 0x5DEECE66D, and c = 0xB.
* <b>Xi</b> is specified by <b>xsubi[3]</b>.
* @param xsubi[3] Indicates the array of the initialized value of <b>Xi</b>.
* @return Returns nonnegative long integers evenly distributed between [0, 2^31) if the operation is successful.
* @since 1.0
* @version 1.0
*/
long int nrand48(unsigned short xsubi[3]);

/**
* @brief Generates pseudo-random numbers evenly distributed between [-2^31, 2^31).
*
* @attention This function works by generating a 48-bit integer sequence, and <b>X(i+1)</b> must be generated by using the value of <b>Xi</b>. The correlation is obtained by using a linear congruential formula, Xn+1 = (aXn + c) mod m, where n = 0, m = 2^48, a = 0x5DEECE66D, and c = 0xB.
* Before this function is called, one of the functions <b>srand48()</b>, <b>seed48()</b>, and <b>lcong48()</b> must be used to initialize <b>Xi</b>.
* @return Returns signed long integers evenly distributed between [-2^31, 2^31) if the operation is successful.
* @since 1.0
* @version 1.0
*/
long mrand48(void);

/**
* @brief Generates pseudo-random numbers evenly distributed between [-2^31, 2^31).
*
* @attention This function works by generating a 48-bit integer sequence, and <b>X(i+1)</b> must be generated by using the value of <b>Xi</b>. The correlation is obtained by using a linear congruential formula, Xn+1 = (aXn + c) mod m, where n = 0, m = 2^48, a = 0x5DEECE66D, and c = 0xB.
* <b>Xi</b> is specified by <b>xsubi[3]</b>.
* @param xsubi[3] Indicates the array of the initialized value of <b>Xi</b>.
* @return Returns signed long integers evenly distributed between [-2^31, 2^31) if the operation is successful.
* @since 1.0
* @version 1.0
*/
long jrand48(unsigned short xsubi[3]);

/**
* @brief Sets the start seed value for the pseudo-random number generator.
*
* @attention This function sets the high order 32 bits of <b>Xi</b> to <b>seedval</b>. The lower order 16 bits are set to <b>0x330E</b>.
* @param seedval Indicates the 48-bit start seed value.
* @since 1.0
* @version 1.0
*/
void srand48(long int seedval);

/**
* @brief Generates an evenly distributed pseudo-random seed.
*
* @attention This function is an initialization function and can be called before calling {@link drand48}, {@link lrand48}, or {@link mrand48}.
* @param seed16v Indicates the 48-bit seed.
* @return Returns the pointer to the pseudo-random seed.
* @since 1.0
* @version 1.0
*/
unsigned short *seed48(unsigned short [3]);

/**
* @brief Sets the seed and related algorithm parameters for the pseudo-random number generator.
*
* @attention This function works by generating a 48-bit integer sequence, and <b>X(i+1)</b> must be generated by using the value of <b>Xi</b>. The correlation is obtained by using a linear congruential formula, Xn+1 = (aXn + c) mod m, where n = 0, m = 2^48, a = 0x5DEECE66D, and c = 0xB.
* <b>param[0 - 2]</b> is the initialized value of <b>Xi</b>. <b>param [3-5]</b> specifies the value of <b>a</b>, and <b>param[6]</b> specifies the value of <b>c</b>. After this function is called, calling <b>srand48 ()</b> or <b>seed48 ()</b> will restore the standard values of <b>a</b> and <b>c</b>.
* @param param[7] Indicates the parameters of the random number generator.
* @since 1.0
* @version 1.0
*/
void lcong48(unsigned short param[7]);

/**
* @brief Creates a unique temporary file name.
*
* @param template Indicates the name of the file to be created. The last six characters are <b>XXXXXX</b>.
* @return Returns the pointer to the created file name if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -----------------------
* EINVAL | The last six characters are not <b>XXXXXX</b>.
* ENOENT | The file name already exists.
* @since 1.0
* @version 1.0
*/
char *mktemp(char *template);

/**
* @brief Creates and opens a unique temporary file.
*
* @param template Indicates the name of the file to be created.
* @param suffixlen Indicates the length of the placeholder suffix <b>XXXXXX</b>.
* @return Returns the descriptor of the opened file if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | The <b>template</b> length is less than (6 + <b>suffixlen</b>), or the last six characters of <b>template</b> are not <b>XXXXXX</b>.
* EEXIST | Failed to create a unique temporary file name.
* Other | For details, see {@link open()}.
* @see open
* @since 1.0
* @version 1.0
*/
int mkstemps(char *template, int suffixlen);


/**
* @brief Creates and opens a unique temporary file.
*
* @param template Indicates the name of the file to be created.
* @param suffixlen Indicates the length of the placeholder suffix <b>XXXXXX</b>.
* @param flags Indicates the flag for creating a file.
* @return Returns the descriptor of the opened file if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | The <b>template</b> length is less than (6 + <b>suffixlen</b>), or the last six characters of <b>template</b> are not <b>XXXXXX</b>.
* EEXIST | Failed to create a unique temporary file name.
* Other | For details, see {@link open()}.
* @see open
* @since 1.0
* @version 1.0
*/
int mkostemps(char *template, int suffixlen, int flags);

/**
* @brief Allocates memory with the specified size and aligns the allocated memory by page size.
*
* @param size Indicates the size of the memory block to be allocated.
* @return Returns the pointer to the allocated memory block if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails and <b>size</b> is <b>0</b>.
* errno | Description
* -------|---------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *valloc(size_t size);

/**
* @brief Converts a double-precision floating-point number into a string.
*
* The conversion result does not contain decimal places. \n
* @param number Indicates the double-precision floating-point number to be converted.
* @param ndigits Indicates the number of valid digits to be stored.
* @param decpt Indicates the decimal point position.
* @param sign Indicates the symbol of the number to be converted.
* @return Returns the pointer to the generated string.
* @since  1.0
* @version  1.0
*/
char *ecvt(double number, int ndigits, int *decpt, int *sign);

/**
* @brief Converts a floating-point number to a string.
*
* @param number Indicates the floating-point number to be converted.
* @param ndigits Indicates the number of decimal places.
* @param decpt Indicates the position of the decimal point, which can be obtained by users.
* @param sign Indicates the symbol of <b>number</b>, which can be obtained by users. <b>0</b> indicates a positive number, and <b>1</b> indicates a negative number.
* @return Returns the converted string.
* @since  1.0
* @version  1.0
*/
char *fcvt(double number, int ndigits, int *decpt, int *sign);

/**
* @brief Converts a floating-point number to a string.
*
* @param x Indicates the floating-point number to convert.
* @param n Indicates the number of decimal places.
* @param b Indicates the pointer to the output conversion result.
* @return Returns the converted string.
* @since  1.0
* @version  1.0
*/
char *gcvt(double x, int n, char *b);

#ifdef __cplusplus
}
#endif

#endif
/** @} */
