/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file string.h
*
* @brief Declares commonly used functions for string operations.
*
* You can use the functions provided in this file to perform the mathematical operations required during development. The operations include copying, \n
* searching for, locating, and appending a string, as well as comparing two strings. \n
* You must pay attention to memory management during function calls. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef	 _STRING_H
#define	 _STRING_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Copies a string (overlapping not allowed).
*
* This function copies <b>n</b> bytes from the source memory area pointed to by <b>src</b> to the destination memory area pointed to by <b>dest</b>. The two memory areas cannot overlap. If they overlap, use {@link memmove()} instead. \n
* @param dest Indicates the pointer to the destination memory area.
* @param src Indicates the pointer to the source memory area.
* @param n Indicates the size of the string to be copied, in bytes.
* @return Returns the pointer (<b>dest</b>) to the destination memory area.
* @since 1.0
* @version 1.0
*/
void *memcpy(void *__restrict dest, const void *__restrict src, size_t n);

/**
* @brief Copies a string (overlapping allowed).
*
* This function copies <b>n</b> bytes from the source memory area pointed to by <b>src</b> to the destination memory area pointed to by <b>dest</b>. The two memory areas can overlap. \n
* @param dest Indicates the pointer to the destination memory area.
* @param src Indicates the pointer to the source memory area.
* @param n Indicates the size of the string to be copied, in bytes.
* @return Returns the pointer (<b>dest</b>) to the destination memory area.
* @see memcpy()
* @since 1.0
* @version 1.0
*/
void *memmove(void *dest, const void *src, size_t n);

/**
* @brief Copies a character to the specified memory area.
*
* This function copies the character specified by <b>c</b> to the n-byte memory area pointed to by <b>s</b>. \n
* @param s Indicates the pointer to the memory area to hold the character.
* @param c Indicates the character to be copied.
* @param n Indicates the size of the memory area.
* @return Returns the pointer (<b>s</b>) to the memory area.
* @since 1.0
* @version 1.0
*/
void *memset(void *s, int c, size_t n);

/**
* @brief Compares two memory areas.
*
* This function checks whether the first <b>n</b> bytes of the two memory areas pointed to by <b>s1</b> and <b>s2</b> are equal. \n
* @param s1 Indicates the pointer to memory area 1 for comparison.
* @param s2 Indicates the pointer to memory area 2 for comparison.
* @param n Indicates the size of the memory area to be compared, in bytes.
* @return Returns <b>0</b> if the first <b>n</b> bytes of two memory areas are the same or <b>n</b> is 0; returns the difference between the first pair of bytes that differ in <b>s1</b> and <b>s2</b> if the contents of two memory areas are different.
* @since 1.0
* @version 1.0
*/
int memcmp(const void *s1, const void *s2, size_t n);

/**
* @brief Searches for a character in the specified memory area.
*
* This function searches for the first occurrence of the character specified by <b>c</b> in the n-byte memory area pointed to by <b>s</b>. Both the character <b>c</b> and the memory area pointed to by <b>s</b> are interpreted as unsigned characters. \n
* @param s Indicates the pointer to the start of the bytes to be searched.
* @param c Indicates the character to be searched for.
* @param n Indicates the length of the memory area to be searched.
* @return Returns the pointer to the character if found; returns <b>NULL</b> if no matching character is found.
* @since 1.0
* @version 1.0
*/
void *memchr(const void *s, int c, size_t n);

/**
* @brief Copies a string.
*
* This function copies the source string pointed to by <b>s</b> to the destination string pointed to by <b>s</b> and returns the pointer to the destination string. \n
* @attention The terminating null byte <b>\0</b> in the source string is also copied to the destination string. The two strings cannot overlap.
* The destination string must be large enough to receive the copied string. You should be aware of a buffer overflow.
* @param dest Indicates the pointer to the destination string.
* @param src Indicates the pointer to the source string.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
char *strcpy(char *dest, const char *src);

/**
* @brief Copies <b>n</b> characters of a string.
*
* This function copies <b>n</b> bytes of the source string pointed to by <b>src</b> to the destination string pointed to by <b>dest</b> and returns the pointer to the destination string. \n
* @attention The null byte <b>\0</b> in the source string is also copied to the destination string. The two strings cannot overlap.
* The destination string must be large enough to receive the copied string. You should be aware of a buffer overflow. If the length of the source string
* is less than <b>n</b>, this function copies the null byte <b>\0</b> to the destination string.
* @param dest Indicates the pointer to the destination string.
* @param src Indicates the pointer to the source string.
* @param n Indicates the number of characters to be copied from the source string.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
char *strncpy(char *dest, const char *src, size_t n);

/**
* @brief Appends a string to another one.
*
* This function appends the source string to the destination string. \n
* @attention The null byte <b>\0</b> of the destination string will be overwritten by the first byte of the source string. The destination string must have enough space for data storage.
* Otherwise, a buffer overflow may occur. If the destination string is not large enough, the program behavior will be unpredictable, and buffer overflows are the favorite choice for attacking secure programs.
* @param dest Indicates the pointer to the destination string.
* @param src Indicates the pointer to the source string.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
char *strcat(char *dest, const char *src);

/**
* @brief Appends the first <b>n</b> bytes of a string to another one.
*
* This function appends the first <b>n</b> bytes of the source string to the destination string. \n
* @attention The null byte <b>\0</b> of the destination string will be overwritten by the first byte of the source string. The destination string must have enough space for data storage.
* Otherwise, a buffer overflow may occur. If the destination string is not large enough, the program behavior will be unpredictable, and buffer overflows are the favorite choice for attacking secure programs.
* @param dest Indicates the pointer to the destination string.
* @param src Indicates the pointer to the source string.
* @param n Indicates the number of bytes to be appended.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
char *strncat(char *dest, const char *src, size_t n);

/**
* @brief Compares two strings by characters.
*
* @attention The two strings are compared based on the ASCII code.
* @param s1 Indicates the pointer to string 1 for comparison.
* @param s2 Indicates the pointer to string 2 for comparison.
* @return Returns <b>0</b> if the operation is successful (the two strings are the same);
* returns the result of the ASCII character of string 1 minus the ASCII character of string 2 if the first pair of characters is at different positions in string 1 and string 2.
* @since 1.0
* @version 1.0
*/
int strcmp(const char *s1, const char *s2);

/**
* @brief Compares the first n characters of two strings.
*
* @attention The two strings are compared based on the ASCII code.
* @param s1 Indicates the pointer to string 1 for comparison.
* @param s2 Indicates the pointer to string 2 for comparison.
* @param n Indicates the number of characters for comparison.
* @return Returns <b>0</b> if the operation is successful (the first n characters of the two strings are the same); returns
* the result of the ASCII character of string 1 minus the ASCII character of string 2 if the first pair of characters is at different positions in string 1 and string 2.
* @since 1.0
* @version 1.0
*/
int strncmp(const char *s1, const char *s2, size_t n);

/**
* @brief Compares two strings by character for the program's current locale.
*
* @attention By default, the program's current locale specified by <b>LC_COLLATE</b> is POSIX or C.
* @param s1 Indicates the pointer to string 1 for comparison.
* @param s2 Indicates the pointer to string 2 for comparison.
* @return Returns <b>0</b> if the operation is successful (the two strings are the same); returns an integer less than <b>0</b> if the value of a character in string 1 is less than the value of the corresponding character in string 2 when the first unmatched character appears;
returns an integer greater than <b>0</b> if the value of a character in string 1 is greater than the value of the corresponding character in string 2 when the first unmatched character appears.
* @since 1.0
* @version 1.0
*/
int strcoll(const char *s1, const char *s2);

/**
* @brief Compares two strings by character for the specified locale.
*
* @param s1 Indicates the pointer to string 1 for comparison.
* @param s2 Indicates the pointer to string 2 for comparison.
* @param locale Indicates the locale. This parameter is ignored currently
* @return Returns <b>0</b> if the operation is successful (the two strings are the same); returns an integer less than <b>0</b> if the value of a character in string 1 is less than the value of the corresponding character in string 2 when the first unmatched character appears;
returns an integer greater than <b>0</b> if the value of a character in string 1 is greater than the value of the corresponding character in string 2 when the first unmatched character appears.
* @since 1.0
* @version 1.0
*/
int strcoll_l(const char *s1, const char *s2, locale_t locale);

/**
* @brief Converts the first n characters of the source string pointed to by <b>src</b> based on the program's current locale specified by {@link LC_COLLATE}, and places them in the destination string pointed to by <b>dest</b>.
*
* @attention If the return value is greater than or equal to <b>n</b>, the content of the destination string is uncertain.
* @param dest Indicates the pointer to the destination string.
* @param src Indicates the pointer to the source string.
* @param n Indicates the maximum string space.
* @return Returns the number of bytes in the destination string (excluding the terminating null byte <b>`\0'</b>).
* @since 1.0
* @version 1.0
*/
size_t strxfrm(char *dest, const char *src, size_t n);


/**
* @brief Locates the first occurrence of a character in a string.
*
*  \n
* @param s Indicates the pointer to the string to be searched.
* @param c Indicates the character to be searched for.
* @return Returns the pointer to the position that the character appears for the first time in the string if the operation is successful; returns <b>NULL</b> if the character is not found in the string.
* @since 1.0
* @version 1.0
*/
char *strchr(const char *s, int c);

/**
* @brief Locates the last occurrence of a character in a string.
*
* @param s Indicates the pointer to the string to be searched.
* @param c Indicates the character to be searched for.
* @return Returns the pointer to the position that the character appears for the last time in the string if the operation is successful; returns <b>NULL</b> if the character is not found in the string.
* @since 1.0
* @version 1.0
*/
char *strrchr(const char *s, int c);

/**
* @brief Obtains the length of the initial segment of a string that contains characters not in reject.
*
* This function is used to determine whether the characters in the string pointed to by <b>s</b> are in the string pointed to by <b>reject</b>. \n
* @param s Indicates the pointer to the string to be retrieved.
* @param reject Indicates the pointer to the string containing all unmatched characters.
* @return Returns the total number of consecutive unmatched characters in the initial segment of the string pointed to by <b>s</b> if the operation is successful; returns the length of the string pointed to by <b>s</b>
* if its characters cannot be found in the string pointed to by <b>reject</b>.
* @since 1.0
* @version 1.0
*/
size_t strcspn(const char *s, const char * reject);

/**
* @brief Obtains the length of the initial segment of a string that contains characters in accept.
*
* This function is used to determine whether the characters in the string pointed to by <b>s</b> are in the string pointed to by <b>accept</b>. \n
* @param s Indicates the pointer to the string to be retrieved.
* @param accept Indicates the pointer to the string containing all matched characters.
* @return Returns the total number of consecutive matched characters in the initial segment of the string pointed to by <b>s</b> if the operation is successful; returns <b>0</b> if its characters cannot be found in the string pointed to by <b>accept</b>.
* @since 1.0
* @version 1.0
*/
size_t strspn(const char *s, const char *accept);

/**
* @brief Searches for any of a set of characters in a string.
*
* This function is used to determine whether the characters in the string pointed to by <b>s</b> are in the string pointed to by <b>accept</b>. \n
* @param s Indicates the pointer to the string to be searched.
* @param accept Indicates the pointer to the string containing all matched characters.
* @return Returns the pointer to a character in the string pointed to by <b>s</b> that first matches one of the characters in the string pointed to by <b>accept</b> if the operation is successful; returns <b>NULL</b> if no characters are found in the string pointed to by <b>accept</b>.
* @since 1.0
* @version 1.0
*/
char *strpbrk(const char *s, const char *accept);

/**
* @brief Searches for a needle string in its haystack string.
*
* @attention The null byte <b>\0</b> in the needle string is not searched for.
* @param haystack Indicates the pointer to the haystack string.
* @param needle Indicates the pointer to the needle string.
* @return Returns the pointer to the first occurrence of the needle string in the haystack string if the operation is successful; returns <b>NULL</b> if the needle string is not found in the haystack string.
* @since 1.0
* @version 1.0
*/
char *strstr(const char *haystack, const char *needle);


/**
* @brief Separates a string into a series of tokens separated by a delimiter.
*
* @attention In each call of this function, the string is separated only once. You can call this function multiple times to separate the string into a set of tokens.
* @param str Indicates the pointer to the first token in the string to be separated. This parameter is set to <b>NULL</b> in subsequent calls to this function.
* @param delim Indicates the pointer to the delimiter used to separate tokens.
* @return Returns the pointer to the first token in the string if the operation is successful; returns <b>NULL</b> if no delimiter is found in the string.
* @since 1.0
* @version 1.0
*/
char *strtok(char *str, const char *delim);

/**
* @brief Calculates the length of a string.
*
* The length of a string does not include the terminating null byte <b>\0</b> of the string. \n
* @param s Indicates the pointer to the string.
* @return Returns the string length.
* @since 1.0
* @version 1.0
*/
size_t strlen(const char *s);

#include <strings.h>

/**
* @brief Obtains an error description string of the specified error code.
*
* This function queries the string representation of the error description matching the error code specified by <b>errnum</b>. \n
* @param errnum Indicates the error code.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno  | description
* ------ | ---------------------------------------------
* EINVAL | <b>errnum</b> is invalid.
* @since 1.0
* @version 1.0
*/
char *strerror(int errnum);

/**
* @brief Separates a string into a series of tokens separated by a delimiter, with the <b>saveptr</b> parameter specified.
*
* @attention In each call of this function, the string is separated only once. You can call this function multiple times to separate the string into a set of tokens.
* <b>saveptr</b> is specified to maintain context between successive calls that parse the same string.
* @param str Indicates the pointer to the first token in the string to be separated. This parameter is set to <b>NULL</b> in subsequent calls to this function.
* @param delim Indicates the pointer to the delimiter used to separate tokens.
* @param saveptr Indicates the double pointer to the <b>char *</b> variable.
* @return Returns the pointer to the first token in the string if the operation is successful; returns <b>NULL</b> if no delimiter is found in the string.
* @since 1.0
* @version 1.0
*/
char *strtok_r(char *str, const char *delim, char **saveptr);

/**
* @brief Obtains an error description string of the specified error code for the specified locale.
*
* This function queries the string representation of the error description matching the error code specified by <b>errnum</b>. \n
* @param errnum Indicates the error code.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the appropriate error description string if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno  | description
* ------ | ---------------------------------------------
* EINVAL | <b>errnum</b> is invalid.
* ERANGE | The buffer is insufficient to contain the error description string.
* @since 1.0
* @version 1.0
*/
char *strerror_l(int errnum, locale_t locale);

/**
* @brief Obtains an error description string of the specified error code.
*
* This function queries the string representation of the error description matching the error code specified by <b>errnum</b>. \n
* @param errnum Indicates the error code.
* @param buf Indicates the pointer to the buffer for storing the error description string.
* @param buflen Indicates the length of the <b>buf</b> parameter.
* @return Returns the appropriate error description string if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno  | description
* ------ | ---------------------------------------------
* EINVAL | <b>errnum</b> is invalid.
* ERANGE | The buffer is insufficient to contain the error description string.
* @since 1.0
* @version 1.0
*/
char* strerror_r(int errnum, char *buf, size_t buflen);

/**
* @brief Copies a string.
*
* This function copies the source string to the destination string and returns the pointer to the terminating null byte <b>\0</b> of the destination string. \n
* @attention The null byte <b>\0</b> in the source string is also copied to the destination string. The two strings cannot overlap.
* The destination string must be large enough to receive the copied string. You should be aware of a buffer overflow.
* @param dest Indicates the pointer to the destination string.
* @param src Indicates the pointer to the source string.
* @return Returns the pointer to the terminating null byte <b>\0</b> of the destination string.
* @since 1.0
* @version 1.0
*/
char *stpcpy(char *dest, const char *src);

/**
* @brief Copies n characters of a string.
*
* This function copies n characters of the source string to the destination string and returns the pointer to the terminating null byte <b>\0</b> of the destination string. \n
* @attention The null byte <b>\0</b> in the source string is also copied to the destination string. The two strings cannot overlap.
* The destination string must be large enough to receive the copied string. You should be aware of a buffer overflow. If the length of the source string
* is less than <b>n</b>, this function copies the null byte <b>\0</b> to the destination string.
* @param dest Indicates the pointer to the destination string.
* @param src Indicates the pointer to the source string.
* @param n Indicates the number of characters to be copied.
* @return Returns the pointer to the terminating null byte <b>\0</b> of the destination string if the operation is successful; returns the pointer to the <b>(dest+n)</b> position if the destination string does not end with <b>\0</b>.
* @since 1.0
* @version 1.0
*/
char *stpncpy(char *dest, const char *src, size_t n);

/**
* @brief Calculates the length of a string.
*
* The actual length of a string does not include the terminating null byte <b>\0</b> of the string. \n
* @param s Indicates the pointer to the string.
* @param maxlen Indicates the maximum length of the string.
* @return Returns the actual length of the string if it is less than the maximum length specified by <b>maxlen</b>; returns the value of <b>maxlen</b> if the actual length is greater than the maximum length specified by <b>maxlen</b>.
* @since 1.0
* @version 1.0
*/
size_t strnlen(const char *s, size_t maxlen);

/**
* @brief Copies a string to a new position.
*
* @attention The memory space is dynamically allocated in the function. Therefore, when the returned string is not required, the allocated memory space needs to be released. Otherwise, memory leakage occurs.
* @param s Indicates the pointer to the string to be copied.
* @return Returns the pointer to the memory space holding the copied string if the operation is successful; returns <b>NULL</b> if the memory space for the copied string is insufficient.
* @since 1.0
* @version 1.0
*/
char *strdup(const char *s);

/**
* @brief Copies n characters of a string to a new position.
*
* @attention The memory space is dynamically allocated in the function. Therefore, when the returned string is not required, the allocated memory space needs to be released. Otherwise, memory leakage occurs.
* If the string is longer than <b>n</b>, only <b>n-1</b> characters are copied and a terminating null byte <b>\0</b> is added.
* @param s Indicates the pointer to the string to be copied.
* @param n Indicates the total number of characters to be copied.
* @return Returns the pointer to the memory space holding the copied string if the operation is successful; returns <b>NULL</b> if the memory space for the copied string is insufficient.
* @since 1.0
* @version 1.0
*/
char *strndup(const char *s, size_t n);

/**
* @brief Returns a string describing the signal number.
*
* @param sig Indicates the signal number to be parsed.
* @return Returns the signal description associated with the specified signal number if the operation is successful; returns <b>NULL</b> or an unknown signal message for an invalid signal number if the operation fails.
* @since 1.0
* @version 1.0
*/
char *strsignal(int sig);

/**
* @brief Copies a memory area to another one.
*
* This function copies no more than <b>n</b> bytes from the source memory area pointed to by <b>src</b> to the destination memory area pointed to by <b>dest</b>. The copy stops when a character specified by <b>c</b> is found. The result for two overlapped memory areas is undefined.\n
* @param dest Indicates the pointer to the destination memory area.
* @param src Indicates the pointer to the source memory area.
* @param c Indicates the character for stopping copying.
* @param n Indicates the size of the string to be copied, in bytes.
* @return Returns the pointer to the next character of the character <b>c</b> in the destination memory area if the operation is successful; returns <b>NULL</b> if the character <b>c</b> is not found in the n bytes of the source memory area.
* @since 1.0
* @version 1.0
*/
void *memccpy(void *__restrict dest, const void *__restrict src, int c, size_t n);


/**
* @brief Separates a string into a series of tokens separated by a delimiter.
*
* The function starts from <b>*stringp</b> and checks whether the string matches a character in the string representing the delimiter. This operation is terminated by overwriting the delimiter with a null byte <b>\0</b>. \n
* @param stringp Indicates the pointer to the first token of the source string.
* @param delim Indicates the pointer to the string representing the delimiter.
* @return Returns the pointer to the first token; returns <b>NULL</b> if <b>*stringp</b> is <b>NULL</b> or the delimiter is not found in the source string.
* @since 1.0
* @version 1.0
*/
char *strsep(char **stringp, const char *delim);

/**
* @brief Appends the first <b>n</b> bytes of a string to another one.
*
* This function appends the first <b>n</b> bytes of the source string to the destination string. \n
* @attention The null byte <b>\0</b> of the destination string will be overwritten by the first byte of the source string, and the source string will end with <b>\0</b>.
* @param d Indicates the pointer to the destination string.
* @param s Indicates the pointer to the source string.
* @param n Indicates the number of characters to be copied from the source string.
* @return Returns <b>strlen(src)</b> plus <b>MIN(n, strlen(initial d))</b>.
* @since 1.0
* @version 1.0
*/
size_t strlcat(char *d, const char *s, size_t n);

/**
* @brief Copies a string.
*
* This function copies n bytes of the source string pointed to by <b>s</b> to the destination string pointed to by <b>d</b>. \n
* @attention If the value of <b>n</b> is the same as the length of the destination string, the last character of the destination string will be <b>\0</b>.
* @param d Indicates the pointer to the destination string.
* @param s Indicates the pointer to the source string.
* @param n Indicates the number of characters to be copied from the source string.
* @return Returns the length of the source string, excluding the null byte <b>\0</b>.
* @since 1.0
* @version 1.0
*/
size_t strlcpy(char *d, const char *s, size_t n);

/**
* @brief Copies a string to a new position.
*
* @attention {@link strdupa()} internally calls {@link alloca()} to allocate space for variables. You should pay attention to the stack space size.
* @return Returns the pointer to the space allocated for the copied string if the operation is successful; returns <b>NULL</b> if the space fails to be allocated.
* @since 1.0
* @version 1.0
*/
#define	 strdupa(x)	 strcpy(alloca(strlen(x)+1),x)

/**
* @brief Compares strings of two versions (string 1 and string 2) and returns the comparison result.
*
* @attention {@link strcmp} compares two strings based on the lexicographic order, while this function compares two strings based on the locale specified by {@link LC_COLLATE}.
* @param s1 Indicates the pointer to string 1 for comparison.
* @param s2 Indicates the pointer to string 2 for comparison.
* @return Returns any of the following values: Assuming that a difference is found at the xth character,
* Input | Return Value
*  -------| --------
* The value for s1[x] is equal to that for s2[x]. | The difference between the values
* The values for the first n characters of string 1 and string 2 are the same. | <b>0</b>
* @since 1.0
* @version 1.0
*/
int strverscmp(const char *s1, const char *s2);

/**
* @brief Searches for a needle string in its haystack string and returns a pointer.
*
* @attention The case difference is ignored during comparison.
* @param haystack Indicates the pointer to the haystack string.
* @param needle Indicates the pointer to the needle string.
* @return Returns the pointer to the beginning of the first needle string found in the haystack string if the operation is successful; returns <b>NULL</b> if the needle string is not found in the haystack string.
* @since 1.0
* @version 1.0
*/
char *strcasestr(const char *haystack, const char *needle);

/**
* @brief Searches for a needle string in its haystack string.
*
* This function searches for the start position of the first occurrence of the needle string whose length is <b>needlelen</b> in the memory area holding the haystack string which starts at the position pointed to by <b>haystack</b>. \n
* @attention This is an extended GNU function.
* @param haystack Indicates the pointer to the start position of the memory area to be searched.
* @param haystacklen Indicates the length (in bytes) of the memory area to be searched.
* @param needle Indicates the pointer to the start position of the needle string to be searched for.
* @param needlelen Indicates the length (in bytes) of the needle string to be searched for.
* @return Returns the start position of the needle string in the memory area if the position is found; returns <b>NULL</b> if the position is not found.
* @since 1.0
* @version 1.0
*/
void *memmem(const void *haystack, size_t haystacklen, const void *needle, size_t needlelen);

/**
* @brief Searches for a character in the specified memory area.
*
* This function searches for the first occurrence of the character specified by <b>c</b> in the n-byte memory area pointed to by <b>s</b>. This function searches backward from the end of the n bytes pointed to by s, while the {@link memchr()} function searches forward from the beginning. \n
* @param s Indicates the pointer to the start of the bytes to be searched.
* @param c Indicates the character to be searched for.
* @param n Indicates the length of the memory area to be searched.
* @return Returns the pointer to the character if found; returns <b>NULL</b> if no matching character is found.
* @since 1.0
* @version 1.0
*/
void *memrchr(const void *s, int c, size_t n);

/**
* @brief Copies a string (overlapping not allowed).
*
* This function copies <b>n</b> bytes from the source memory area pointed to by <b>src</b> to the destination memory area pointed to by <b>dest</b>. The two memory areas cannot overlap. This function is similar to {@link memcpy()} except for the return value. This function returns the pointer to the last byte written during the copy. \n
* @param dest Indicates the pointer to the destination memory area.
* @param src Indicates the pointer to the source memory area.
* @param n Indicates the size of the string to be copied, in bytes.
* @return Returns the pointer to the last byte written during the copy.
* @since 1.0
* @version 1.0
*/
void *mempcpy(void *dest, const void *src, size_t n);

#ifdef __cplusplus
}
#endif

#endif
/** @} */
