/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file strings.h
*
* @brief Declares commonly used functions for byte sequence operations.
*
* You can use the functions provided in this file to perform the mathematical operations required during development. Example operations include comparing, \n
* copying byte sequences, and setting a byte sequence to 0. \n
* You must pay attention to memory management during function calls. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef	 _STRINGS_H
#define	 _STRINGS_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Compares byte sequences.
*
* Specifically, this function checks whether the first <b>n</b> bytes of the two byte sequences pointed to by <b>s1</b> and <b>s2</b> are equal. If they are equal or <b>n</b> is 0, this function returns <b>0</b>. Otherwise, it returns a non-zero value. \n
* @attention This function is deprecated (marked as LEGACY in POSIX.1-2001).
* Its specifications have been removed since POSIX.1-2008. You can use {@link memcmp()} in new projects.
* @param s1 Indicates the pointer to byte sequence 1 for comparison.
* @param s2 Indicates the pointer to byte sequence 2 for comparison.
* @param n Indicates the number of bytes to be compared.
* @return Returns <b>0</b> if the two byte sequences are equal or <b>n</b> is 0; returns a non-zero value if the operation fails.
* @since 1.0
* @version 1.0
*/
int bcmp(const void *s1, const void *s2, size_t n);
/**
* @brief Copies byte sequences.
*
* Specifically, this function copies the first <b>n</b> bytes from the memory area pointed to by <b>src</b> to that pointed to by <b>dest</b>. This function can always be called successfully even if the two specified areas overlap. \n
* @attention This function is deprecated (marked as LEGACY in POSIX.1-2001).
* Its specifications have been removed since POSIX.1-2008. You can use {@link memcpy()} and {@link memmove()} in new projects.
* @param src Indicates the pointer to the source memory area.
* @param dest Indicates the pointer to the destination memory area.
* @param n Indicates the number of bytes to be copied.
* @since 1.0
* @version 1.0
*/
void bcopy(const void *src, void *dest, size_t n);
/**
* @brief Sets byte sequences to zero.
*
* Specifically, this function sets the first <b>n</b> bytes in the memory area starting at <b>s</b> to zero. \n
* @attention This function is deprecated (marked as LEGACY in POSIX.1-2001).
* Its specifications have been removed since POSIX.1-2008. You can use {@link memset()} in new projects.
* @param s Indicates the pointer to the first token in the byte sequence to be set to zero.
* @param n Indicates the number of bytes to be set to zero.
* @since 1.0
* @version 1.0
*/
void bzero(void *s, size_t n);

/**
* @brief Searches for the first position of the matched character in a string.
*
* @param s Indicates the pointer to the string to be searched.
* @param c Indicates the character to be matched.
* @return Returns the pointer to the matched character if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
char *index(const char *s, int c);

/**
* @brief Searches for the last position of the matched character in a string.
*
* @param s Indicates the pointer to the string to be searched.
* @param c Indicates the character to be matched.
* @return Returns the pointer to the matched character if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
char *rindex(const char *s, int c);

/**
* @brief Searches for the first bit in a word of the integer type.
*
* Specifically, this function searches for the least significant bit (position 1) set in the word. \n
* @param i Indicates the long integer to be searched.
* @return Returns the position of the first bit if the operation is successful; returns <b>0</b> if no bits are set in the word.
* @since  1.0
* @version  1.0
*/
int ffs(int i);

/**
* @brief Searches for the first bit in a word of the long integer type.
*
* Specifically, this function searches for the least significant bit (position 1) set in the word. \n
* @param i Indicates the long integer to be searched.
* @return Returns the position of the first bit if the operation is successful; returns <b>0</b> if no bits are set in the word.
* @since  1.0
* @version  1.0
*/
int ffsl(long int i);

/**
* @brief Searches for the first bit in a word of the 8-byte long integer type.
*
* Specifically, this function searches for the least significant bit (position 1) set in the word.  \n
* @param i Indicates the 8-byte long integer to be searched.
* @return Returns the position of the first bit if the operation is successful; returns <b>0</b> if no bits are set in the word.
* @since  1.0
* @version  1.0
*/
int ffsll(long long int i);

/**
* @brief Compares two strings (string 1 and string 2), regardless of the letter case.
*
* @param _l Indicates the pointer to string 1 for comparison.
* @param _r Indicates the pointer to string 2 for comparison.
* @return Returns <b>0</b> if the two strings are equal; returns a value greater than <b>0</b> if <b>_l</b> is greater than <b>_r</b>; returns a value less than <b>0</b> if <b>_l</b> is less than <b>_r</b>.
* @since 1.0
* @version 1.0
*/
int strcasecmp(const char *_l, const char *_r);

/**
* @brief Compares a specified length of two strings (string 1 and string 2), regardless of the letter case.
*
* @param _l Indicates the pointer to string 1 for comparison.
* @param _r Indicates the pointer to string 2 for comparison.
* @param n Indicates the string length to be compared.
* @return Returns <b>0</b> if the two strings are equal; returns a value greater than <b>0</b> if <b>_l</b> is greater than <b>_r</b>; returns a value less than <b>0</b> if <b>_l</b> is less than <b>_r</b>.
* @since 1.0
* @version 1.0
*/
int strncasecmp(const char *_l, const char *_r, size_t n);

#ifdef __cplusplus
}
#endif

#endif
/** @} */