/**
* @addtogroup PROCESS
* @{
*
* @brief Provides process- and thread-related structures and functions.
*
* @since 1.0
* @version 1.0
*/

/**
* @file capability.h
*
* @brief Provides functions and related data structures for obtaining and setting process capabilities. \n
*
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _SYS_CAPABILITY_H
#define _SYS_CAPABILITY_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Obtains the capability information of a specified process based on the input parameters.
*
* @param hdr_ptr Indicates the data structure required for this function call, including the API version and the target process identifier (PID).
* @param data_ptr Indicates the buffer space for storing the process capabilities.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
*  ----------   | ---------
* EINVAL | Incorrect parameter.
* EPERM | No permission.
* ESRCH | No process matching the specified PID is found.
* ENOTSUP | The version passed to <b>hdr_ptr</b> is not supported.
* @since  1.0
* @version  1.0
*/
int capget(cap_user_header_t hdr_ptr, cap_user_data_t data_ptr);

/**
* @brief Sets the capability information for a specified process based on the input parameters.
*
* @param hdr_ptr Indicates the data structure required for this function call, including the API version and the target PID.
* Only the version <b>_LINUX_CAPABILITY_VERSION_3 0x20080522</b> is supported.
* @param data_ptr Indicates the buffer space for storing the customized process capabilities.
* Currently, the following capabilities are supported (other values do not take effect):
* CAP_DAC_OVERRIDE: Ignores discretionary access control (DAC) restrictions.
* CAP_DAC_READ_SEARCH: Ignores DAC restrictions on file read and search.
* CAP_CHOWN: Changes the permissions of file ownership.
* CAP_SETUID: Allows changing the user ID of the process.
* CAP_SETGID: Allows changing the group ID of the process.
* CAP_KILL: Allows the current application to send signals to other applications with different UID.
* CAP_SETPCAP: Allows changing the capabilities.
* CAP_SYS_NICE: Allows priority increase and priority setting for other processes.
* CAP_SYS_TIME: Allows changing the system clock.
* CAP_FOWNER: Allows other users (not the file owner) to modify file permission configurations.
* The <b>cap_user_data_t</b> structure contains three members: <b>effective</b>, <b>permitted</b>, and <b>inheritable</b>. <b>inheritable</b> is not implemented.
* <b>effective</b> is included in <b>permitted</b>. <b>permitted</b> and <b>inheritable</b> can only be decreased.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the creation fails.
* errno | Description
*  ----------   | ---------
* EINVAL | Incorrect parameter.
* EPERM | No permission.
* ESRCH | No process matching the specified PID is found.
* ENOTSUP | The version passed to <b>hdr_ptr</b> is not supported.
* ENOMEM | Insufficient memory.
* @since  1.0
* @version  1.0
*/
int capset(cap_user_header_t hdr_ptr, const cap_user_data_t data_ptr);

#ifdef __cplusplus
}
#endif

#endif /* _SYS_CAPABILITY_H */