/**
* @addtogroup MEM
* @{
*
* @brief Provides structures and functions for performing memory operations, including memory mapping, remapping, allocation, and release.
*
* @since 1.0
* @version 1.0
*/

/**
* @file mman.h
*
* @brief Declares structures and functions for performing memory operations, including memory mapping, remapping, unmapping, and attribute setting.
*
*
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _SYS_MMAN_H
#define	 _SYS_MMAN_H
#ifdef __cplusplus
extern "C" {
#endif

/** @brief Indicates the return value of functions such as {@link mmap()} when the operation fails. */
#define MAP_FAILED ((void *) -1)

/** @brief Indicates the mapping attribute that the updates to the mapping are visible to other processes mapping the same file and are carried through to the underlying file. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define MAP_SHARED     0x01
/** @brief Indicates the mapping attribute that the updates to the mapping are not visible to other mapping processes and are not carried through to the underlying file. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define MAP_PRIVATE    0x02
/** @brief Indicates the mapping attribute that specifies the mapping as fixed mapping. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define MAP_FIXED      0x10
/** @brief Indicates the mapping attribute that specifies the mapping as anonymous mapping without a specified file or device. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define MAP_ANON       0x20
/** @brief Indicates the mapping attribute that specifies the mapping as anonymous mapping without a specified file or device. This macro is the synonym for {@link MAP_ANON} and is used as an input parameter passed to functions such as {@link mmap()}. */
#define MAP_ANONYMOUS  MAP_ANON

/** @brief Indicates that no permission is granted to the current process for accessing the mapping area. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define PROT_NONE      0
/** @brief Indicates that the current process is granted the read permission on the mapping area. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define PROT_READ      1
/** @brief Indicates that the current process is granted the write permission on the mapping area. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define PROT_WRITE     2
/** @brief Indicates that the current process is granted the execute permission on the mapping area. This macro is used as an input parameter passed to functions such as {@link mmap()}. */
#define PROT_EXEC      4

/** @brief Indicates the remapping attribute that allows the mapping to be relocated to a new address. This macro is used as an input parameter passed to functions such as {@link mremap()}. */
#define MREMAP_MAYMOVE 1
/** @brief Indicates the remapping attribute that specifies the mapping as fixed mapping. This macro is used as an input parameter passed to functions such as {@link mremap()}. */
#define MREMAP_FIXED 2

/**
* @brief Creates a mapping between the virtual address space of the calling process and a file or device.
*
* The start address for the mapping is specified by <b>addr</b>, and the length to map is specified by <b>length</b>.
* The contents of the mapping are initialized starting at <b>offset</b> with the specified <b>length</b> in the file referred to by the file descriptor <b>fd</b>.
* \n
* @attention If the file mapping is successful, <b>fd</b> cannot be closed before the mapping is deleted. (This rule does not conform to the Portable Operating System Interface (POSIX) standard. You should pay special attention to this rule.)
* @attention This function can only be used to mapping a file in the JFFS2/NFS/FAT/RAMFS file systems.
* @param addr Indicates the pointer to the start address of the mapping. If this parameter is <b>NULL</b>, the kernel determines the address to start.
* @param length Indicates the length of the mapping, in bytes.
* @param prot Indicates the permission to be granted on the mapping area. The permission to grant must not conflict with the open mode of the file. The value of this parameter is the bitwise OR combination of one or more of the following constants:
* prot | Description
* -----------|--------
* PROT_EXEC | Executable
* PROT_READ | Readable
* PROT_WRITE | Writable
* @param flags Specifies whether updates are visible to other processes mapping the same segment, and whether updates are carried through to the underlying file. The following table describes available values.
* flags | Description
* --------------|--------------------------------------------
* MAP_SHARED | Updates to the mapping are visible to other processes mapping the same file, and are carried through to the underlying file.
* MAP_PRIVATE | Updates to the mapping are invisible to other processes mapping the same file, and are not carried through to the underlying file.
* MAP_ANONYMOUS | No file or device is specified for the mapping (anonymous mapping). Both <b>fd</b> and <b>offset</b> are ignored.
* @param fd Indicates the file or device to map.
* @param offset Indicates the offset into the file where the mapping will start.
* @return Returns the pointer to the address where the mapping is placed if the operation is successful; returns {@link MAP_FAILED} and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|------------------------------------------------------------------------------
* EACCES | The file descriptor specified by <b>fd</b> refers to a non-regular file.<br>The file descriptor specified by <b>fd</b> is invalid.<br>{@link MAP_SHARED} and {@link PROT_WRITE} are specified, but the file identified by <b>fd</b> is not opened in {@link O_RDWR} mode.<br>{@link PROT_WRITE} is specified, but the file is append-only.
* EBADF | The file descriptor specified by <b>fd</b> is invalid, and {@link MAP_ANONYMOUS} is not specified in <b>flags</b>.
* EINVAL | The length and offset are too large, <b>addr</b> is not page-aligned, or the length is <b>0</b>.<br>Neither {@link MAP_SHARED} nor {@link MAP_ANONYMOUS} is specified in <b>flags</b>, or both are specified.
* EAGAIN | The file identified by <b>fd</b> has been locked.
* ENFILE | The total number of open files exceeds the system limit.
* ENODEV | The file identified by <b>fd</b> does not support memory mapping.
* ENOMEM | Insufficient memory.
* EPERM | {@link PROT_EXEC} is specified in <b>prot</b>, but the mapped area belongs to a file on a file system that was mounted non-executable, or the file seal does not allow this operation.
* @since 1.0
* @version 1.0
*/
void *mmap(void *addr, size_t length, int prot, int flags, int fd, off_t offset);

/**
* @brief Removes all mappings for the specified virtual address space.
*
* After all mappings are removed, any references to addresses within the specified range will generate invalid memory references.
* The memory region will also be automatically unmapped when the process is terminated. Closing the file descriptor does not remove mappings from the region. \n
* @param addr Indicates the pointer to the start address of the memory region to unmap, which is the return value of {@link mmap()}.
* @param length Indicates the length of the address range to unmap, which should be the length specified in {@link mmap()}.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|-----------
* EINVAL | Invalid input parameters.
* @since 1.0
* @version 1.0
*/
int munmap(void *addr, size_t length);

/**
* @brief Sets protection attributes for the memory pages contained in the memory region starting from <b>addr</b> with the specified length.
*
* The address specified by <b>addr</b> must be page-aligned.
* If the process attempts to access memory in a manner that violates the protection attributes, an access exception will occur, and the process will be terminated. \n
* @param addr Indicates the pointer to the start address of the memory region to modify, which must be a multiple of the page size.
* @param len Indicates the length of the memory region to modify, in bytes.
* @param prot Indicates the permission of the memory region to modify, which can be a bitwise OR combination of one or more of the constants listed in <b>prot</b>.
* prot | Description
* -----------|--------
* PROT_EXEC | Executable
* PROT_READ | Readable
* PROT_WRITE | Writable
* PROT_NONE | Not accessible
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|-----------------------------------------------------------------------------------------
* EACCES | The memory region cannot be granted the specified permission.<br>This error can occur, for example, when you use {@link mmap} to map a file with <b>prot</b> set to {@link PROT_READ} and then use this function to set <b>prot</b> to {@link PROT_WRITE}.
* EINVAL | <b>addr</b> is an invalid pointer or it points to an address that is not a multiple of the page size.
* ENOMEM | Internal kernel structures cannot be allocated due to insufficient memory, or addresses in the specified range are invalid for the address space of the process.<br>The total number of mappings with different attributes exceeds the maximum number allowed by the system if the protection attribute of the memory region is changed.
* @since 1.0
* @version 1.0
*/
int mprotect(void *addr, size_t len, int prot);

/**
* @brief Remaps a virtual memory region.
*
* This function expands or shrinks an existing memory mapping, and may also move the mapping at the same time depending on the settings of <b>flags</b> and the available virtual address space. \n
* @param old_address Indicates the old address of the virtual memory region to expand or shrink.
* @param old_size Indicates the old size of the virtual memory region.
* @param new_size Indicates the requested size of the virtual memory region after the expansion or shrinking.
* @param flags Specifies whether the existing memory can be mapped to a new or specified address.
* flags | Description
* ---------------|-------------------------------------------------------------------------------------------------
* MREMAP_MAYMOVE | By default, if no enough space is available to expand a mapping at its current location, the operation fails. If this flag is specified, the kernel is allowed to relocate the mapping to a new virtual address when necessary.
* MREMAP_FIXED | If this flag is specified, the <b>new_address</b> parameter is enabled in this function and the memory is mapped to a new address. This flag must be used together with {@link MREMAP_MAYMOVE}.
* @param new_address Indicates the new address of the virtual memory region. This parameter is optional and is used when {@link MREMAP_FIXED} is specified in <b>flags</b>.
* @return Returns the pointer to the new mapping address if the operation is successful; returns {@link MAP_FAILED} and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|-----------------------------------------------------------------------------------------------
* EAGAIN | The memory segment to expand is locked.
* EFAULT | Some memory addresses in the range from <b>old_address</b> to <b>old_address</b>+<b>old_size</b> are invalid for this process.
* EINVAL | <b>old_address</b> is not a multiple of the page size, or a value other than {@link MREMAP_MAYMOVE} and {@link MREMAP_FIXED} is specified in <b>flags</b>.<br><b>new_size</b> is <b>0</b>, <b>new_size</b> or <b>new_address</b> is invalid, or {@link MREMAP_FIXED} is specified in <b>flags</b> without also specifying {@link MREMAP_MAYMOVE}.<br><b>old_size</b> is <b>0</b> but <b>old_address</b> does not point to a shared memory mapping; <b>old_size</b> is <b>0</b> but {@link MREMAP_MAYMOVE} is not specified in <b>flags</b>; or the new memory address range specified by <b>new_address</b> and <b>new_size</b> overlaps the old memory address range specified by <b>old_address</b> and <b>old_size</b>.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *mremap(void *old_address, size_t old_size, size_t new_size, int flags, ...);

#ifdef __cplusplus
}
#endif
#endif
/** @} */