/**
* @addtogroup FS
* @{
*
* @brief Provides functions for file system, include file operations, directory operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file mount.h
*
* @brief Provides functions for mounting and unmounting a file system.
*
* @since 1.0
* @version 1.0
*/

#ifndef _SYS_MOUNT_H
#define _SYS_MOUNT_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Mounts a file system.
*
* Mounts a device (referenced by a file system name) to a specified directory. \n
* @param source Indicates the pointer to the device to mount.
* @param target Indicates the pointer to the path of the mount point.
* @param filesystemtype Indicates the pointer to the file system type.
* @param mountflags Indicates the mounting flag.
* @param data Indicates the pointer to the mounting data.
* @return Returns <b>0</b> if the mounting is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* --------------- | --------------------------------
* EACCES | A component of the path was not searchable.
* EBUSY | <b>source</b> is already mounted.
* EFAULT | One of the pointer parameters points outside the user address space.
* EINVAL | <b>source</b> cannot be mounted.
* ELOOP | Too many links encountered during path name resolution.
* EMFILE | (Non-block device) Virtual device table is full.
* ENAMETOOLONG | The path name length is greater than the value of {@link MAXPATHLEN}.
* ENODEV | No such file system type is configured in the kernel.
* ENOENT | A path name is null or has a nonexistent component.
* ENOMEM | Insufficient memory.
* ENOTBLK | <b>source</b> is not a block device.
* ENOTDIR | <b>target</b> or the prefix of <b>source</b> is not a directory.
* ENGIO | No such device.
* EPERM | No permission.
* @since 1.0
* @version 1.0
*/
int mount(const char *source, const char *target, const char *filesystemtype, unsigned long mountflags, const void *data);

/**
* @brief Unmounts a file system.
*
* @param target Indicates the pointer to the path of the file system to unmount.
* @return Returns <b>0</b> if the file system is unmounted successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------- | ---------------------------
* EBUSY | <b>target</b> could not be unmounted because it is busy.
* EFAULT | <b>target</b> points outside the user address space.
* EINVAL | <b>target</b> is not a mount point.
* ENAMETOOLONG | The path length is greater than the value of {@link MAXPATHLEN}.
* ENOENT | A path name is null or has a nonexistent component.
* ENOMEM | Insufficient memory.
* EPERM | No permission.
* @since 1.0
* @version 1.0
*/
int umount(const char *target);

/**
* @brief Unmounts a file system.
*
* This function is used to unmount a file system. Currently, the value of <b>flags</b> can only be <b>0</b>. In this case, this function is the same as <b>umount()</b>. \n
* @param target Indicates the pointer to the path of the file system to unmount.
* @param flags Indicates the control flag. (Currently, the value of <b>flags</b> can only be <b>0</b>. In this case, this function is the same as <b>umount()</b>.)
* @return Returns <b>0</b> if the file system is unmounted successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------- | ------------------------------
* EBUSY | <b>target</b> could not be unmounted because it is busy.
* EFAULT | <b>target</b> points outside the user address space.
* EINVAL | <b>target</b> is not a mount point.
* ENAMETOOLONG | The path length is greater than the value of {@link MAXPATHLEN}.
* ENOENT | A path name is null or has a nonexistent component.
* ENOMEM | Insufficient memory.
* EPERM | No permission.
* ENOSYS | The function call is not supported. (<b>flags</b> value is not 0.)
* @since 1.0
* @version 1.0
*/
int umount2(const char *target, int flags);

#ifdef __cplusplus
}
#endif

#endif
/** @} */