/**
* @addtogroup IO
* @{
*
* @brief Provides data structures and functions for performing input/output (I/O) operations, such as printf and scanf.
*
* @since 1.0
* @version 1.0
*/

/**
* @file select.h
*
* @brief Provides functions and structures related to I/O multiplexing.
*
* These functions can be used to implement I/O multiplexing.
*
* @since 1.0
* @version 1.0
*/
#ifndef _SYS_SELECT_H
#define _SYS_SELECT_H
#ifdef __cplusplus
extern "C" {
#endif

/**@brief Defines the size of <b>fd_set</b>, that is, the maximum number of monitored files. */
#define FD_SETSIZE 1024

/**@brief Defines <b>fd_set</b> as the alias of the element type. */
typedef unsigned long fd_mask;

/**
* @brief Defines a file descriptor set.
*
* This structure is used to specify whether the file descriptor set exists. \n
*/
typedef struct {
/** Whether the descriptor exists is indicated by each bit. */
unsigned long fds_bits[FD_SETSIZE / 8 / sizeof(long)];
} fd_set;

/**
* @brief Clears all elements in the file descriptor set.
*
* @param s Indicates the descriptor set of the <b>fd_set</b> type.
* @since 1.0
* @version 1.0
*/
#define FD_ZERO(s) do { int __i; unsigned long *__b=(s)->fds_bits; for(__i=sizeof (fd_set)/sizeof (long); __i; __i--) *__b++=0; } while(0)

/**
* @brief Adds a file descriptor to a set.
*
* @param d Indicates the file descriptor of the <b>int</b> type.
* @param s Indicates the descriptor set of the <b>fd_set</b> type.
* @since 1.0
* @version 1.0
*/
#define FD_SET(d, s)   ((s)->fds_bits[(d)/(8*sizeof(long))] |= (1UL<<((d)%(8*sizeof(long)))))

/**
* @brief Removes a file descriptor from a set.
*
* @param d Indicates the file descriptor of the <b>int</b> type.
* @param s Indicates the descriptor set of the <b>fd_set</b> type.
* @since 1.0
* @version 1.0
*/
#define FD_CLR(d, s)   ((s)->fds_bits[(d)/(8*sizeof(long))] &= ~(1UL<<((d)%(8*sizeof(long)))))

/**
* @brief Checks whether a file descriptor is in a set.
*
* @param d Indicates the file descriptor of the <b>int</b> type.
* @param s Indicates the descriptor set of the <b>fd_set</b> type.
* @return Returns <b>0</b> if the file descriptor is not in the set; returns <b>1</b> if the file descriptor is in the set.
* @since 1.0
* @version 1.0
*/
#define FD_ISSET(d, s) !!((s)->fds_bits[(d)/(8*sizeof(long))] & (1UL<<((d)%(8*sizeof(long)))))

/**
* @brief Monitors the I/O events of multiple file descriptors.
*
* @param nfds Indicates the maximum value of the file descriptor of the <b>int</b> type.
* @param readfds Indicates the descriptor set of the readable event to observe, which is the <b>fd_set *</b> type.
* @param writefds Indicates the descriptor set of the writable event to observe, which is the <b>fd_set *</b> type.
* @param exceptfds Indicates the descriptor set of exception event to observe, which is the <b>fd_set *</b> type.
* @param timeout Indicates the timeout interval, which is the <b>struct timeval *</b> type.
* @return Returns the total number of file descriptors of the observed events (that is, the sum of all file descriptors in <b>readfds</b>, <b>writefds</b>, and <b>exceptfds</b>) if the operation is successful;
* returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ---------- | -------------------
* EBADF | An invalid file descriptor is given in one of the sets.
* EINVAL | <b>nfds</b> is less than 0.
* ENOMEM | Failed to apply for memory.
* @since 1.0
* @version 1.0
*/
int select(int nfds, fd_set *__restrict readfds, fd_set *__restrict writefds, fd_set *__restrict exceptfds, struct timeval *__restrict timeout);

#ifdef __cplusplus
}
#endif
#endif

/** @} */