/**
* @addtogroup MEM
* @{
*
* @brief Provides structures and functions for performing memory operations, including memory mapping, remapping, allocation, and release.
*
* @since 1.0
* @version 1.0
*/

/**
* @file shm.h
*
* @brief Declares APIs for creating, mapping, deleting, and controlling shared memory.
*
* @since 1.0
* @version 1.0
*/
#ifndef _SYS_SHM_H
#define _SYS_SHM_H

#ifdef __cplusplus
extern "C" {
#endif


#define __NEED_time_t
#define __NEED_size_t
#define __NEED_pid_t


#include <sys/ipc.h>


/** @brief Aligns the shared memory address. */
#define SHMLBA 4096

/**
* @brief Stores information about a shared memory segment.
*/
struct shmid_ds {
/** Operation permissions on this segment */
struct ipc_perm shm_perm;
/** Segment size, in bytes */
size_t shm_segsz;
/** Time when the last process is attached to this segment */
time_t shm_atime;
int __unused1;
/** Time when the last process is detached from this segment */
time_t shm_dtime;
int __unused2;
/** Time when the last process modified this segment */
time_t shm_ctime;
int __unused3;
/** ID of the process that created this segment */
pid_t shm_cpid;
/** ID of the last process that operated this segment */
pid_t shm_lpid;
/** Number of processes attached to this segment */
unsigned long shm_nattch;
/** Padding field */
unsigned long __pad1;
/** Padding field */
unsigned long __pad2;
};

/**
* @brief Describes limitations and attributes of system-level shared memory.
*/
struct shminfo {
/** Maximum size of a shared memory segment allowed by the system */
unsigned long shmmax;
/** Minimum size of a shared memory segment allowed by the system */
unsigned long shmmin;
/** Maximum number of shared memory segments allowed by the system */
unsigned long shmmni;
/** Maximum number of shared memory segments that can be attached by a process */
unsigned long shmseg;
/** Maximum size of all shared memory segments allowed by the system */
unsigned long shmall;
/** Reserved */
unsigned long __unused[4];
};

/**
* @brief Describes system resource information about the shared memory.
*/
struct shm_info {
/** Number of existing shared memory segments */
int __used_ids;
/** Total number of shared memory pages */
unsigned long shm_tot;
/** Number of resident shared memory pages */
unsigned long shm_rss;
/** Number of shared memory pages in the swap partition */
unsigned long shm_swp;
/** Number of swap attempts */
unsigned long __swap_attempts;
/** Number of successful swap attempts */
unsigned long __swap_successes;
};

/** @brief Indicates that the shared memory segment is readable. This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmget()}. */
#define SHM_R 0400
/** @brief Indicates that the shared memory segment is writable. This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmget()}. */
#define SHM_W 0200

/** @brief Indicates that the attached shared memory segment is read-only. This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmat()}. */
#define SHM_RDONLY 010000
/** @brief Indicates that the shared memory address can be rounded to a value meeting the requirement ({@link SHMLBA}). This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmat()}. */
#define SHM_RND    020000
/** @brief Indicates that the memory segment can be remapped. This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmat()}. */
#define SHM_REMAP  040000
/** @brief Indicates that the attached shared memory segment is executable. This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmat()}. */
#define SHM_EXEC   0100000

/** @brief Locks the shared memory segment in memory so that it cannot be swapped to the swap partition. This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmget()}. */
#define SHM_LOCK 11
/** @brief Unlocks the shared memory segment. This macro is used for setting the <b>shmflg</b> parameter passed to functions such as {@link shmget()}. */
#define SHM_UNLOCK 12
/** @brief Obtains a {@link shmid_ds} data structure. This macro is used for setting the <b>cmd</b> parameter passed to {@link shmctl()}. */
#define SHM_STAT (13 | (IPC_STAT & 0x100))
/** @brief Obtains a {@link shm_info} data structure that includes system resource information about this shared memory segment. This macro is used for setting the <b>cmd</b> parameter passed to {@link shmctl()}. */
#define SHM_INFO 14
/** @brief Obtains a {@link shmid_ds} data structure without permission check. This macro is used for setting the <b>cmd</b> parameter passed to {@link shmctl()}. */
#define SHM_STAT_ANY (15 | (IPC_STAT & 0x100))

/**
* @brief Attaches the shared memory segment identified by <b>shmid</b> to the address space of the current process.
*
* @param shmid Identifies the shared memory segment to attach, which is obtained by {@link shmget()}.
* @param shmaddr Indicates the space address of the current process to attach. If this parameter is <b>NULL</b>, the system chooses an unused address to attach.
* @param shmflg Indicates the attribute of the shared memory segment to attach, which is the bitwise OR combination of one or more of the following flags:
* shmflg | Description
* -----------|--------
* SHM_RND | Round the attached address. If <b>shmaddr</b> is not <b>NULL</b> and {@link SHM_RND} is specified in <b>shmflg</b>, the address is rounded down to the nearest multiple of {@link SHMLBA}.
* SHM_EXEC | Allow the contents of the attached shared memory segment to be executable.
* SHM_RDONLY | Set the contents of the attached shared memory segment to be read-only.
* SHM_REMAP | Replace any existing mapping in the address range starting at <b>shmaddr</b> and continuing for the size of the segment during the current memory mapping.<br>When {@link SHM_REMAP} is not specified, an <b>EINVAL</b> error will be returned if a mapping exists in the address range.
* @return Returns the pointer to the address of the attached shared memory segment if the operation is successful; returns <b>(void *)-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|------------------------------------------------------------------------------
* EACCES | The current process does not have the required permission on the requested mapping type.
* EIDRM | The shared memory segment specified by <b>shmid</b> has been removed.
* EINVAL | The value of <b>shmid</b> is invalid.<br><b>shmaddr</b> is not aligned (not page-aligned and {@link SHM_RND} is not specified) or its value is invalid.<br>{@link SHM_REMAP} is specified, but <b>shmaddr</b> is <b>NULL</b>.
* ENOMEM | No page tables can be allocated.
* @since 1.0
* @version 1.0
*/
void *shmat(int shmid, const void *shmaddr, int shmflg);

/**
* @brief Performs a control operation specified by the <b>cmd</b> parameter on the shared memory segment identified by <b>shmid</b>.
*
*
* @param shmid Identifies the shared memory segment to attach, which is obtained by {@link shmget()}.
* @param cmd Indicates the control operation to perform. The following table describes available values.
* cmd | Description
* -----------|--------
* IPC_STAT | Copy information from the kernel data structure associated with the shared memory segment identified by <b>shmid</b> into the {@link shmid_ds} structure pointed to by <b>buf</b>.
* SHM_STAT | Return a {@link shmid_ds} data structure.<br>When {@link SHM_STAT} is specified, <b>shmid</b> does not identify a shared memory segment. Instead, it indicates the index to the kernel's internal array that maintains information about all shared memory segments.
* IPC_SET | Write the values of some fields in the {@link shmid_ds} data structure pointed to by <b>buf</b> to the kernel data structure associated with the specified shared memory segment, updating also its <b>shm_ctime</b>member.<br>The following fields can be modified: <b>shm_perm.uid</b>, <b>shm_perm.gid</b>, and <b>shm_perm.mode</b>.
* IPC_RMID | Mark the shared memory segment to be destroyed.
* IPC_INFO | Return information about system-wide shared memory limits and parameters in the structure pointed to by <b>buf</b>.<br>The returned structure is of the {@link shminfo} type, and a cast is required.
* SHM_INFO | Return a {@link shm_info} structure that contains information about system resources consumed by the specified shared memory segment.
* @param buf Indicates the pointer to the buffer for storing the information to set or obtain.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|------------------------------------------------------------------------------
* EACCES | {@link IPC_STAT} or {@link SHM_STAT} is specified in <b>cmb</b>, but the <b>mode</b> field in {@link shm_perm} does not allow any read operation on the shared memory segment identified by <b>shmid</b>.
* EFAULT | {@link SHM_STAT} or {@link IPC_STAT} is specified in <b>cmd</b>, but the address pointed to by <b>buf</b> is not accessible.
* EIDRM | <b>shmid</b> points to a removed shared memory segment.
* EINVAL | The value of <b>shmid</b> or <b>cmd</b> is invalid.<br>When {@link SHM_STAT} is specified in <b>cmd</b>, the value is considered invalid if the index specified by <b>shmid</B> refers to an unused array slot.
* ENOMEM | No page tables can be allocated.
* EPERM | {@link IPC_SET} or {@link IPC_RMID} is attempted, but the effective user ID of the calling process is not that of the creator or the owner, and the process is not privileged.
* @since 1.0
* @version 1.0
*/
int shmctl(int shmid, int cmd, struct shmid_ds *buf);

/**
* @brief Detaches the shared memory segment attached to the address pointed to by <b>shmaddr</b> from the address space of the calling process.
*
* @param shmaddr Indicates the pointer to the address of the shared memory segment to detach. The value of this parameter should be the pointer previously returned by {@link shmat()}.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|-----------------------------------------------------
* EINVAL | No shared memory segment is attached to <b>shmaddr</b>, or <b>shmaddr</b> is not aligned.
* @since 1.0
* @version 1.0
*/
int shmdt(const void *shmaddr);

/**
* @brief Obtains or creates a shared memory segment with the specified size based on the ID specified by <b>key</b>.
*
* If the shared memory segment associated with <b>key</b> does not exist and {@link IPC_CREAT} is specified in <b>shmflg</b>
* or if <b>key</b> has the value {@link IPC_PRIVATE}, this function creates a shared memory segment with the size equal to the value of <b>size</b> rounded up to a multiple of {@link PAGE_SIZE}. \n
* @param key Indicates the key of the shared memory segment.
* @param size Indicates the size of the shared memory segment to create. The actual size is rounded up to a multiple of {@link PAGE_SIZE}.
* @param shmflg Indicates the attribute of the shared memory segment, which is the bitwise OR combination of one or more of the following flags:
* shmflg | Description
* -----------|-----------------------------------------------------------------------------------
* IPC_CREAT | Create a new shared memory segment. If this flag is not specified, this function will find the segment with the specified <b>key</b> and check to see whether the user has permission to access the segment.
* IPC_EXCL | Used together with {@link IPC_CREAT} to ensure that this function is called to create a shared memory segment. The calling fails if the segment already exists.
* @return Returns the ID of the shared memory segment if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ----------|------------------------------------------------------------------------------
* EACCES | The current process does not have the permission to obtain the specified shared memory segment.
* EEXIST | {@link IPC_EXCL} and {@link IPC_CREAT} are specified, but the shared memory segment associated with <b>key</b> already exists.
* EINVAL | The size specified for the shared memory segment to create is less than the value of <b>shmmin</b> in {@link shminfo} or greater than the value of <b>shmmax</b>.<br>The shared memory segment associated with <b>key</b> already exists, but the value of <b>size</b> is greater than the size of the segment.
* ENOENT | No segment exists for the given key, and {@link IPC_CREAT} is not specified.
* ENOMEM | Insufficient memory.
* ENOSPC | Allocating a segment of the requested size would cause the system to exceed the system-wide limit on shared memory (<b>shmall</b> in {@link shminfo}).
* @since 1.0
* @version 1.0
*/
int shmget(key_t key, size_t size, int shmflg);

#ifdef __cplusplus
}
#endif

#endif
/** @} */