/**
* @addtogroup NET
* @{
*
* @brief Provides functions and data structures related to network operations.
*
* @since 1.0
* @version 1.0
*/

/**
* @file socket.h
*
* @brief Provides functions and data structures related to network operations.
*
* For example, you can use the functions to send and receive network data, manage network addresses, and convert bytes.
*
* @since 1.0
* @version 1.0
*/
#ifndef	 _SYS_SOCKET_H
#define	 _SYS_SOCKET_H
#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Defines options for socket level.
*/
#define SOL_SOCKET      1

/**
* @brief Describes the socket address information.
*/
struct sockaddr {
/** Address family */
sa_family_t sa_family;
/** Protocol address */
char sa_data[14];
};

/**
* @brief Creates a socket and returns its descriptor.
* @attention Only the following socket domains are supported: <b>AF_INET</b>, <b>AF_INET6</b>.
* Only the socket types <b>SOCK_RAW</b>, <b>SOCK_DGRAM</b> and <b>SOCK_STREAM</b> are supported, and the OR operation between the extra socket flag and the socket type is not supported.
* @param domain Indicates the socket domain.
* @param type Indicates the socket type.
* @param protocol Indicates the socket protocol.
* @return Returns the descriptor for the new socket if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EAFNOSUPPORT   | Invalid socket domain
* ESOCKTNOSUPPORT| Unsupported socket domain
* EPROTONOSUPPORT| Unsupported protocol
* EINVAL         | Invalid input parameter
* ENOBUFS        | Connection setup error
* ENFILE         | File table addition exception
* @since 1.0
* @version 1.0
*/
int socket(int domain, int type, int protocol);

/**
* @brief Shuts down a socket.
* @param sockfd Indicates the socket descriptor.
* @param how Indicates how to shut down the socket. You can disable either receptions by <b>SHUT_RD</b> or transmissions by <b>SHUT_WR</b>, or both by <b>SHUT_RDWR</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* ENOTCONN       | Socket not connected
* ENOMEM         | Memory copy exception
* EIO             | Incorrect connection type obtained based on the socket descriptor
* @since 1.0
* @version 1.0
*/
int shutdown(int sockfd, int how);

/**
* @brief Binds a local protocol address to a socket.
* @param sockfd Indicates the socket descriptor.
* @param addr Indicates the address to be bound to the socket.
* @param addrlen Indicates the length of the address structure pointed to by <b>addr</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* @since 1.0
* @version 1.0
*/
int bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen);

/**
* @brief Initiates a connection to a socket.
* @param sockfd Indicates the socket descriptor.
* @param addr Indicates the pointer to the address to be connected.
* @param addrlen Indicates the length of the address structure pointed to by <b>addr</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* EIO            | Incorrect connection type obtained based on the socket descriptor
* ECONNREFUSED   | Restart due to connection failure
* ECONNABORTED   | Shut-down due to connection failure
* @since 1.0
* @version 1.0
*/
int connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen);

/**
* @brief Listens for network connections.
* @param sockfd Indicates the socket descriptor.
* @param backlog Indicates the maximum length of the queue to accept incoming connection requests.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* EIO             | Incorrect connection type obtained based on the socket descriptor
* @since 1.0
* @version 1.0
*/
int listen(int sockfd, int backlog);

/**
* @brief Accepts incoming connection requests.
* @param sockfd Indicates the socket descriptor.
* @param addr Indicates the pointer to the socket address requiring for the connection.
* @param addrlen Indicates the length of the address structure pointed to by <b>addr</b>.
* @return Returns a valid socket if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EOPNOTSUPP     | The socket requiring for the connection is not of the <b>NETCONN_TCP</b> type.
* EWOULDBLOCK    | Failure to receive events in non-blocking mode.
* ENFILE         | Failure to creat a new socket.
* ECONNABORTED   | Failure to receive the remote information.
* ENOMEM         | Memory copy exception.
* @since 1.0
* @version 1.0
*/
int accept(int sockfd, struct sockaddr *__restrict addr, socklen_t *__restrict addrlen);

/**
* @brief Retrieves the local address of the specified socket.
* @param fd Indicates the socket descriptor.
* @param addr Indicates the pointer to the address to be retrieved.
* @param len Indicates the pointer to the length of the address pointed to by <b>addr</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* ENOMEM         | Memory copy exception
* @since 1.0
* @version 1.0
*/
int getsockname(int fd, struct sockaddr *restrict addr, socklen_t *restrict len);

/**
* @brief Retrieves the peer address of the specified socket.
* @param fd Indicates the socket descriptor.
* @param addr Indicates the pointer to the address to be retrieved.
* @param len Indicates the pointer to the length of the address pointed to by <b>addr</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* ENOMEM         | Memory copy exception
* @since 1.0
* @version 1.0
*/
int getpeername(int fd, struct sockaddr *restrict addr, socklen_t *restrict len);

/**
* @brief Sends data to another socket.
* @param fd Indicates the socket descriptor.
* @param buf Indicates the pointer to a buffer for storing the data to be sent.
* @param len Indicates the length of the buffer pointed to by <b>buf</b>.
* @param flags Indicates the socket flags. This parameter has the following values:
* value          | Description
* -------------- | ---------------------------------
* MSG_DONTWAIT   | Enables the non-blocking operation.
* MSG_MORE       | The caller has more data to send.
* MSG_NOSIGNAL   | No signals are generated.
* @return Returns the length of the sent data if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* EDESTADDRREQ   | Invalid connection
* ENOMEM         | Memory copy exception
* EOPNOTSUPP     | Unsupported message type
* ENOTCONN       | Socket not connected
* EIO             | Incorrect connection type obtained based on the socket descriptor
* @since 1.0
* @version 1.0
*/
ssize_t send(int fd, const void *buf, size_t len, int flags);

/**
* @brief Receives data from another socket.
* @param fd Indicates the socket descriptor.
* @param buf Indicates the pointer to a buffer for storing the data to be received.
* @param len Indicates the length of the buffer pointed to by <b>buf</b>.
* @param flags Indicates the socket flags. This parameter has the following values:
* value          | Description
* -------------- | ---------------------------------
* MSG_DONTWAIT   | Enables the non-blocking operation.
* MSG_MORE       | The caller has more data to send.
* MSG_NOSIGNAL   | No signals are generated.
* @return Returns the length of the received data if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* ENOMEM         | Memory copy exception
* EOPNOTSUPP     | Unsupported message type
* ENOTCONN       | Socket not connected
* EWOULDBLOCK    | Blocking operation
* ENOTCONN       | Connection closed
* @since 1.0
* @version 1.0
*/
ssize_t recv(int fd, void *buf, size_t len, int flags);

/**
* @brief Sends data to another socket.
* @param fd Indicates the socket descriptor.
* @param buf Indicates the pointer to a buffer for storing the data to be sent.
* @param len Indicates the length of the buffer pointed to by <b>buf</b>.
* @param flags Indicates the socket flags. This parameter has the following values:
* value          | Description
* -------------- | ---------------------------------
* MSG_DONTWAIT   | Enables the non-blocking operation.
* MSG_MORE       | The caller has more data to send.
* MSG_NOSIGNAL   | No signals are generated.
* @param addr Indicates the pointer to the destination network address.
* @param alen Indicates the length of the destination network address.
* @return Returns the length of the sent data if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* EDESTADDRREQ   | Invalid connection
* ENOMEM         | Memory copy exception
* EOPNOTSUPP     | Unsupported message type
* ENOTCONN       | Socket not connected
* EIO             | Incorrect connection type obtained based on the socket descriptor
* @since 1.0
* @version 1.0
*/
ssize_t sendto(int fd, const void *buf, size_t len, int flags, const struct sockaddr *addr, socklen_t alen);

/**
* @brief Receives data from a specified socket.
* @param fd Indicates the socket descriptor.
* @param buf Indicates the pointer to a buffer for storing the data to be received.
* @param len Indicates the length of the buffer pointed to by <b>buf</b>.
* @param flags Indicates the socket flags. This parameter has the following values:
* value          | Description
* -------------- | ---------------------------------
* MSG_DONTWAIT   | Enables the non-blocking operation.
* MSG_MORE       | The caller has more data to send.
* MSG_NOSIGNAL   | No signals are generated.
* @param addr Indicates the pointer to the source address.
* @param alen Indicates the size of the source address.
* @return Returns the length of the received data if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* ENOMEM         | Memory copy exception
* EOPNOTSUPP     | Unsupported message type
* ENOTCONN       | Socket not connected
* ENOTCONN       | Connection closed
* @since 1.0
* @version 1.0
*/
ssize_t recvfrom(int fd, void *__restrict buf, size_t len, int flags, struct sockaddr *__restrict addr, socklen_t *__restrict alen);

/**
* @brief Sends data to another socket.
* @attention This function does not support ancillary data.
* @param fd Indicates the socket descriptor.
* @param msg Indicates the pointer to the address of the message header to be sent.
* @param flags Indicates the socket flags. This parameter has the following values:
* value          | Description
* -------------- | ---------------------------------
* MSG_DONTWAIT   | Enables the non-blocking operation.
* MSG_MORE       | The caller has more data to send.
* MSG_NOSIGNAL   | No signals are generated.
* @return Returns the length of the sent data if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* EDESTADDRREQ   | Invalid connection
* ENOMEM         | Memory copy exception
* EIO             | Incorrect connection type obtained based on the socket descriptor
* EOPNOTSUPP     | Unsupported message type
* @since 1.0
* @version 1.0
*/
ssize_t sendmsg(int fd, const struct msghdr *msg, int flags);

/**
* @brief Receives data from a specified socket.
* @attention This function does not allow sending ancillary data. Currently, the length of the array defined by <b>msg_iov</b> can only be <b>1</b>.
* @param fd Indicates the socket descriptor.
* @param msg Indicates the pointer to the address to receive the message header.
* @param flags Indicates the socket flags. This parameter has the following values:
* value          | Description
* -------------- | ---------------------------------
* MSG_DONTWAIT   | Enables the non-blocking operation.
* MSG_MORE       | The caller has more data to send.
* MSG_NOSIGNAL   | No signals are generated.
* @return Returns the length of the received data if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Invalid input parameter
* EBADF          | Invalid socket or socket status exception
* EDESTADDRREQ   | Invalid connection
* ENOMEM         | Memory copy exception
* EIO             | Incorrect connection type obtained based on the socket descriptor
* @since 1.0
* @version 1.0
*/
ssize_t recvmsg(int fd, struct msghdr *msg, int flags);

/**
* @brief Retrieves the socket options.
* @param fd Indicates the socket descriptor.
* @param level Indicates the protocol level at which the option resides. Only {@link SOL_SOCKET} is supported.
* @param optname Indicates the socket options to be retrieved, which has the following values:
* value           | Description
* --------------- | -------------------------------------------------------------
* SO_BROADCAST    | Permits sending broadcast messages.
* SO_RCVTIMEO     | Receiving data times out.
* SO_RCVBUF       | Sets or gets the maximum socket receiving buffer in bytes.
* SO_TYPE         | Gets the socket type.
* SO_DONTROUTE    | Prohibits messages from being sent via a gateway.
* SO_BINDTODEVICE | Binds this socket to a particular device.
* @param optval Indicates the pointer to the option data.
* @param optlen Indicates the pointer to the size of the buffer pointed to by <b>optval</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Small <b>optlen</b>
* EBADF          | Invalid socket or socket status exception
* EFAULT         | The input parameter is a null pointer.
* ENOBUFS        | Retrieved length exception
* ENOMEM         | Memory copy exception
* @since 1.0
* @version 1.0
*/
int getsockopt(int fd, int level, int optname, void *__restrict optval, socklen_t *__restrict optlen);

/**
* @brief Sets the socket options.
* @param fd Indicates the socket descriptor.
* @param level Indicates the protocol level at which the option resides. Only {@link SOL_SOCKET} is supported.
* @param optname Indicates the socket options to set, which has the following values:
* value           | Description
* --------------- | ---------------------------------
* SO_BROADCAST    | Permits sending broadcast messages.
* SO_RCVTIMEO     | Receiving data times out.
* SO_RCVBUF       | Sets or gets the maximum socket receiving buffer in bytes.
* SO_DONTROUTE    | Prohibits messages from being sent via a gateway.
* SO_BINDTODEVICE | Binds this socket to a particular device.
* @param optval Indicates the pointer to the option data.
* @param optlen Indicates the size of the buffer pointed to by <b>optval</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno          | Description
* -------------- | ---------------------------------
* EINVAL         | Small <b>optlen</b>
* EBADF          | Invalid socket or socket status exception
* EFAULT         | The input parameter is a null pointer.
* ENOBUFS        | Retrieved length exception
* ENOMEM         | Memory copy exception
* @since 1.0
* @version 1.0
*/
int setsockopt(int fd, int level, int optname, const void *optval, socklen_t optlen);

#ifdef __cplusplus
}
#endif
#endif/** @} */