/**
* @addtogroup FS
* @{
*
* @brief Provides functions for file system, include file operations, directory operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file stat.h
*
* @brief Provides functions for file and directory operations.
*
* These functions can be used for file system operations, such as file read/write, directory traversal, and file system mounting.
*
* @since 1.0
* @version 1.0
*/
#ifndef	 _SYS_STAT_H
#define	 _SYS_STAT_H


#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Defines the file information structure.
*/
struct stat {
#if defined(__ARMEB__)
/** Device ID used by the file */
unsigned short st_dev;
unsigned short __pad1;
#else
/** Device ID used by the file */
unsigned long  st_dev;
#endif
/** Inode number */
unsigned long  st_ino;
/** File mode, including <b>S_IFDIR</b> (Directory), <b>S_IFCHR</b> (Character device), <b>S_IFBLK</b> (Block device), <b>S_IFREG</b> (Regular file), <b>S_IFIFO</b> (FIFO), <b>S_IFLNK</b> (Symbolic link), and <b>S_IFSOCK</b> (Socket) */
unsigned short st_mode;
/** Number of hard links */
unsigned short st_nlink;
/** User ID of owner */
unsigned short st_uid;
/** Group ID */
unsigned short st_gid;
#if defined(__ARMEB__)
/** Device ID of the device file */
unsigned short st_rdev;
unsigned short __pad2;
#else
/** Device ID of the device file */
unsigned long  st_rdev;
#endif
/** File size in bytes */
unsigned long  st_size;
/** Size of the disk block that contains the file */
unsigned long  st_blksize;
/** Number of disk blocks occupied by the file */
unsigned long  st_blocks;
/** Time when the file is accessed for the last time which is represented by the number of seconds passed since 00:00:00 on January 1, 1970 */
unsigned long  st_atime;
/** Time when the file is accessed for the last time, which is represented by the number of nanoseconds */
unsigned long  st_atime_nsec;
/** Time when the file is modified for the last time which is represented by the number of seconds passed since 00:00:00 on January 1, 1970 */
unsigned long  st_mtime;
/** Time when the file is modified for the last time, which is represented by the number of nanoseconds */
unsigned long  st_mtime_nsec;
/** Time when the file state is modified for the last time, which is represented by the number of seconds */
unsigned long  st_ctime;
/** Time when the file state is modified for the last time, which is represented by the number of nanoseconds */
unsigned long  st_ctime_nsec;
unsigned long  __unused4;
unsigned long  __unused5;
};

/**
* @brief Obtains file information.
*
* @param path Indicates the file path.
* @param buf Indicates the pointer to a <b>stat</b> structure into which file information is placed.
* @return Returns <b>0</b> if file information is obtained successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | -----------------------------------------------------------------------
* EINVAL | Invalid parameter.
* ENOENT | A component of <b>path</b> does not name an existing file or <b>path</b> is an empty string.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOMEM | Insufficient memory.
* ENOSYS | The function is not supported.
* EAGAIN | The file list is null.
* EACCES | The permission bit of the file mode does not allow the requested access, or the search permission is denied on the path prefix component.
* EEXIST | The file or directory already exists.
* EIO | An I/O error occurs when data is read from or written to the file system.
* EROFS | The physical drive is write-protected.
* ENOSPC | No available space on the device.
* ENFILE | The number of opened files in the system has reached the maximum.
* ENOTEMPTY | The directory is not empty.
* EISDIR | The named file is a directory.
* ENOTDIR | A component of the <b>path</b> prefix is the name of an existing file. The file is neither a directory nor a symbolic link of a directory.
* EPERM | Operation not permitted or access denied because the access is prohibited or the directory is full.
* EBUSY | The operation is rejected according to the file sharing policy.
* EPROTO | The protocol in NFS is incorrect.
* ENODEV | No such device.
* EFAULT | Incorrect IP address.
* @since 1.0
* @version 1.0
*/
int stat(const char *__restrict path, struct stat *__restrict buf);

/**
* @brief Obtains file status information.
*
* @param fd Indicates the file descriptor.
* @param buf Indicates the pointer to a <b>stat</b> structure into which file information is placed.
* @return Returns <b>0</b> if file information is obtained successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | --------------------------------------------------
* EBADF | <b>fd</b> is an invalid file descriptor.
* EAGAIN | The file list is null.
* EINVAL | Invalid parameter.
* ENOENT | A component of <b>path</b> does not name an existing file or <b>path</b> is an empty string.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOMEM | Insufficient memory.
* ENOSYS | The function is not supported.
* EACCES | Search permission is denied on a component of the path prefix.
* EIO | An I/O error occurs when data is read from or written to the file system.
* EROFS | The physical drive is write-protected.
* ENOSPC | No available space on the device.
* EISDIR | The named file is a directory.
* EPERM | Operation not permitted or access denied because the access is prohibited or the directory is full.
* EBUSY | The operation is rejected according to the file sharing policy.
* EPROTO | The protocol in NFS is incorrect.
* ENODEV | No such device.
* EFAULT | Incorrect IP address.
* @since 1.0
* @version 1.0
*/
int fstat(int fd, struct stat *buf);

/**
* @brief Obtains file information.
*
* @param path Indicates the file path.
* @param buf Indicates the pointer to a <b>stat</b> structure into which file information is placed.
* @return Returns <b>0</b> if file information is obtained successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | -----------------------------------------------------------------------
* EINVAL | Invalid parameter.
* ENOENT | A component of <b>path</b> does not name an existing file or <b>path</b> is an empty string.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOMEM | Insufficient memory.
* ENOSYS | The function is not supported.
* EAGAIN | The file list is null.
* EACCES | The permission bit of the file mode does not allow the requested access, or the search permission is denied on the path prefix component.
* EEXIST | The file or directory already exists.
* EIO | An I/O error occurs when data is read from or written to the file system.
* EROFS | The physical drive is write-protected.
* ENOSPC | No available space on the device.
* ENFILE | The number of opened files in the system has reached the maximum.
* ENOTEMPTY | The directory is not empty.
* EISDIR | The named file is a directory.
* ENOTDIR | A component of the <b>path</b> prefix is the name of an existing file. The file is neither a directory nor a symbolic link of a directory.
* EPERM | Operation not permitted or access denied because the access is prohibited or the directory is full.
* EBUSY | The operation is rejected according to the file sharing policy.
* EPROTO | The protocol in NFS is incorrect.
* ENODEV | No such device.
* EFAULT | Incorrect IP address.
* @since 1.0
* @version 1.0
*/
int lstat(const char *__restrict path, struct stat *__restrict buf);

/**
* @brief Creates a directory.
*
* @attention This function can NOT be used in the PROC file system.
* @param pathname Indicates the pointer to the name of the directory to create.
* @param mode Indicates the directory permission.
* @return Returns <b>0</b> if the directory is created successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | ---------------------------------------------------------------------
* EINVAL | The <b>pathname</b> is a null pointer or an empty string.
* EACCES | The permission bit of the file mode does not allow the requested access, or the search permission is denied on the path prefix directory.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOENT | The directory component in the path name does not exist.
* EEXIST | The file or directory already exists.
* EIO | An I/O error occurs when data is read from or written to the file system.
* EROFS | The physical drive is write-protected.
* ENOSPC | No available space on the device.
* EPERM | Operation not permitted or access denied because the access is prohibited or the directory is full.
* EBUSY | The operation is rejected according to the file sharing policy.
* ENOSYS | The function is not supported.
* EPROTO | The protocol in NFS is incorrect.
* EFAULT | Incorrect IP address.
* @since 1.0
* @version 1.0
*/
int mkdir(const char *pathname, mode_t mode);

/**
* @brief Creates a named pipe to implement inter-process communication.
*
* @attention Permission control is not supported. The number of pipes that can be created is not limited. Pipes can be created only in the <b>/dev</b> directory.
* @param name Indicates the pointer to the name of the pipe to create.
* @param mode Not supported.
* @return Returns <b>0</b> if the pipe is created successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* -------|------------------------------------------
* EINVAL | Invalid input parameter (for example, empty address, file name too long or the file is not in the <b>/dev</b> directory).
* EEXIST | The file already exists.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int mkfifo(const char *name, mode_t mode);

/**
* @brief Creates a directory.
*
* @attention This function can NOT be used in the PROC file system.
* @attention Currently, the relative path is not supported. (<b>fd</b> must be {@link AT_FDCWD}, and the <b>pathname</b> argument is an absolute path.)
* @param fd Indicates the file descriptor of the target directory. (Currently, only {@link AT_FDCWD} is supported.)
* @param pathname Indicates the pointer to the name of the directory to create.
* @param mode Indicates the directory permission.
* @return Returns <b>0</b> if the directory is created successfully; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | ---------------------------------------------------------------------
* EINVAL | The <b>pathname</b> is a null pointer or an empty string.
* EACCES | The permission bit of the file mode does not allow the requested access, or the search permission is denied on the path prefix directory.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOENT | The directory component in the path name does not exist.
* EEXIST | The file or directory already exists.
* EIO | An I/O error occurs when data is read from or written to the file system.
* EROFS | The physical drive is write-protected.
* ENOSPC | No available space on the device.
* EPERM | Operation not permitted or access denied because the access is prohibited or the directory is full.
* EBUSY | The operation is rejected according to the file sharing policy.
* ENOSYS | The function is not supported.
* EPROTO | The protocol in NFS is incorrect.
* EFAULT | Incorrect IP address.
* @since 1.0
* @version 1.0
*/
int mkdirat(int fd, const char *pathname, mode_t mode);


/**
* @brief Modifies permissions for a file.
*
* @attention This function can be used only in the JFFS2 file system.
* @param pathname Indicates the pointer to the path of the file.
* @param mode Indicates the new file permission.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table otherwise.
* errno | Description
* ------------ | ---------------------------------------------------------------------
* EINVAL | The <b>pathname</b> is a null pointer or an empty string.
* EACCES | The permission bit of the file mode does not allow the requested access, or the search permission is denied on the path prefix directory.
* EFAULT | Incorrect IP address.
* ENAMETOOLONG | The path name length is greater than the value of {@link NAME_MAX}.
* ENOENT | The directory component in the path name does not exist.
* ENOMEM | Insufficient memory.
* ENOTDIR | A component of the <b>path</b> prefix is the name of an existing file. The file is neither a directory nor a symbolic link of a directory.
* EPERM | Operation not permitted or access denied because the access is prohibited or the directory is full.
* EROFS | The physical drive is write-protected.
* EEXIST | The file or directory already exists.
* ENOSYS | The function is not supported.

* @since 1.0
* @version 1.0
*/
int chmod(const char *pathname, mode_t mode);

/**
* @brief Sets umask for a process.
*
* This function sets the umask of the current process file and returns the previous value of the umask. umask is used to turn off permission bits corresponding to in the <b>mode</b> parameter. \n
* @param mode Used to specify the umask value of a process.
* @return Returns the previous value of the umask. This function call always succeeds.
* @since  1.0
* @version  1.0
*/
mode_t umask(mode_t mode);

#ifdef __cplusplus
}
#endif
#endif
/** @} */