/**
* @addtogroup TIME-SYS
* @{
*
* @brief Provides time-related structures and functions.
*
* @since 1.0
* @version 1.0
*/

/**
* @file time.h
*
* @brief Provides structures and functions related to the system time.
*
* You can read and write the system time and set timers.
*
* @since 1.0
* @version 1.0
*/
#ifndef _SYS_TIME_H
#define _SYS_TIME_H
#ifdef __cplusplus
extern "C" {
#endif

#include <sys/select.h>
/**
* @brief Describes a period of time, accurate to microseconds.
*
*/
struct timeval {
/** Second */
time_t tv_sec;
/** Microsecond */
suseconds_t tv_usec;
};

/**
* @brief Describes a period of time, accurate to nanoseconds.
*
*/
struct timespec {
/** Second */
time_t tv_sec;
/** Nanosecond */
long tv_nsec;
};

/**
* @brief Obtains the system time and time zone.
*
* @attention Currently, there is no time zone structure. The return value of <b>ovalue</b> is empty.
* @param value Indicates the pointer to the <b>timeval</b> structure that contains the time.
* @param ovalue This parameter is used for compatibility only.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect input.
* @since 1.0
* @version 1.0
*/
int gettimeofday(struct timeval *__restrict value, void *__restrict ovalue);

/**
* @brief Defines the timer that runs in real time.
*/
#define ITIMER_REAL    0

/**
* @brief Describes a timer.
*
*/
struct itimerval {
/** Timer interval. The value <b>0</b> indicates a one-shot timer. */
struct timeval it_interval;
/** Timer value */
struct timeval it_value;
};

/**
* @brief Obtains the value of an intermittent timer.
*
*
* @param w Indicates the timer type. Only {@link ITIMER_REAL} is supported.
* @param old Indicates the pointer to the timer value.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect input or timer type.
* @since 1.0
* @version 1.0
*/
int getitimer(int w, struct itimerval *old);

/**
* @brief Sets the value for a timer.
*
*
* @param w Indicates the timer type. Currently, only {@link ITIMER_REAL} is supported.
* @param value Indicates the pointer to the timer value to set.
* @param ovalue Indicates the pointer to the original timer value before the current setting, which can be <b>NULL</b>.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect input or timer type.
* @since 1.0
* @version 1.0
*/
int setitimer(int w, const struct itimerval *__restrict value, struct itimerval *__restrict ovalue);

/**
* @brief Describes a time zone.
*
*/
struct timezone {
/** Minutes west of UTC */
int tz_minuteswest;
/** DST offset */
int tz_dsttime;
};

/**
* @brief Sets the system time and time zone.
*
* @attention <b>tz</b> is invalid. The time is set internally using <b>clock_settime</b>.
* @param tv Indicates the pointer to the time to set.
* @param tz Indicates the pointer to the time zone to set. This parameter is not used yet.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect input.
* @since 1.0
* @version 1.0
*/
int settimeofday(const struct timeval *tv, const struct timezone *tz);

#ifdef __cplusplus
}
#endif

#endif/** @} */