/**
* @addtogroup IO
* @{
*
* @brief Provides data structures and functions for performing input/output (I/O) operations, such as printf and scanf.
*
* @since 1.0
* @version 1.0
*/

/**
* @file uio.h
*
* @brief Provides functions and structures related to input/output (I/O) to a file or buffer.
*
* You can use the functions provided in this file to read files to the buffer or read information from the buffer to files.
*
* @since 1.0
* @version 1.0
*/

#ifndef _SYS_UIO_H
#define _SYS_UIO_H

#ifdef __cplusplus
extern "C" {
#endif


/**
* @brief Reads the data of <b>fd</b> to <b>iov</b>.
*
* @param fd Indicates the file descriptor.
* @param iov Indicates the pointer to the multi-group buffer.
* @param iovcnt Indicates the number of buffers. For details, see {@link IOV_MAX}.
* @return Returns the number of bytes read if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value if the operation fails. \n
* If <b>errno</b> is set to <b>EINVAL</b>, the sum of the <b>iov_len</b> values overflows an <b>ssize_t</b> value, or the vector count <b>iovcnt</b> is less than 0 or greater than the permitted maximum.
* @since 1.0
* @version 1.0
*/
ssize_t readv(int fd, const struct iovec *iov, int iovcnt);

/**
* @brief Writes a given length of data into a file.
*
* This function is used to write the <b>iovcnt</b> length of <b>iov</b> to <b>fd</b>. \n
* @param fd Indicates the file descriptor.
* @param iov Indicates the pointer to the multi-group buffer.
* @param iovcnt Indicates the number of buffers. For details, see {@link IOV_MAX}.
* @return Returns the number of bytes written if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value if the operation fails. \n
* If <b>errno</b> is set to <b>EINVAL</b>, the sum of the <b>iov_len</b> values overflows an <b>ssize_t</b> value, or the vector count <b>iovcnt</b> is less than 0 or greater than the permitted maximum.
* @since 1.0
* @version 1.0
*/
ssize_t writev(int fd, const struct iovec *iov, int iovcnt);

/**
* @brief Reads the data in a file whose <b>fd</b> offset is <b>offset</b> to the multi-group buffer space pointed to by <b>iov</b>.
*
* @param fd Indicates the file descriptor.
* @param iov Indicates the pointer to the multi-group buffer.
* @param iovcnt Indicates the number of buffers. For details, see {@link IOV_MAX}.
* @param offset Indicates the offset of the file descriptor to read.
* @return Returns the number of bytes read if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value if the operation fails. \n
* If <b>errno</b> is set to <b>EINVAL</b>, the sum of the <b>iov_len</b> values overflows an <b>ssize_t</b> value, or the vector count <b>iovcnt</b> is less than 0 or greater than the permitted maximum.
* @since 1.0
* @version 1.0
*/
ssize_t preadv(int fd, const struct iovec *iov, int iovcnt,off_t offset);

/**
* @brief Writes the data of a multi-group buffer space pointed to by <b>iov</b> to the offset of <b>fd</b>.
*
* @param fd Indicates the file descriptor.
* @param iov Indicates the pointer to the multi-group buffer.
* @param iovcnt Indicates the number of buffers. For details, see {@link IOV_MAX}.
* @param offset Indicates the offset of the file descriptor to write.
* @return Returns the number of bytes written if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value if the operation fails. \n
* If <b>errno</b> is set to <b>EINVAL</b>, the sum of the <b>iov_len</b> values overflows an <b>ssize_t</b> value, or the vector count <b>iovcnt</b> is less than 0 or greater than the permitted maximum.
* @since 1.0
* @version 1.0
*/
ssize_t pwritev(int fd, const struct iovec *iov, int iovcnt,off_t offset);

#ifdef __cplusplus
}
#endif

#endif

/** @} */