/**
* @addtogroup PROCESS
* @{
*
* @brief Provides process- and thread-related structures and functions.
*
* @since 1.0
* @version 1.0
*/

/**
* @file wait.h
*
* @brief Provides process- and thread-related structures and functions (for example, waiting for child processes to end and reclaiming resources).
*
*
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _SYS_WAIT_H
#define	 _SYS_WAIT_H
#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Waits for any child process to end and reclaims its resources.
*
* @param status Indicates the pointer to the obtained status information. If this parameter is not <b>NULL</b>, the status information is stored in the int value that it points to.\n
* You can use the following macros defined in {@link private.h} to check the integer (the macro uses the integer as a parameter instead of the pointer that points to it): \n
* <ul><li>{@link WIFEXITED(status)}: If the child process ends normally, <b>true</b> is returned. Otherwise, <b>false</b> is returned.</li> \n
* <li>{@link WEXITSTATUS(status)}: If {@link WIFEXITED(status)} is <b>true</b>, this macro can be used to obtain the exit code that the child process passed to {@link exit()}.</li> \n
* <li>{@link WTERMSIG(status)}: After a child process ends abnormally, the parent process can obtain the child process exit code {@link SIGUSR2} through {@link WTERMSIG}, indicating that the child process ends abnormally. This is the only case supported.</li></ul> \n
* @return Returns the child process ID if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------------------------------
* ECHILD | The child process does not exist, or the specified process group does not exist.
* ESRCH | The child process ends abnormally.
* @since 1.0
* @version 1.0
*/
pid_t wait(int *status);

/**
* @brief Waits for a specified child process to end and reclaims its resources.
*
* @param pid Indicates the ID of the child process to wait for.\n
* <ul><li>If this parameter is less than <b>-1</b>, the system waits for any child process whose process group ID is the absolute value of <b>pid</b>.</li> \n
* <li>If this parameter is <b>-1</b>, the system waits for any child process. In this case, this function is equivalent to {@link wait()}.</li> \n
* <li>If this parameter is <b>0</b>, the system waits for a child process whose process group ID is the same as the calling process (any process that is in the same process group as the calling process).</li> \n
* <li>If this parameter is greater than <b>0</b>, the system waits for the child process whose process ID is the value of <b>pid</b>.</li></ul> \n
* @param status Indicates the pointer to the obtained status information. If this parameter is not <b>NULL</b>, the status information is stored in the int value that it points to.\n
* You can use the following macros defined in {@link private.h} to check the integer (the macro uses the integer as a parameter instead of the pointer that points to it): \n
* <ul><li>{@link WIFEXITED(status)}: If the child process ends normally, <b>true</b> is returned. Otherwise, <b>false</b> is returned.</li> \n
* <li>{@link WEXITSTATUS(status)}: If {@link WIFEXITED(status)} is <b>true</b>, this macro can be used to obtain the exit code that the child process passed to {@link exit()}.</li> \n
* <li>{@link WTERMSIG(status)}: After a child process ends abnormally, the parent process can obtain the child process exit code {@link SIGUSR2} through {@link WTERMSIG}, indicating that the child process ends abnormally. This is the only case supported.</li></ul> \n
* {@link WIFSIGNALED}, {@link WIFSTOPPED}, {@link WSTOPSIG}, {@link WCOREDUMP}, {@link WIFCONTINUED}, and {@link WUNTRACED} are not supported. \n
* @param options Provides some options to control the behavior of this function. If you do not want to use these options, set this parameter to <b>0</b>. \n
* {@link WNOHANG}: If the child process specified by <b>pid</b> is not ended, this function returns <b>0</b> immediately instead of blocking the calling process. If the child process is ended, the process ID of the child process is returned.
* {@link WUNTRACED}, {@link WEXITED}, {@link WSTOPPED}, {@link WCONTINUED}, and {@link WNOWAIT} are not supported.
* @return Returns the child process ID if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|-----------------------------------------
* EINVAL | Invalid options.
* EOPNOTSUPP | Unsupported options.
* ECHILD | The child process does not exist, or the specified process group does not exist.
* ESRCH | The child process ends abnormally.
* @since 1.0
* @version 1.0
*/
pid_t waitpid(pid_t pid, int *status, int options);


#ifdef __cplusplus
}
#endif
#endif


/** @} */