/**
* @addtogroup TIME
* @{
*
* @brief Provides time-related structures and functions.
*
* @since 1.0
* @version 1.0
*/

/**
* @file time.h
*
* @brief Provides time-related structures and functions.
*
* You can perform time-related management operations, including obtaining and setting broken-down time and system ticks, and manipulating timers.
*
* @since 1.0
* @version 1.0
*/
#ifndef	_TIME_H
#define _TIME_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Describes date and time information.
*
*/
struct tm {
/** Second. The value ranges from 0 to 60. */
int tm_sec;
/** Minute. The value ranges from 0 to 59. */
int tm_min;
/** Hour. The value ranges from 0 to 23. */
int tm_hour;
/** Day. The value ranges from 1 to 31. */
int tm_mday;
/** Month. The value ranges from 0 to 11. */
int tm_mon;
/** Year. The value is the actual year minus 1900. */
int tm_year;
/** Week. The value ranges from 0 to 6. The value <b>0</b> indicates Sunday, the value <b>1</b> indicates Monday, and so on. */
int tm_wday;
/* Number of days starting from January 1. The value ranges from 0 to 365. The value <b>0</b> indicates January 1, the value <b>1</b> indicates January 2, and so on. */
int tm_yday;
/** Daylight saving time (DST) */
int tm_isdst;
/** Seconds east of UTC */
long __tm_gmtoff;
/** Time zone */
const char *__tm_zone;
};

/**
* @brief Obtains the time.
*
* The time obtained is the total number of seconds elapsed since January 1, 1970 00:00:00 (UTC). \n
* @param t Indicates the pointer to the number of seconds. You can also pass <b>NULL</b> to use the return value.
* @return Returns the number of seconds.
* @since 1.0
* @version 1.0
*/
time_t time(time_t *t);

/**
* @brief Calculates the difference between two times, in seconds.
*
* @param time1 Indicates the first time.
* @param time2 Indicates the second time.
* @return Returns the difference, represented using a double.
* @since 1.0
* @version 1.0
*/
double difftime(time_t time1, time_t time2);

/**
* @brief Converts the broken-down time in the {@link tm} structure into seconds.
*
* @param tm Indicates the pointer to the broken-down time in the {@link tm} structure.
* @return Returns the total number of seconds elapsed since January 1, 1970 00:00:00 (UTC) if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EOVERFLOW | Inappropriate input parameter.
* @since 1.0
* @version 1.0
*/
time_t mktime(struct tm *tm);

/**
* @brief Converts the broken-down time in the {@link tm} structure to a string in the required format.
*
* @param s Indicates the pointer to the string.
* @param n Indicates the size of the buffer for storing the string.
* @param f Indicates the pointer to the required format.
* @param tm Indicates the pointer to the broken-down time in the {@link tm} structure.
* @return Returns the number of bytes in the string if the conversion is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
size_t strftime(char *restrict s, size_t n, const char *restrict f, const struct tm *restrict tm);

/**
* @brief Converts the number of seconds to the UTC time in the {@link tm} structure.
*
* @param t Indicates the pointer to the number of seconds to convert.
* @return Returns the pointer to the UTC time in the {@link tm} structure if the conversion is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EOVERFLOW | Inappropriate input parameter.
* @since 1.0
* @version 1.0
*/
struct tm *gmtime(const time_t *t);

/**
* @brief Converts the number of seconds to the local time in the {@link tm} structure.
*
* @param t Indicates the pointer to the number of seconds to convert.
* @return Returns the pointer to the local time in the {@link tm} structure if the conversion is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EOVERFLOW | Inappropriate input parameter.
* @since 1.0
* @version 1.0
*/
struct tm *localtime(const time_t *t);

/**
* @brief Converts the broken-down time in the {@link tm} structure into a string.
*
* @param tm Indicates the pointer to the broken-down time in the {@link tm} structure.
* @return Returns the string in the format of <b>week month day hour:minute:second year</b>, for example, <b>Thu Jan 1 08:00:00 1970</b>. If the conversion fails, the program ends.
* @since 1.0
* @version 1.0
*/
char *asctime(const struct tm *tm);

/**
* @brief Converts the date and time into a string.
*
* @param timep Indicates the number of seconds to convert.
* @return Returns the string in the format of <b>week month day hour:minute:second year</b>, for example, <b>Thu Jan 1 08:00:00 1970</b>. If the conversion fails, the program ends.
* @since 1.0
* @version 1.0
*/
char *ctime(const time_t *timep);


/**
* @brief Converts the broken-down time in the {@link tm} structure to a string in a specified programming language and format.
*
* @param s Indicates the pointer to the string.
* @param n Indicates the size of the buffer for storing the string.
* @param f Indicates the pointer to the required format.
* @param tm Indicates the pointer to the broken-down time in the {@link tm} structure.
* @param loc Indicates the required programming language. Currently, only C programming language is supported.
* @return Returns the number of bytes in the string if the conversion is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
size_t strftime_l(char *  __restrict s, size_t n, const char *  __restrict f, const struct tm *  __restrict tm, locale_t loc);

/**
* @brief Converts the number of seconds to the UTC time in the {@link tm} structure. (This function is reentrant.)
*
* This function is used in the multi-task environment. \n
* @param t Indicates the pointer to the number of seconds to convert.
* @param tm Indicates the pointer to the {@link tm} structure.
* @return Returns the pointer to the UTC time in the {@link tm} structure if the conversion is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EOVERFLOW | Inappropriate input parameter.
* @since 1.0
* @version 1.0
*/
struct tm *gmtime_r(const time_t *__restrict t, struct tm *__restrict tm);

/**
* @brief Converts the number of seconds to the local time in the {@link tm} structure. (This function is reentrant.)
*
* @param t Indicates the pointer to the number of seconds to convert.
* @param tm Indicates the pointer to the {@link tm} structure.
* @return Returns the pointer to the local time in the {@link tm} structure if the conversion is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EOVERFLOW | Inappropriate input parameter.
* @since 1.0
* @version 1.0
*/
struct tm *localtime_r(const time_t *__restrict t, struct tm *__restrict tm);

/**
* @brief Converts the broken-down time in the {@link tm} structure into a string. (This function is reentrant.)
*
* @param tm Indicates the pointer to the broken-down time in the {@link tm} structure.
* @param buf Indicates the pointer to the buffer for storing the string.
* @return Returns the string in the format of <b>week month day hour:minute:second year</b>, for example, <b>Thu Jan 1 08:00:00 1970</b>. If the conversion fails, the program ends.
* @since 1.0
* @version 1.0
*/
char *asctime_r(const struct tm *__restrict tm, char *__restrict buf);

/**
* @brief Converts the date and time into a string. (This function is reentrant.)
*
* @param t Indicates the pointer to the number of seconds to convert.
* @param buf Indicates the pointer to the buffer for storing the string.
* @return Returns the string in the format of <b>week month day hour:minute:second year</b>, for example, <b>Thu Jan 1 08:00:00 1970</b>. If the conversion fails, the program ends.
* @since 1.0
* @version 1.0
*/
char *ctime_r(const time_t *t, char *buf);

/**
* @brief Sets a timer.
*
*/
struct itimerspec {
/** Interval between two consecutive timer triggers */
struct timespec it_interval;
/** Initial timeout duration */
struct timespec it_value;
};

/**
* @brief Defines the clock that runs in real time.
*
*/
#define CLOCK_REALTIME           0

/**
* @brief Pauses the current thread until a specified time arrives.
*
* A sleeping thread cannot be woken up by a signal. \n
*
* @param tspec1 Indicates the pointer to the minimum duration that the current thread is paused. Currently, the unit of precision is tick, and the discrepancy is fewer than 2 ticks.
* @param tspec2 This parameter is not used yet.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* @since 1.0
* @version 1.0
*/
int nanosleep(const struct timespec *tspec1, struct timespec *tspec2);

/**
* @brief Obtains the precision of a clock.
*
* @param id Indicates the clock ID. Currently, the following values are supported: <b>CLOCK_REALTIME</b>, <b>CLOCK_REALTIME_COARSE</b>, <b>CLOCK_MONOTONIC</b>, <b>CLOCK_MONOTONIC_COARSE</b>, and <b>CLOCK_MONOTONIC_RAW</b>.
* @param tspec Indicates the pointer to the data obtained.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> otherwise.
* @since 1.0
* @version 1.0
*/
int clock_getres(clockid_t id, struct timespec *tspec);

/**
* @brief Obtains the time of a clock.
*
* @param id Indicates the clock ID. Currently, the following values are supported: <b>CLOCK_REALTIME</b>, <b>CLOCK_REALTIME_COARSE</b>, <b>CLOCK_MONOTONIC</b>, <b>CLOCK_MONOTONIC_COARSE</b>, and <b>CLOCK_MONOTONIC_RAW</b>.
* @param tspec Indicates the pointer to the time obtained.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* @since 1.0
* @version 1.0
*/
int clock_gettime(clockid_t id, struct timespec *tspec);

/**
* @brief Sets the time for a clock.
*
* @param id Indicates the clock ID. Only <b>CLOCK_REALTIME</b> is supported.
* @param tspec Indicates the pointer to the time to set.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* @since 1.0
* @version 1.0
*/
int clock_settime(clockid_t id, const struct timespec *tspec);

/**
* @brief Pauses the current thread until a specified time of a clock arrives.
*
* A sleeping thread cannot be woken up by a signal. \n
*
* @param id Indicates the clock ID. Only <b>CLOCK_REALTIME</b> is supported.
* @param flag Indicates the clock type. This parameter must be set to <b>0</b>
* @param tspec1 Indicates the pointer to the minimum duration that the current thread is paused. Currently, the unit of precision is tick, and the discrepancy is fewer than 2 ticks.
* @param tspec2 This parameter is not used yet.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* @since 1.0
* @version 1.0
*/
int clock_nanosleep(clockid_t id, int flag, const struct timespec *tspec1, struct timespec *tspec2);

/**
* @brief Creates a timer for the process.
*
* @attention <b>sigev_notify</b> in the <b>sigevent</b> structure must be <b>SIGEV_SIGNAL</b>.
* @param id Indicates the clock ID. Only <b>CLOCK_REALTIME</b> is supported.
* @param evp Indicates the pointer to the asynchronous notification signal and action, which can be <b>NULL</b>.
* @param t Indicates the pointer to the timer ID.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* ENOTSUP | The value of <b>evp</b> is not <b>NULL</b> and <b>sigev_notify</b> is not <b>SIGEV_SIGNAL</b>.
* @since 1.0
* @version 1.0
*/
int timer_create(clockid_t id, struct sigevent *__restrict evp, timer_t *__restrict t);

/**
* @brief Deletes a timer for the process.
*
* @param t Indicates the ID of the timer to delete.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* @since 1.0
* @version 1.0
*/
int timer_delete(timer_t t);

/**
* @brief Sets a timer for the process.
*
* @param t Indicates the ID of the timer to set.
* @param flags Indicates the type of the timer to set. This parameter is not supported and must be set to <b>0</b>.
* @param val Indicates the pointer to the timer duration and interval to set.
* @param old Indicates the pointer to the timer duration and interval before the current setting.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* ENOSYS | The value of <b>flags</b> is not <b>0</b>.
* EINVAL | Incorrect parameter or timer operation failure.
* @since 1.0
* @version 1.0
*/
int timer_settime(timer_t t, int flags, const struct itimerspec *__restrict val, struct itimerspec *__restrict old);

/**
* @brief Obtains a timer of the process.
*
* @param t Indicates the ID of the timer to obtain.
* @param tspec Indicates the pointer to the timer duration and interval.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect parameter or timer operation failure.
* @since 1.0
* @version 1.0
*/
int timer_gettime(timer_t t, struct itimerspec *tspec);

/**
* @brief Obtains the number of times that a timer overruns.
*
* @param t Indicates the ID of the timer to obtain.
* @return Returns the number of times if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* @since 1.0
* @version 1.0
*/
int timer_getoverrun(timer_t t);

/**
* @brief Converts a time string to the broken-down time in the {@link tm} structure.
*
* This function parses the input string <b>s</b> based on <b>format</b> and stores the result in the {@link tm} structure. \n
* @param s Indicates the pointer to the string that contains only time data.
* @param format Indicates the pointer to the required format.
* @param tm Indicates the pointer to the {@link tm} structure.
* @return Returns the pointer to the position to which the string has been processed (the next character to be processed) upon the conversion completion if the operation is successful; returns <b>NULL</b> otherwise.
* @since 1.0
* @version 1.0
*/
char *strptime(const char *s, const char *format, struct tm *tm);

/**
* @brief Converts a time string to the broken-down time in the {@link tm} structure.
*
* @param buf Indicates the pointer to the time string. The format is specified by the file defined by the environment variable <b>DATEMSK</b>.
* @return Returns the time in the {@link tm} structure if the operation is successful; returns <b>NULL</b> otherwise.
* @since 1.0
* @version 1.0
*/
struct tm *getdate(const char *buf);

/**
* @brief Sets the system time.
*
* @param t Indicates the pointer to the number of seconds to set.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Invalid input.
* @since 1.0
* @version 1.0
*/
int stime(const time_t *t);

/**
* @brief Converts the broken-down time in the {@link tm} structure to the number of seconds.
*
* @param tm Indicates the pointer to the broken-down time in the {@link tm} structure to convert.
* @return Returns the number of seconds if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EOVERFLOW | The input parameter is too long.
* @since 1.0
* @version 1.0
*/
time_t timegm(struct tm *tm);

#ifdef __cplusplus
}
#endif

#endif
/** @} */