/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file unistd.h
*
* @brief Provides functions and data structures related to process operations.
*
* For example, you can use the functions to create and copy a process, and obtain a process ID.
*
* @since 1.0
* @version 1.0
*/

#ifndef	 _UNISTD_H
#define	 _UNISTD_H

#ifdef __cplusplus
extern "C" {
#endif


/**@brief Descriptor ID of the standard input file */
#define STDIN_FILENO  0
/**@brief Descriptor ID of the standard output file */
#define STDOUT_FILENO 1
/**@brief Descriptor ID of the standard error file */
#define STDERR_FILENO 2

/**@brief Relocation starts from the file header. */
#define SEEK_SET 0
/**@brief Relocation starts from the position of the currently accessed file. */
#define SEEK_CUR 1
/**@brief Relocation starts from the end of the file. */
#define SEEK_END 2

/**@brief NULL */
#define NULL ((void*)0)

/**
* @brief Creates an anonymous pipe.
*
* @attention A maximum of 32 anonymous pipes can be created, and the internal buffer of a pipe is 1023 bytes.
* @param pipefd Indicates the pointer to the buffer for storing the created pipe descriptor. The buffer has two sizes of the <b>int</b> type, one for the read-end descriptor and the other for the write-end descriptor.
* @return Returns <b>0</b> if the creation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the creation fails.
* errno | Description
* -------|------------------------------------------
* EFAULT | <b>pipefd</b> is invalid.
* ENOMEM | Creation failed due to insufficient space.
* ENFILE | Failed to create an anonymous pipe because the number of anonymous pipes has reached the upper limit.
* @since 1.0
* @version 1.0
*/
int pipe(int pipefd[2]);
/**
* @brief Closes a file with a specified file descriptor.
*
* @param fd Indicates the descriptor of the file to be closed.
* @return Returns <b>0</b> if the file is closed; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the file fails to be closed.
* errno | Description
* ------ | ----------------------------------------------------
* EBADF | <b>fd</b> is invalid.
* EINVAL | The file is not owned by the current process.
* ENOENT | The file does not exist.
* ENODEV | The disk partition is not found.
* @since 1.0
* @version 1.0
*/
int close(int fd);

/**
* @brief Copies a specified file descriptor.
*
* @attention This function can be used only in the NFS file system.
* @param oldfd Indicates the file descriptor to be copied.
* @return Returns a non-negative integer (minimum descriptor available to the current process) if the copy is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the copy fails.
* errno | Description
* ------ | -----------------------------------------------
* EBADF | <b>oldfd</b> is invalid.
* EMFILE | Failed to allocate file descriptors because too many files are opened.
* ENOENT | The index node cannot be found based on the path name associated with <b>oldfd</b>.
* ENOSYS | The operation is not supported.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int dup(int oldfd);

/**
* @brief Copies the descriptor of the target file to a specified descriptor.
*
* @param oldfd Indicates the file descriptor to be copied.
* @param newfd Indicates the new file descriptor for copying.
* @return Returns a non-negative integer (descriptor of the new file) if the copy is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the copy fails.
* errno | Description
* ------ | -----------------------------------------------------
* EBADF | <b>oldfd</b> is invalid.
* EMFILE | Failed to allocate file descriptors because too many files are opened.
* ENOENT | The index node cannot be found based on the path name associated with <b>oldfd</b>.
* ENOSYS | The operation is not supported.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int dup2(int oldfd, int newfd);

/**
* @brief Sets the offset of the pointer to the file.
*
* The pointer to the file is used to read and write data. \n
* @param fd Indicates the file descriptor.
* @param offset Indicates the pointer offset to be set.
* @param whence Indicates the pointer position. The value can be <b>SEEK_CUR</b> (current position), <b>SEEK_SET</b> (beginning of the file), or <b>SEEK_END</b> (end of the file).
* @return Returns the offset to the beginning of the file if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -----------------------
* EBADF | <b>fd</b> is invalid.
* ENOSYS | This operation is not supported.
* EINVAL | <b>whence</b> is invalid.
* EIO | A hardware error occurs at the I/O layer.
* @since 1.0
* @version 1.0
*/
off_t lseek(int fd, off_t offset, int whence);

/**
* @brief Synchronizes a file associated with a specified file descriptor to the storage device.
*
* @param fd Indicates the descriptor of the file to be synchronized.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -----------------------
* EBADF | <b>fd</b> is invalid.
* EINVAL | <b>fd</b> is invalid for this device.
* ENOSYS | The operation is not supported.
* @since 1.0
* @version 1.0
*/
int fsync(int fd);

/**
* @brief Reads the file contents and saves them in a specified buffer location.
*
* @param fd Indicates the descriptor of the file to read.
* @param buf Indicates the pointer to the buffer for storing the data read.
* @param size Indicates the length of the content to read. If the length exceeds that of the buffer, memory corruption may occur.
* @return Returns the number of bytes read if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -----------------------
* EBADF | <b>fd</b> is invalid.
* EACCES | The file cannot be read.
* EFAULT | <b>buf</b> is set to <b>NULL</b>.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
ssize_t read(int fd, void *buf, size_t size);

/**
* @brief Writes the specified content to the file.
*
* @attention This function can NOT be used in the PROC file system.
* @param fd Indicates the descriptor of the file into which content is to be written.
* @param buf Indicates the pointer to the content to be written to the file.
* @param size Indicates the length of the written data in the unit of bytes.
* @return Returns the number of bytes written if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ----------------------------------------------
* EBADF | <b>fd</b> is invalid.
* EACCES | The file cannot be written.
* EFAULT | <b>buf</b> is an inaccessible address.
* EAGAIN | {@link O_NONBLOCK} is used to select non-blocking I/O, and the write would block.
* ENOSYS | The operation is not supported.
* EINVAL | The current position of the file pointer is less than <b>0</b>.
* ENOSPC | The storage device to which the file is written has no space.
* ENOMEM | Insufficient memory.
* EFBIG | An attempt is made to write a file whose length is greater than the defined maximum file size.
* @since 1.0
* @version 1.0
*/
ssize_t write(int fd, const void *buf, size_t size);

/**
* @brief Reads data whose offset is <b>offset</b> and length is <b>count</b> from <b>fd</b> to the buffer.
*
* @attention The <b>fd</b> offset does not change after the function is called.
* This function is helpful in multithreaded applications. They allow multiple threads to execute the I/O on the same file descriptor without being affected by changes in other file offsets.
* It is not an error if the number of transferred bytes is less than the number of requested bytes after the function is called.
* @param fd Indicates the file descriptor to read.
* @param buf Indicates the pointer to the buffer for storing data.
* @param count Indicates the length of the data to read.
* @param offset Indicates the offset of the file descriptor to read.
* @return Returns the number of records read if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value if the operation fails.
* @since 1.0
* @version 1.0
*/
ssize_t pread(int fd, void *buf, size_t count, off_t offset);

/**
* @brief Writes data from the buffer to <b>fd</b> whose offset is <b>offset</b> and length is <b>count</b>.
*
* @attention The <b>fd</b> offset does not change after the function is called.
* This function is helpful in multithreaded applications. They allow multiple threads to execute the I/O on the same file descriptor without being affected by changes in other file offsets.
* It is not an error if the number of transferred bytes is less than the number of requested bytes after the function is called. For details, see {@link write()}.
* @param fd Indicates the file descriptor to write.
* @param buf Indicates the pointer to the buffer to which data is written.
* @param count Indicates the size of the data to write.
* @param offset Indicates the offset of the file to write.
* @return Returns the number of records written if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value if the operation fails. Global variables of the <b>int</b> type can be directly obtained.
* @since 1.0
* @version 1.0
*/
ssize_t pwrite(int fd, const void *buf, size_t count, off_t offset);

/**
* @brief Deletes a specified file.
*
* @attention This function can NOT be used in the PROC file system.
* @param path Indicates the path of the file to be deleted.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------------ | -------------------------------------------------------------
* EINVAL | <b>path</b> is a null pointer or an empty string.
* ENAMETOOLONG | The length of the path name is greater than {@link NAME_MAX}.
* ENOMEM | Insufficient memory.
* ENOENT | The file does not exist.
* ENOSYS | The operation is not supported.
* ENOTEMPTY | The path is a pseudo file used as a directory, or the directory is not empty.
* EISDIR | The specified path is a directory.
* ENXIO | No such device or address.
* EACCES | The search permission on the path prefix component is denied, or the write permission of the directory in the path is denied.
* @since 1.0
* @version 1.0
*/
int unlink(const char *path);

/**
* @brief Deletes a specified file.
*
* @attention This function can NOT be used in the PROC file system.
* @attention This function does not support relative paths.
* @param fd Indicates the descriptor of the directory where the file to be deleted is located. Currently, this parameter can only be set to {@link AT_FDCWD}.
* @param path Indicates the path of the file to be deleted. This parameter must be set to an absolute path.
* @param flag Indicates the operation flag. The value can only be <b>0</b> (deleting a file) or {@link AT_REMOVEDIR} (deleting a directory, equivalent to {@link rmdir}). If this parameter is set to other values, the function will fail.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------------ | -------------------------------------------------------------
* EINVAL | <b>path</b> is a null pointer or an empty string.
* ENAMETOOLONG | The length of the path name is greater than {@link NAME_MAX}.
* ENOMEM | Insufficient memory.
* ENOENT | The file does not exist.
* ENOSYS | The operation is not supported.
* ENOTEMPTY | The path is a pseudo file used as a directory, or the directory is not empty.
* EISDIR | The specified path is a directory.
* ENXIO | No such device or address.
* EACCES | The search permission on the path prefix component is denied, or the write permission of the directory in the path is denied.
* @since 1.0
* @version 1.0
*/
int unlinkat(int fd, const char *path, int flag);

/**
* @brief Deletes a directory.
*
* @attention This function can NOT be used in the PROC file system.
* @param path Indicates the name of the directory to be deleted.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------------ | ---------------------------------------------
* EINVAL | <b>path</b> is a null pointer or an empty string.
* ENAMETOOLONG | The length of the path name is greater than {@link PATH_MAX}.
* ENOMEM | Insufficient memory.
* ENOENT | The directory does not exist.
* EPERM | The path is a mount point, or it is neither a directory nor a file.
* ENOSYS | The operation is not supported.
* ENOTEMPTY | The directory is not empty.
* ENOTDIR | The specified path is not a directory.
* ENXIO | No such device or address.
* @since 1.0
* @version 1.0
*/
int rmdir(const char *path);

/**
* @brief Truncates a file to a specified size based on the file path.
*
* @attention This function can be used only in the FAT file system.
* @param path Indicates the path of the file to be truncated.
* @param length Indicates the length of the file to be truncated.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------------ | -----------------------------------------------------------
* EACCES | The file has no write permission.
* EBADF | Failed to open the file in the specified path.
* ENAMETOOLONG | Failed to open the file because the length of path is greater than {@link PATH_MAX}.
* ENOMEM | Insufficient memory.
* EPERM | Failed to obtain the file structure corresponding to the file descriptor when opening the file.
* EMFILE | Failed to open the file because the maximum number of files that can be opened has been reached.
* EINVAL | The target length is less than <b>0</b>.
* ENOSYS | The operation is not supported.
* EPERM | Failed to obtain the file structure corresponding to the file specified by <b>path</b>, or the file access is rejected.
* @since 1.0
* @version 1.0
*/
int truncate(const char *path, off_t length);

/**
* @brief Truncates a file to a specified length.
*
* @attention This function can be used only in the FAT file system based on the file descriptor.
* @param fd Indicates the file descriptor.
* @param length Indicates the length of the file to be truncated.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -----------------------------------------------------------
* EACCES | The file has no write permission.
* EBADF | Failed to open the file in the specified path.
* EINVAL | The target length is less than <b>0</b>.
* ENOSYS | The operation is not supported.
* EPERM | Failed to obtain the file structure corresponding to the file specified by <b>path</b>, or the file access is rejected.
* @since 1.0
* @version 1.0
*/
int ftruncate(int fd, off_t length);

/**@brief Existing file */
#define F_OK 0
/**@brief Readable file */
#define R_OK 4
/**@brief Writable file */
#define W_OK 2
/**@brief Executable file */
#define X_OK 1

/**
* @brief Checks whether a file has the corresponding permission.
*
* @param path Indicates the path of the file to be checked.
* @param mode Indicates the access permission to be checked.
* @return Returns <b>0</b> if the current file has the permission to be checked; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the current file does not have the permission to be checked or other errors occur.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EACCES | The permission bit of the file mode does not allow the requested access, or the search permission is denied on the path prefix component.
* EFAULT | Invalid address.
* ENOENT | The component of <b>path</b> does not exist or the path is an empty string.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int access(const char *path, int mode);

/**
* @brief Switches the current working directory to a specified directory.
*
* @param path Indicates the target working directory.
* @return Returns <b>0</b> if the working directory is switched; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the working directory fails to be switched and the current working directory remains unchanged.
* errno | Description
* ------------ | -----------------------
* EINVAL | <b>path</b> is set to <b>NULL</b>.
* ENAMETOOLONG | The length of the path name is greater than {@link PATH_MAX}.
* ENOMEM | Insufficient memory.
* ENOTDIR | <b>path</b> is not a directory name.
* ENOENT | The path does not exist.
* @since 1.0
* @version 1.0
*/
int chdir(const char *path);
/**
* @brief Obtains the current working directory.
*
* @param buf Indicates the pointer to the buffer of the array for storing the absolute path name of the current working directory.
* @param size Indicates the size of the array to which the buffer points.
* @return Returns <b>buf</b> if the operation is successful; returns <b>NULL</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------------ | ----------------------------------------------
* EINVAL | <b>buf</b> is set to <b>NULL</b>.
* ERANGE | <b>size</b> is less than the length of the current working directory (including the terminating null byte \0).
* ENAMETOOLONG | Failed to copy the name of the current working directory to the buffer using <b>memcpy_s</b>.
* @since 1.0
* @version 1.0
*/
char *getcwd(char *buf, size_t size);

/**
* @brief Arranges a signal to be sent to the current process after the number of seconds specified by <b>seconds</b>.
*
*  \n
* @attention An exception occurs when this function is used together with <b>setitimer()</b>, {@link ualarm()}, and <b>sleep()</b>. This function shares a timer with <b>setitimer()</b> and {@link ualarm()}. You are not advised to use this function together with <b>sleep()</b>.
* @param seconds Indicates an unsigned integer <b>seconds</b> needs to be transferred.
* @return Returns the remaining seconds until the countdown of <b>seconds</b> ends; returns <b>0</b> if no alarm clock is set.
* @since  1.0
* @version  1.0
*/
unsigned int alarm(unsigned int seconds);

/**
* @brief Sleeps for a period of time.
*
* This function is used to stop the current thread until the specified time. A sleeping thread cannot be woken up by a signal. \n
* @param seconds Indicates the sleeping seconds.
* @return Returns the seconds if the operation is successful; returns <b>0</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect input.
* @since  1.0
* @version  1.0
*/
unsigned sleep(unsigned seconds);

/**
* @brief Waits for signal.
*
* This function is used to stop the current process or thread until a signal is delivered. \n
* @return Returns when a signal was caught and returns <b>-1</b> and sets <b>errno</b> to EINTR.
* errno | Description
* ----- | ---------------------------------
* EINTR | A signal was caught during execution.
* @since  1.0
* @version  1.0
*/
int pause(void);

/**
* @brief Creates a new process that inherits from the user-mode data of its parent process.
*
* @return Returns <b>0</b> in the child process and returns the ID of the created child process in the parent process if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------------------------------------
* EAGAIN | Insufficient process or thread control blocks.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
pid_t fork(void);

/**
* @brief Executes a specified user program file in ELF format.
*
* @param path Indicates the full path of the ELF file.
* @param arg Indicates the list of program parameters maintained by the pointer to the array. The last element of the array must be a null pointer.
* @param envp Indicates the pointer to the string for storing new environment variables.
* @return Returns <b>-1</b> and sets <b>errno</b> to the corresponding error code {@link perror} if the operation fails.
* @since 1.0
* @version 1.0
*/
int execve(const char *path, char *const arg[], char *const envp[]);

/**
* @brief Executes a specified user program file in ELF format.
*
* @attention This function is slightly different from {@link execve} in input parameters. The <b>arg</b> parameter is a string array for storing new environment variables.
* @param path Indicates the full path of the ELF file.
* @param arg Indicates the list of program parameters maintained by the pointer to the array. The last element of the array must be a null pointer.
* @return Returns <b>-1</b> and sets <b>errno</b> to the corresponding error code {@link perror} if the operation fails.
* @since 1.0
* @version 1.0
*/
int execv(const char *path, char *const arg[]);

/**
* @brief Executes a specified user program file in ELF format.
*
* @param path Indicates the full path of the ELF file.
* @param arg Indicates the list of parameters required by the program. The <b>arg</b> parameter must point to a new environment variable array, that is, the environment variable of the new program.
* @return Returns <b>-1</b> and sets <b>errno</b> to the corresponding error code {@link perror} if the operation fails.
* @since 1.0
* @version 1.0
*/
int execle(const char *path, const char *arg, ...);

/**
* @brief Executes a specified user program file in ELF format.
*
* @param path Indicates the full path of the ELF file.
* @param arg Indicates the list of parameters required by the program, ending with a null pointer.
* @return Returns <b>-1</b> and sets <b>errno</b> to the corresponding error code {@link perror} if the operation fails.
* @since 1.0
* @version 1.0
*/
int execl(const char *path, const char *arg, ...);

/**
* @brief Executes a specified user program file in ELF format.
*
* @param path Indicates the name or path name of the program file to be executed.
* @param arg Indicates the list of parameters required by the program, ending with a null pointer.
* @return Returns <b>-1</b> and sets <b>errno</b> to the corresponding error code {@link perror} if the operation fails. \n
* If the specified file name contains a slash (/), the file is executed. If it does not contain a slash (/), the current user program traverses the path list specified by the path environment variable. If the path environment variable is not defined, the current user program traverses ("/usr/local/bin:/bin:/usr/bin"). \n
* The path list and file name are combined as an executable file and the file is executed. \n
* If the file does not have the execute permission, the system continues to search for other path lists. If no other file is found, <b>EACCES</b> is returned. \n
* If no valid executable file exists in the list of all paths, <b>-1</b> is returned.
* @since 1.0
* @version 1.0
*/
int execvp(const char *path, char *const arg[]);

/**
* @brief Executes a specified user program file in ELF format.
*
* @param path Indicates the name or path name of the program file to be executed.
* @param arg Indicates the list of parameters required by the program, ending with a null pointer.
* @return Returns <b>-1</b> and sets <b>errno</b> to the corresponding error code {@link perror} if the operation fails. \n
* If the specified file name contains a slash (/), the file is executed. If it does not contain a slash (/), the current user program traverses the path list specified by the path environment variable. If the path environment variable is not defined, the current user program traverses ("/usr/local/bin:/bin:/usr/bin"). \n
* The path list and file name are combined as an executable file and the file is executed. \n
* If the file does not have the execute permission, the system continues to search for other path lists. If no other file is found, <b>EACCES</b> is returned. \n
* If no valid executable file exists in the list of all paths, <b>-1</b> is returned.
* @since 1.0
* @version 1.0
*/
int execlp(const char *path, const char *arg, ...);

/**
* @brief Exits the process immediately and closes all opened file descriptors in the process.
*
* The subprocess is taken over by process 1. \n
* @param status Indicates the exit status of the process. The status is returned to the parent process of the current process and can be collected by the {@link wait()} series functions.
* @since 1.0
* @version 1.0
*/
_Noreturn void _exit(int status);

/**
* @brief Swaps bytes.
*
* This function is used to copy n bytes from the <b>from</b> array to the <b>to</b> array, exchanging adjacent even and odd bytes.
* @param from source array.
* @param to destination array.
* @param n number of bytes.
* @since 1.0
* @version 1.0
*/
void swab(const void *from, void *to, ssize_t n);

/**
* @brief Obtains the process ID.

* @return Returns the process ID.
* @since 1.0
* @version 1.0
*/
pid_t getpid(void);

/**
* @brief Obtains the parent process ID.
*
* @return Returns the parent process ID.
* @since 1.0
* @version 1.0
*/
pid_t getppid(void);

/**
* @brief Obtains the ID of the process group of the calling process.
*
* @return Returns the process group ID if the operation is successful.
* @since 1.0
* @version 1.0
*/
pid_t getpgrp(void);

/**
* @brief Obtains the ID of the process group whose process ID is specified by <b>pid</b>.
*
* @param pid Indicates the process ID. If the value is <b>0</b>, the process group ID of the current process is obtained.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid process ID.
* ESRCH | The specified process cannot be found.
* @since 1.0
* @version 1.0
*/
pid_t getpgid(pid_t pid);

/**
* @brief Sets the ID of the process group whose process ID is specified by <b>pid</b>.
*
* @param pid Indicates the process ID. If the value is <b>0</b>, the process group ID of the current process is obtained.
* @param pgid Indicates the process group ID.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|---------------------------------------
* EINVAL | Invalid process ID.
* ESRCH | The specified process cannot be found.
* EPERM | No permission to operate the process group.
* EACCES | The child process has performed <b>exec</b>, and the parent process does not have the operation permission.
* @since 1.0
* @version 1.0
*/
int setpgid(pid_t pid, pid_t pgid);

/**
* @brief Parses command-line arguments based on the specified option.
*
* @param argc Indicates the number of arguments when the file of the function is executed.
* @param argv Indicates the argument content when the file of the function is executed.
* @param optstring Indicates the string containing the optional arguments of the executable file.
* Character Option Expression | Description
*---------------------  | -------------------
* Only characters that indicate different options are available. | <b>abc</b> indicates that the file has three options <b>-a</b>, <b>-b</b>, and <b>-c</b>.
* The character is followed by a colon. | <b>b</b> indicates that <b>-b</b> must be followed by an argument in the executable file that contains the <b>getopt()</b> function.
* The character is followed by two colons. | <b>b</b> indicates that <b>-b</b> can be followed by or without the corresponding argument in the executable file that contains the <b>getopt()</b> function.
* If this function obtains an option without arguments, the value of <b>optind</b> increases by 1. If this function obtains an option with arguments and the result is saved to <b>optarg</b>, the value of <b>optind</b> increases by 2.
* @return Returns the parsed option character if the parsing is successful;
* returns <b>?</b> if an option character that is not in <b>optstring</b> is encountered during parsing.
* When an option is encountered but there is no argument during parsing, this function returns <b>:</b> if the first character of <b>optstring</b> is a colon (:); returns <b>?</b> otherwise.
* Returns <b>-1</b> in the following cases:
* if all options have been parsed or <b>argv[optind]</b> is a null pointer;
* if the first character of the string to which <b>argv[optind]</b> points is not <b>-</b>, that is, the non-option <b>argv</b> parameter cannot appear in the middle of the parameter list. If a non-option parameter is encountered during parsing, the parsing is terminated.
* if the string to which <b>argv[optind]</b> points is <b>-</b>.
* @since  1.0
* @version  1.0
*/
int getopt(int argc, char * const argv[], const char *optstring);

/**
* @brief Obtains the real user ID (UID) of the calling process.
*
* @return Returns the real UID.
* @since 1.0
* @version 1.0
*/
uid_t getuid(void);

/**
* @brief Obtains the effective user ID (UID) of the calling process.
*
* @return Returns the effective UID.
* @since 1.0
* @version 1.0
*/
uid_t geteuid(void);

/**
* @brief Obtains the real group ID (GID) of the calling process.
*
* @return Returns the real GID.
* @since 1.0
* @version 1.0
*/
gid_t getgid(void);

/**
* @brief Obtains the effective group ID (GID) of the calling process.
*
* @return Returns the effective GID.
* @since 1.0
* @version 1.0
*/
gid_t getegid(void);

/**
* @brief Obtains a list of supplementary user group IDs specific to the calling process.
*
* @param size Indicates the maximum size allowed for the supplementary group <b>list</b>. If the value is <b>0</b>, this function obtains the total number of supplementary groups.
* @param list Indicates the list of supplementary groups.
* @return Returns the number of supplementary groups of the calling process if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>size</b> is invalid.
* EFAULT | Invalid address.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int getgroups(int size, gid_t list[]);

/**
* @brief Sets the real user ID for the calling process.
*
* @attention The real user ID is the same as the effective user ID and saved user ID.
* @param uid Indicates the user ID to set.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>uid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int setuid(uid_t uid);

/**
* @brief Sets the effective user ID of the calling process.
*
* @attention The real user ID is the same as the effective user ID and saved user ID.
* @param euid Indicates the effective user ID to set.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>euid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int seteuid(uid_t euid);

/**
* @brief Sets the user group ID of the calling process.
*
* @attention The real user group ID is the same as the effective user group ID and saved user group ID.
* @param gid Indicates the user group ID to set.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>gid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int setgid(gid_t gid);

/**
* @brief Sets the effective user group ID of the calling process.
*
* @attention The real user group ID is the same as the effective user group ID and saved user group ID.
* @param egid Indicates the effective user group ID to set.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>egid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int setegid(gid_t egid);

/**
* @brief Obtains the configuration value of a file.
*
* @param path Indicates the file path.
* @param name Indicates the name of the configuration value.
* @return Returns the actual value of the configuration file if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | -------------------------
* EINVAL | The value of <b>name</b> does not match any existing configuration option.
* @since 1.0
* @version 1.0
*/
long pathconf(const char *path, int name);

/**
* @brief Sets the real and effective user IDs of the calling process.
*
* @attention The real user ID is the same as the effective user ID and saved user ID.
* @param ruid Indicates the real user ID to set. If the value is <b>-1</b>, the real user ID is left unchanged.
* @param euid Indicates the effective user ID to set. If the value is <b>-1</b>, the effective user ID is left unchanged.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>ruid</b> or <b>euid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int setreuid(uid_t ruid, uid_t euid);

/**
* @brief Sets the real and effective user group IDs of the calling process.
*
* @attention The real user group ID is the same as the effective user group ID and saved user group ID.
* @param rgid Indicates the real user group ID to set. If the value is <b>-1</b>, the real user group ID is left unchanged.
* @param egid Indicates the effective user group ID to set. If the value is <b>-1</b>, the effective user group ID is left unchanged.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>rgid</b> or <b>egid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int setregid(gid_t rgid, gid_t egid);

/**
* @brief Sets the process group ID of the calling process.
*
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value if the operation fails.
* @since 1.0
* @version 1.0
*/
pid_t setpgrp(void);


/**
* @brief Sleeps for several microseconds.
*
* This function is used to suspend the current thread for the specified duration. A sleeping thread cannot be woken up by a signal. \n
* @param useconds Indicates the sleeping time in microseconds. The precision unit is tick, and the deviation is less than 2 ticks.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect input parameter.
* @since 1.0
* @version 1.0
*/
int usleep(unsigned useconds);

/**
* @brief Sets a microsecond-level timer.
*
* @param value Indicates the number of microseconds to be set for the timer. The precision unit is tick, and the deviation is ±1 tick.
* @param interval Indicates the interval for triggering the timer. The precision unit is tick, and the deviation is ±1 tick.
* @return Returns the remaining microseconds of the previous timer if the operation is successful; returns <b>0</b> if the operation fails.
* errno | Description
* -------------- | ---------------------------------
* EINVAL | Incorrect input parameter.
* @since 1.0
* @version 1.0
*/
unsigned ualarm(unsigned value, unsigned interval);

/**
* @brief Sets the supplementary user group list of the calling process.
*
* @param size Indicates the maximum size of the supplementary user group list. If the value is <b>0</b>, the list is cleared. The maximum value is <b>255</b>.
* @param list Indicates the target supplementary group list.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>size</b> is invalid.
* EFAULT | Invalid address.
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
int setgroups(size_t size, const gid_t *list);

/**
* @brief Sets the real, effective, and saved user IDs of the calling process.
*
* @attention The real user ID is the same as the effective user ID and saved user ID.
* @param ruid Indicates the real user ID to set. If the value is <b>-1</b>, the real user ID is left unchanged.
* @param euid Indicates the effective user ID to set. If the value is <b>-1</b>, the effective user ID is left unchanged.
* @param suid Indicates the saved user ID to set. If the value is <b>-1</b>, the saved user ID is left unchanged.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>ruid</b>, <b>euid</b>, or <b>suid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int setresuid(uid_t ruid, uid_t euid, uid_t suid);

/**
* @brief Sets the real, effective, and saved group IDs of the calling process.
*
* @attention The real user group ID is the same as the effective user group ID and saved user group ID.
* @param rgid Indicates the real user group ID to set. If the value is <b>-1</b>, the real user group ID is left unchanged.
* @param egid Indicates the effective user group ID to set. If the value is <b>-1</b>, the effective user group ID is left unchanged.
* @param sgid Indicates the saved group ID to set. If the value is <b>-1</b>, the saved user group ID is left unchanged.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EINVAL | <b>rgid</b>, <b>egid</b>, or <b>sgid</b> is invalid.
* EPERM | No setting permission.
* @since 1.0
* @version 1.0
*/
int setresgid(gid_t rgid, gid_t egid, gid_t sgid);

/**
* @brief Obtains the real, effective, and saved user IDs of the calling process.
*
* @param ruid Indicates the pointer to the real user ID.
* @param euid Indicates the pointer to the effective user ID.
* @param suid Indicates the pointer to the saved user ID.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EFAULT | Invalid address.
* @since 1.0
* @version 1.0
*/
int getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);

/**
* @brief Obtains the real, effective, and saved user group IDs of the calling process.
*
* @param rgid Indicates the pointer to the real user group ID.
* @param egid Indicates the pointer to the effective user group ID.
* @param sgid Indicates the pointer to the saved user group ID.
* @return Returns <b>0</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* ------ | ---------------------------------------------------------------------
* EFAULT | Invalid address.
* @since 1.0
* @version 1.0
*/
int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid);

/**
* @brief Obtains the name of the current working directory.
*
* @return Returns the name of the current working directory if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
char *get_current_dir_name(void);

/**
* @brief Adjusts the heap size of a process.
*
* @param increment Indicates the size of the memory space to obtain.
* @return Returns the pointer to the memory space if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the operation fails.
* errno | Description
* -------|------------------------------------------
* ENOMEM | Insufficient memory.
* @since 1.0
* @version 1.0
*/
void *sbrk(intptr_t increment);

/**
* @brief Obtains the memory page size.
*
* @return Returns the number of bytes in a memory page.
* @since 1.0
* @version 1.0
*/
int getpagesize(void);

/**
* @brief Changes the user and group ownership of a file.
*
* @attention This function can be used only in the JFFS2 file system.
* @param pathname Indicates the pointer to the file path.
* @param owner Indicates the new file owner.
* @param group Indicates the new file group.
* @return Returns <b>0</b> if the change is successful; returns <b>-1</b> and sets <b>errno</b> to a value in the following table if the change fails.
* errno | Description
* ------------ | ---------------------------------------------------------------------
* EINVAL | <b>pathname</b> is a null pointer or an empty string.
* EACCES | The permission bit of the file mode does not allow the requested access, or the search permission is denied on the path prefix directory.
* EFAULT | Invalid address.
* ENAMETOOLONG | The length of the path name is greater than {@link NAME_MAX}.
* ENOENT | The directory component in the path name does not exist.
* ENOMEM | Insufficient memory.
* ENOTDIR | A component of the path prefix is the name of an existing file, which is neither a directory nor a symbolic link of a directory.
* EPERM | The operation is not allowed or access is denied because the access is prohibited or the directory is full.
* EROFS | The physical drive is write-protected.
* EEXIST | The file or directory already exists.
* ENOSYS | The operation is not supported.

* @since 1.0
* @version 1.0
*/
int chown(const char *pathname, uid_t owner, gid_t group);

#ifdef __cplusplus
}
#endif

#endif

/** @} */
