/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file wchar.h
*
* @brief Declares commonly used functions related to wide characters.
*
* You can use functions provided in this file to perform operations such as reading, writing, comparing, concatenating, copying, and searching wide characters. \n
* @since 1.0  \n
* @version 1.0  \n
*/

#ifndef _WCHAR_H
#define _WCHAR_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Copies the wide characters pointed to by <b>src</b> to the wide character array pointed to by <b>dest</b>, including the terminating null character <b>'\0'</b>. \n
*
* *
* @attention Ensure that the array pointed to by <b>dest</b> is large enough to accommodate the source characters. If the <b>dest</b> space is insufficient, unexpected exceptions may occur. Whenever a program reads data from or copies data to a buffer, it should first check whether there is enough space for the data. \n
* @param dest Indicates the pointer to the array to accommodate the copied wide characters.
* @param src Indicates the pointer to the wide characters to be copied.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
wchar_t *wcscpy(wchar_t *dest, const wchar_t *src);

/**
* @brief Copies the first <b>n</b> wide characters pointed to by <b>src</b> to the wide character array pointed to by <b>dest</b>.
*
* @attention Ensure that the array pointed to by <b>dest</b> is large enough to accommodate the source characters. If the <b>dest</b> space is insufficient, unexpected exceptions may occur. Whenever a program reads data from or copies data to a buffer, it should first check whether there is enough space for the data. \n
* If the first <b>n</b> wide characters pointed to by <b>src</b> do not contain null characters, the character string copied to <b>dest</b> does not end with <b>null</b>.
* @param dest Indicates the pointer to the array to accommodate the copied wide characters.
* @param src Indicates the pointer to the wide characters to be copied.
* @param n Indicates the maximum number of wide characters to be copied.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
wchar_t *wcsncpy(wchar_t *dest, const wchar_t *src, size_t n);

/**
* @brief Appends a copy of the wide characters pointed to by <b>src</b> to the end of the wide character array pointed to by <b>dest</b> and adds a terminating null character <b>'\0'</b>.
*
*
* @attention Ensure that the wide character array pointed to by <b>dest</b> has enough space to accommodate the concatenated characters ({@link wcslen} (<b>dest</b>) + {@link wcslen} (<b>src</b>) + 1).
* @param dest Indicates the pointer to the array to accommodate the concatenated wide characters.
* @param src Indicates the pointer to the wide characters to be concatenated.
* @return Returns the pointer to the destination string after concatenation.
* @since  1.0
* @version 1.0
*/
wchar_t *wcscat(wchar_t *dest, const wchar_t *src);

/**
* @brief Appends a copy of the first <b>n</b> wide characters pointed to by <b>src</b> to the end of the wide characters pointed to by <b>dest</b> and adds a terminating null character <b>'\0'</b>.
*
*
* @attention Ensure that the wide character array pointed to by <b>dest</b> has enough space to accommodate the concatenated characters ({@link wcslen} (<b>dest</b>) + <b>n</b> + 1).
* @param dest Indicates the pointer to the array to accommodate the concatenated wide characters.
* @param src Indicates the pointer to the wide characters to be concatenated.
* @param n Indicates the maximum number of wide characters to be concatenated.
* @return Returns the pointer to the destination string after concatenation.
* @since  1.0
* @version 1.0
*/
wchar_t *wcsncat(wchar_t *dest, const wchar_t *src, size_t n);

/**
* @brief Compares each character in the string pointed to by <b>s1</b> with that in the string pointed to by <b>s2</b> in ASCII-code order.
*
* This comparison starts from the first character in the two strings. \n
* @param s1 Indicates the pointer to string 1 for comparison.
* @param s2 Indicates the pointer to string 2 for comparison.
* @return Returns <b>0</b> if the two strings are equal; returns the result of subtracting the ASCII code of string 2 from the ASCII code of string 1 if the first mismatch is found.
* @since  1.0
* @version 1.0
*/
int wcscmp(const wchar_t *s1, const wchar_t *s2);

/**
* @brief Compares the first <b>n</b> characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b> in ASCII-code order.
*
*
* @attention If <b>n</b> is greater than the length of string 1 or string 2, the comparison stops when reaching the terminating null character <b>L'\0'</b> for the first time.
* @param s1 Indicates the pointer to string 1 for comparison.
* @param s2 Indicates the pointer to string 2 for comparison.
* @param n Indicates the maximum number of characters to be compared.
* @return Returns <b>0</b> if the two strings are equal; returns the result of subtracting the ASCII code of string 2 from the ASCII code of string 1 if the first mismatch is found.
* @since 1.0
* @version 1.0
*/
int wcsncmp(const wchar_t *s1, const wchar_t *s2, size_t n);

/**
* @brief Compares the wide characters in the string pointed to by <b>ws1</b> with those in the string pointed to by <b>ws2</b> based on the specified locale {@link LC_COLLATE}.
*
*
* @attention By default, the program's current locale specified by {@link LC_COLLATE} is <b>POSIX</b> or <b>C</b>.
* @param ws1 Indicates the pointer to string 1 for comparison.
* @param ws2 Indicates the pointer to string 2 for comparison.
* @return Returns <b>0</b> if the two strings are equal; returns the result of subtracting the ASCII code of string 2 from the ASCII code of string 1 if the first mismatch is found.
* @since 1.0
* @version 1.0
*/
int wcscoll(const wchar_t *ws1, const wchar_t *ws2);

/**
* @brief Compares the first <b>n</b> wide characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b>.
*
*
* @param s1 Indicates the pointer to wide character string 1 for comparison.
* @param s2 Indicates the pointer to wide character string 2 for comparison.
* @param n Indicates the maximum number of wide characters to be compared.
* @return Returns <b>0</b> if the two strings are equal; returns a positive integer if <b>s1</b> is greater than <b>s2</b>; returns a negative integer if <b>s1</b> is less than <b>s2</b>.
* @since 1.0
* @version 1.0
*/
size_t wcsxfrm(wchar_t *s1, const wchar_t *s2, size_t n);

/**
* @brief Locates the first occurrence of the wide character pointed to by <b>wc</b> in the wide character string pointed to by <b>wcs</b>.
*
*
* @param wcs Indicates the pointer to the wide character string to match the specified wide character.
* @param wc Indicates the wide character to be located.
* @return Returns the pointer to the first occurrence of the wide character; returns <b>NULL</b> if no match is found in the wide character string.
* @since  1.0
* @version 1.0
*/
wchar_t *wcschr(const wchar_t *wcs, wchar_t wc);

/**
* @brief Locates the last occurrence of the wide character pointed to by <b>wc</b> in the wide character string pointed to by <b>wcs</b>.
*
*
* @param wcs Indicates the pointer to the wide character string to match the specified wide character.
* @param wc Indicates the wide character to be located.
* @return Returns the pointer to the last occurrence of the wide character; returns <b>NULL</b> if no match is found in the wide character string.
* @since  1.0
* @version 1.0
*/
wchar_t *wcsrchr(const wchar_t *wcs, wchar_t wc);

/**
* @brief Scans the wide character string pointed to by <b>wcs</b> for any wide characters specified in <b>reject</b> and obtains the number of unmatched characters in <b>wcs</b>.
*
*
* @param wcs Indicates the pointer to the wide character string to be scanned.
* @param accept Indicates the pointer to the wide characters to be matched with the string.
* @return Returns the number of unmatched characters in <b>wcs</b>.
* @since  1.0
* @version 1.0
*/
size_t wcscspn(const wchar_t *wcs, const wchar_t *accept);

/**
* @brief Scans the wide character string pointed to by <b>wcs</b> for any wide characters specified in <b>reject</b> and obtains the number matched characters in <b>wcs</b>.
*
* @param wcs Indicates the pointer to the wide character string to be scanned.
* @param accept Indicates the pointer to the wide characters to be matched with the string.
* @return Returns the number of matched characters in <b>wcs</b>.
* @since  1.0
* @version 1.0
*/
size_t wcsspn(const wchar_t *wcs, const wchar_t *accept);

/**
* @brief Scans the wide character string pointed to by <b>wcs</b> for any wide characters specified in <b>accept</b> and obtains the first occurrence of the matched character.
*
*
* @param wcs Indicates the pointer to the wide character string to be scanned.
* @param accept Indicates the pointer to the wide characters to be matched with the string.
* @return Returns the pointer to the first occurrence of the matched character; returns <b>NULL</b> if no match is found.
* @since  1.0
* @version 1.0
*/
wchar_t *wcspbrk(const wchar_t *wcs, const wchar_t *accept);

/**
* @brief Splits a wide character string pointed to by <b>wcs</b> into tokens using the given delimiter.
*
*
* @attention In each call to this function, the string is split only once. You can call this function multiple times to split the string into a set of tokens and use <b>ptr</b> to point to the rest part in the source string.
* @param wcs Indicates the pointer to the first token in the wide character string to be split. This parameter is set to <b>NULL</b> in subsequent calls to this function.
* @param delim Indicates the pointer to the delimiter used to split tokens.
* @param ptr Indicates the double pointer to the rest part in the source string after splitting.
* @return Returns the pointer to the first token in the string if the operation is successful; returns <b>NULL</b> if no delimiter is found.
* @since 1.0
* @version 1.0
*/
wchar_t *wcstok(wchar_t *wcs, const wchar_t *delim, wchar_t **ptr);

/**
* @brief Calculates the length of a wide character string pointed to by <b>s</b>.
*
* The length does not include the terminating null character <b>'\0'</b>. \n
* @param s Indicates the pointer to the wide character string to be calculated. This parameter cannot be <b>NULL</b>.
* @return Returns the length of the string.
* @since  1.0
* @version 1.0
*/
size_t wcslen(const wchar_t *s);

/**
* @brief Searches the wide character string pointed to by <b>dest</b> for the first occurrence of the wide character string pointed to by <b>src</b>.
*
* This match does not include the terminating null character <b>'\0'</b>. \n
* @param haystack Indicates the pointer to the wide character string to be searched.
* @param needle Indicates the pointer to the wide character string to match.
* @return Returns the pointer to the first occurrence of the specified wide character string.
* @since  1.0
* @version 1.0
*/
wchar_t *wcswcs(const wchar_t *haystack, const wchar_t *needle);

/**
* @brief Searches for the first position of the matched wide character in a wide character string.
*
* @param haystack Indicates the pointer to the wide character string to be searched.
* @param needle Indicates the pointer to the wide character to be matched.
* @return Returns the pointer to the matched wide character if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
wchar_t *wcswcs(const wchar_t *haystack, const wchar_t *needle);

/**
* @brief Searches for the first position of the matched wide character within the specified number of characters in a wide character string.
*
* @param s Indicates the pointer to the wide character string to be searched.
* @param c Indicates the wide character to be matched.
* @param n Indicates the number of characters to be searched in the wide character string.
* @return Returns the pointer to the matched wide character if the operation is successful; returns <b>NULL</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
wchar_t *wmemchr(const wchar_t *s, wchar_t c, size_t n);

/**
* @brief Compares the first <b>count</b> characters in the string pointed to by <b>lhs</b> with the first <b>count</b> characters in the string pointed to by <b>rhs</b>.
*
* @attention This function compares character strings in lexicographical order. It is locale-insensitive and does not pay attention to the value of the <b>wchar_t</b> object. It does not stop comparison even when encountering a null or invalid wide character.
* @param lhs Indicates the pointer to string 1 for comparison.
* @param rhs Indicates the pointer to string 2 for comparison.
* @param count Indicates the number of characters to compare.
* @return Returns <b>0</b> if the first <b>count</b> characters of both strings are equal. \n
* If the wide character value of <b>lhs</b> is greater than that of <b>rhs</b> at the index (<b>i</b>) where the first mismatch is found, this function returns a value greater than <b>0</b>. \n
* If the wide character value of <b>lhs</b> is less than that of <b>rhs</b> at the index (<b>i</b>) where the first mismatch is found, this function returns a value less than <b>0</b>. \n
* @since 1.0
* @version 1.0
*/
int wmemcmp(const wchar_t *lhs, const wchar_t *rhs, size_t count);

/**
* @brief Copies <b>count</b> successive characters from the wide character array pointed to by <b>src</b> to the wide character array pointed to by <b>dest</b>.
*
*
* @attention If the arrays overlap (using the same memory), the behavior is undefined. If <b>count</b> is <b>0</b>, this function does nothing.
* @param dest Indicates the pointer to the wide character array to copy characters to.
* @param src Indicates the pointer to the wide character array to copy characters from.
* @param count Indicates the number of wide characters to copy.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
wchar_t *wmemcpy(wchar_t *dest, const wchar_t *src, size_t count);

/**
* @brief Copies <b>count</b> successive characters from the wide character array pointed to by <b>src</b> to the wide character array pointed to by <b>dest</b> (with possible array overlapping).
*
*
* @attention If <b>count</b> is <b>0</b>, this function does nothing. The arrays may overlap. If the arrays overlap, a copying operation is triggered, as if wide characters were copied to a temporary wide character array and then copied from the temporary array to the array pointed to by <b>dest</b>.
* @param dest Indicates the pointer to the wide character array to copy characters to.
* @param src Indicates the pointer to the wide character array to copy characters from.
* @param count Indicates the number of wide characters to copy.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
wchar_t *wmemmove(wchar_t *dest, const wchar_t *src, size_t count);

/**
* @brief Fills <b>count</b> characters specified by <b>ch</b> to the wide character array pointed to by <b>dest</b>.
*
*
* @attention If <b>count</b> is <b>0</b>, this function does nothing. This function is locale-insensitive and does not pay attention to the value of the <b>wchar_t</b> object, which can be null or an invalid wide character.
* @param dest Indicates the pointer to the wide character array to fill characters to.
* @param ch Indicates the wide characters to be filled.
* @param count Indicates the number of wide characters to be filled.
* @return Returns the pointer to the destination string.
* @since 1.0
* @version 1.0
*/
wchar_t *wmemset(wchar_t *dest, wchar_t ch, size_t count);

/**
* @brief Converts a single-byte character <b>c</b> into its wide-character representation.
*
* @attention If <b>c</b> is <b>EOF</b> or its length is not 1, <b>WEOF</b> is returned.
* @param c Indicates the character to be converted.
* @return Returns the wide-character representation if the operation is successful; returns <b>WEOF</b> if <b>c</b> is <b>EOF</b> or its length is not 1.
* @since 1.0
* @version 1.0
*/
wint_t btowc(int c);

/**
* @brief Converts a wide character <b>c</b> into its single-byte representation.
*
* This function works only if the multi-byte character representation of this wide character <b>c</b> is a single-byte character in its initial state. \n
* @attention Use this function with caution. Internationalized programs should never distinguish between single-byte and multi-byte representations.
* @param c Indicates the wide character to be converted.
* @return Returns the converted single-byte representation if the operation is successful; returns <b>EOF</b> otherwise.
* @since  1.0
* @version 1.0
*/
int wctob(wint_t c);

/**
* @brief Checks whether the <b>mbstate_t</b> object pointed to by <b>ps</b> is in the initial state.
*
*
* @param ps Indicates the pointer to the <b>mbstate_t</b> object to check.
* @return Returns a non-zero value if <b>ps</b> points to an object that is in the initial state or it is a null pointer; returns <b>0</b> otherwise.
* @since  1.0
* @version 1.0
*/
int mbsinit(const mbstate_t *ps);

/**
* @brief Converts the wide character specified by <b>wc</b> into a character string and stores the string to the beginning of the character array pointed to by <b>s</b>.
*
*
* @param s Indicates the pointer to the address storing the converted string.
* @param wc Indicates the wide character to be converted.
* @param ps Indicates the pointer to the <b>mbstate_t</b> object describing the conversion state.
* @return Returns the number of characters written to the character string if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to {@link EILSEQ} if the operation fails.
* @since  1.0
* @version 1.0
*/
size_t wcrtomb(char *s, wchar_t wc, mbstate_t *ps);

/**
* @brief Determines the number of bytes in a character string pointed to by <b>s</b>.
*
* If <b>s</b> is not <b>NULL</b>, this function checks at most <b>n</b> bytes in the string of <b>s</b>.
* It also uses a static anonymous shift state pointed to by <b>ps</b>. \n
* @attention The behavior of this function depends on the {@link LC_CTYPE} type of the current locale.
* @param s Indicates the pointer to the string to check.
* @param n Indicates the maximum number of bytes to check.
* @param ps Indicates the pointer to the <b>mbstate_t</b> object describing the conversion state.
* @return Returns the number of bytes in the string if the operation is successful. \n
* If <b>s</b> is a null pointer, this function returns <b>0</b>. \n
* If the characters in <b>s</b> are invalid or cannot be parsed, this function returns <b>-1</b>. \n
* If the first <b>n</b> bytes in the string of <b>s</b> do not contribute to a complete multi-byte character, this function returns <b>(size_t)-2</b>. \n
* If the first <b>n</b> bytes in the string of <b>s</b> contain an invalid multi-byte character, this function returns <b>(size_t)-1</b> and sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
size_t mbrlen(const char *s, size_t n, mbstate_t *ps);

/**
* @brief Converts a multi-byte character string with a length of <b>len</b> into a wide character string.
*
*
* @attention <b>dest</b> cannot be <b>NULL</b>.
* @param dest Indicates the pointer to the destination wide character array to store the converted string.
* @param src Indicates the double pointer to the source string to be converted.
* @param len Indicates the number of wide characters to be converted.
* @param ps Indicates the pointer to the <b>mbstate_t</b> object describing the conversion state.
* @return Returns the number of characters written to the destination wide character array. \n
* If an invalid character is encountered, this function returns <b>len-1</b> and sets <b>errno</b> to {@link EILSEQ}. \n
* If <b>len</b> non-<b>L'\0'</b> characters have been successfully written to the destination wide character array, <b>src</b> points to the next character to be converted, and this function returns the number of wide characters successfully written to the destination array. \n
* If the converted wide character is <b>L'\0'</b>, this function returns <b>0</b> and resets <b>* ps</b> to the initial state.
* @since  1.0
* @version 1.0
*/
size_t mbsrtowcs(wchar_t *dest, const char **src, size_t len, mbstate_t *ps);

/**
* @brief Converts a wide character string into a multi-byte string.
*
*
* @param dest Indicates the pointer to the character string array to accommodate the multi-byte string.
* @param src Indicates the double pointer to the source string to be converted.
* @param len Indicates the maximum number of bytes to be written to the destination string array.
* @param ps Indicates the pointer to the <b>mbstate_t</b> object describing the conversion state.
* @return Returns the number of converted bytes, excluding the terminating null byte <b>'\0'</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to {@link EILSEQ} if the operation fails.
* @since  1.0
* @version 1.0
*/
size_t wcsrtombs(char *dest, const wchar_t **src,size_t len, mbstate_t *ps);

/**
* @brief Converts a wide character string pointed to by <b>str</b> into a floating-point value and assigns the next character in <b>str</b> after the floating-point value to <b>endptr</b>.
*
*
* @attention If the first character in <b>str</b> is a space, this function discards it until the first non-space character is found.
* @param str Indicates the pointer to the wide character string to be converted.
* @param endptr Indicates the double pointer to the next character in <b>str</b> after the floating-point value.
* @return Returns the converted value if the operation is successful. \n
* If no conversion can be performed, this function returns <b>0</b>. \n
* If the correct value is out of the range of representable values for the type, this function returns a positive or negative {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}. \n
* @since  1.0
* @version 1.0
*/
float wcstof(const wchar_t *str, wchar_t **endptr);

/**
* @brief Converts a wide character string pointed to by <b>str</b> into a double value and assigns the next character in <b>str</b> after the double value to <b>endptr</b>.
*
*
* @attention If the first character in <b>str</b> is a space, this function discards it until the first non-space character is found.
* @param str Indicates the pointer to the wide character string to be converted.
* @param endptr Indicates the double pointer to the next character in <b>str</b> after the double value.
* @return Returns the converted value if the operation is successful. \n
* If no conversion can be performed, this function returns <b>0</b>.\n
* If the correct value is out of the range of representable values for the type, this function returns a positive or negative {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}. \n
* @since  1.0
* @version 1.0
*/
double wcstod(const wchar_t *str, wchar_t **endptr);

/**
* @brief Converts a wide character string pointed to by <b>str</b> into a long double value and assigns the next character in <b>str</b> after the long double value to <b>endptr</b>.
*
*
* @attention If the first character in <b>str</b> is a space, this function discards it until the first non-space character is found.
* @param str Indicates the pointer to the wide character string to be converted.
* @param endptr Indicates the double pointer to the next character in <b>str</b> after the long double value.
* @return Returns the converted value if the operation is successful. \n
* If no conversion can be performed, this function returns <b>0</b>.\n
* If the correct value is out of the range of representable values for the type, this function returns a positive or negative {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}.\n
* @since  1.0
* @version 1.0
*/
long double wcstold(const wchar_t *str, wchar_t **endptr);

/**
* @brief Converts a wide character string pointed to by <b>str</b> into a long value.
*
*
* @attention If the first character in <b>str</b> is a space, this function discards it until the first non-space character is found.
* @param str Indicates the pointer to the wide character string to be converted.
* @param endptr Indicates the double pointer to the next character in <b>str</b> after the long value.
* @param base Indicates the base of the converted value.
* @return Returns the converted value if the operation is successful. \n
* If no conversion can be performed, this function returns <b>0</b>.\n
* If the correct value is out of the range of representable values for the type, this function returns a positive or negative {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}.\n
* If the correct value would cause an underflow, this function returns a value no greater than the smallest normalized positive number. \n
* @since  1.0
* @version 1.0
*/
long wcstol(const wchar_t *str, wchar_t **endptr, int base);

/**
* @brief Converts a wide character string pointed to by <b>str</b> into an unsigned long value of a specified base.
*
*
* @attention This function discards as many spaces as possible until the first non-space character is found.
* @param str Indicates the pointer to the wide character string to be converted.
* @param endptr Indicates the double pointer to the next character in <b>str</b> after the unsigned long value.
* @param base Indicates the base of the converted value.
* @return Returns the converted value if the operation is successful. \n
* If no conversion can be performed, this function returns <b>0</b>.\n
* If the correct value is out of the range of representable values for the type, this function returns a positive or negative {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}.\n
* If the correct value would cause an underflow, this function returns a value no greater than the smallest normalized positive number. \n
* @since  1.0
* @version 1.0
*/
unsigned long wcstoul(const wchar_t *str, wchar_t **endptr, int base);

/**
* @brief Converts a wide character string pointed to by <b>str</b> into a long long value of a specified base.
*
*
* @attention This function discards as many spaces as possible until the first non-space character is found.
* @param str Indicates the pointer to the wide character string to be converted.
* @param endptr Indicates the double pointer to the next character in <b>str</b> after the long long value.
* @param base Indicates the base of the converted value.
* @return Returns the converted value if the operation is successful. \n
* If no conversion can be performed, this function returns <b>0</b>. \n
* If the correct value is out of the range of representable values for the type, this function returns a positive or negative {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}.\n
* If the correct value would cause an underflow, this function returns a value no greater than the smallest normalized positive number. \n
* @since  1.0
* @version 1.0
*/
long long wcstoll(const wchar_t *str, wchar_t **endptr, int base);

/**
* @brief Converts a wide character string pointed to by <b>str</b> into an unsigned long long value of a specified base.
*
*
* @attention This function discards as many spaces as possible until the first non-space character is found.
* @param str Indicates the pointer to the wide character string to be converted.
* @param endptr Indicates the double pointer to the next character in <b>str</b> after the unsigned long long value.
* @param base Indicates the base of the converted value.
* @return Returns the converted value if the operation is successful. \n
* If no conversion can be performed, this function returns <b>0</b>.\n
* If the correct value is out of the range of representable values for the type, this function returns a positive or negative {@link HUGE_VAL} and sets <b>errno</b> to {@link ERANGE}.\n
* If the correct value would cause an underflow, this function returns a value no greater than the smallest normalized positive number. \n
* @since  1.0
* @version 1.0
*/
unsigned long long wcstoull(const wchar_t *str, wchar_t **endptr, int base);

/**
* @brief Sets and determines the orientation of the file stream.
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param mode Indicates the orientation of the stream. The value can be: \n
* A negative value indicating that the stream is byte-oriented \n
* A positive value indicating that the stream is wide-oriented \n
* <b>0</b> indicating that the stream orientation is not set \n
* @return Returns a positive value if the stream is wide-oriented; returns a negative value if the stream is byte-oriented; returns <b>0</b> if the stream has no orientation.
* @since  1.0
* @version 1.0
*/
int fwide(FILE *stream, int mode);

/**
* @brief Prints formatted data to the standard output (stdout).
*
*
* @attention This function is the wide character equivalent of the {@link printf} function.
* @param format Indicates the pointer to the string that may contain the format specifiers.
* @param ... Indicates the variable arguments specifying the data to print.
* @return Returns the total number of output characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns <b>-1</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
int wprintf(const wchar_t *format, ...);

/**
* @brief Prints wide character strings to a specified file stream.
*
*
* @param fp Indicates the pointer to the stream to print data.
* @param fmt Indicates the pointer to the output format of the data.
* @return Returns the number of wide characters that are successfully printed; returns <b>-1</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
int fwprintf(FILE *__restrict fp, const wchar_t *__restrict fmt, ...);

/**
* @brief Prints formatted data to a specified string.
*
*
* @attention A maximum of <b>maxlen-1</b> valid characters can be printed to the destination string.
* @param wcs Indicates the pointer to the destination string.
* @param maxlen Indicates the maximum length of the destination string.
* @param format Indicates the pointer to the format string that contains the formatted data.
* @param .... Indicates the variable arguments written to the formatted data.
* @return Returns the number of formatted characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns <b>-1</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
int swprintf(wchar_t *wcs, size_t maxlen, const wchar_t *format, ...);

/**
* @brief Prints formatted data from a variable argument list to the standard output (stdout).
*
*
* @attention This function is generally used together with {@link va_start} and {@link va_end} and is the wide character equivalent of the {@link vprintf} function.
* @param format Indicates the pointer to the string that may contain the format specifiers.
* @param args Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the total number of output characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns a negative value otherwise.
* @since 1.0
* @version 1.0
*/
int vwprintf(const wchar_t *format, va_list args);

/**
* @brief Prints formatted data from a variable argument list specified by <b>args</b> to a specified file stream.
*
*
* @attention This function is generally used together with {@link va_start} and {@link va_end} and is the wide character equivalent of the {@link vfprintf} function.
* @param stream Indicates the pointer to a file object that identifies an output stream.
* @param format Indicates the pointer to the wide character string that may contain the format specifiers.
* @param args Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of written characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns <b>-1</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
int vfwprintf(FILE *stream, const wchar_t *format, __isoc_va_list args);

/**
* @brief Prints formatted data from a variable argument list specified by <b>args</b> to a specified string.
*
*
* @attention The destination wide character string must end with the terminating null character <b>'\0'</b>. This function is generally used together with {@link va_start} and {@link va_end} and is the wide character equivalent of the {@link vsnprintf} function.
* @param wcs Indicates the pointer to the destination wide character string.
* @param maxlen Indicates the maximum length of the destination wide character string.
* @param format Indicates the pointer to the string that may contain the format specifiers.
* @param args Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of written characters (excluding the terminating null character <b>'\0'</b>) if the operation is successful; returns <b>-1</b> if the operation fails.
* @since 1.0
* @version 1.0
*/
int vswprintf(wchar_t *wcs, size_t maxlen, const wchar_t *format, __isoc_va_list args);

/**
* @brief Reads formatted data from the standard input (stdin) and stores it based on the wide string format into the locations pointed to by the variable arguments.
*
*
* @attention This function is the wide character equivalent of the {@link scanf} function.
* @param format Indicates the pointer to the string that may contain the format specifiers.
* @param ... Indicates the variable arguments specifying the data to read.
* @return Returns the number of read bytes if the operation is successful; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when wide characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int wscanf(const wchar_t *format, ...);

/**
* @brief Reads formatted data from a specified stream and stores it based on the wide string format into the locations pointed to by the variable arguments.
*
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @param format Indicates the pointer to the string that may contain the format specifiers.
* @param  ... Indicates the variable arguments storing the input data.
* @return Returns the number of variable arguments that are successfully stored.
* @since 1.0
* @version 1.0
*/
int fwscanf(FILE *stream, const wchar_t *format, ...);

/**
* @brief Reads data from a wide character string pointed to by <b>ws</b> and stores it based on the wide string format into the locations pointed to by the variable arguments.
*
*
* @attention An asterisk ( * ) right after the percent symbol (such as <b>%*d</b> and <b>%*s</b>) in the format denotes that the value for the format will be read but will not be stored into a variable.
* @param ws Indicates the pointer to the wide character string to read.
* @param format Indicates the pointer to the format string that contains the formatted data.
* @param .... Indicates the variable arguments specifying the data to store.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading.
* @since 1.0
* @version 1.0
*/
int swscanf(const wchar_t *ws, const wchar_t *format, ...);

/**
* @brief Reads data from the stdin and stores it based on the wide string format into the locations pointed to by the elements in the variable argument list identified by <b>arg</b>.
*
*
* @attention This function is generally used together with {@link va_start} and {@link va_end} and is the wide character equivalent of the {@link vscanf} function.
* @param format Indicates the pointer to the string that may contain the format specifiers.
* @param arg Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when wide characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int vwscanf(const wchar_t *format, va_list arg);

/**
* @brief Reads data from a specified file stream and stores it based on the wide string format into the locations pointed to by the elements in the variable argument list identified by <b>arg</b>.
*
*
* @attention This function is generally used together with {@link va_start} and {@link va_end} and is the wide character equivalent of the {@link vfscanf} function.
* @param stream Indicates the pointer to a file object that identifies an output stream.
* @param format Indicates the pointer to the wide character string that may contain the format specifiers.
* @param arg Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when wide characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int vfwscanf(FILE *stream, const wchar_t *format, va_list arg);

/**
* @brief Reads data from a string pointed to by <b>ws</b> and stores it based on the wide string format into the locations pointed to by the elements in the variable argument list identified by <b>arg</b>.
*
*
* @attention This function is generally used together with {@link va_start} and {@link va_end} and is the wide character equivalent of the {@link vsscanf} function.
* @param ws Indicates the pointer to the wide character string to read.
* @param format Indicates the pointer to the wide character string that may contain the format specifiers.
* @param arg Indicates a value identifying a variable argument list initialized by using <b>va_start</b>.
* @return Returns the number of fields successfully assigned to the argument list if the operation is successful; returns <b>0</b> if no fields are assigned; returns <b>EOF</b> if a reading error occurs or the end-of-file is reached during data reading. \n
* If an encoding error occurs when wide characters are parsed, this function sets <b>errno</b> to {@link EILSEQ}.
* @since 1.0
* @version 1.0
*/
int vswscanf(const wchar_t *ws, const wchar_t *format, va_list arg);

/**
* @brief Reads a wide character from a specified file stream.
*
*
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the wide character if the operation is successful; returns <b>WEOF</b> if the operation fails or the end-of-file is reached during data reading.
* @since  1.0
* @version 1.0
*/
wint_t fgetwc(FILE *stream);

/**
* @brief Reads the first wide character from a specified file stream.
*
*
* @attention This function is similar to {@link fgetwc} except that it is implemented as a macro.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the wide character if the operation is successful; returns <b>WEOF</b> if an error occurs.
* @since 1.0
* @version 1.0
*/
wint_t getwc(FILE *stream);

/**
* @brief Reads a wide character from the stdin.
*
*
* @attention For details about unlocked objects, see {@link unlocked_stdio}.
* @return Returns the wide character if the operation is successful; returns <b>WEOF</b> if an error occurs.
* @since 1.0
* @version 1.0
*/
wint_t getwchar(void);

/**
* @brief Writes a wide character <b>wc</b> to a specified file stream.
*
*
* @param wc Indicates the wide character to write.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the wide character if the operation is successful; returns <b>WEOF</b> otherwise.
* @since  1.0
* @version 1.0
*/
wint_t fputwc(wchar_t wc, FILE *stream);

/**
* @brief Writes a wide character <b>wc</b> to a specified file stream.
*
*
* @attention This function is similar to {@link fputwc} only except that it is implemented as a macro.
* @param wc Indicates the wide character to write.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the wide character if the operation is successful; returns <b>WEOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
wint_t putwc(wchar_t wc, FILE *stream);

/**
* @brief Writes a wide character <b>wc</b> to the stdout.
*
*
* @param wc Indicates the wide character to write.
* @return Returns the wide character if the operation is successful; returns <b>WEOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
wint_t putwchar(wchar_t wc);

/**
* @brief Reads wide characters from a specified file stream.
*
* @param ws Indicates the pointer to the wide character array that stores the characters to read.
* @param n Indicates the number of characters to read.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns the pointer to the read string if the operation is successful; returns a null pointer if no characters are read or the end-of-file is reached during data reading.
* @since  1.0
* @version 1.0
*/
wchar_t *fgetws(wchar_t *ws, int n, FILE *stream);

/**
* @brief Writes a wide string pointed to by <b>ws</b> to a specified file stream.
*
*
* @param ws Indicates the pointer to the wide string to write.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns a non-negative value if the operation is successful; returns <b>-1</b> (EOF) otherwise.
* @since  1.0
* @version 1.0
*/
int fputws(const wchar_t *ws, FILE *stream);

/**
* @brief Pushes a character back into a specified file stream.
*
*
* @attention If you call this function multiple times without any character inserting or reading operation or file stream repositioning, the calls will fail.
* @param ch Indicates the character to push back.
* @param stream Indicates the pointer to the file object that identifies a stream.
* @return Returns <b>ch</b> if the operation is successful; returns <b>EOF</b> otherwise.
* @since 1.0
* @version 1.0
*/
wint_t ungetwc(wint_t ch, FILE *stream);

/**
* @brief Converts a multi-byte character string with a length of <b>n</b> into a wide character string.
*
*
* @param pwc Indicates the pointer to the destination wide character array to store the converted string.
* @param s Indicates the pointer to the multi-byte character string to be converted.
* @param n Indicates the number of bytes to read from <b>s</b>.
* @param ps Indicates the pointer to the <b>mbstate_t</b> object describing the conversion state.
* @return Returns the number of characters written to the destination wide character array. \n
* If the first <b>n</b> bytes in the string of <b>s</b> do not contribute to a complete multi-byte character, this function returns <b>(size_t)-2</b>. \n
* If the first <b>n</b> bytes in the string of <b>s</b> contain an invalid multi-byte character, this function returns <b>(size_t)-1</b> and sets <b>errno</b> to {@link EILSEQ}. \n
* If the converted wide character is <b>L'\ 0'</b>, this function returns <b>0</b> and resets <b>* ps</b> to the initial state. \n
* @since  1.0
* @version 1.0
*/
size_t mbrtowc(wchar_t *pwc, const char *s, size_t n, mbstate_t *ps);

/**
* @brief Converts a multi-byte character string with a length of <b>n</b> into a wide character string that can hold a total of <b>nwc</b> wide characters.
*
*
* @attention The <b>dest</b> value cannot be <b>NULL</b>.
* @param dest Indicates the pointer to the destination wide character array to store the converted string.
* @param src Indicates the double pointer to the source string to be converted.
* @param nwc Indicates the number of wide characters that can be written to the destination wide character array.
* @param len Indicates the number of characters to be converted.
* @param ps Indicates the pointer to the <b>mbstate_t</b> object describing the conversion state.
* @return Returns the number of characters written to the destination wide character array. \n
* If the first <b>n</b> bytes in the string of <b>s</b> contain an invalid multi-byte character, this function returns <b>(size_t)-1</b> and sets <b>errno</b> to {@link EILSEQ}. \n
* If more than <b>nwc</b> characters will be written or more than <b>len</b> characters are to be converted, <b>src</b> points to the next character to be converted, and this function returns the number of characters successfully written to the destination wide character array. \n
* If the converted wide character is <b>L'\ 0'</b>, this function returns <b>0</b> and resets <b>* ps</b> to the initial state. \n
* @since  1.0
* @version 1.0
*/
size_t mbsnrtowcs(wchar_t *dest, const char **src, size_t nwc, size_t len, mbstate_t *ps);

/**
* @brief Converts <b>nwc</b> wide characters in the string pointed to by <b>src</b> into a character string.
*
*
* @param dest Indicates the pointer to character array to accommodate the character string.
* @param src Indicates the double pointer to the source wide character string to be converted.
* @param nwc Indicates the number of wide characters to be converted.
* @param len Indicates the maximum number of bytes to be written to the destination string.
* @param ps Indicates the pointer to the <b>mbstate_t</b> object describing the conversion state.
* @return Returns the number of converted bytes, excluding the terminating null byte <b>'\0'</b> if the operation is successful; returns <b>-1</b> and sets <b>errno</b> to {@link EILSEQ} if the operation fails.
* @since  1.0
* @version 1.0
*/
size_t wcsnrtombs(char *dest, const wchar_t **src, size_t nwc,size_t len, mbstate_t *ps);

/**
* @brief Copies a specified wide character string to a newly allocated buffer.
*
*
* @attention This function allocates memory for variables. It is best practice to release the memory if you do not use the returned wide character string.
* @param s Indicates the pointer to the wide character string to copy.
* @return Returns the pointer to the new wide string buffer.
* @since  1.0
* @version 1.0
*/
wchar_t *wcsdup(const wchar_t *s);

/**
* @brief Calculates the length of a wide character string pointed to by <b>s</b>.
*
* If the length of the string exceeds <b>maxlen</b>, <b>maxlen</b> is returned. \n
* @attention The length of the string does not include the terminating null character <b>'\0'</b>.
* @param s Indicates the pointer to the wide character string whose length is to be calculated.
* @param maxlen Indicates the maximum length of the string whose length is to be calculated.
* @return Returns the length of the wide character string if the operation is successful; returns <b>maxlen</b> if the length of the string exceeds <b>maxlen</b>.
* @since  1.0
* @version 1.0
*/
size_t wcsnlen(const wchar_t *s, size_t maxlen);

/**
* @brief Copies the wide characters (including the terminating null character <b>'\0'</b>) pointed to by <b>src</b> to the wide character array pointed to by <b>dest</b>.
*
*
* @attention Ensure that the array pointed to by <b>dest</b> has sufficient space to hold the wide characters to be written.
* @param dest Indicates the pointer to the wide character array to copy the wide characters to.
* @param src Indicates the pointer to the wide characters to copy.
* @return Returns the pointer to the end of the destination wide character array.
* @since  1.0
* @version 1.0
*/
wchar_t *wcpcpy(wchar_t *dest, const wchar_t *src);

/**
* @brief Copies <b>n</b> wide characters (including the terminating null character <b>'\0'</b>) pointed to by <b>src</b> to the wide character array pointed to by <b>dest</b>.
*
*
* @attention Ensure that the array pointed to by <b>dest</b> has sufficient space to hold the wide characters to be written.
* @param dest Indicates the pointer to the array to accommodate the copied wide characters.
* @param src Indicates the pointer to the wide character string to be copied.
* @param n Indicates the maximum number of wide characters to be copied.
* @return Returns the pointer to the end of the destination wide character array.
* @since  1.0
* @version 1.0
*/
wchar_t *wcpncpy(wchar_t *dest, const wchar_t *src, size_t n);

/**
* @brief Compares the wide characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b>, with their case differences ignored.
*
*
* @param s1 Indicates the pointer to wide character string 1 for comparison.
* @param s2 Indicates the pointer to wide character string 2 for comparison.
* @return Returns <b>0</b> if the two wide character strings are equal; returns the code difference between the two wide character strings if they are not equal.
* @since  1.0
* @version 1.0
*/
int wcscasecmp(const wchar_t *s1, const wchar_t *s2);

/**
* @brief Compares the wide characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b> based on the specified <b>locale</b> environment, with their case differences ignored.
*
*
* @param s1 Indicates the pointer to wide character string 1 for comparison.
* @param s2 Indicates the pointer to wide character string 2 for comparison.
* @param locale Indicates the locale.
* @return Returns <b>0</b> if the two wide character strings are equal; returns the code difference between the two wide character strings if they are not equal.
* @since  1.0
* @version 1.0
*/
int wcscasecmp_l(const wchar_t *s1, const wchar_t *s2, locale_t locale);

/**
* @brief Compares a maximum of <b>n</b> wide characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b>, with their case differences ignored.
*
*
* @param s1 Indicates the pointer to wide character string 1 for comparison.
* @param s2 Indicates the pointer to wide character string 2 for comparison.
* @param n Indicates the maximum number of wide characters to be compared.
* @return Returns <b>0</b> if the two wide character strings are equal; returns the code difference between the two wide character strings if they are not equal.
* @since  1.0
* @version 1.0
*/
int wcsncasecmp(const wchar_t *s1, const wchar_t *s2, size_t n);

/**
* @brief Compares a maximum of <b>n</b> wide characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b> based on the specified <b>locale</b> environment, with their case differences ignored.
*
*
* @param s1 Indicates the pointer to wide character string 1 for comparison.
* @param s2 Indicates the pointer to wide character string 2 for comparison.
* @param n Indicates the maximum number of wide characters to be compared.
* @param locale Indicates the locale.
* @return Returns <b>0</b> if the two wide character strings are equal; returns the code difference between the two wide character strings if they are not equal.
* @since  1.0
* @version 1.0
*/
int wcsncasecmp_l(const wchar_t *s1, const wchar_t *s2, size_t n, locale_t locale);

/**
* @brief Compares wide characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b> based on the specified <b>locale</b> environment.
*
*
* @param s1 Indicates the pointer to wide character string 1 for comparison.
* @param s2 Indicates the pointer to wide character string 2 for comparison.
* @param locale Indicates the locale.
* @return Returns <b>0</b> if the two wide character strings are equal; returns the code difference between the two wide character strings if they are not equal.
* @since  1.0
* @version 1.0
*/
int wcscoll_l(const wchar_t *s1, const wchar_t *s2, locale_t locale);

/**
* @brief Converts the date and time in the <b>tm</b> structure to a wide character string.
*
*
* @param wcs Indicates the pointer to the wide character array to accommodate the converted string.
* @param n Indicates the size of the buffer for the wide character array.
* @param f Indicates the pointer to the required format.
* @param tm Indicates the pointer to the broken-down time in the <b>tm</b> structure.
* @return Returns the number of bytes in the string if the conversion is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
size_t wcsftime(wchar_t *__restrict wcs, size_t n, const wchar_t *__restrict f, const struct tm *__restrict tm);

/**
* @brief Compares the first <b>n</b> wide characters in the string pointed to by <b>s1</b> with those in the string pointed to by <b>s2</b> based on the specified <b>locale</b> environment.
*
*
* @param s1 Indicates the pointer to wide character string 1 for comparison.
* @param s2 Indicates the pointer to wide character string 2 for comparison.
* @param n Indicates the maximum number of wide characters to be compared.
* @param locale Indicates the locale.
* @return Returns <b>0</b> if the two wide character strings are equal; returns the code difference between the two wide character strings if they are not equal.
* @since  1.0
* @version 1.0
*/
size_t wcsxfrm_l(wchar_t *s1, const wchar_t *s2, size_t n, locale_t locale);

#ifdef __cplusplus
}
#endif

#endif

/** @} */