/**
* @addtogroup UTILS
* @{
*
* @brief Provides functions and data structures related to system operations, such as string conversion and I/O, string operations, process operations and so on.
*
* @since 1.0
* @version 1.0
*/

/**
* @file wctype.h
*
* @brief Provides functions to detect, translate, and map wide characters.
*
* Wide characters are classified into the following types: <b>alnum</b>, <b>alpha</b>, <b>upper</b>, <b>lower</b>, <b>digit</b>, <b>xdigit</b>, <b>graph</b>, <b>cntrl</b>, <b>punct</b>, and <b>space</b>.
* You can use these functions to detect the type of wide characters and translate the type for the current locale {@link LC_CTYPE} or a specified locale.
* You can use these functions to convert the case of wide characters, map the wide characters, and obtain the mapping descriptor for conversion.
* @since 1.0
* @version 1.0
*/
#ifndef _WCTYPE_H
#define _WCTYPE_H

#ifdef __cplusplus
extern "C" {
#endif

/**
* @brief Describes the wide character transliteration types.
*/
typedef const int * wctrans_t;

/**
* @brief Checks whether a wide character is a letter or digit.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswalnum(wint_t wc);
/**
* @brief Checks whether a wide character is a letter.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswalpha(wint_t wc);
/**
* @brief Checks whether a wide character is a space or tab character (\t).
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswblank(wint_t wc);
/**
* @brief Checks whether a wide character is a control character.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswcntrl(wint_t wc);
/**
* @brief Checks whether a wide character is a decimal digit.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswdigit(wint_t wc);

/**
* @brief Checks whether a wide character is visible.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswgraph(wint_t wc);
/**
* @brief Checks whether a wide character is a lowercase letter.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswlower(wint_t wc);
/**
* @brief Checks whether a wide character is printable.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswprint(wint_t wc);
/**
* @brief Checks whether a wide character is a punctuation mark.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswpunct(wint_t wc);

/**
* @brief Checks whether a wide character is an uppercase letter.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswupper(wint_t wc);

/**
* @brief Checks whether a wide character is a hexadecimal digit.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswxdigit(wint_t wc);
/**
* @brief Checks whether the character specified by <b>wc</b> belongs to the <b>desc</b> class.
*
* @param wc Indicates the wide character.
* @param desc Indicates the wide character class.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswctype(wint_t wc, wctype_t desc);
/**
* @brief Translates the type of a wide character based on the conversion mapping relationship.
*
* @param wc Indicates the wide character.
* @param desc Indicates the mapping relationship, which can be obtained from {@link wctrans()}.
* @return Returns the translated wide character if the operation is successful; returns <b>WEOF</b> if <b>wc</b> is <b>WEOF</b>.
* @since 1.0
* @version 1.0
*/
wint_t    towctrans(wint_t wc, wctrans_t desc);

/**
* @brief Converts an uppercase wide character to lowercase.
*
* @param wc Indicates the wide character.
* @return Returns the lowercase wide character if the conversion is successful; returns <b>wc</b> unchanged otherwise.
* @since 1.0
* @version 1.0
*/
wint_t    towlower(wint_t wc);

/**
* @brief Converts a lowercase wide character to uppercase.
*
* @param wc Indicates the wide character.
* @return Returns the uppercase wide character if the conversion is successful; returns <b>wc</b> unchanged otherwise.
* @since 1.0
* @version 1.0
*/
wint_t    towupper(wint_t wc);

/**
* @brief Determines a mapping which can map a wide character to another wide character.
*
* In the {@link LC_CTYPE} locale, the mapping is returned based on the <b>name</b> parameter. \n
* @param name Indicates the mapping name. The value can only be {@link tolower} (converting an uppercase wide character to lowercase) or {@link toupper} (converting a lowercase wide character to uppercase).
* @return Returns the mapping corresponding to <b>name</b> if the operation is successful; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
wctrans_t wctrans(const char *name);
/**
* @brief Checks whether a wide character type exists in the {@link LC_CTYPE} locale.
*
* @attention The wide character types are as follows:
* Type | Description
* ------------ | -----------------------------
* alnum | Letters and digits
* alpha | Letters
* cntrl | Control characters
* digit | Decimal digits
* graph | Visible characters
* lower | Lowercase letters
* print | Printable characters
* punct | Punctuation marks
* space | Space
* upper | Uppercase letters
* xdigit | Hexadecimal digits
* @param name Indicates the pointer to the wide character type.
* @return Returns the type descriptor if the check is successful; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
wctype_t wctype(const char *name);

/**
* @brief Checks whether a wide character is a letter or digit for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswalnum_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether a wide character is alphabetic for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswalpha_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether a wide character is a blank or <b>\t</b> character for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswblank_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether a wide character is a control character for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswcntrl_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether a wide character is a decimal digit for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswdigit_l(wint_t wc, locale_t locale);

/**
* @brief Checks whether a wide character is visible for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswgraph_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether a wide character is in lowercase for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswlower_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether a wide character is printable for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswprint_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether <b>wc</b> is a punctuation wide character for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswpunct_l(wint_t wc, locale_t locale);

/**
* @brief Checks whether a wide character belongs to the wide-character class <b>space</b>.
*
* @param wc Indicates the wide character.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int       iswspace(wint_t wc);

/**
* @brief Checks whether a wide character belongs to the wide-character class <b>space</b> for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswspace_l(wint_t wc, locale_t locale);

/**
* @brief Checks whether a wide character is in uppercase for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswupper_l(wint_t wc, locale_t locale);

/**
* @brief Checks whether a wide character is a hexadecimal digit for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswxdigit_l(wint_t wc, locale_t locale);
/**
* @brief Checks whether the character specified by <b>wc</b> belongs to the <b>desc</b> class for the specified locale.
*
* @param wc Indicates the wide character.
* @param desc Indicates the wide character class.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns a non-zero value if the operation is successful; returns <b>0</b> otherwise.
* @since 1.0
* @version 1.0
*/
int iswctype_l(wint_t wc, wctype_t desc, locale_t locale);

/**
* @brief Converts an uppercase wide character to lowercase for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the lowercase wide character if the conversion is successful; returns <b>wc</b> unchanged otherwise.
* @link duplocale
* @since 1.0
* @version 1.0
*/
wint_t towlower_l(wint_t wc, locale_t locale);

/**
* @brief Converts a lowercase wide character to uppercase for the specified locale.
*
* @param wc Indicates the wide character.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the uppercase wide character if the conversion is successful; returns <b>wc</b> unchanged otherwise.
* @link duplocale
* @since 1.0
* @version 1.0
*/
wint_t towupper_l(wint_t wc, locale_t locale);

/**
* @brief Translates the type of a wide character based on the translation mapping relationship for the specified locale.
*
* @param wc Indicates the wide character.
* @param desc Indicates the mapping relationship, which can be obtained from {@link wctrans_l()}.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the translated wide character if the operation is successful; returns <b>WEOF</b> if <b>wc</b> is <b>WEOF</b>.
* @since 1.0
* @version 1.0
*/
wint_t towctrans_l(wint_t wc, wctype_t desc, locale_t locale);

/**
* @brief Determines a mapping which can map a wide character to another wide character.
*
* In the {@link LC_CTYPE} locale, the mapping is returned based on the <b>name</b> parameter. \n
* @param name Indicates the mapping name. The value can only be <b>tolower</b> (converting an uppercase wide character to lowercase) or <b>toupper</b> (converting a lowercase wide character to uppercase).
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the mapping corresponding to <b>name</b> if the operation is successful; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
wctrans_t wctrans_l(const char *name, locale_t locale);

/**
* @brief Checks whether a wide character type exists for the specified locale.
*
* @attention The wide character types are as follows:
* Type | Description
* ------------ | -----------------------------
* alnum | Letters and digits
* alpha | Letters
* cntrl | Control characters
* digit | Decimal digits
* graph | Visible characters
* lower | Lowercase letters
* print | Printable characters
* punct | Punctuation marks
* space | Space
* upper | Uppercase letters
* xdigit | Hexadecimal digits
* @param name Indicates the pointer to the wide character type.
* @param locale Indicates the locale. This parameter is ignored currently.
* @return Returns the type descriptor if the check is successful; returns <b>0</b> otherwise.
* @since  1.0
* @version  1.0
*/
wctype_t wctype_l(const char *name, locale_t locale);

#ifdef __cplusplus
}
#endif

#endif

/** @} */
